@@ -1,5 +1,5 @@
 /*
- * $Id: casts.h 13160 2007-03-20 15:15:45Z cbiere $
+ * $Id: casts.h 14409 2007-08-11 16:51:00Z cbiere $
  *
  * Copyright (c) 2006 Christian Biere
  *
@@ -48,10 +48,10 @@ deconstify_gboolean(const gboolean *p)
 	return (gboolean *) p;
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gchar *
-deconstify_gchar(const gchar *p)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE char *
+deconstify_gchar(const char *p)
 {
-	return (gchar *) p;
+	return (char *) p;
 }
 
 static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE guint32 *
@@ -60,76 +60,84 @@ deconstify_guint32(const guint32 *p)
 	return (guint32 *) p;
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gpointer
-deconstify_gpointer(gconstpointer p)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE void * 
+deconstify_gpointer(const void *p)
 {
-	return (gpointer) p;
+	return (void *) p;
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gconstpointer
-cast_to_gconstpointer(gconstpointer p)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE const void *
+cast_to_gconstpointer(const void *p)
 {
 	return p; 
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gpointer
-cast_to_gpointer(gpointer p)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE void *
+cast_to_gpointer(void *p)
 {
 	return p;
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gchar *
-cast_to_gchar_ptr(gpointer p)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE char *
+cast_to_gchar_ptr(void *p)
 {
 	return p;
 }
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE guchar *
-cast_to_guchar_ptr(gpointer p)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE unsigned char *
+cast_to_guchar_ptr(void *p)
 {
 	return p;
 }
 
-/**
- * FIXME: Use uintptr_t if available.
- */
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gulong
-cast_ptr_to_uintptr(gpointer p)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE void *
+ulong_to_pointer(unsigned long value)
 {
-	gulong u = (gulong) p;
-	STATIC_ASSERT(sizeof u >= sizeof p);
-	return u;
+	return (void *) value;
 }
 
-/**
- * FIXME: Use uintptr_t if available.
- */
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gpointer
-cast_uintptr_to_ptr(gulong u)
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE unsigned long
+pointer_to_ulong(const void *p)
 {
-	gpointer p = (gpointer) u;
-	STATIC_ASSERT(sizeof p >= sizeof u);
-	return p;
+	return (unsigned long) p;
+}
+
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE void *
+uint_to_pointer(unsigned value)
+{
+	return ulong_to_pointer(value);
+}
+
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE unsigned
+pointer_to_uint(const void *p)
+{
+	return pointer_to_ulong(p);
+}
+
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE void *
+int_to_pointer(int value)
+{
+	return uint_to_pointer(value);
+}
+
+static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE int
+pointer_to_int(const void *p)
+{
+	return pointer_to_uint(p);
 }
 
 typedef void (*func_ptr_t)(void);
 
-static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE gpointer
-cast_func_to_gpointer(func_ptr_t f)
-{
-	return (gpointer) f;
-}
-
 static inline G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE func_ptr_t
-cast_gpointer_to_func(gconstpointer p)
+cast_pointer_to_func(const void *p)
 {
 	return (func_ptr_t) p;
 }
 
 static inline size_t G_GNUC_CONST WARN_UNUSED_RESULT ALWAYS_INLINE
-ptr_diff(gconstpointer a, gconstpointer b)
+ptr_diff(const void *a, const void *b)
 {
-	return (gchar *) a - (gchar *) b;
+	return (const char *) a - (const char *) b;
 }
 
 /**
