@@ -1,5 +1,5 @@
 /*
- * $Id: alive.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: alive.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_alive_h_
 #define _core_alive_h_
 
-#include <glib.h>
+#include "common.h"
 
 struct gnutella_node;
 
