@@ -1,5 +1,5 @@
 /*
- * $Id: ban.h 9244 2005-08-17 13:44:05Z cbiere $
+ * $Id: ban.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_ban_h_
 #define _core_ban_h_
 
-#include <glib.h>
+#include "common.h"
 #include "lib/host_addr.h"
 
 struct gnutella_socket;
@@ -58,7 +58,7 @@ ban_type_t ban_allow(const host_addr_t addr);
 void ban_record(const host_addr_t addr, const gchar *msg);
 void ban_force(struct gnutella_socket *s);
 gint ban_delay(const host_addr_t addr);
-gchar *ban_message(const host_addr_t addr);
+const gchar *ban_message(const host_addr_t addr);
 gboolean ban_is_banned(const host_addr_t addr);
 void ban_max_recompute(void);
 
