@@ -1,5 +1,5 @@
 /*
- * $Id: bh_download.c 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: bh_download.c 13864 2007-06-17 17:32:42Z cbiere $
  *
  * Copyright (c) 2005, Raphael Manfredi
  *
@@ -35,13 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: bh_download.c 12147 2006-10-27 19:55:09Z cbiere $")
+RCSID("$Id: bh_download.c 13864 2007-06-17 17:32:42Z cbiere $")
 
 #include "bh_download.h"
 #include "downloads.h"
 #include "pmsg.h"
 #include "bsched.h"
 #include "gnet_stats.h"
+#include "rx_inflate.h"
 
 #include "lib/atoms.h"
 #include "lib/endian.h"
@@ -56,7 +57,7 @@ struct browse_ctx {
 	rxdrv_t *rx;					/**< RX stack top */
 	gnet_host_t host;				/**< Host we're browsing, for logging */
 	gnet_search_t sh;				/**< Search ID to which hits are given */
-	gchar *vendor;					/**< Vendor version string */
+	const gchar *vendor;			/**< Vendor version string (atom) */
 	gnutella_header_t header;		/**< Received header */
 	gchar *data;					/**< Where payload data is stored */
 	guint data_size;				/**< Size of data buffer */
@@ -79,8 +80,6 @@ browse_host_dl_create(gpointer owner, gnet_host_t *host, gnet_search_t sh)
 	bc->owner = owner;
 	bc->host = *host;			/* Struct copy */
 	bc->sh = sh;
-	bc->data = g_malloc(BH_DL_DEFAULT_SIZE);
-	bc->data_size = BH_DL_DEFAULT_SIZE;
 
 	return bc;
 }
@@ -134,8 +133,8 @@ browse_data_read(struct browse_ctx *bc, pmsg_t *mb)
 		 */
 
 		if (bc->size > bc->data_size) {
-			bc->data = g_realloc(bc->data, bc->size);
-			bc->data_size = bc->size;
+			bc->data_size = MAX(BH_DL_DEFAULT_SIZE, bc->size);
+			bc->data = g_realloc(bc->data, bc->data_size);
 		}
 
 		bc->pos = 0;
@@ -227,7 +226,7 @@ browse_data_ind(rxdrv_t *rx, pmsg_t *mb)
 	download_check(d);
 
 	if (!error) {
-		download_browse_maybe_finished(d);
+		download_maybe_finished(d);
 		download_check(d);
 	}
 
@@ -244,7 +243,7 @@ browse_rx_given(gpointer o, ssize_t r)
 {
 	struct browse_ctx *bc = o;
 
-	download_browse_received(bc->owner, r);
+	download_data_received(bc->owner, r);
 }
 
 static G_GNUC_PRINTF(2, 3) void
@@ -325,7 +324,7 @@ browse_host_dl_receive(
 		struct rx_link_args args;
 
 		args.cb = &browse_rx_link_cb;
-		args.bs = bws.in;
+		args.bws = BSCHED_BWS_IN;
 		args.wio = wio;
 
 		bc->rx = rx_make(bc, &bc->host, rx_link_get_ops(), &args);
@@ -398,26 +397,33 @@ void
 browse_host_dl_close(struct browse_ctx *bc)
 {
 	g_assert(bc != NULL);
-	g_assert(bc->rx != NULL);
 
-	rx_disable(bc->rx);
+	if (bc->rx) {
+		rx_disable(bc->rx);
+	}
 }
 
 /**
  * Terminate host browsing.
  */
 void
-browse_host_dl_free(struct browse_ctx *bc)
+browse_host_dl_free(struct browse_ctx **ptr)
 {
-	g_assert(bc != NULL);
+	struct browse_ctx *bc = *ptr;
 
-	atom_str_free_null(&bc->vendor);
-	if (bc->rx)
-		rx_free(bc->rx);
-	if (!bc->closed)
-		search_dissociate_browse(bc->sh, bc->owner);
-	G_FREE_NULL(bc->data);
-	wfree(bc, sizeof *bc);
+	if (bc) {	
+		atom_str_free_null(&bc->vendor);
+		if (bc->rx) {
+			rx_free(bc->rx);
+			bc->rx = NULL;
+		}
+		if (!bc->closed) {
+			search_dissociate_browse(bc->sh, bc->owner);
+		}
+		G_FREE_NULL(bc->data);
+		wfree(bc, sizeof *bc);
+		*ptr = NULL;
+	}
 }
 
 /**
