@@ -1,5 +1,5 @@
 /*
- * $Id: bh_upload.h 11897 2006-09-06 06:47:45Z cbiere $
+ * $Id: bh_upload.h 13345 2007-04-16 20:18:13Z cbiere $
  *
  * Copyright (c) 2005, Christian Biere & Raphael Manfredi
  *
@@ -39,12 +39,7 @@
 
 #include "common.h"
 
-#include "tx.h"
-#include "tx_link.h"
-#include "tx_deflate.h"
-
-#include "if/core/hosts.h"
-#include "if/core/wrap.h"
+#include "special_upload.h"
 
 /**
  * The version of the Browse-Host protocol we support.
@@ -55,32 +50,27 @@
 /**
  * Flags for browse_host_open().
  */
-#define BH_DEFLATE	(1 << 0)		/**< Deflate output */
-#define BH_GZIP		(1 << 1)		/**< gzip output */
-#define BH_CHUNKED	(1 << 2)		/**< Emit chunked data */
-#define BH_HTML		(1 << 3)		/**< Emit HTML data */
-#define BH_QHITS	(1 << 4)		/**< Emit Gnutella query hits */
 
-struct gnutella_socket;
-struct bio_source;
-
-typedef void (*bh_closed_t)(gpointer arg);
-typedef void (*bh_writable_t)(gpointer arg);
-
-struct special_ctx {
-	txdrv_t *tx;
-	ssize_t (*read)(gpointer ctx, gpointer dest, size_t size);
-	ssize_t (*write)(gpointer ctx, gconstpointer data, size_t size);
-	void (*flush)(gpointer ctx, bh_closed_t cb, gpointer arg);
-	void (*close)(gpointer ctx, gboolean fully_served);
+enum {
+	BH_F_DEFLATE = 1 << 0,		/**< Deflate output */
+	BH_F_GZIP	 = 1 << 1,		/**< gzip output */
+	BH_F_CHUNKED = 1 << 2,		/**< Emit chunked data */
+	BH_F_HTML	 = 1 << 3,		/**< Emit HTML data */
+	BH_F_QHITS	 = 1 << 4		/**< Emit Gnutella query hits */
 };
 
-struct special_ctx *browse_host_open(
-	gpointer owner, gnet_host_t *host,
-	bh_writable_t writable,
-	struct tx_deflate_cb *deflate_cb,
-	struct tx_link_cb *link_cb,
-	wrap_io_t *wio,
+struct gnutella_host;
+struct tx_deflate_cb;
+struct tx_link_cb;
+struct wrap_io;
+
+struct special_upload *browse_host_open(
+	gpointer owner,
+	struct gnutella_host *host,
+	special_upload_writable_t writable,
+	const struct tx_deflate_cb *deflate_cb,
+	const struct tx_link_cb *link_cb,
+	struct wrap_io *wio,
 	gint flags);
 
 #endif /* _core_bh_upload_h_ */
