@@ -1,5 +1,5 @@
 /*
- * $Id: bogons.h 9244 2005-08-17 13:44:05Z cbiere $
+ * $Id: bogons.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_bogons_h_
 #define _core_bogons_h_
 
-#include <glib.h>
+#include "common.h"
 #include "lib/host_addr.h"
 
 gboolean bogons_check(const host_addr_t addr);
