@@ -1,5 +1,5 @@
 /*
- * $Id: clock.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: clock.c 14955 2007-09-28 19:25:45Z bpringlemeir $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: clock.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: clock.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
 
 #include "clock.h"
 
@@ -121,7 +121,7 @@ val_destroy(cqueue_t *unused_cq, gpointer obj)
 /**
  * Create a value for the `used' table.
  */
-struct used_val *
+static struct used_val *
 val_create(const host_addr_t addr, gint precision)
 {
 	struct used_val *v = walloc(sizeof *v);
