@@ -1,5 +1,5 @@
 /*
- * $Id: dmesh.c 13997 2007-06-30 22:43:57Z cbiere $
+ * $Id: dmesh.c 14869 2007-09-09 23:43:27Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: dmesh.c 13997 2007-06-30 22:43:57Z cbiere $")
+RCSID("$Id: dmesh.c 14869 2007-09-09 23:43:27Z cbiere $")
 
 #include "gnutella.h"
 #include "downloads.h"
@@ -49,6 +49,7 @@ RCSID("$Id: dmesh.c 13997 2007-06-30 22:43:57Z cbiere $")
 #include "fileinfo.h"
 #include "settings.h"
 #include "hosts.h"
+#include "g2_cache.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -1369,6 +1370,9 @@ dmesh_fill_alternate(const struct sha1 *sha1, gnet_host_t *hvec, gint hcnt)
 		if (NET_TYPE_IPV4 != host_addr_net(dme->url.addr))
 			continue;
 
+		if (g2_cache_lookup(dme->url.addr, dme->url.port))
+			continue;			/* Don't pollute with G2-only entries */
+
 		g_assert(i < MAX_ENTRIES);
 		selected[i++] = dme;
 	}
@@ -1594,6 +1598,7 @@ dmesh_alternate_location(const struct sha1 *sha1,
 			fi->done * 100 / fi->size > MIN_PFSP_PCT
 		) && upload_is_enabled()
 	) {
+		static const char tls_hex[] = "tls=8";	/* Only us at index zero */
 		size_t url_len;
 		struct dmesh_entry ourselves;
 		time_t now = tm_time();
@@ -1610,9 +1615,12 @@ dmesh_alternate_location(const struct sha1 *sha1,
 		url_len = dmesh_entry_compact(&ourselves, url, sizeof url);
 		g_assert((size_t) -1 != url_len && url_len < sizeof url);
 
-		if (!header_fmt_value_fits(fmt, url_len, maxslen))
+		if (!header_fmt_value_fits(fmt, url_len + strlen(tls_hex), maxslen))
 			goto nomore;
 
+		if (tls_enabled()) {
+			header_fmt_append_value(fmt, tls_hex);
+		}
 		header_fmt_append_value(fmt, url);
 		added = TRUE;
 	}
@@ -1674,6 +1682,9 @@ dmesh_alternate_location(const struct sha1 *sha1,
 		if (dme->url.idx != URN_INDEX)
 			continue;
 
+		if (g2_cache_lookup(dme->url.addr, dme->url.port))
+			continue;			/* Don't pollute with G2-only entries */
+
 		g_assert(i < MAX_ENTRIES);
 
 		selected[i++] = dme;
@@ -2271,6 +2282,7 @@ dmesh_multiple_downloads(const struct sha1 *sha1,
 			blank_guid,
 			NULL,	/* hostname */
 			sha1,
+			NULL,	/* TTH */
 			now,
 			fi,
 			NULL,	/* proxies */
