@@ -1,5 +1,5 @@
 /*
- * $Id: extensions.c 13831 2007-06-16 01:22:39Z cbiere $
+ * $Id: extensions.c 15154 2008-03-19 00:25:28Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: extensions.c 13831 2007-06-16 01:22:39Z cbiere $")
+RCSID("$Id: extensions.c 15154 2008-03-19 00:25:28Z cbiere $")
 
 #include <zlib.h>
 
@@ -147,6 +147,7 @@ static const struct rwtable urntable[] =
 {
 	{ "bitprint",		EXT_T_URN_BITPRINT },
 	{ "sha1",			EXT_T_URN_SHA1 },
+	{ "ttroot",			EXT_T_URN_TTH },
 };
 
 /** GGEP extension table (sorted) */
@@ -181,6 +182,7 @@ static const struct rwtable ggeptable[] =
 	GGEP_ID(P),			/**< Push alt-locs in HEAD Pongs */
 	GGEP_ID(PATH),		/**< Shared file path, in query hits */
 	GGEP_ID(PHC),		/**< Packed host caches (UHC) in pongs */
+	GGEP_ID(PR),		/**< Partial Result, in queries and hits */
 	GGEP_ID(PUSH),		/**< Push proxy info, in qhits */
 	GGEP_ID(PUSH_TLS),	/**< TLS-capability bitmap for GGEP PUSH */
 	GGEP_ID(Q),			/**< Queue status in HEAD Pongs */
