@@ -1,5 +1,5 @@
 /*
- * $Id: extensions.h 13831 2007-06-16 01:22:39Z cbiere $
+ * $Id: extensions.h 15154 2008-03-19 00:25:28Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -68,6 +68,7 @@ typedef enum ext_token {
 	EXT_T_UNKNOWN = 0,		/**< Unknown */
 	EXT_T_URN_BITPRINT,		/**< urn:bitprint: */
 	EXT_T_URN_SHA1,			/**< urn:sha1: */
+	EXT_T_URN_TTH,			/**< urn:ttroot: */
 	EXT_T_URN_EMPTY,		/**< urn: */
 	EXT_T_URN_BAD,			/**< urn */
 	EXT_T_XML,				/**< XML payload */
@@ -101,6 +102,7 @@ typedef enum ext_token {
 	EXT_T_GGEP_P,			/**< Push alt-locs in HEAD Pongs */
 	EXT_T_GGEP_PATH,		/**< Shared file path, in query hits */
 	EXT_T_GGEP_PHC,			/**< Packed HostCaches, in pongs (UHC) */
+	EXT_T_GGEP_PR,			/**< Partial Result, in queries and hits */
 	EXT_T_GGEP_PUSH,		/**< Push proxy info, in query hits */
 	EXT_T_GGEP_PUSH_TLS,	/**< TLS-capability bitmap for GGEP PUSH */
 	EXT_T_GGEP_Q,			/**< Queue status in HEAD Pongs */
