@@ -1,5 +1,5 @@
 /*
- * $Id: features.c,v 1.12 2005/08/15 13:07:51 rmanfredi Exp $
+ * $Id: features.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: features.c,v 1.12 2005/08/15 13:07:51 rmanfredi Exp $");
+RCSID("$Id: features.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "features.h"
 
@@ -54,18 +54,56 @@ struct header_x_feature {
 	int minor;
 };
 
-xfeatures_t xfeatures;
+struct features {
+	GList *list;
+};
+
+static struct features * 
+features_get(xfeature_t xf)
+{
+	static struct features features[NUM_FEATURES];
+	guint i;
+
+	i = (guint) xf;
+	g_return_val_if_fail(i < G_N_ELEMENTS(features), NULL);
+	return &features[i];
+}
 
 /***
  *** X-Features header parsing utilities
  ***/
 
+/**
+ * Removes all memory used by the header_features_add.
+ */
+static void
+header_features_cleanup(xfeature_t xf)
+{
+	struct features *features;
+	GList *cur;
+
+ 	features = features_get(xf);
+	g_return_if_fail(features);
+
+   	cur = g_list_first(features->list);
+	for (/* NOTHING */; NULL != cur; cur = g_list_next(cur)) {
+		struct header_x_feature *header = cur->data;
+
+		G_FREE_NULL(header->name);
+		wfree(header, sizeof *header);
+	}
+	g_list_free(features->list);
+	features->list = NULL;
+}
+
 void
 features_close(void)
 {
-	header_features_cleanup(&xfeatures.uploads);
-	header_features_cleanup(&xfeatures.downloads);
-	header_features_cleanup(&xfeatures.connections);
+	guint i;
+
+	for (i = 0; i < NUM_FEATURES; i++) {
+		header_features_cleanup(i);
+	}
 }
 
 /**
@@ -73,37 +111,23 @@ features_close(void)
  * header.
  */
 void
-header_features_add(struct xfeature_t *xfeatures,
-	gchar *feature_name,
+header_features_add(xfeature_t xf,
+	const gchar *feature_name,
 	int feature_version_major,
 	int feature_version_minor)
 {
-	struct header_x_feature *feature = walloc(sizeof(*feature));
+	struct header_x_feature *item;
+	struct features *features;
 
-	feature->name = g_strdup(feature_name);
-	feature->major = feature_version_major;
-	feature->minor = feature_version_minor;
+	features = features_get(xf);
+	g_return_if_fail(features);
 
-	xfeatures->features = g_list_append(xfeatures->features, feature);
-}
+	item = walloc(sizeof *item);
+	item->name = g_strdup(feature_name);
+	item->major = feature_version_major;
+	item->minor = feature_version_minor;
 
-/**
- * Removes all memory used by the header_features_add.
- */
-void
-header_features_cleanup(struct xfeature_t *xfeatures)
-{
-	GList *cur;
-	for(cur = g_list_first(xfeatures->features);
-		cur != g_list_last(xfeatures->features);
-		cur = g_list_next(cur)) {
-
-		struct header_x_feature *feature =
-			(struct header_x_feature *) cur->data;
-
-		G_FREE_NULL(feature->name);
-		wfree(feature, sizeof(*feature));
-	}
+	features->list = g_list_append(features->list, item);
 }
 
 /**
@@ -117,10 +141,10 @@ header_features_cleanup(struct xfeature_t *xfeatures)
  * *rw is changed too *rw + bytes written
  */
 void
-header_features_generate(struct xfeature_t *xfeatures,
-	gchar *buf, size_t len, size_t *rw)
+header_features_generate(xfeature_t xf, gchar *dst, size_t len, size_t *rw)
 {
 	static const char hdr[] = "X-Features";
+	struct features *features;
 	GList *cur;
 	gpointer fmt;
 
@@ -131,30 +155,28 @@ header_features_generate(struct xfeature_t *xfeatures,
 	if (len - *rw < (sizeof(hdr) + sizeof(": \r\n") - 1))
 		return;
 
-	if (g_list_first(xfeatures->features) == NULL)
+	features = features_get(xf);
+	g_return_if_fail(features);
+
+	if (g_list_first(features->list) == NULL)
 		return;
 
 	fmt = header_fmt_make(hdr, ", ", len - *rw);
 
-	for (
-		cur = g_list_first(xfeatures->features);
-		cur != NULL;
-		cur = g_list_next(cur)
-	) {
-		gchar feature_version[50];
-		struct header_x_feature *feature =
-			(struct header_x_feature *) cur->data;
+	for (cur = g_list_first(features->list); cur; cur = g_list_next(cur)) {
+		struct header_x_feature *item = cur->data;
+		gchar buf[50];
 
-		gm_snprintf(feature_version, sizeof(feature_version), "%s/%d.%d",
-			feature->name, feature->major, feature->minor);
+		gm_snprintf(buf, sizeof buf, "%s/%d.%d",
+			item->name, item->major, item->minor);
 
-		header_fmt_append_value(fmt, feature_version);
+		header_fmt_append_value(fmt, buf);
 	}
 
 	header_fmt_end(fmt);
 
 	if ((size_t) header_fmt_length(fmt) < len - *rw) {
-		*rw += gm_snprintf(&buf[*rw], len - *rw, "%s", header_fmt_string(fmt));
+		*rw += gm_snprintf(&dst[*rw], len - *rw, "%s", header_fmt_string(fmt));
 	}
 
 	header_fmt_free(fmt);
@@ -162,17 +184,20 @@ header_features_generate(struct xfeature_t *xfeatures,
 
 /**
  * Retrieves the major and minor version from a feature in the X-Features
- * header, if no support was found both major and minor are 0.
+ * header, if no support was found both major and minor are 0 and FALSE
+ * is returned.
  */
-void
+gboolean
 header_get_feature(const gchar *feature_name, const header_t *header,
 	guint *feature_version_major, guint *feature_version_minor)
 {
 	gchar *buf = NULL;
 	gchar *start;
 
-	*feature_version_major = 0;
-	*feature_version_minor = 0;
+	if (feature_version_major)
+		*feature_version_major = 0;
+	if (feature_version_minor)
+		*feature_version_minor = 0;
 
 	buf = header_get(header, "X-Features");
 
@@ -190,7 +215,7 @@ header_get_feature(const gchar *feature_name, const header_t *header,
 		 * Better safe than sorry.
 		 */
 
-		return;
+		return FALSE;
 	}
 
 	/*
@@ -198,13 +223,14 @@ header_get_feature(const gchar *feature_name, const header_t *header,
 	 * feature.  If we look for "bar", then we must not match on "foobar".
 	 */
 
-	for (start = buf;;) {
+	start = buf;
+	for (;;) {
 		gint pc;			/* Previous char */
 
 		buf = ascii_strcasestr(buf, feature_name);
 
 		if (buf == NULL)
-			return;
+			return FALSE;
 		if (buf == start)
 			break;
 
@@ -225,18 +251,20 @@ header_get_feature(const gchar *feature_name, const header_t *header,
 
 	if (*buf != '/') {
 		g_warning("[header] Malformed X-Features header, ignoring");
-		if (dbg > 2)
+		if (GNET_PROPERTY(dbg) > 2)
 			header_dump(header, stderr);
 
-		return;
+		return FALSE;
 	}
 
 	buf++;
 
 	if (*buf == '\0')
-		return;
+		return FALSE;
 
-	parse_major_minor(buf, NULL, feature_version_major, feature_version_minor);
+	
+	return 0 == parse_major_minor(buf, NULL,
+					feature_version_major, feature_version_minor);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
