@@ -1,5 +1,5 @@
 /*
- * $Id: features.h 12201 2006-11-03 05:14:47Z cbiere $
+ * $Id: features.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_features_h_
 #define _core_features_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "lib/header.h"		/* For header_t */
 
