@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.h 13987 2007-06-29 16:57:00Z cbiere $
+ * $Id: fileinfo.h 14801 2007-09-07 22:02:33Z cbiere $
  *
  * Copyright (c) 2002, Vidar Madsen
  *
@@ -46,21 +46,6 @@
 
 #define FI_LOW_SRC_COUNT	5			/**< Few sources known if beneath */
 
-/*
- * Operating flags.
- */
-
-enum {
-	FI_F_SUSPEND	= 1 << 0,	/**< Marked "suspended" new downloads */
-	FI_F_DISCARD	= 1 << 1,	/**< Discard fileinfo if refcount = 0 */
-	FI_F_TRANSIENT	= 1 << 2,	/**< Don't persist to disk */
-	FI_F_MARK		= 1 << 3,	/**< Marked during traversal */
-	FI_F_PAUSED		= 1 << 4,	/**< Paused by user */
-	FI_F_SEEDING	= 1 << 5,	/**< Seeding after successful download */
-	FI_F_STRIPPED	= 1 << 6,	/**< Fileinfo trailler has been stripped */
-	FI_F_FETCH_TTH	= 1 << 7	/**< Tigertree data is being downloaded */
-};
-
 /*
  * Public interface.
  */
@@ -71,7 +56,7 @@ void file_info_scandir(const gchar *dir);
 gint file_info_has_trailer(const gchar *path);
 void file_info_retrieve(void);
 void file_info_store(void);
-void file_info_store_binary(fileinfo_t *fi);
+void file_info_store_binary(fileinfo_t *fi, gboolean force);
 void file_info_store_if_dirty(void);
 void file_info_set_discard(fileinfo_t *fi, gboolean state);
 enum dl_chunk_status file_info_find_hole(
@@ -111,7 +96,9 @@ void file_info_remove_source(
 void file_info_timer(void);
 void file_info_unlink(fileinfo_t *fi);
 void file_info_upload_stop(fileinfo_t *fi, const gchar *reason);
-void file_info_changed(fileinfo_t *fi);
+void file_info_pause(fileinfo_t *);
+void file_info_resume(fileinfo_t *);
+void file_info_changed(fileinfo_t *);
 
 shared_file_t *file_info_shared_sha1(const struct sha1 *sha1);
 gint file_info_available_ranges(fileinfo_t *fi, gchar *buf, gint size);
@@ -130,6 +117,15 @@ typedef void (*file_info_foreach_cb)(gnet_fi_t fi_handle, gpointer udata);
 void file_info_foreach(file_info_foreach_cb callback, gpointer udata);
 gboolean file_info_purge(fileinfo_t *fi);
 
+char *file_info_unique_filename(const gchar *path, const gchar *file,
+	const gchar *ext);
+
+void fi_src_info_changed(struct download *);
+void fi_src_ranges_changed(struct download *);
+void fi_src_status_changed(struct download *);
+
+GSList *file_info_get_sources(const fileinfo_t *);
+
 /***
  *** Inlined routines.
  ***/
@@ -140,7 +136,7 @@ gboolean file_info_purge(fileinfo_t *fi);
 static inline G_GNUC_CONST WARN_UNUSED_RESULT guint32
 fi_alive_count(const fileinfo_t *fi)
 {
-	return fi->aqueued_count + fi->pqueued_count + fi->recvcount;
+	return fi->active_queued + fi->passive_queued + fi->recvcount;
 }
 
 #endif /* _core_fileinfo_h_ */
