@@ -1,5 +1,5 @@
 /*
- * $Id: geo_ip.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: geo_ip.c 13873 2007-06-18 03:52:15Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: geo_ip.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: geo_ip.c 13873 2007-06-18 03:52:15Z cbiere $")
 
 #include "geo_ip.h"
 #include "settings.h"
@@ -56,18 +56,17 @@ RCSID("$Id: geo_ip.c 11437 2006-08-05 00:24:20Z cbiere $")
 static const gchar gip_file[] = "geo-ip.txt";
 static const gchar gip_what[] = "Geographic IP mappings";
 
-static gpointer geo_db;			/**< The database of bogus CIDR ranges */
+static struct iprange_db *geo_db;	/**< The database of bogus CIDR ranges */
 
 /**
  * Context used during ip_range_split() calls.
  */
 struct range_context {
-	guint32 ip1;				/**< Original lower IP in global range */
-	guint32 ip2;				/**< Original upper IP in global range */
-	gint country;				/**< Country code (numerical encoded) */
-	gint count;					/**< Amount of ranges we added, for stats */
 	gchar *line;				/**< The line from the input file */
 	gint linenum;				/**< Line number in input file, for errors */
+	guint32 ip1;				/**< Original lower IP in global range */
+	guint32 ip2;				/**< Original upper IP in global range */
+	guint16 country;			/**< Country code (numerical encoded) */
 };
 
 /**
@@ -78,28 +77,22 @@ struct range_context {
 static void
 gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 {
-	struct range_context *ctx = (struct range_context *) udata;
+	struct range_context *ctx = udata;
 	iprange_err_t error;
 	gpointer ccode;
+	guint cc;
 
-	if (dbg > 4)
+	if (GNET_PROPERTY(dbg) > 4)
 		printf("GEO adding %s/%d for \"%s\"\n",
 			ip_to_string(ip), bits, ctx->line);
 
-	ccode = GUINT_TO_POINTER(ctx->country);
+	cc = ctx->country;
+	ccode = GUINT_TO_POINTER(cc);
 	error = iprange_add_cidr(geo_db, ip, bits, ccode);
 
 	switch (error) {
 	case IPR_ERR_OK:
 		break;
-	case IPR_ERR_RANGE_OVERLAP:
-		error = iprange_add_cidr_force(geo_db, ip, bits, ccode, NULL);
-		if (error == IPR_ERR_OK) {
-			g_warning("%s, line %d: "
-				"entry \"%s\" (%s/%d) superseded earlier smaller range",
-				gip_file, ctx->linenum, ctx->line, ip_to_string(ip), bits);
-			break;
-		}
 		/* FALL THROUGH */
 	default:
 		g_warning("%s, line %d: rejected entry \"%s\" (%s/%d): %s",
@@ -107,8 +100,6 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 			iprange_strerror(error));
 		return;
 	}
-
-	ctx->count++;
 }
 
 /**
@@ -116,18 +107,18 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
  *
  * @return The amount of entries loaded.
  */
-static gint
+static guint
 gip_load(FILE *f)
 {
 	gchar line[1024];
 	gchar *p;
 	gint linenum = 0;
 	const gchar *end;
-	gint c, code;
+	guint16 code;
+	gint c;
 	struct range_context ctx;
 
-	geo_db = iprange_make(NULL, NULL);
-	ctx.count = 0;
+	geo_db = iprange_new();
 
 	while (fgets(line, sizeof(line), f)) {
 		linenum++;
@@ -214,7 +205,7 @@ gip_load(FILE *f)
 		}
 
 		code = iso3166_encode_cc(end);
-		if (-1 == code) {
+		if (ISO3166_INVALID == code) {
 			g_warning("%s, line %d: bad country code in \"%s\"",
 				gip_file, linenum, line);
 			continue;
@@ -235,17 +226,14 @@ gip_load(FILE *f)
 		ip_range_split(ctx.ip1, ctx.ip2, gip_add_cidr, &ctx);
 	}
 
-	if (dbg) {
-		iprange_stats_t stats;
+	iprange_sync(geo_db);
 
-		iprange_get_stats(geo_db, &stats);
-
-		g_message("loaded %d geographical IP ranges", ctx.count);
-		g_message("geo IP stats: count=%d level2=%d heads=%d enlisted=%d",
-			stats.count, stats.level2, stats.heads, stats.enlisted);
+	if (GNET_PROPERTY(dbg)) {
+		g_message("loaded %u geographical IP ranges (%u hosts)",
+			iprange_get_item_count(geo_db),
+			iprange_get_host_count(geo_db));
 	}
-
-	return ctx.count;
+	return iprange_get_item_count(geo_db);
 }
 
 /**
@@ -257,7 +245,7 @@ gip_changed(const gchar *filename, gpointer unused)
 {
 	FILE *f;
 	gchar buf[80];
-	gint count;
+	guint count;
 
 	(void) unused;
 
@@ -269,7 +257,7 @@ gip_changed(const gchar *filename, gpointer unused)
 	count = gip_load(f);
 	fclose(f);
 
-	gm_snprintf(buf, sizeof(buf), "Reloaded %d geographic IP ranges.", count);
+	gm_snprintf(buf, sizeof(buf), "Reloaded %u geographic IP ranges.", count);
 	gcu_statusbar_message(buf);
 }
 
@@ -333,10 +321,7 @@ gip_init(void)
 void
 gip_close(void)
 {
-	if (geo_db) {
-		iprange_free_each(geo_db, NULL);
-		geo_db = NULL;
-	}
+	iprange_free(&geo_db);
 }
 
 /**
@@ -344,9 +329,9 @@ gip_close(void)
  *
  * @param ha the host address to look up.
  * @return the country mapped to this IP address as an numerical encoded
- * country code, * or -1 when unknown.
+ *         country code, or ISO3166_INVALID when unknown.
  */
-gint
+guint16
 gip_country(const host_addr_t ha)
 {
 	host_addr_t to;
@@ -360,9 +345,9 @@ gip_country(const host_addr_t ha)
 
 		ip = host_addr_ipv4(to);
 		if (geo_db && NULL != (code = iprange_get(geo_db, ip)))
-			return (GPOINTER_TO_INT(code) >> 1) - 1;
+			return (GPOINTER_TO_UINT(code) >> 1) - 1;
 	}
-	return -1;
+	return ISO3166_INVALID;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
