@@ -1,5 +1,5 @@
 /*
- * $Id: geo_ip.h 9450 2005-09-10 08:17:29Z daichik $
+ * $Id: geo_ip.h 13871 2007-06-18 03:28:40Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,10 +36,10 @@
 #ifndef _core_geo_ip_h_
 #define _core_geo_ip_h_
 
-#include <glib.h>
+#include "common.h"
 #include "lib/host_addr.h"
 
-gint gip_country(const host_addr_t addr);
+guint16 gip_country(const host_addr_t addr);
 void gip_init(void);
 void gip_close(void);
 
