@@ -1,5 +1,5 @@
 /*
- * $Id: ggep.h,v 1.10 2005/08/16 04:15:22 cbiere Exp $
+ * $Id: ggep.h 12993 2007-03-05 22:51:40Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,13 +36,13 @@
 #ifndef _core_ggep_h_
 #define _core_ggep_h_
 
+#include "common.h"
+
 #include "extensions.h"
 #include "lib/cobs.h"
 #include "lib/zlib_util.h"
 
-#include <glib.h>
-
-#define GGEP_MAGIC		'\xc3'		/**< GGEP extension prefix */
+#define GGEP_MAGIC		0xC3U		/**< GGEP extension prefix */
 
 /*
  * GGEP Extension Header Flags.
