@@ -1,5 +1,5 @@
 /*
- * $Id: ggep_type.h 13285 2007-04-13 01:03:20Z cbiere $
+ * $Id: ggep_type.h 15056 2007-12-09 01:32:27Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -57,7 +57,8 @@ typedef enum ggept_status {
 
 struct gnutella_host;
 
-ggept_status_t ggept_h_sha1_extract(extvec_t *exv, struct sha1 *sha1);
+ggept_status_t ggept_h_sha1_extract(extvec_t *, struct sha1 *);
+ggept_status_t ggept_h_tth_extract(extvec_t *, struct tth *);
 
 /** Decompiled payload of "GTKGV1" */
 struct ggep_gtkgv1 {
