@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: gnet_stats.c 14601 2007-08-20 19:26:41Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: gnet_stats.c 14601 2007-08-20 19:26:41Z cbiere $")
 
 #include "gnet_stats.h"
 #include "gmsg.h"
@@ -92,6 +92,8 @@ gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
 		N_("Payload inflating error"),		 /**< MSG_DROP_INFLATE_ERROR */
 		N_("Unknown header flags present"),/**< MSG_DROP_UNKNOWN_HEADER_FLAGS */
 		N_("Own search results"),			 /**< MSG_DROP_OWN_RESULTS */
+		N_("Ancient query format"),			 /**< MSG_DROP_ANCIENT_QUERY */
+		N_("Blank Servent ID"),				 /**< MSG_DROP_BLANK_SERVENT_ID */
 	};
 
 	STATIC_ASSERT(G_N_ELEMENTS(msg_drop_reasons) == MSG_DROP_REASON_COUNT);
