@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.h 10066 2005-11-09 19:16:53Z cbiere $
+ * $Id: hcache.h 13263 2007-04-07 00:23:15Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi, Richard Eckart
  *
@@ -72,14 +72,13 @@ gboolean hcache_node_is_bad(const host_addr_t addr);
 
 void hcache_prune(hcache_type_t type);
 
-gboolean hcache_read_finished(void);
-gint hcache_size(host_type_t type);
+guint hcache_size(host_type_t type);
 gboolean hcache_is_low(host_type_t type);
 
 gint hcache_fill_caught_array(
 	host_type_t type, gnet_host_t *hosts, gint hcount);
 
-void hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port);
+gboolean hcache_get_caught(host_type_t type, host_addr_t *addr, guint16 *port);
 gboolean hcache_find_nearby(host_type_t type,
 	host_addr_t *addr, guint16 *port);
 
