@@ -1,5 +1,5 @@
 /*
- * $Id: hosts.h 9053 2005-08-07 22:32:24Z cbiere $
+ * $Id: hosts.h 12576 2006-12-05 19:59:05Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -26,7 +26,7 @@
 #ifndef _core_hosts_h_
 #define _core_hosts_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "if/core/hosts.h"
 
@@ -48,7 +48,7 @@ void host_add_semi_pong(const host_addr_t addr, guint16 port);
 void host_shutdown(void);
 void host_close(void);
 
-const gchar *host_to_string(const gnet_host_t *h);
+const gchar *gnet_host_to_string(const gnet_host_t *h);
 
 void parse_netmasks(const gchar *value);
 gboolean host_is_nearby(const host_addr_t addr);
