@@ -1,5 +1,5 @@
 /*
- * $Id: http.c 13949 2007-06-24 00:46:05Z cbiere $
+ * $Id: http.c 14707 2007-08-27 05:53:17Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: http.c 13949 2007-06-24 00:46:05Z cbiere $")
+RCSID("$Id: http.c 14707 2007-08-27 05:53:17Z rmanfredi $")
 
 #include "nodes.h"
 #include "http.h"
@@ -663,7 +663,7 @@ http_url_parse(const gchar *url, guint16 *port, const gchar **host,
 	}
 
 	if (GNET_PROPERTY(http_debug) > 4) {
-		g_message("URL \"%s\" -> host=\"%s\", port=%u, path=\"%s\"\n",
+		g_message("URL \"%s\" -> host=\"%s\", port=%u, path=\"%s\"",
 			url, *host, (unsigned) *port, *path);
 	}
 
@@ -2214,7 +2214,7 @@ http_got_header(struct http_async *ha, header_t *header)
 	guint http_major = 0, http_minor = 0;
 
 	if (GNET_PROPERTY(http_debug) > 2) {
-		g_message("----Got HTTP reply from %s:\n",
+		g_message("----Got HTTP reply from %s:",
 			host_addr_to_string(s->addr));
 		g_message("%s", status);
 		header_dump(header, stderr);
