@@ -1,5 +1,5 @@
 /*
- * $Id: http.h 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: http.h 13932 2007-06-22 23:57:20Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,6 +36,8 @@
 #ifndef _core_http_h_
 #define _core_http_h_
 
+#include "common.h"
+
 #include "if/core/http.h"
 #include "lib/host_addr.h"
 
@@ -61,15 +63,15 @@ typedef enum {
  * The callback used to generate custom headers.
  *
  * @param `buf' is where the callback can generate extra data.
- * @param `retlen' is initially filled with the room available in `buf'.
+ * @param `size' is the size of buf in bytes.
  * @param `arg' is user-supplied data.
  * @param `flags' are extra flags passed by callback invoker
  *
- * The callback is expected to fill `buf' and return the length of written
- * data into `retlen'.
+ * The callback is expected to fill `buf' and return the amount of bytes
+ * written to buf.
  */
-typedef void (*http_status_cb_t)(
-	gchar *buf, gint *retlen, gpointer arg, guint32 flags);
+typedef size_t (*http_status_cb_t)(
+	gchar *buf, size_t size, gpointer arg, guint32 flags);
 
 typedef struct {
 	http_extra_type_t he_type;		/**< Union discriminent */
@@ -86,6 +88,29 @@ typedef struct {
 #define he_cb	u.u_cbk.u_cb
 #define he_arg	u.u_cbk.u_arg
 
+static inline void
+http_extra_callback_set(http_extra_desc_t *he,
+	http_status_cb_t callback, gpointer user_arg)
+{
+	he->he_type = HTTP_EXTRA_CALLBACK;
+	he->he_cb = callback;
+	he->he_arg = user_arg;
+}
+
+static inline void
+http_extra_line_set(http_extra_desc_t *he, const gchar *msg)
+{
+	he->he_type = HTTP_EXTRA_LINE;
+	he->he_msg = msg;
+}
+
+static inline void
+http_extra_body_set(http_extra_desc_t *he, const gchar *body)
+{
+	he->he_type = HTTP_EXTRA_BODY;
+	he->he_msg = body;
+}
+
 /*
  * Flags used during callback invocation.
  */
@@ -96,18 +121,19 @@ typedef struct {
 #define HTTP_CBF_SHOW_RANGES	0x00000008	/**< Show available ranges */
 
 struct header;
+struct http_async;
 
 /**
  * Callback used from asynchronous request to indicate that we got headers.
  * Indicates whether we should continue or not, given the HTTP response code.
  */
 typedef gboolean (*http_header_cb_t)(
-	gpointer h, struct header *header, gint code, const gchar *message);
+	struct http_async *, struct header *, gint code, const gchar *message);
 
 /**
  * Callback used from asynchronous request to indicate that data is available.
  */
-typedef void (*http_data_cb_t)(gpointer h, gchar *data, gint len);
+typedef void (*http_data_cb_t)(struct http_async *, gchar *data, gint len);
 
 typedef enum {				/**< Type of error reported by http_error_cb_t */
 	HTTP_ASYNC_SYSERR,		/**< System error, value is errno */
@@ -118,8 +144,8 @@ typedef enum {				/**< Type of error reported by http_error_cb_t */
 
 typedef struct {
 	struct header *header;	/**< Parsed HTTP header */
-	gint code;				/**< HTTP status code */
 	const gchar *message;	/**< HTTP status message */
+	gint code;				/**< HTTP status code */
 } http_error_t;
 
 /**
@@ -127,10 +153,11 @@ typedef struct {
  * The type of `val' depends on the `error'.
  */
 
-typedef void (*http_error_cb_t)(gpointer h, http_errtype_t error, gpointer val);
+typedef void (*http_error_cb_t)(
+			struct http_async *, http_errtype_t error, gpointer val);
 
 /**
- * Callabck to free user opaque data.
+ * Callback to free user opaque data.
  */
 
 typedef void (*http_user_free_t)(gpointer data);
@@ -139,7 +166,7 @@ typedef void (*http_user_free_t)(gpointer data);
  * Asynchronous operations that the user may redefine.
  */
 
-typedef size_t (*http_op_request_t)(gpointer handle, gchar *buf, size_t len,
+typedef size_t (*http_op_request_t)(struct http_async *, gchar *buf, size_t len,
 	const gchar *verb, const gchar *path, const gchar *host, guint16 port);
 
 /*
@@ -187,7 +214,7 @@ extern http_url_error_t http_url_errno;
  * Callback to notify about state changes in HTTP request.
  */
 
-typedef void (*http_state_change_t)(gpointer handle, http_state_t newstate);
+typedef void (*http_state_change_t)(struct http_async *, http_state_t newstate);
 
 /**
  * HTTP data buffered when it cannot be sent out immediately.
@@ -219,8 +246,10 @@ gboolean http_send_status(struct gnutella_socket *s,
 	gint code, gboolean keep_alive, http_extra_desc_t *hev, gint hevcnt,
 	const gchar *reason, ...) G_GNUC_PRINTF(6, 7);
 
-void http_hostname_add(
-	gchar *buf, gint *retval, gpointer arg, guint32 flags);
+size_t http_hostname_add(
+	gchar *buf, size_t size, gpointer arg, guint32 flags);
+size_t http_retry_after_add(
+	gchar *buf, size_t size, gpointer arg, guint32 flags);
 
 gint http_status_parse(const gchar *line,
 	const gchar *proto, const gchar **msg, guint *major, guint *minor);
@@ -245,14 +274,14 @@ const gchar *http_url_strerror(http_url_error_t errnum);
 gboolean http_url_parse(
 	const gchar *url, guint16 *port, const gchar **host, const gchar **path);
 
-gpointer http_async_get(
-	gchar *url,
+struct http_async *http_async_get(
+	const gchar *url,
 	http_header_cb_t header_ind,
 	http_data_cb_t data_ind,
 	http_error_cb_t error_ind);
 
-gpointer http_async_get_addr(
-	gchar *path,
+struct http_async *http_async_get_addr(
+	const gchar *path,
 	const host_addr_t,
 	guint16 port,
 	http_header_cb_t header_ind,
@@ -261,23 +290,26 @@ gpointer http_async_get_addr(
 
 const gchar *http_async_strerror(guint errnum);
 const gchar *http_async_info(
-	gpointer handle, const gchar **req, gchar **path,
+	struct http_async *handle, const gchar **req, const gchar **path,
 	host_addr_t *addr, guint16 *port);
-void http_async_connected(gpointer handle);
-void http_async_close(gpointer handle);
-void http_async_cancel(gpointer handle);
-void http_async_error(gpointer handle, gint code);
-http_state_t http_async_state(gpointer handle);
+void http_async_connected(struct http_async *handle);
+void http_async_close(struct http_async *handle);
+void http_async_cancel(struct http_async *handle);
+void http_async_error(struct http_async *handle, gint code);
+http_state_t http_async_state(struct http_async *handle);
 
-void http_async_set_opaque(gpointer handle, gpointer data, http_user_free_t fn);
-gpointer http_async_get_opaque(gpointer handle);
-void http_async_log_error(gpointer handle, http_errtype_t type, gpointer v);
-void http_async_log_error_dbg(
-	gpointer handle, http_errtype_t type, gpointer v, guint32 dbg_level);
+void http_async_set_opaque(struct http_async *handle,
+		gpointer data, http_user_free_t fn);
+gpointer http_async_get_opaque(struct http_async *handle);
+void http_async_log_error(struct http_async *handle,
+		http_errtype_t type, gpointer v);
+void http_async_log_error_dbg(struct http_async *handle,
+		http_errtype_t type, gpointer v, guint32 dbg_level);
 
-void http_async_on_state_change(gpointer handle, http_state_change_t fn);
-void http_async_allow_redirects(gpointer handle, gboolean allow);
-void http_async_set_op_request(gpointer handle, http_op_request_t op);
+void http_async_on_state_change(struct http_async *handle,
+		http_state_change_t fn);
+void http_async_allow_redirects(struct http_async *handle, gboolean allow);
+void http_async_set_op_request(struct http_async *handle, http_op_request_t op);
 
 void http_close(void);
 
