@@ -1,5 +1,5 @@
 /*
- * $Id: huge.c 13977 2007-06-28 00:10:51Z cbiere $
+ * $Id: huge.c 15050 2007-12-07 18:49:19Z cbiere $
  *
  * Copyright (c) 2002-2003, Ch. Tronche & Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: huge.c 13977 2007-06-28 00:10:51Z cbiere $")
+RCSID("$Id: huge.c 15050 2007-12-07 18:49:19Z cbiere $")
 
 #include "huge.h"
 #include "share.h"
@@ -778,11 +778,14 @@ huge_sha1_extract32(const gchar *buf, size_t len, struct sha1 *sha1,
 bad:
 	if (GNET_PROPERTY(dbg)) {
 		if (is_printable(buf, len)) {
-			g_warning("%s has bad SHA1 (len=%d): %.*s",
-				gmsg_infostr(header), (gint) len, (gint) len, buf);
+			g_warning("%s has bad SHA1 (len=%u): %.*s",
+				gmsg_infostr(header),
+				(unsigned) len,
+				(int) MIN(len, (size_t) INT_MAX),
+				buf);
 		} else {
-			g_warning("%s has bad SHA1 (len=%d)",
-					gmsg_infostr(header), (gint) len);
+			g_warning("%s has bad SHA1 (len=%u)",
+				gmsg_infostr(header), (unsigned) len);
 			if (len)
 				dump_hex(stderr, "Base32 SHA1", buf, len);
 		}
@@ -791,6 +794,36 @@ bad:
 	return FALSE;
 }
 
+gboolean
+huge_tth_extract32(const gchar *buf, size_t len, struct tth *tth,
+	gconstpointer header)
+{
+	if (len != TTH_BASE32_SIZE)
+		goto bad;
+
+	if (TTH_RAW_SIZE != base32_decode(tth->data, sizeof tth->data, buf, len))
+		goto bad;
+
+	return TRUE;
+
+bad:
+	if (GNET_PROPERTY(dbg)) {
+		if (is_printable(buf, len)) {
+			g_warning("%s has bad TTH (len=%u): %.*s",
+				gmsg_infostr(header),
+				(unsigned) len,
+				(int) MIN(len, (size_t) INT_MAX),
+				buf);
+		} else {
+			g_warning("%s has bad TTH (len=%u",
+				gmsg_infostr(header), (unsigned) len);
+			if (len)
+				dump_hex(stderr, "Base32 TTH", buf, len);
+		}
+	}
+	return FALSE;
+}
+
 /**
  * Parse the "X-Gnutella-Alternate-Location" header if present to learn
  * about other sources for this file.
