@@ -1,5 +1,5 @@
 /*
- * $Id: inet.c 11825 2006-08-30 14:44:06Z cbiere $
+ * $Id: inet.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: inet.c 11825 2006-08-30 14:44:06Z cbiere $")
+RCSID("$Id: inet.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "inet.h"
 #include "nodes.h"				/* For node_become_firewalled() */
@@ -79,9 +79,9 @@ static time_t fw_time = 0;				/**< When we last became firewalled */
  * mode.
  */
 
-static gpointer incoming_ev = NULL;			/**< Callout queue timer */
-static gpointer incoming_udp_ev = NULL;		/**< Idem */
-static gpointer solicited_udp_ev = NULL;	/**< Idem */
+static cevent_t *incoming_ev;		/**< Callout queue timer */
+static cevent_t *incoming_udp_ev;	/**< Idem */
+static cevent_t *solicited_udp_ev;	/**< Idem */
 
 /*
  * Unfortunately, to accurately detect true unsolicited UDP traffic, we have
@@ -96,7 +96,7 @@ static GHashTable *outgoing_udp = NULL;		/**< Maps "IP" => "ip_record" */
 
 struct ip_record {
 	host_addr_t addr;			/**< The IP address to which we sent data */
-	gpointer timeout_ev;		/**< The expiration time for the fw breach */
+	cevent_t *timeout_ev;		/**< The expiration time for the fw breach */
 };
 
 /***
@@ -112,8 +112,8 @@ struct ip_record {
 
 #define OUTGOING_WINDOW		150			/**< Outgoing monitoring window */
 
-static guint32 activity_seen = 0;		/**< Activity recorded in period */
-static gpointer outgoing_ev = NULL;		/**< Callout queue timer */
+static gboolean activity_seen;			/**< Activity recorded in period */
+static cevent_t *outgoing_ev;			/**< Callout queue timer */
 
 static void inet_set_is_connected(gboolean val);
 
@@ -139,8 +139,7 @@ ip_record_make(const host_addr_t addr)
 static void
 ip_record_free(struct ip_record *ipr)
 {
-	if (ipr->timeout_ev)
-		cq_cancel(callout_queue, ipr->timeout_ev);
+	cq_cancel(callout_queue, &ipr->timeout_ev);
 	wfree(ipr, sizeof *ipr);
 }
 
@@ -179,8 +178,11 @@ ip_record_destroy(cqueue_t *unused_cq, gpointer obj)
 }
 
 /**
- * @returns whether ip is that of the local machine of in the same local
- * network area.
+ * Checks whether a host address is considered being "local".
+ *
+ * @param addr The host address to check.
+ * @returns TRUE if the IP address is that of the local machine or
+ *			a private address. Otherwise FALSE is returned.
  */
 static gboolean
 is_local_addr(const host_addr_t addr)
@@ -193,7 +195,10 @@ is_local_addr(const host_addr_t addr)
 	 *		 cause blocking of several seconds which is not acceptable
 	 *		 except during the startup phase.
 	 */
-	if (NET_USE_IPV4 == network_protocol || NET_USE_BOTH == network_protocol) {
+	if (
+		NET_USE_IPV4 == GNET_PROPERTY(network_protocol) ||
+		NET_USE_BOTH == GNET_PROPERTY(network_protocol)
+	) {
 		if (!is_host_addr(our_addr)) {
 			static gboolean tried;
 
@@ -218,11 +223,12 @@ is_local_addr(const host_addr_t addr)
 				}
 			}
 		}
-		if (host_addr_equal(addr, listen_addr()))	/* Ourselves */
-			return TRUE;
 	}
 	
-	if (NET_USE_IPV6 == network_protocol || NET_USE_BOTH == network_protocol) {
+	if (
+		NET_USE_IPV6 == GNET_PROPERTY(network_protocol) ||
+		NET_USE_BOTH == GNET_PROPERTY(network_protocol)
+	) {
 		if (!is_host_addr(our_addr_v6)) {
 			static gboolean tried;
 
@@ -247,28 +253,18 @@ is_local_addr(const host_addr_t addr)
 				}
 			}
 		}
-
-		if (host_addr_equal(addr, listen_addr6()))	/* Ourselves */
-			return TRUE;
 	}
 
+	if (is_my_address(addr))
+		return TRUE;
+
 	switch (host_addr_net(addr)) {
 	case NET_TYPE_IPV4:
-		{
-			static host_addr_t loopback;
-
-			if (!is_host_addr(loopback))
-				string_to_host_addr("127.0.0.0", NULL, &loopback);
-
-			return	host_addr_matches(addr, loopback, 8) ||
-					host_addr_matches(addr, our_addr, 24); /* Same LAN/24 */
-		}
 	case NET_TYPE_IPV6:
-		return	host_addr_matches(addr, ipv6_link_local, 10) ||
-				host_addr_matches(addr, ipv6_site_local, 10);
+		return is_private_addr(addr);
 	case NET_TYPE_LOCAL:
 	case NET_TYPE_NONE:
-		return FALSE;
+		return TRUE;
 	}
 	g_assert_not_reached();
 	return FALSE;
@@ -286,12 +282,7 @@ inet_firewalled(void)
 {
 	gnet_prop_set_boolean_val(PROP_IS_FIREWALLED, TRUE);
 	fw_time = tm_time();
-
-	if (incoming_ev) {
-		cq_cancel(callout_queue, incoming_ev);
-		incoming_ev = NULL;
-	}
-
+	cq_cancel(callout_queue, &incoming_ev);
 	node_became_firewalled();
 }
 
@@ -302,12 +293,7 @@ void
 inet_udp_firewalled(void)
 {
 	gnet_prop_set_boolean_val(PROP_IS_UDP_FIREWALLED, TRUE);
-
-	if (incoming_udp_ev) {
-		cq_cancel(callout_queue, incoming_udp_ev);
-		incoming_udp_ev = NULL;
-	}
-
+	cq_cancel(callout_queue, &incoming_udp_ev);
 	node_became_udp_firewalled();
 }
 
@@ -322,8 +308,8 @@ got_no_udp_solicited(cqueue_t *unused_cq, gpointer unused_obj)
 	(void) unused_cq;
 	(void) unused_obj;
 
-	if (dbg)
-		printf("FW: got no solicited UDP traffic for %d secs\n",
+	if (GNET_PROPERTY(dbg))
+		g_message("FW: got no solicited UDP traffic for %d secs",
 			FW_SOLICITED_WINDOW);
 
 	solicited_udp_ev = NULL;
@@ -339,11 +325,10 @@ inet_udp_got_solicited(void)
 	gnet_prop_set_boolean_val(PROP_RECV_SOLICITED_UDP, TRUE);
 
 	if (solicited_udp_ev == NULL) {
-		activity_seen++;
 		cq_insert(callout_queue,
 			FW_SOLICITED_WINDOW * 1000, got_no_udp_solicited, NULL);
-		if (dbg)
-			printf("FW: got solicited UDP traffic\n");
+		if (GNET_PROPERTY(dbg))
+			g_message("FW: got solicited UDP traffic");
 	} else
 		cq_resched(callout_queue,
 			solicited_udp_ev, FW_SOLICITED_WINDOW * 1000);
@@ -359,7 +344,7 @@ got_no_connection(cqueue_t *unused_cq, gpointer unused_obj)
 	(void) unused_cq;
 	(void) unused_obj;
 
-	if (dbg)
+	if (GNET_PROPERTY(dbg))
 		g_message("FW: got no connection to port %u for %d secs",
 			socket_listen_port(), FW_INCOMING_WINDOW);
 
@@ -377,7 +362,7 @@ got_no_udp_unsolicited(cqueue_t *unused_cq, gpointer unused_obj)
 	(void) unused_cq;
 	(void) unused_obj;
 
-	if (dbg)
+	if (GNET_PROPERTY(dbg))
 		g_message("FW: got no unsolicited UDP datagram to port %u for %d secs",
 			socket_listen_port(), FW_INCOMING_WINDOW);
 
@@ -395,7 +380,7 @@ inet_not_firewalled(void)
 	gnet_prop_set_boolean_val(PROP_IS_FIREWALLED, FALSE);
 	node_proxy_cancel_all();
 
-	if (dbg)
+	if (GNET_PROPERTY(dbg))
 		g_message("FW: we're not TCP-firewalled for port %u",
 			socket_listen_port());
 }
@@ -408,7 +393,7 @@ inet_udp_not_firewalled(void)
 {
 	gnet_prop_set_boolean_val(PROP_IS_UDP_FIREWALLED, FALSE);
 
-	if (dbg)
+	if (GNET_PROPERTY(dbg))
 		g_message("FW: we're not UDP-firewalled for port %u",
 			socket_listen_port());
 }
@@ -419,17 +404,17 @@ inet_udp_not_firewalled(void)
 void
 inet_got_incoming(const host_addr_t addr)
 {
-	gboolean is_local = is_local_addr(addr);
+	if (is_local_addr(addr))
+		return;
 
 	/*
 	 * If we get an incoming connection from the outside, we're surely
 	 * connected to the Internet.
 	 */
 
-	if (!is_local)
-		activity_seen++;				/* In case we have a timer set */
+	activity_seen = TRUE;				/* In case we have a timer set */
 
-	if (!is_inet_connected && !is_local)
+	if (!GNET_PROPERTY(is_inet_connected))
 		inet_set_is_connected(TRUE);
 
 	/*
@@ -438,7 +423,7 @@ inet_got_incoming(const host_addr_t addr)
 	 * got an incoming connection.
 	 */
 
-	if (!is_firewalled) {
+	if (!GNET_PROPERTY(is_firewalled)) {
 		g_assert(incoming_ev);
 		cq_resched(callout_queue, incoming_ev, FW_INCOMING_WINDOW * 1000);
 		return;
@@ -449,12 +434,10 @@ inet_got_incoming(const host_addr_t addr)
 	 * If we're not, then we're not firewalled.
 	 */
 
-	if (!is_local)
-		inet_not_firewalled();
+	inet_not_firewalled();
 
-	incoming_ev = cq_insert(
-		callout_queue, FW_INCOMING_WINDOW * 1000,
-		got_no_connection, NULL);
+	incoming_ev = cq_insert(callout_queue, FW_INCOMING_WINDOW * 1000,
+					got_no_connection, NULL);
 }
 
 /**
@@ -474,7 +457,7 @@ inet_udp_got_unsolicited_incoming(void)
 	 * got an incoming connection.
 	 */
 
-	if (!is_udp_firewalled) {
+	if (!GNET_PROPERTY(is_udp_firewalled)) {
 		g_assert(incoming_udp_ev);
 		cq_resched(callout_queue, incoming_udp_ev, FW_INCOMING_WINDOW * 1000);
 		return;
@@ -484,9 +467,8 @@ inet_udp_got_unsolicited_incoming(void)
 
 	g_assert(incoming_udp_ev == NULL);
 
-	incoming_udp_ev = cq_insert(
-		callout_queue, FW_INCOMING_WINDOW * 1000,
-		got_no_udp_unsolicited, NULL);
+	incoming_udp_ev = cq_insert(callout_queue, FW_INCOMING_WINDOW * 1000,
+							got_no_udp_unsolicited, NULL);
 }
 
 /**
@@ -495,12 +477,12 @@ inet_udp_got_unsolicited_incoming(void)
 void
 inet_udp_got_incoming(const host_addr_t addr)
 {
-	gboolean is_local = is_local_addr(addr);
+	if (is_local_addr(addr))
+		return;
 
-	if (!is_local)
-		activity_seen++;				/* In case we have a timer set */
+	activity_seen = TRUE;				/* In case we have a timer set */
 
-	if (!is_inet_connected && !is_local)
+	if (!GNET_PROPERTY(is_inet_connected))
 		inet_set_is_connected(TRUE);
 
 	/*
@@ -509,14 +491,9 @@ inet_udp_got_incoming(const host_addr_t addr)
 	 * some data to that IP address.
 	 */
 
-	if (!is_local_addr(addr)) {
-		gpointer ipr;
-
-		ipr = g_hash_table_lookup(outgoing_udp, &addr);
-		inet_udp_got_solicited();
-		if (NULL == ipr)
-			inet_udp_got_unsolicited_incoming();
-	}
+	inet_udp_got_solicited();
+	if (NULL == g_hash_table_lookup(outgoing_udp, &addr))
+		inet_udp_got_unsolicited_incoming();
 }
 
 /**
@@ -551,8 +528,9 @@ inet_can_answer_ping(void)
 {
 	gint elapsed;
 
-	if (!is_firewalled)
-		return current_peermode != NODE_P_LEAF;	/* Leaves don't send pongs */
+	/* Leaves don't send pongs */
+	if (!GNET_PROPERTY(is_firewalled))
+		return GNET_PROPERTY(current_peermode) != NODE_P_LEAF;
 
 	if (!is_host_addr(listen_addr()) && !is_host_addr(listen_addr6()))
 		return FALSE;		/* We don't know our local IP, we can't reply */
@@ -593,8 +571,8 @@ inet_set_is_connected(gboolean val)
 {
 	gnet_prop_set_boolean_val(PROP_IS_INET_CONNECTED, val);
 
-	if (dbg)
-		printf("FW: we're %sconnected to the Internet\n",
+	if (GNET_PROPERTY(dbg))
+		g_message("FW: we're %sconnected to the Internet",
 			val ? "" : "no longer ");
 }
 
@@ -610,10 +588,12 @@ check_outgoing_connection(cqueue_t *unused_cq, gpointer unused_obj)
 	(void) unused_cq;
 	(void) unused_obj;
 
-	if (activity_seen == 0)				/* Nothing over the period */
+	if (activity_seen) {
+		activity_seen = FALSE;
+	} else {
+		/* Nothing over the period */
 		inet_set_is_connected(FALSE);
-
-	activity_seen = 0;
+	}
 }
 
 /**
@@ -634,7 +614,7 @@ inet_connection_attempted(const host_addr_t addr)
 	 */
 
 	if (!outgoing_ev) {
-		activity_seen = 0;
+		activity_seen = FALSE;
 		outgoing_ev = cq_insert(
 			callout_queue, OUTGOING_WINDOW * 1000,
 			check_outgoing_connection, NULL);
@@ -654,9 +634,9 @@ inet_connection_succeeded(const host_addr_t addr)
 	if (is_local_addr(addr))
 		return;
 
-	activity_seen++;
+	activity_seen = TRUE;
 
-	if (!is_inet_connected)
+	if (!GNET_PROPERTY(is_inet_connected))
 		inet_set_is_connected(TRUE);
 }
 
@@ -666,7 +646,7 @@ inet_connection_succeeded(const host_addr_t addr)
 void
 inet_read_activity(void)
 {
-	activity_seen++;
+	activity_seen = TRUE;
 
 	/*
 	 * We're not sure activity was not with a local node, so don't
@@ -684,7 +664,7 @@ inet_init(void)
 	 * If we persisted "is_firewalled" to FALSE, arm the no-connection timer.
 	 */
 
-	if (!is_firewalled)
+	if (!GNET_PROPERTY(is_firewalled))
 		incoming_ev = cq_insert(
 			callout_queue, FW_INCOMING_WINDOW * 1000,
 			got_no_connection, NULL);
@@ -693,7 +673,7 @@ inet_init(void)
 	 * If we persisted "is_udp_firewalled" to FALSE, idem.
 	 */
 
-	if (!is_udp_firewalled)
+	if (!GNET_PROPERTY(is_udp_firewalled))
 		incoming_udp_ev = cq_insert(
 			callout_queue, FW_INCOMING_WINDOW * 1000,
 			got_no_udp_unsolicited, NULL);
@@ -702,7 +682,7 @@ inet_init(void)
 	 * If we persisted "recv_solicited_udp" to TRUE, idem.
 	 */
 
-	if (recv_solicited_udp)
+	if (GNET_PROPERTY(recv_solicited_udp))
 		solicited_udp_ev = cq_insert(
 			callout_queue, FW_SOLICITED_WINDOW * 1000,
 			got_no_udp_solicited, NULL);
@@ -736,12 +716,9 @@ inet_close(void)
 	g_hash_table_foreach(outgoing_udp, free_ip_record, NULL);
 	g_hash_table_destroy(outgoing_udp);
 
-	if (incoming_ev)
-		cq_cancel(callout_queue, incoming_ev);
-	if (incoming_udp_ev)
-		cq_cancel(callout_queue, incoming_udp_ev);
-	if (solicited_udp_ev)
-		cq_cancel(callout_queue, solicited_udp_ev);
+	cq_cancel(callout_queue, &incoming_ev);
+	cq_cancel(callout_queue, &incoming_udp_ev);
+	cq_cancel(callout_queue, &solicited_udp_ev);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
