@@ -1,5 +1,5 @@
 /*
- * $Id: local_shell.c 13738 2007-05-25 12:38:56Z rmanfredi $
+ * $Id: local_shell.c 14955 2007-09-28 19:25:45Z bpringlemeir $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -78,10 +78,11 @@ socket_set_nonblocking(int fd)
 
 #include "common.h"
 
-RCSID("$Id: local_shell.c 13738 2007-05-25 12:38:56Z rmanfredi $");
+RCSID("$Id: local_shell.c 14955 2007-09-28 19:25:45Z bpringlemeir $");
 
 #include "lib/misc.h"
 #include "lib/socket.h"
+#include "core/local_shell.h"
 
 #include "lib/override.h"
 
@@ -400,9 +401,11 @@ local_shell_mainloop(int fd)
 			 * the shell would not terminate before another write()
 			 * (which should gain 0 or EPIPE) is attempted.
 			 */
-			if (0 == client.fill || server.hup) {
-				return 0;
+			if (client.fill > 0) {
+				fprintf(stderr, "Server hung up unexpectedly!\n");
+				return -1;
 			}
+			return 0;
 		}
 		if (client.eof && 0 == client.fill) {
 			if ((server.eof && 0 == server.fill) || client.hup) {
