@@ -1,5 +1,5 @@
 /*
- * $Id: move.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: move.c 14233 2007-07-29 22:52:01Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: move.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: move.c 14233 2007-07-29 22:52:01Z cbiere $")
 
 #include "downloads.h"
 #include "fileinfo.h"
@@ -220,7 +220,7 @@ d_start(struct bgtask *h, gpointer ctx, gpointer item)
 
 	name = file_info_readable_filename(d->file_info);
 
-	md->target = unique_filename(we->dest, name, we->ext, NULL);
+	md->target = file_info_unique_filename(we->dest, name, we->ext);
 	if (NULL == md->target)
 		goto abort_read;
 
@@ -273,7 +273,11 @@ d_end(struct bgtask *h, gpointer ctx, gpointer item)
 	close(md->rd);
 	md->rd = -1;
 
-	close(md->wd);
+	if (-1 == close(md->wd)) {
+		md->error = errno;
+		g_warning("error whilst closing copy target \"%s\": %s",
+			md->target, g_strerror(errno));
+	}
 	md->wd = -1;
 
 	/*
@@ -284,8 +288,35 @@ d_end(struct bgtask *h, gpointer ctx, gpointer item)
 	 */
 
 	if (md->error == 0) {
+		struct stat buf;
+
 		g_assert(md->copied == md->size);
 
+		/*
+		 * As a precaution, stat() the file.  When moving the file accross
+		 * NFS where the target filesystem is full, write() or close() may
+		 * not return ENOSPC.  Double-check here otherwise they'll lose
+		 * a perfectly good file.
+		 *		--RAM, 2007-07-30.
+		 */
+
+		if (-1 == stat(md->target, &buf)) {
+			md->error = errno;
+			g_warning("cannot stat copy target \"%s\": %s",
+				md->target, g_strerror(errno));
+			goto error;
+		}
+
+		if (
+			!S_ISREG(buf.st_mode) ||
+			(filesize_t) 0 + buf.st_size != (off_t) 0 + md->copied
+		) {
+			md->error = ENOSPC;
+			g_warning("target size mismatch for \"%s\": got only %s",
+				md->target, off_t_to_string(buf.st_size));
+			goto error;
+		}
+
 		if (-1 == unlink(download_pathname(md->d)))
 			g_warning("cannot unlink \"%s\": %s",
 				download_basename(md->d), g_strerror(errno));
@@ -295,6 +326,7 @@ d_end(struct bgtask *h, gpointer ctx, gpointer item)
 				md->target, g_strerror(errno));
 	}
 
+error:
 	elapsed = delta_time(tm_time(), md->start);
 	elapsed = MAX(1, elapsed);		/* time warp? clock not monotic? */
 
