@@ -1,5 +1,5 @@
 /*
- * $Id: move.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: move.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_move_h_
 #define _core_move_h_
 
-#include <glib.h>
+#include "common.h"
 
 /*
  * Public interface.
