@@ -1,5 +1,5 @@
 /*
- * $Id: mq.c 12265 2006-11-08 06:43:13Z cbiere $
+ * $Id: mq.c 13789 2007-05-29 22:21:28Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: mq.c 12265 2006-11-08 06:43:13Z cbiere $")
+RCSID("$Id: mq.c 13789 2007-05-29 22:21:28Z cbiere $")
 
 #include "nodes.h"
 #include "mq.h"
@@ -43,9 +43,9 @@ RCSID("$Id: mq.c 12265 2006-11-08 06:43:13Z cbiere $")
 #include "gmsg.h"
 #include "gnet_stats.h"
 
-#include "lib/walloc.h"
-#include "lib/glib-missing.h"	/* For gm_snprintf() */
 #include "lib/cq.h"
+#include "lib/glib-missing.h"	/* For gm_snprintf() */
+#include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -57,10 +57,69 @@ static gboolean make_room_header(
 	mqueue_t *q, gchar *header, guint prio, gint needed, gint *offset);
 static void mq_swift_timer(cqueue_t *cq, gpointer obj);
 
+gboolean
+mq_is_flow_controlled(const struct mqueue *q)
+{
+	return 0 != (q->flags & MQ_FLOWC);
+}
+
+gboolean
+mq_is_swift_controlled(const struct mqueue *q)
+{
+	return 0 != (q->flags & MQ_SWIFT);
+}
+
+gint
+mq_maxsize(const struct mqueue *q)
+{
+	return q->maxsize;
+}
+
+gint mq_size(const struct mqueue *q)
+{
+	return q->size;
+}
+
+gint
+mq_lowat(const struct mqueue *q)
+{
+	return q->lowat;
+}
+
+gint
+mq_hiwat(const struct mqueue *q)
+{
+	return q->hiwat;
+}
+
+gint
+mq_count(const struct mqueue *q)
+{
+	return q->count;
+}
+
+gint
+mq_pending(const struct mqueue *q)
+{
+	return q->size + tx_pending(q->tx_drv);
+}
+
+struct bio_source *
+mq_bio(const struct mqueue *q)
+{
+	return tx_bio_source(q->tx_drv);
+}
+
+struct gnutella_node *
+mq_node(const struct mqueue *q)
+{
+	return q->node;
+}
+
 /**
  * Compute readable queue information.
  */
-static const gchar *
+const gchar *
 mq_info(const mqueue_t *q)
 {
 	static gchar buf[160];
@@ -97,7 +156,8 @@ static GHashTable *qown = NULL;
 /**
  * Add linkable into queue
  */
-static void mq_add_linkable(mqueue_t *q, GList *l)
+static void
+mq_add_linkable(mqueue_t *q, GList *l)
 {
 	mqueue_t *owner;
 
@@ -124,13 +184,13 @@ static void mq_add_linkable(mqueue_t *q, GList *l)
 /**
  * Remove linkable from queue
  */
-static void mq_remove_linkable(mqueue_t *q, GList *l)
+static void
+mq_remove_linkable(mqueue_t *q, GList *l)
 {
 	mqueue_t *owner;
 
 	g_assert(q->magic == MQ_MAGIC);
 	g_assert(l != NULL);
-	g_assert(l->data != NULL);
 	g_assert(qown != NULL);		/* Must have added something before */
 
 	owner = g_hash_table_lookup(qown, l);
@@ -226,12 +286,14 @@ mq_free(mqueue_t *q)
 {
 	GList *l;
 	gint n;
+	pmsg_t *mb;
 
 	tx_free(q->tx_drv);		/* Get rid of lower layers */
 
 	for (n = 0, l = q->qhead; l; l = g_list_next(l)) {
 		n++;
 		pmsg_free(l->data);
+		l->data = NULL;
 		mq_remove_linkable(q, l);
 	}
 
@@ -240,10 +302,14 @@ mq_free(mqueue_t *q)
 	if (q->qlink)
 		qlink_free(q);
 
-	if (q->swift_ev)
-		cq_cancel(callout_queue, q->swift_ev);
-
+	cq_cancel(callout_queue, &q->swift_ev);
 	g_list_free(q->qhead);
+
+	while ((mb = slist_shift(q->qwait)))
+		pmsg_free(mb);
+	slist_free(&q->qwait);
+
+	q->qhead = NULL;
 	q->magic = 0;
 	wfree(q, sizeof(*q));
 }
@@ -264,10 +330,14 @@ mq_rmlink_prev(mqueue_t *q, GList *l, gint size)
 	q->qhead = g_list_remove_link(q->qhead, l);
 	if (q->qtail == l)
 		q->qtail = prev;
+
+	g_assert(q->size >= size);
 	q->size -= size;
+	g_assert(q->count > 0);
 	q->count--;
 
 	pmsg_free(l->data);
+	l->data = NULL;
 	g_list_free_1(l);
 
 	return prev;
@@ -352,7 +422,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 
 		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH);
 		gnutella_header_set_hops(&q->header, 1);
-		gnutella_header_set_ttl(&q->header, max_ttl);
+		gnutella_header_set_ttl(&q->header, GNET_PROPERTY(max_ttl));
 
 		if (needed > 0)
 			make_room_header(q, (gchar*) &q->header, PMSG_P_DATA, needed, NULL);
@@ -387,10 +457,15 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 * removing enough data from the queue.
 		 */
 
-		for (ttl = hard_ttl_limit; needed > 0 && ttl >= 0; ttl--) {
-			gint old_size = q->size;
+		for (ttl = GNET_PROPERTY(hard_ttl_limit); ttl >= 0; ttl--) {
+			gint old_size;
 
-			gnutella_header_set_hops(&q->header, hard_ttl_limit - ttl);
+			if (needed <= 0)
+				break;
+
+			old_size = q->size;
+			gnutella_header_set_hops(&q->header,
+				GNET_PROPERTY(hard_ttl_limit) - ttl);
 			gnutella_header_set_ttl(&q->header, ttl);
 
 			if (
@@ -479,7 +554,7 @@ mq_enter_flowc(mqueue_t *q)
 
 	node_tx_enter_flowc(q->node);	/* Signal flow control */
 
-	if (dbg > 4)
+	if (GNET_PROPERTY(dbg) > 4)
 		printf("entering FLOWC for node %s (%d bytes queued)\n",
 			node_addr(q->node), q->size);
 }
@@ -492,7 +567,7 @@ mq_leave_flowc(mqueue_t *q)
 {
 	g_assert(q->flags & MQ_FLOWC);
 
-	if (dbg > 4)
+	if (GNET_PROPERTY(dbg) > 4)
 		printf("leaving %s for node %s (%d bytes queued)\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			node_addr(q->node), q->size);
@@ -501,11 +576,7 @@ mq_leave_flowc(mqueue_t *q)
 	if (q->qlink)
 		qlink_free(q);
 
-	if (q->swift_ev) {
-		cq_cancel(callout_queue, q->swift_ev);
-		q->swift_ev = NULL;
-	}
-
+	cq_cancel(callout_queue, &q->swift_ev);
 	node_tx_leave_flowc(q->node);	/* Signal end flow control */
 }
 
@@ -973,7 +1044,7 @@ make_room_header(
 	g_assert(needed > 0);
 	mq_check(q, 0);
 
-	if (dbg > 5)
+	if (GNET_PROPERTY(dbg) > 5)
 		printf("%s try to make room for %d bytes in queue 0x%lx (node %s)\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			needed, (gulong) q, node_addr(q->node));
@@ -1088,7 +1159,7 @@ restart:
 		 * Drop message.
 		 */
 
-		if (dbg > 4)
+		if (GNET_PROPERTY(dbg) > 4)
 			gmsg_log_dropped(pmsg_start(cmb),
 				"to %s node %s, in favor of %s",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
@@ -1100,8 +1171,8 @@ restart:
 		g_assert(q->qlink[n] == item);
 
 		needed -= cmb_size;
-		(void) mq_rmlink_prev(q, item, cmb_size);
 		q->qlink[n] = NULL;
+		(void) mq_rmlink_prev(q, item, cmb_size);
 
 		dropped++;
 
@@ -1111,7 +1182,7 @@ restart:
 	if (dropped)
 		node_add_txdrop(q->node, dropped);	/* Dropped during TX */
 
-	if (dbg > 5)
+	if (GNET_PROPERTY(dbg) > 5)
 		printf("%s end purge: %d bytes (count=%d) for node %s, need=%d\n",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
 			q->size, q->count, node_addr(q->node), needed);
@@ -1171,7 +1242,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		!make_room(q, mb, msize, &qlink_offset)
 	) {
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
-		if (dbg > 4)
+		if (GNET_PROPERTY(dbg) > 4)
 			gmsg_log_dropped(pmsg_start(mb),
 				"to FLOWC node %s, %d bytes queued",
 				node_addr(q->node), q->size);
@@ -1209,23 +1280,30 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		gnet_stats_count_flowc(pmsg_start(mb));
 
 		if (has_normal_prio) {
-			if (dbg > 4)
+			if (GNET_PROPERTY(dbg) > 4)
 				gmsg_log_dropped(pmsg_start(mb),
 					"to FLOWC node %s, %d bytes queued [FULL]",
 					node_addr(q->node), q->size);
 
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 		} else {
-			if (dbg > 4)
+			if (GNET_PROPERTY(dbg) > 4)
 				gmsg_log_dropped(pmsg_start(mb),
 					"to FLOWC node %s, %d bytes queued [KILLING]",
 					node_addr(q->node), q->size);
 
-			/* XXX: Is the check for UDP the correct fix or just a
-			 *		workaround? node_bye_v() asserts that the node isn't
-			 *		the UDP node.
-			 *		-- cbiere, 2004-12-20
+			/*
+			 * Is the check for UDP the correct fix or just a
+			 * workaround? node_bye_v() asserts that the node isn't
+			 * the UDP node.	-- cbiere, 2004-12-20
+			 *
+			 * Yes, it's the correct fix: node_bye() should only be called
+			 * when a TCP connection is made.  We could teach about UDP
+			 * in node_bye(), but I don't like routines with an unclear
+			 * contract (which attempt to "do the right thing" depending on
+			 * the parameter values). --RAM, 2006-12-29
 			 */
+
 			if (!NODE_IS_UDP(q->node)) {
 				node_bye(q->node, 502, "Send queue reached %d bytes",
 					q->maxsize);
