@@ -1,5 +1,5 @@
 /*
- * $Id: namesize.c 11945 2006-09-12 20:50:34Z cbiere $
+ * $Id: namesize.c 12446 2006-11-24 14:55:48Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"			/* For safe compilation under USE_DMALLOC */
 
-RCSID("$Id: namesize.c 11945 2006-09-12 20:50:34Z cbiere $")
+RCSID("$Id: namesize.c 12446 2006-11-24 14:55:48Z cbiere $")
 
 #include "namesize.h"
 
@@ -90,7 +90,8 @@ namesize_make(const gchar *name, filesize_t size)
 void
 namesize_free(namesize_t *ns)
 {
-	atom_str_free(ns->name);
+	atom_str_free_null(&ns->name);
 	wfree(ns, sizeof(*ns));
 }
 
+/* vi: set ts=4 sw=4 cindent: */
