@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 13870 2007-06-18 03:27:25Z cbiere $
+ * $Id: nodes.h 15053 2007-12-08 01:15:45Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -224,6 +224,8 @@ typedef struct gnutella_node {
 	host_addr_t gnet_qhit_addr;	/**< When != 0, last IP we got in query hit */
 	const gchar *guid;			/**< GUID of node (atom) seen on the network */
 
+	guint32 svn_release_revision;	/**< Latest SVN release known by the node */
+
 	guint32 n_ping_throttle;  /**< Number of pings we throttled */
 	guint32 n_ping_accepted;  /**< Number of pings we accepted */
 	guint32 n_ping_special;	  /**< Number of special pings we received */
@@ -279,24 +281,24 @@ enum {
 	NODE_F_WRITABLE		= 1 << 6,	/**< Node is writable */
 	NODE_F_READABLE		= 1 << 7,	/**< Node is readable, process queries */
 	NODE_F_BYE_SENT		= 1 << 8,	/**< Bye message was queued */
-	NODE_F_NODELAY		= 1 << 9,	/**< TCP_NODELAY was activated */
-	NODE_F_NOREAD		= 1 << 10,	/**< Prevent further reading from node */
-	NODE_F_EOF_WAIT		= 1 << 11,	/**< At final shutdown, waiting EOF */
-	NODE_F_CLOSING		= 1 << 12,	/**< Initiated bye or shutdown */
-	NODE_F_ULTRA		= 1 << 13,	/**< Is one of our ultra nodes */
-	NODE_F_LEAF			= 1 << 14,	/**< Is one of our leaves */
-	NODE_F_CRAWLER		= 1 << 15,	/**< Is a Gnutella Crawler */
-	NODE_F_FAKE_NAME	= 1 << 16,	/**< Was unable to validate GTKG name */
-	NODE_F_PROXY		= 1 << 17,	/**< Sent a push-proxy request to it */
-	NODE_F_PROXIED		= 1 << 18,	/**< We are push-proxy for that node */
-	NODE_F_QRP_SENT		= 1 << 19,	/**< Undergone 1 complete QRP sending */
-	NODE_F_TSYNC_WAIT	= 1 << 20,	/**< Time sync pending via TCP */
-	NODE_F_TSYNC_TCP	= 1 << 21,	/**< No replies via UDP, use TCP */
-	NODE_F_GTKG			= 1 << 22,	/**< Node is another gtk-gnutella */
-	NODE_F_FORCE		= 1 << 23,	/**< Connection is forced */
-	NODE_F_NO_OOB_PROXY	= 1 << 24,	/**< Do not OOB proxy the leaf */
-	NODE_F_TLS			= 1 << 25,	/**< TLS-tunneled */
-	NODE_F_CAN_TLS		= 1 << 26	/**< Indicated support for TLS */
+	NODE_F_NOREAD		= 1 << 9,	/**< Prevent further reading from node */
+	NODE_F_EOF_WAIT		= 1 << 10,	/**< At final shutdown, waiting EOF */
+	NODE_F_CLOSING		= 1 << 11,	/**< Initiated bye or shutdown */
+	NODE_F_ULTRA		= 1 << 12,	/**< Is one of our ultra nodes */
+	NODE_F_LEAF			= 1 << 13,	/**< Is one of our leaves */
+	NODE_F_CRAWLER		= 1 << 14,	/**< Is a Gnutella Crawler */
+	NODE_F_FAKE_NAME	= 1 << 15,	/**< Was unable to validate GTKG name */
+	NODE_F_PROXY		= 1 << 16,	/**< Sent a push-proxy request to it */
+	NODE_F_PROXIED		= 1 << 17,	/**< We are push-proxy for that node */
+	NODE_F_QRP_SENT		= 1 << 18,	/**< Undergone 1 complete QRP sending */
+	NODE_F_TSYNC_WAIT	= 1 << 19,	/**< Time sync pending via TCP */
+	NODE_F_TSYNC_TCP	= 1 << 20,	/**< No replies via UDP, use TCP */
+	NODE_F_GTKG			= 1 << 21,	/**< Node is another gtk-gnutella */
+	NODE_F_FORCE		= 1 << 22,	/**< Connection is forced */
+	NODE_F_NO_OOB_PROXY	= 1 << 23,	/**< Do not OOB proxy the leaf */
+	NODE_F_TLS			= 1 << 24,	/**< TLS-tunneled */
+	NODE_F_CAN_TLS		= 1 << 25,	/**< Indicated support for TLS */
+	NODE_F_VMSG_SUPPORT	= 1 << 26	/**< Indicated which VMSGs are supported */
 };
 
 /*
@@ -326,7 +328,8 @@ enum {
 	NODE_A_CAN_VENDOR	= 1 << 18,	/**< Node supports vendor messages */
 	NODE_A_CAN_ULTRA	= 1 << 19,	/**< Node is ultra capable */
 	NODE_A_CAN_INFLATE	= 1 << 20,	/**< Node capable of inflating */
-	NODE_A_CAN_HEAD		= 1 << 21	/**< Supports HEAD ping (vendor message) */
+	NODE_A_CAN_HEAD		= 1 << 21,	/**< Supports HEAD ping (vendor message) */
+	NODE_A_CAN_SVN_NOTIFY =	1 << 22	/**< Supports SVN release notifications */
 };
  
 /*
@@ -610,6 +613,7 @@ gnutella_node_t *node_browse_prepare(
 	gchar *data, guint32 size);
 void node_browse_cleanup(gnutella_node_t *n);
 void node_kill_hostiles(void);
+void node_supports_tls(struct gnutella_node *);
 
 node_id_t node_id_get_self(void);
 gboolean node_id_self(const node_id_t node_id);
