@@ -1,5 +1,5 @@
 /*
- * $Id: oob.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: oob.h 13164 2007-03-20 20:20:00Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,22 +36,23 @@
 #ifndef _core_oob_h_
 #define _core_oob_h_
 
-#include <glib.h>
+#include "common.h"
 
 /*
  * Public interface.
  */
 
 struct gnutella_node;
+struct array;
 
 void oob_init(void);
 void oob_shutdown(void);
 void oob_close(void);
 
-void oob_got_results(
-	struct gnutella_node *n, GSList *files, gint count, gboolean use_ggep_h);
-void oob_deliver_hits(struct gnutella_node *n,
-	const gchar *muid, guint8 wanted);
+void oob_got_results(struct gnutella_node *n, GSList *files,
+		gint count, gboolean secure_oob, gboolean ggep_h);
+void oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted,
+	const struct array *token);
 
 #endif /* _core_oob_h_ */
 
