@@ -1,5 +1,5 @@
 /*
- * $Id: pcache.c 13893 2007-06-19 17:33:36Z cbiere $
+ * $Id: pcache.c 15129 2008-03-11 10:40:37Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pcache.c 13893 2007-06-19 17:33:36Z cbiere $")
+RCSID("$Id: pcache.c 15129 2008-03-11 10:40:37Z rmanfredi $")
 
 #include "alive.h"
 #include "extensions.h"
@@ -189,10 +189,8 @@ build_ping_msg(const gchar *muid, guint8 ttl, gboolean uhc, guint32 *size)
 		ggep_stream_init(&gs, ggep, sizeof msg_init.buf - sizeof *m);
 		
 		spp = GNET_PROPERTY(current_peermode) == NODE_P_LEAF ? 0x0 : 0x1;
-#ifdef HAS_GNUTLS
-		spp |= 0x02;
-#endif	/* HAS_GNUTLS */
-		
+		spp |= tls_enabled() ? 0x02 : 0;
+
 		ok = ggep_stream_pack(&gs, GGEP_NAME(SCP), &spp, sizeof spp, 0);
 		g_assert(ok);
 
@@ -614,9 +612,7 @@ send_personal_info(struct gnutella_node *n, gboolean control,
 		local_meta.flags |= PONG_META_HAS_IPV6;
 	}
 
-#ifdef HAS_GNUTLS
-	local_meta.flags |= PONG_META_HAS_TLS;
-#endif	/* HAS_GNUTLS */
+	local_meta.flags |= tls_enabled() ? PONG_META_HAS_TLS : 0;
 
 	send_pong(n, control, flags, 0,
 		MIN(gnutella_header_get_hops(&n->header) + 1U, GNET_PROPERTY(max_ttl)),
@@ -893,6 +889,12 @@ free_cached_pong(struct cached_pong *cp)
  *
  * Fills `addr' and `port' with the pong value and return TRUE if we
  * got a pong.  Otherwise return FALSE.
+ *
+ * XXX This routine is no longer called anywhere.
+ * XXX Need to think about whether we should just remove this pcache recent
+ * XXX pong caching alltogether, or whether we need to keep it for the pong
+ * XXX demultiplexing logic?
+ * XXX		--RAM, 2008-03-11
  */
 gboolean
 pcache_get_recent(host_type_t type, host_addr_t *addr, guint16 *port)
