@@ -1,5 +1,5 @@
 /*
- * $Id: pproxy.c 13815 2007-06-14 05:41:30Z cbiere $
+ * $Id: pproxy.c 14868 2007-09-09 23:41:25Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: pproxy.c 13815 2007-06-14 05:41:30Z cbiere $")
+RCSID("$Id: pproxy.c 14868 2007-09-09 23:41:25Z cbiere $")
 
 #include "pproxy.h"
 #include "http.h"
@@ -543,11 +543,10 @@ build_push(guint8 ttl, guint8 hops, const gchar *guid,
 
 	ggep_stream_init(&gs, p, size);
 
-#ifdef HAS_GNUTLS
-	supports_tls = supports_tls
-		|| is_my_address_and_port(addr_v4, port)
-		|| is_my_address_and_port(addr_v6, port);
-#endif /* HAS_GNUTLS */
+	if (!supports_tls && tls_enabled()) {
+		supports_tls = is_my_address_and_port(addr_v4, port)
+			|| is_my_address_and_port(addr_v6, port);
+	}
 	
 	if (supports_tls) {
 		if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0)) {
@@ -626,6 +625,58 @@ validate_vendor(gchar *vendor, gchar *token, const host_addr_t addr)
 	return result;
 }
 
+static void
+pproxy_fetch_addresses(struct pproxy *pp, const char *buf)
+{
+	const gchar *endptr;
+	host_addr_t addr;
+	guint16 port;
+
+	if (NULL == buf)
+		return;
+
+	if (!string_to_host_addr_port(buf, &endptr, &addr, &port))
+		return;
+
+	pp->addr_v4 = zero_host_addr;
+	pp->addr_v6 = zero_host_addr;
+	pp->port = port;
+
+	switch (host_addr_net(addr)) {
+	case NET_TYPE_IPV4:
+		pp->addr_v4 = addr;
+		break;
+	case NET_TYPE_IPV6:
+		pp->addr_v6 = addr;
+		break;
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		break;
+	}
+
+	/* Allow a second address */
+	endptr = skip_ascii_spaces(endptr);
+	if (',' == *endptr) {
+		endptr = skip_ascii_spaces(&endptr[1]);
+		if (
+			string_to_host_addr_port(endptr, NULL, &addr, &port) &&
+			port == pp->port
+		) {
+			switch (host_addr_net(addr)) {
+			case NET_TYPE_IPV4:
+				pp->addr_v4 = addr;
+				break;
+			case NET_TYPE_IPV6:
+				pp->addr_v6 = addr;
+				break;
+			case NET_TYPE_LOCAL:
+			case NET_TYPE_NONE:
+				break;
+			}
+		}
+	}
+}
+
 /**
  * Called once all the HTTP headers have been read to proceed with
  * the push proxyfication.
@@ -636,7 +687,7 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	struct gnutella_socket *s = pp->socket;
 	const gchar *request = getline_str(s->getline);
 	struct gnutella_node *n;
-	gchar *buf;
+	const gchar *buf;
 	gchar *token;
 	gchar *user_agent;
 	GSList *nodes;
@@ -678,60 +729,12 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	 */
 
 	buf = header_get(header, "X-Node");
-
-	if (buf == NULL) {
-		pproxy_error_remove(pp, 400,
-			"Malformed push-proxy request: missing X-Node header");
-		return;
+	if (buf) {
+		pproxy_fetch_addresses(pp, buf);
 	}
-
-	{
-		const gchar *endptr;
-		host_addr_t addr;
-		guint16 port;
-
-		if (!string_to_host_addr_port(buf, &endptr, &addr, &port)) {
-			pproxy_error_remove(pp, 400,
-				"Malformed push-proxy request: cannot parse X-Node");
-			return;
-		}
-
-		pp->addr_v4 = zero_host_addr;
-		pp->addr_v6 = zero_host_addr;
-		pp->port = port;
-
-		switch (host_addr_net(addr)) {
-		case NET_TYPE_IPV4:
-			pp->addr_v4 = addr;
-			break;
-		case NET_TYPE_IPV6:
-			pp->addr_v6 = addr;
-			break;
-		case NET_TYPE_LOCAL:
-		case NET_TYPE_NONE:
-			break;
-		}
-		
-		endptr = skip_ascii_spaces(endptr);
-		if (',' == *endptr) {
-			endptr = skip_ascii_spaces(&endptr[1]);
-			if (
-				string_to_host_addr_port(endptr, NULL, &addr, &port) &&
-				port == pp->port
-			) {
-				switch (host_addr_net(addr)) {
-				case NET_TYPE_IPV4:
-					pp->addr_v4 = addr;
-					break;
-				case NET_TYPE_IPV6:
-					pp->addr_v6 = addr;
-					break;
-				case NET_TYPE_LOCAL:
-				case NET_TYPE_NONE:
-					break;
-				}
-			}
-		}
+	buf = header_get(header, "X-Node-IPv6");
+	if (buf) {
+		pproxy_fetch_addresses(pp, buf);
 	}
 
 	if (!host_is_valid(pp->addr_v4, pp->port)) {
@@ -743,7 +746,7 @@ pproxy_request(struct pproxy *pp, header_t *header)
 	
 	if (!is_host_addr(pp->addr_v4) && !is_host_addr(pp->addr_v6)) {
 		pproxy_error_remove(pp, 400,
-			"Malformed push-proxy request: supplied address is unreachable");
+			"Malformed push-proxy request: supplied no valid address");
 		return;
 	}
 
@@ -1132,14 +1135,10 @@ cproxy_build_request(struct http_async *unused_handle, gchar *buf, size_t len,
 	const gchar *verb, const gchar *path, const gchar *unused_host,
 	guint16 unused_port)
 {
-	gchar addr_buf[128];
+	gchar addr_v4_buf[128];
 	gchar addr_v6_buf[128];
 	host_addr_t addr;
-
-	/*
-	 * Note that we send an HTTP/1.0 request here, hence we need neither
-	 * the Host: nor the Connection: header.
-	 */
+	gboolean has_ipv4 = FALSE;
 
 	(void) unused_handle;
 	(void) unused_host;
@@ -1147,30 +1146,42 @@ cproxy_build_request(struct http_async *unused_handle, gchar *buf, size_t len,
 	g_assert(len <= INT_MAX);
 
 	addr = listen_addr();
+	addr_v4_buf[0] = '\0';
 	if (is_host_addr(addr)) {
-		gm_snprintf(addr_buf, sizeof addr_buf, "X-Node: %s\r\n",
-			host_addr_port_to_string(addr, GNET_PROPERTY(listen_port)));
-	} else {
-		addr_buf[0] = '\0';
+		has_ipv4 = TRUE;
+		concat_strings(addr_v4_buf, sizeof addr_v4_buf,
+			"X-Node: ",
+			host_addr_port_to_string(addr, GNET_PROPERTY(listen_port)),
+			"\r\n",
+			(void *) 0);
 	}
+
 	addr = listen_addr6();
+	addr_v6_buf[0] = '\0';
 	if (is_host_addr(addr)) {
-		gm_snprintf(addr_v6_buf, sizeof addr_v6_buf, "X-Node: %s\r\n",
-			host_addr_port_to_string(addr, GNET_PROPERTY(listen_port)));
-	} else {
-		addr_v6_buf[0] = '\0';
+		/* Older clients only know X-Node, so if we don't have an IPv4
+		 * address, use the X-Node header instead. If they don't support
+		 * IPv6 we lose anyway.
+		 */
+		concat_strings(addr_v6_buf, sizeof addr_v6_buf,
+			has_ipv4 ? "X-Node-IPv6: " : "X-Node: ",
+			host_addr_port_to_string(addr, GNET_PROPERTY(listen_port)),
+			"\r\n",
+			(void *) 0);
 	}
 	
 	return gm_snprintf(buf, len,
-		"%s %s HTTP/1.0\r\n"
+		"%s %s HTTP/1.1\r\n"
 		"User-Agent: %s\r\n"
+		"Connection: close\r\n"
+		"Host:\r\n"
 		"X-Token: %s\r\n"
 		"%s"
 		"%s"
 		"\r\n",
 		verb, path, version_string,
 		tok_version(),
-		addr_buf,
+		addr_v4_buf,
 		addr_v6_buf);
 }
 
@@ -1205,9 +1216,7 @@ cproxy_create(struct download *d, const host_addr_t addr, guint16 port,
 
 	concat_strings(path, sizeof path,
 		"/gnutella/push-proxy?ServerId=", guid_base32_str(guid),
-#ifdef HAS_GNUTLS
-		"&tls=true",
-#endif /* HAS_GNUTLS */
+		tls_enabled() ? "&tls=true" : "",
 		(void *) 0);
 
 	/*
@@ -1231,7 +1240,7 @@ cproxy_create(struct download *d, const host_addr_t addr, guint16 port,
 	cp->addr = addr;
 	cp->port = port;
 	cp->guid = atom_guid_get(guid);
-	cp->file_idx = file_idx;
+	cp->file_idx = file_idx == URN_INDEX ? 0 : file_idx;
 	cp->http_handle = handle;
 	cp->flags = 0;
 
