@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.c 13893 2007-06-19 17:33:36Z cbiere $
+ * $Id: qhit.c 15058 2007-12-09 01:36:11Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: qhit.c 13893 2007-06-19 17:33:36Z cbiere $")
+RCSID("$Id: qhit.c 15058 2007-12-09 01:36:11Z cbiere $")
 
 #include "bsched.h"
 #include "dmesh.h"		/* For dmesh_fill_alternate() */
@@ -62,7 +62,7 @@ RCSID("$Id: qhit.c 13893 2007-06-19 17:33:36Z cbiere $")
 
 #define QHIT_SIZE_THRESHOLD	2016	/**< Flush query hits larger than this */
 #define QHIT_MAX_RESULTS	255		/**< Maximum amount of hits in a query hit */
-#define QHIT_MAX_ALT		15		/**< Send out 15 alt-locs per entry, max */
+#define QHIT_MAX_ALT		10		/**< Send out 10 alt-locs per entry, max */
 #define QHIT_MAX_PROXIES	5		/**< Send out 5 push-proxies at most */
 #define QHIT_MAX_GGEP		512		/**< Allocated room for trailing GGEP */
 
@@ -177,9 +177,8 @@ found_write(gconstpointer data, size_t length)
 	g_assert(length <= INT_MAX);
 	g_assert(!f->open);
 
-	if (length > sizeof f->data - f->pos) {
+	if (length > sizeof f->data - f->pos)
 		return FALSE;
-	}
 
 	g_assert(f->pos < sizeof f->data);
 	memcpy(&f->data[f->pos], data, length);
@@ -516,12 +515,12 @@ flush_match(void)
 		}
 	}
 
-#ifdef HAS_GNUTLS
-	if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0))
-		g_warning("could not write GGEP \"TLS\" extension into query hit");
-	if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
-		g_warning("could not write GGEP \"GTKG.TLS\" extension into query hit");
-#endif /* HAS_GNUTLS */
+	if (tls_enabled()) {
+		if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0))
+			g_warning("could not write GGEP \"TLS\" extension into QHIT");
+		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
+			g_warning("could not write GGEP \"GTKG.TLS\" extension into QHIT");
+	}
 
 
 	/*
@@ -648,16 +647,20 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (sha1_available && !found_ggep_h()) {
-		static const gchar urnsha1[] = "urn:sha1:";
-		const gchar *b32 = sha1_base32(shared_file_sha1(sf));
+		const struct sha1 * const sha1 = shared_file_sha1(sf);
+		const struct tth * const tth = shared_file_tth(sf);
 
 		/* Good old way: ASCII URN */
-		if (!found_write(urnsha1, CONST_STRLEN(urnsha1)))
-			return FALSE;
-		if (!found_write(b32, SHA1_BASE32_SIZE))
+		if (!found_write(sha1_to_urn_string(sha1), SHA1_URN_LENGTH))
 			return FALSE;
 		if (!found_write("\x1c", 1))
 			return FALSE;
+		if (tth) {
+			if (!found_write(tth_to_urn_string(tth), TTH_URN_LENGTH))
+				return FALSE;
+			if (!found_write("\x1c", 1))
+				return FALSE;
+		}
 	}
 
 	/*
@@ -670,16 +673,19 @@ add_file(const struct shared_file *sf)
 
 	/*
 	 * Emit the SHA1 as GGEP "H" if they said they understand it.
+	 * Modern way: GGEP "H" for binary URN
 	 */
 
 	if (sha1_available && found_ggep_h()) {
-		/* Modern way: GGEP "H" for binary URN */
-		guint8 type = GGEP_H_SHA1;
+		const struct sha1 * const sha1 = shared_file_sha1(sf);
+		const struct tth * const tth = shared_file_tth(sf);
+		const guint8 type = tth ? GGEP_H_BITPRINT : GGEP_H_SHA1;
 
 		ok =
 			ggep_stream_begin(&gs, GGEP_NAME(H), GGEP_W_COBS) &&
 			ggep_stream_write(&gs, &type, 1) &&
-			ggep_stream_write(&gs, shared_file_sha1(sf), SHA1_RAW_SIZE) &&
+			ggep_stream_write(&gs, sha1->data, SHA1_RAW_SIZE) &&
+			(tth ? ggep_stream_write(&gs, tth->data, TTH_RAW_SIZE) : TRUE) &&
 			ggep_stream_end(&gs);
 
 		if (!ok)
