@@ -1,5 +1,5 @@
 /*
- * $Id: routing.c 13783 2007-05-29 16:57:45Z cbiere $
+ * $Id: routing.c 15033 2007-11-30 19:41:48Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: routing.c 13783 2007-05-29 16:57:45Z cbiere $")
+RCSID("$Id: routing.c 15033 2007-11-30 19:41:48Z cbiere $")
 
 #include "search.h"			/* For search_passive. */
 #include "routing.h"
@@ -560,7 +560,7 @@ get_next_entry(void)
  *
  * @return the new location of the revitalized entry
  */
-void
+static void
 revitalize_entry(struct message *entry, gboolean force)
 {
 	struct message **relocated;
@@ -714,7 +714,6 @@ routing_init(void)
 {
     gchar guid_buf[GUID_RAW_SIZE];
 	guint32 i;
-	gboolean need_guid = TRUE;
 
 	/*
 	 * Make sure it segfaults if we try to access it, but it must be
@@ -741,39 +740,13 @@ routing_init(void)
 			atom_guid_get(g), GUINT_TO_POINTER(1));
 	}
 
-	/*
-	 * Only generate a new GUID for this servent if all entries are 0.
-	 * The empty initialization happens in config_init(), but it can be
-	 * overridden by the GUID read from the configuration file
-	 * (persistent GUID).
-	 *		--RAM, 08/03/2002
-	 */
-
-    gnet_prop_get_storage(PROP_SERVENT_GUID, guid_buf, sizeof(guid_buf));
-
-	/* The last byte (#15) is a marker */
-	for (i = 0; i < GUID_RAW_SIZE - 1; i++) {
-		if (guid_buf[i]) {
-			need_guid = FALSE;
-			break;
-		}
-	}
-
-	if (!guid_is_gtkg(guid_buf, NULL, NULL, NULL))
-		need_guid = TRUE;
-
-retry:
-	if (need_guid)
-		guid_ping_muid(guid_buf);		/* We want a "modern" GUID */
-
-	/*
-	 * If by extraordinary, we have generated a banned GUID, retry.
-	 */
-
-	if (is_banned_push(guid_buf)) {
-		need_guid = TRUE;
-		goto retry;
-	}
+	/* The Servent GUID is not persistent. A new is generated each session. */
+	do {
+		guid_random_muid(guid_buf);
+		/*
+		 * If by extraordinary, we have generated a banned GUID, retry.
+		 */
+	} while (is_banned_push(guid_buf));
 
     gnet_prop_set_storage(PROP_SERVENT_GUID, guid_buf, sizeof(guid_buf));
 	g_assert(guid_is_gtkg(guid_buf, NULL, NULL, NULL));
@@ -2081,23 +2054,28 @@ route_exists_for_reply(const gchar *muid, guint8 function)
 GSList *
 route_towards_guid(const gchar *guid)
 {
+	struct gnutella_node *node;
 	struct message *m;
-	GSList *nodes = NULL;
 
-	if (
-		!is_banned_push(guid) &&
-		find_message(guid, QUERY_HIT_ROUTE_SAVE, &m) &&
-		NULL != m->routes
-	) {
-		GSList *sl;
+	if (is_banned_push(guid))
+		return NULL;
+	
+	node = node_by_guid(guid);
+	if (node)
+		return g_slist_prepend(NULL, node);
+	
+	if (find_message(guid, QUERY_HIT_ROUTE_SAVE, &m) && m->routes) {
+		GSList *iter, *nodes = NULL;
 		
 		revitalize_entry(m, TRUE);
-		for (sl = m->routes; NULL != sl; sl = g_slist_next(sl)) {
-			struct route_data *rd = sl->data;
+		for (iter = m->routes; NULL != iter; iter = g_slist_next(iter)) {
+			struct route_data *rd = iter->data;
 			nodes = g_slist_prepend(nodes, rd->node);
 		}
+		return nodes;
 	}
-	return nodes;
+
+	return NULL;
 }
 
 /**
