@@ -1,5 +1,5 @@
 /*
- * $Id: rx.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: rx.c 13504 2007-05-05 21:04:17Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -40,7 +40,7 @@
 
 #include "common.h"
 
-RCSID("$Id: rx.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: rx.c 13504 2007-05-05 21:04:17Z rmanfredi $")
 
 #include "rx.h"
 #include "nodes.h"
@@ -144,6 +144,25 @@ rx_set_data_ind(rxdrv_t *rx, rx_data_t data_ind)
 	rx->data_ind = data_ind;
 }
 
+/**
+ * Replace the `data_ind' callback, returning the old one.
+ */
+rx_data_t
+rx_replace_data_ind(rxdrv_t *rx, rx_data_t data_ind)
+{
+	rx_data_t old_data_ind;
+
+	g_assert(rx);
+	g_assert(rx->upper == NULL);			/* Called on topmost driver */
+	g_assert(data_ind != rx_data_ind);		/* Must not use internal routine */
+	g_assert(rx->data_ind != rx_data_ind);	/* data_ind has been set already */
+
+	old_data_ind = rx->data_ind;
+	rx->data_ind = data_ind;
+
+	return old_data_ind;
+}
+
 /**
  * Called when an upper driver (urx) is attached on top of us.
  */
