@@ -1,5 +1,5 @@
 /*
- * $Id: rx_chunk.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: rx_chunk.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2005, Christian Biere
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: rx_chunk.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: rx_chunk.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "pmsg.h"
 #include "hosts.h"				/* For host_ip() */
@@ -142,7 +142,7 @@ parse_chunk(rxdrv_t *rx, const gchar *src, size_t size,
 					if (!(attr->flags & IF_NO_CRLF)) {
 						attr->flags |= IF_NO_CRLF;
 						g_warning("Host %s forgot CRLF after data",
-							host_to_string(&rx->host));
+							gnet_host_to_string(&rx->host));
 					}
 
 					len++;
@@ -283,7 +283,7 @@ parse_chunk(rxdrv_t *rx, const gchar *src, size_t size,
 		 */
 
 		if (CHUNK_STATE_DATA == attr->state) {
-			if (dbg > 9)
+			if (GNET_PROPERTY(dbg) > 9)
 				printf("parse_chunk: chunk size %lu bytes\n",
 					(gulong) attr->data_remain);
 			break;
@@ -345,11 +345,12 @@ dechunk_data(rxdrv_t *rx, pmsg_t *mb)
 			} else {
 				/* Only the first ``data_remain'' bytes are forwarded */
 				mb->m_rptr += attr->data_remain;
-				nmb->m_wptr = nmb->m_rptr + attr->data_remain;
+				nmb->m_wptr =
+					deconstify_gpointer(&nmb->m_rptr[attr->data_remain]);
 				attr->data_remain = 0;
 				attr->state = CHUNK_STATE_DATA_CRLF;
 			}
-			if (dbg > 9)
+			if (GNET_PROPERTY(dbg) > 9)
 				printf("dechunk_data: returning chunk of %u bytes\n",
 					pmsg_size(nmb));
 			return nmb;
