@@ -1,5 +1,5 @@
 /*
- * $Id: rx_link.h 10744 2006-04-20 21:37:35Z cbiere $
+ * $Id: rx_link.h 12583 2006-12-06 05:42:15Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,10 @@
 #ifndef _core_rx_link_h_
 #define _core_rx_link_h_
 
+#include "common.h"
+
 #include "rx.h"
+#include "if/core/bsched.h"
 
 const struct rxdrv_ops *rx_link_get_ops(void);
 
@@ -55,8 +58,8 @@ struct rx_link_cb {
  */
 struct rx_link_args {
 	const struct rx_link_cb *cb;	/**< Callbacks */
-	struct bsched *bs;				/**< Bandwidth scheduler to use */
 	struct wrap_io *wio;			/**< I/O wrapping routines */
+	bsched_bws_t bws;				/**< Bandwidth scheduler to use */
 };
 
 #endif	/* _core_rx_link_h_ */
