@@ -1,5 +1,5 @@
 /*
- * $Id: settings.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: settings.c 15066 2007-12-24 18:04:41Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -61,14 +61,16 @@
 #include "if/bridge/c2ui.h"
 
 #include "lib/bit_array.h"
-#include "lib/getphysmemsize.h"
 #include "lib/cq.h"
 #include "lib/file.h"
+#include "lib/getphysmemsize.h"
 #include "lib/glib-missing.h"
+#include "lib/sha1.h"
 #include "lib/tm.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: settings.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: settings.c 15066 2007-12-24 18:04:41Z cbiere $")
 
 static const gchar config_file[] = "config_gnet";
 static const gchar ul_stats_file[] = "upload_stats";
@@ -385,6 +387,25 @@ settings_ensure_unicity(gboolean check_only)
 	return 0;
 }
 
+static void
+settings_init_session_id(void)
+{
+	SHA1Context ctx;
+	struct sha1 digest;
+	guint32 noise[64];
+	size_t size, i;
+
+	for (i = 0; i < G_N_ELEMENTS(noise); i++) {
+		noise[i] = random_raw();
+	}
+	SHA1Reset(&ctx);
+	SHA1Input(&ctx, &noise, sizeof noise);
+	SHA1Result(&ctx, &digest);
+
+	size = MIN(GUID_RAW_SIZE, sizeof digest.data);
+	gnet_prop_set_storage(PROP_SESSION_ID, digest.data, size);
+}
+
 void
 settings_init(void)
 {
@@ -411,6 +432,7 @@ settings_init(void)
 	gnet_prop_set_guint32_val(PROP_SYS_NOFILE, max_fd);
 	gnet_prop_set_guint64_val(PROP_SYS_PHYSMEM, amount);
 
+	settings_init_session_id();
 	memset(deconstify_gpointer(GNET_PROPERTY(servent_guid)), 0, GUID_RAW_SIZE);
 
 	if (NULL == config_dir || '\0' == config_dir[0])
@@ -500,7 +522,7 @@ settings_home_dir(void)
 /**
  * Gets the IPC directory.
  */
-const gchar *
+static const gchar *
 settings_ipc_dir(void)
 {
 	static const gchar *path;
@@ -1264,7 +1286,7 @@ listen_port_changed(property_t prop)
 		gnet_prop_set_guint32_val(prop, port);
 	}
 
-	if (GNET_PROPERTY(is_firewalled) || !settings_init_running) {
+	if (!settings_init_running) {
 		inet_firewalled();
 		inet_udp_firewalled();
 	}
@@ -1289,8 +1311,7 @@ network_protocol_changed(property_t prop)
 {
 
 	(void) prop;
-	request_new_sockets(GNET_PROPERTY(listen_port),
-		GNET_PROPERTY(is_firewalled) || !settings_init_running);
+	request_new_sockets(GNET_PROPERTY(listen_port), !settings_init_running);
 	return FALSE;
 }
 
@@ -1556,8 +1577,7 @@ forced_local_ip_changed(property_t prop)
 	(void) prop;
 	if (GNET_PROPERTY(force_local_ip) || GNET_PROPERTY(force_local_ip6)) {
 		update_address_lifetime();
-		request_new_sockets(GNET_PROPERTY(listen_port),
-			GNET_PROPERTY(is_firewalled) || !settings_init_running);
+		request_new_sockets(GNET_PROPERTY(listen_port), !settings_init_running);
 	}
     return FALSE;
 }
