@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.c 13934 2007-06-23 01:23:35Z cbiere $
+ * $Id: sockets.c 15053 2007-12-08 01:15:45Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: sockets.c 13934 2007-06-23 01:23:35Z cbiere $")
+RCSID("$Id: sockets.c 15053 2007-12-08 01:15:45Z cbiere $")
 
 #ifdef I_NETDB
 #include <netdb.h>
@@ -61,9 +61,10 @@ RCSID("$Id: sockets.c 13934 2007-06-23 01:23:35Z cbiere $")
 #include "pproxy.h"
 #include "udp.h"
 #include "settings.h"
-#include "shell.h"
 #include "tls_cache.h"
 
+#include "shell/shell.h"
+
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
@@ -83,8 +84,8 @@ RCSID("$Id: sockets.c 13934 2007-06-23 01:23:35Z cbiere $")
 #include "lib/override.h"		/* Must be the last header included */
 
 #ifndef SHUT_WR
-/* XXX: This should be handled by Configure because SHUT_* are sometimes
- *		enums instead of macro definitions.
+/* FIXME: This should be handled by Configure because SHUT_* are sometimes
+ *		  enums instead of macro definitions.
  */
 #define SHUT_WR 1					/**< Shutdown TX side */
 #endif
@@ -92,9 +93,12 @@ RCSID("$Id: sockets.c 13934 2007-06-23 01:23:35Z cbiere $")
 #define RQST_LINE_LENGTH	256	/**< Reasonable estimate for request line */
 #define SOCK_UDP_RECV_BUF	131072	/**< 128K - Large to avoid loosing dgrams */
 
-#define SOCK_ADNS_PENDING	0x01	/**< Don't free() the socket too early */
-#define SOCK_ADNS_FAILED	0x02	/**< Signals error in the ADNS callback */
-#define SOCK_ADNS_BADNAME	0x04	/**< Signals bad host name */
+enum {
+	SOCK_ADNS_PENDING	= 1 << 0,	/**< Don't free() the socket too early */
+	SOCK_ADNS_FAILED	= 1 << 1,	/**< Signals error in the ADNS callback */
+	SOCK_ADNS_BADNAME	= 1 << 2,	/**< Signals bad host name */
+	SOCK_ADNS_ASYNC		= 1 << 3	/**< Signals async resolution */
+};
 
 struct gnutella_socket *s_tcp_listen = NULL;
 struct gnutella_socket *s_tcp_listen6 = NULL;
@@ -140,7 +144,7 @@ socket_free_buffer(struct gnutella_socket *s)
 	}
 }
 
-void
+static void
 socket_dealloc(struct gnutella_socket **s_ptr)
 {
 	struct gnutella_socket *s;
@@ -155,7 +159,7 @@ socket_dealloc(struct gnutella_socket **s_ptr)
 	}
 }
 
-host_addr_t
+static host_addr_t
 socket_ipv6_trt_map(const host_addr_t addr)
 {
 	if (
@@ -228,16 +232,14 @@ socket_evt_set(struct gnutella_socket *s,
 
 	fd = socket_evt_fd(s);
 
-#ifdef HAS_GNUTLS
 	s->tls.cb_cond = cond;
 	s->tls.cb_handler = handler;
 	s->tls.cb_data = data;
 
-	if (GNET_PROPERTY(tls_debug) > 1)
+	if (GNET_PROPERTY(tls_debug) > 4) {
 		g_message("socket_evt_set: fd=%d, cond=%s, handler=%p",
 			fd, inputevt_cond_to_string(cond), handler);
-#endif /* HAS_GNUTLS */
-
+	}
 	s->gdk_tag = inputevt_add(fd, cond, handler, data);
 	g_assert(0 != s->gdk_tag);
 	
@@ -257,8 +259,7 @@ socket_evt_clear(struct gnutella_socket *s)
 	socket_check(s);
 
 	if (s->gdk_tag) {
-#ifdef HAS_GNUTLS
-		if (GNET_PROPERTY(tls_debug) > 1) {
+		if (GNET_PROPERTY(tls_debug) > 4) {
 			gint fd = socket_evt_fd(s);
 			g_message("socket_evt_clear: fd=%d, cond=%s, handler=%p",
 				fd, inputevt_cond_to_string(s->tls.cb_cond), s->tls.cb_handler);
@@ -267,7 +268,7 @@ socket_evt_clear(struct gnutella_socket *s)
 		s->tls.cb_cond = 0;
 		s->tls.cb_handler = NULL;
 		s->tls.cb_data = NULL;
-#endif /* HAS_GNUTLS */
+
 		inputevt_remove(s->gdk_tag);
 		s->gdk_tag = 0;
 	}
@@ -611,7 +612,7 @@ send_socks4(struct gnutella_socket *s)
 		length = sizeof *req;
 	}
 
-	/* XXX: Shouldn't this use the configured username instead? */
+	/* FIXME: Shouldn't this use the configured username instead? */
 	/* Determine the current username */
 	{
 		const struct passwd *user;
@@ -1258,7 +1259,7 @@ socket_destroy(struct gnutella_socket *s, const gchar *reason)
 		break;
 	case SOCK_TYPE_DOWNLOAD:
 		if (s->resource.download) {
-			download_stop(s->resource.download, GTA_DL_ERROR, "%s", reason);
+			download_queue(s->resource.download, "%s", reason);
 			return;
 		}
 		break;
@@ -1291,50 +1292,6 @@ socket_destroy(struct gnutella_socket *s, const gchar *reason)
 	socket_free_null(&s);
 }
 
-struct socket_linger {
-	guint tag;	/* Holds the result of inputevt_add() */
-};
-
-static void
-socket_linger_cb(gpointer data, gint fd, inputevt_cond_t unused_cond)
-{
-	struct socket_linger *ctx = data;
-
-	(void) unused_cond;
-	g_assert(fd >= 0);
-	g_assert(NULL != data);
-	g_assert(0 != ctx->tag);
-
-	if (close(fd)) {
-		gint e = errno;
-
-		if (!is_temporary_error(e))
-			g_warning("close(%d) failed: %s", fd, g_strerror(e));
-
-		/* remove the handler in case of EBADF because it would
-		 * cause looping otherwise */
-		if (EBADF != e)
-			return;
-	} else {
-		g_message("socket_linger_cb: close() succeeded");
-	}
-
-	inputevt_remove(ctx->tag);
-	wfree(ctx, sizeof *ctx);
-}
-
-static void
-socket_linger_close(gint fd)
-{
-	struct socket_linger *ctx;
-
-	g_assert(fd >= 0);
-
-	ctx = walloc(sizeof *ctx);
-	ctx->tag = inputevt_add(fd, INPUT_EVENT_RX, socket_linger_cb, ctx);
-	g_assert(0 != ctx->tag);
-}
-
 /**
  * Dispose of socket, closing connection, removing input callback, and
  * reclaiming attached getline buffer.
@@ -1370,32 +1327,22 @@ socket_free(struct gnutella_socket *s)
 		s->getline = NULL;
 	}
 
-#ifdef HAS_GNUTLS
-	if (s->tls.ctx) {
-		if (s->file_desc != -1 && SOCK_TLS_ESTABLISHED == s->tls.stage) {
-			gboolean is_incoming = SOCK_CONN_INCOMING == s->direction;
-
-			if (!is_incoming) {
+	if (socket_with_tls(s)) {
+		if (s->file_desc != -1 && socket_uses_tls(s)) {
+			if (SOCK_CONN_INCOMING != s->direction) {
 				tls_cache_insert(s->addr, s->port);
 			}
-			tls_bye(s->tls.ctx, is_incoming);
+			tls_bye(s);
 		}
-		tls_free(&s->tls.ctx);
+		tls_free(s);
 	}
-#endif	/* HAS_GNUTLS */
 
 	if (s->file_desc != -1) {
-		if (s->corked)
-			sock_cork(s, FALSE);
-		sock_tx_shutdown(s);
+		socket_cork(s, FALSE);
+		socket_tx_shutdown(s);
 		if (close(s->file_desc)) {
-			gint e = errno;
-
-			if (!is_temporary_error(e))
-				g_warning("close(%d) failed: %s", s->file_desc, g_strerror(e));
-
-			if (EBADF != e) /* just in case, as it would cause looping */
-				socket_linger_close(s->file_desc);
+			g_warning("socket_free: close(%d) failed: %s",
+				s->file_desc, g_strerror(errno));
 		}
 		s->file_desc = -1;
 	}
@@ -1422,18 +1369,15 @@ socket_free_null(struct gnutella_socket **s_ptr)
  */
 static gboolean
 socket_tls_setup(struct gnutella_socket *s)
-#ifdef HAS_GNUTLS
 {
-	if (!s->tls.enabled) {
+	if (!s->tls.enabled)
 		return 0;
-	}
 
 	if (s->tls.stage < SOCK_TLS_INITIALIZED) {
-		s->tls.ctx = tls_init(s);
-		if (!s->tls.ctx) {
+		if (tls_init(s))
 			goto destroy;
-		}
 		s->tls.stage = SOCK_TLS_INITIALIZED;
+		socket_nodelay(s, TRUE);
 	}
 
 	if (s->tls.stage < SOCK_TLS_ESTABLISHED) {
@@ -1460,12 +1404,6 @@ destroy:
 	errno = EIO;
 	return -1;
 }
-#else	/* HAVE_GNUTLS */
-{
-	(void) s;
-	return 0;
-}
-#endif	/* HAVE_GNUTLS */
 
 /**
  * Used for incoming connections, for outgoing too??
@@ -1482,7 +1420,6 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 	ssize_t r;
 	size_t parsed;
 	const gchar *first, *endptr;
-	time_t banlimit;
 
 	(void) source;
 
@@ -1493,8 +1430,10 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 
 	g_assert(0 == s->pos);		/* We read a line, then leave this callback */
 
-#ifdef HAS_GNUTLS
-	if (s->direction == SOCK_CONN_INCOMING) {
+	if (
+		s->direction == SOCK_CONN_INCOMING &&
+		s->tls.enabled
+	) {
 		if (s->tls.enabled && s->tls.stage < SOCK_TLS_INITIALIZED) {
 			ssize_t ret;
 			guchar c;
@@ -1530,7 +1469,6 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 			return;
 		}
 	}
-#endif /* HAS_GNUTLS */
 
 	socket_alloc_buffer(s);
 
@@ -1570,6 +1508,10 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 		s->pos += r;
 	}
 
+	if (!s->getline) {
+		s->getline = getline_make(MAX_LINE_SIZE);
+	}
+
 	/*
 	 * Get first line.
 	 */
@@ -1624,7 +1566,7 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 		return;
 	}
 
-	if (is_strprefix(first, "QUEUE ")) {
+	if (parq_is_enabled() && is_strprefix(first, "QUEUE ")) {
 		parq_download_queue_ack(s);
 		return;
 	}
@@ -1678,18 +1620,23 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 		g_assert_not_reached();
 	}
 
-	/*
-	 * Check for PARQ banning.
-	 * 		-- JA, 29/07/2003
-	 */
+	if (parq_is_enabled()) {
+		time_t banlimit;
+		/*
+		 * Check for PARQ banning.
+		 * 		-- JA, 29/07/2003
+		 */
 
-	banlimit = parq_banned_source_expire(s->addr);
-	if (banlimit) {
-		if (GNET_PROPERTY(socket_debug))
-			g_warning("[sockets] PARQ has banned host %s until %s",
-				host_addr_to_string(s->addr), timestamp_to_string(banlimit));
-		ban_force(s);
-		goto cleanup;
+		banlimit = parq_banned_source_expire(s->addr);
+		if (banlimit) {
+			if (GNET_PROPERTY(socket_debug)) {
+				g_warning("[sockets] PARQ has banned host %s until %s",
+					host_addr_to_string(s->addr),
+					timestamp_to_string(banlimit));
+			}
+			ban_force(s);
+			goto cleanup;
+		}
 	}
 
 	/*
@@ -1742,6 +1689,7 @@ socket_read(gpointer data, gint source, inputevt_cond_t cond)
 		NULL != (endptr = is_strprefix(first, "HELO")) &&
 		(is_ascii_space(endptr[0]) || '\0' == endptr[0])
 	) {
+		getline_set_maxlen(s->getline, SHELL_MAX_LINE_SIZE);
         shell_add(s);
 	} else
 		goto unknown;
@@ -1799,7 +1747,10 @@ socket_connected(gpointer data, gint source, inputevt_cond_t cond)
 
 	if (0 != socket_tls_setup(s)) {
 		if (!is_temporary_error(errno)) {
-			socket_destroy(s, "TLS handshake failed");
+			if (s->type == SOCK_TYPE_DOWNLOAD && s->resource.download)
+				download_fallback_to_push(s->resource.download, FALSE, FALSE);
+			else
+				socket_destroy(s, "TLS handshake failed");
 		}
 		return;
 	}
@@ -2011,6 +1962,37 @@ socket_addr_getsockname(socket_addr_t *p_addr, int fd)
 	return 0;
 }
 
+static int
+socket_addr_getpeername(socket_addr_t *p_addr, int fd)
+{
+	socklen_t addr_len;
+
+	g_return_val_if_fail(p_addr, -1);
+	g_return_val_if_fail(fd >= 0, -1);
+
+	errno = 0;
+	addr_len = socket_addr_init(p_addr, NET_TYPE_IPV4);
+	if (
+		-1 != getpeername(fd, socket_addr_get_sockaddr(p_addr), &addr_len) &&
+		AF_INET == socket_addr_get_family(p_addr)
+	) {
+		return 0;
+	}
+
+#ifdef HAS_IPV6
+	errno = 0;
+	addr_len = socket_addr_init(p_addr, NET_TYPE_IPV6);
+	if (
+		-1 != getpeername(fd, socket_addr_get_sockaddr(p_addr), &addr_len) &&
+		AF_INET6 == socket_addr_get_family(p_addr)
+	) {
+		return 0;
+	}
+#endif	/* HAS_IPV6 */
+
+	return -1;
+}
+
 /**
  * Tries to guess the local IP address.
  */
@@ -2062,7 +2044,7 @@ static void
 socket_accept(gpointer data, gint unused_source, inputevt_cond_t cond)
 {
 	socket_addr_t addr;
-	socklen_t len = sizeof addr;
+	socklen_t addr_len;
 	struct gnutella_socket *s = data;
 	struct gnutella_socket *t = NULL;
 	gint fd;
@@ -2086,7 +2068,8 @@ socket_accept(gpointer data, gint unused_source, inputevt_cond_t cond)
 		return;
 	}
 
-	fd = accept(s->file_desc, (struct sockaddr *) &addr, &len);
+	addr_len = socket_addr_init(&addr, s->net);
+	fd = accept(s->file_desc, socket_addr_get_sockaddr(&addr), &addr_len);
 	if (fd < 0) {
 		/*
 		 * If we ran out of file descriptors, try to reclaim one from the
@@ -2097,7 +2080,9 @@ socket_accept(gpointer data, gint unused_source, inputevt_cond_t cond)
 			(errno == EMFILE || errno == ENFILE) &&
 			reclaim_fd != NULL && (*reclaim_fd)()
 		) {
-			fd = accept(s->file_desc, (struct sockaddr *) &addr, &len);
+			addr_len = socket_addr_init(&addr, s->net);
+			fd = accept(s->file_desc, socket_addr_get_sockaddr(&addr),
+					&addr_len);
 		}
 
 		if (fd < 0) {
@@ -2132,7 +2117,6 @@ socket_accept(gpointer data, gint unused_source, inputevt_cond_t cond)
 	t->file_desc = fd;
 	t->direction = SOCK_CONN_INCOMING;
 	t->type = s->type;
-	t->getline = getline_make(MAX_LINE_SIZE);
 
 	if (SOCK_F_TCP & s->flags) {
 		t->addr = socket_addr_get_addr(&addr);
@@ -2146,16 +2130,33 @@ socket_accept(gpointer data, gint unused_source, inputevt_cond_t cond)
 	}
 	t->net = host_addr_net(t->addr);
 
-#ifdef HAS_GNUTLS
+	if ((SOCK_F_TCP & t->flags) && !is_host_addr(t->addr)) {
+		if (socket_addr_getpeername(&addr, t->file_desc)) {
+			g_warning("getpeername() failed: %s", g_strerror(errno));
+			socket_free_null(&t);
+			return;
+		}
+		t->addr = socket_addr_get_addr(&addr);
+		t->port = socket_addr_get_port(&addr);
+		if (!is_host_addr(t->addr)) {
+			g_warning("Incoming TCP connection from unidentifiable source");
+			socket_free_null(&t);
+			return;
+		}
+		g_warning("Had to use getpeername() after accept(): peer=%s",
+			host_addr_port_to_string(t->addr, t->port));
+	}
+
 	t->tls.enabled = s->tls.enabled; /* Inherit from listening socket */
 	t->tls.stage = SOCK_TLS_NONE;
 	t->tls.ctx = NULL;
 	t->tls.snarf = 0;
 
-	if (GNET_PROPERTY(tls_debug) > 2)
-		g_message("Incoming connection");
-#endif	/* HAS_GNUTLS */
-        
+	if (GNET_PROPERTY(tls_debug) > 2) {
+		g_message("Incoming connection from %s",
+			host_addr_port_to_string(t->addr, t->port));
+	}
+
 	socket_wio_link(t);
 
 	t->flags |= SOCK_F_ESTABLISHED;
@@ -2196,9 +2197,12 @@ socket_accept(gpointer data, gint unused_source, inputevt_cond_t cond)
 
 #if defined(CMSG_FIRSTHDR) && defined(CMSG_NXTHDR)
 static inline const struct cmsghdr *
-cmsg_nxthdr(const struct msghdr *msg, const struct cmsghdr *cmsg)
+cmsg_nxthdr(const struct msghdr *msg_, const struct cmsghdr *cmsg_)
 {
-	return CMSG_NXTHDR((struct msghdr *) msg, (struct cmsghdr *) cmsg);
+	struct msghdr *msg = (struct msghdr *) msg_;
+	struct cmsghdr *cmsg = (struct cmsghdr *) cmsg_;
+    
+	return CMSG_NXTHDR(msg, cmsg);
 }
 #endif	/* CMSG_FIRSTHDR && CMSG_NXTHDR */
 
@@ -2439,7 +2443,7 @@ socket_udp_event(gpointer data, gint unused_source, inputevt_cond_t cond)
 }
 
 static inline void
-socket_set_linger(gint fd)
+socket_set_linger(int fd)
 {
 	g_assert(fd >= 0);
 
@@ -2448,7 +2452,7 @@ socket_set_linger(gint fd)
 
 #ifdef TCP_LINGER2
 	{
-		gint timeout = 20;	/* timeout in seconds for FIN_WAIT_2 */
+		int timeout = 20;	/* timeout in seconds for FIN_WAIT_2 */
 
 		if (setsockopt(fd, sol_tcp(), TCP_LINGER2, &timeout, sizeof timeout))
 			g_warning("setsockopt() for TCP_LINGER2 failed: %s",
@@ -2475,8 +2479,11 @@ socket_set_accept_filters(struct gnutella_socket *s)
 	socket_check(s);
 	g_assert(s->file_desc >= 0);
 
+	if (GNET_PROPERTY(tcp_defer_accept_timeout) <= 0)
+		return;
+
 #if defined(TCP_DEFER_ACCEPT)
-	if (GNET_PROPERTY(tcp_defer_accept_timeout) > 0) {
+	{
 		gint timeout;
 
 		timeout = MIN(GNET_PROPERTY(tcp_defer_accept_timeout), (guint) INT_MAX);
@@ -2513,6 +2520,48 @@ socket_set_accept_filters(struct gnutella_socket *s)
 #endif /* SO_ACCEPTFILTER */
 }
 
+static void
+socket_set_fastack(struct gnutella_socket *s)
+{
+	static const int on = 1;
+
+	socket_check(s);
+	g_return_if_fail(s->file_desc >= 0);
+	
+	if (!(SOCK_F_TCP & s->flags))
+		return;
+
+	(void) on;
+#if defined(TCP_FASTACK)
+	if (setsockopt(s->file_desc, sol_tcp(), TCP_FASTACK, &on, sizeof on)) {
+		g_message("Could not set TCP_FASTACK (fd=%d): %s",
+			s->file_desc, g_strerror(errno));
+	}
+#endif /* TCP_FASTACK */
+}
+
+/**
+ * Enable quick ACK sending at the TCP level, if supported on this platform.
+ * This can really increase the reception of data as data packets are
+ * immediately acknowledged to the sender.
+ */
+void
+socket_set_quickack(struct gnutella_socket *s, int on)
+{
+	socket_check(s);
+	g_return_if_fail(s->file_desc >= 0);
+
+	if (!(SOCK_F_TCP & s->flags))
+		return;
+
+	(void) on;
+#if defined(TCP_QUICKACK)
+	if (setsockopt(s->file_desc, sol_tcp(), TCP_QUICKACK, &on, sizeof on)) {
+		g_message("Could not set TCP_QUICKACK (fd=%d): %s",
+			s->file_desc, g_strerror(errno));
+	}
+#endif	/* TCP_QUICKACK*/
+}
 
 /*
  * Sockets creation
@@ -2532,6 +2581,9 @@ socket_connect_prepare(struct gnutella_socket *s,
 
 	socket_check(s);
 
+	/* Filter out flags which we cannot accept */
+	flags &= (SOCK_F_TLS | SOCK_F_FORCE);
+
 	if (!(s->flags & SOCK_F_FORCE) && hostiles_check(addr)) {
 		g_warning("Not connecting to hostile host %s",
 			host_addr_to_string(addr));
@@ -2583,23 +2635,36 @@ socket_connect_prepare(struct gnutella_socket *s,
 	s->port = port;
 	s->flags |= SOCK_F_TCP | flags;
 
-#ifdef HAS_GNUTLS
-	s->tls.enabled = GNET_PROPERTY(tls_enforce) || (SOCK_F_TLS & flags);
+	s->tls.enabled = tls_enabled() &&
+		(GNET_PROPERTY(tls_enforce) || (SOCK_F_TLS & flags));
 	s->tls.stage = SOCK_TLS_NONE;
 	s->tls.ctx = NULL;
 	s->tls.snarf = 0;
-#endif	/* HAS_GNUTLS */
-        
+
 	socket_wio_link(s);
 
 	setsockopt(s->file_desc, SOL_SOCKET, SO_KEEPALIVE, &enable, sizeof enable);
 	setsockopt(s->file_desc, SOL_SOCKET, SO_REUSEADDR, &enable, sizeof enable);
 
 	socket_set_nonblocking(s->file_desc);
-	set_close_on_exec(fd);
+	set_close_on_exec(s->file_desc);
 	socket_set_linger(s->file_desc);
 	socket_tos_normal(s);
 
+	/*
+	 * Fast ACKs is mainly useful for downloads.
+	 */
+
+	switch (s->type) {
+	case SOCK_TYPE_DOWNLOAD:
+	case SOCK_TYPE_HTTP:
+		socket_set_fastack(s);
+		socket_set_quickack(s, TRUE);
+	default:
+		socket_set_quickack(s, FALSE);
+		break;
+	}
+
 	return 0;
 }
 
@@ -2624,8 +2689,7 @@ socket_connect_finalize(struct gnutella_socket *s, const host_addr_t ha)
 
 	if (!(s->flags & SOCK_F_FORCE) && hostiles_check(ha)) {
 		g_warning("Not connecting to hostile host %s", host_addr_to_string(ha));
-		socket_destroy(s, "Not connecting to hostile host");
-		return -1;
+		goto failure;	/* Not connecting to hostile host */
 	}
 
 	s->addr = ha;
@@ -2686,18 +2750,12 @@ socket_connect_finalize(struct gnutella_socket *s, const host_addr_t ha)
 				g_warning("Proxy isn't properly configured (%s:%u)",
 					GNET_PROPERTY(proxy_hostname), GNET_PROPERTY(proxy_port));
 			}
-			socket_destroy(s, "Check the proxy configuration");
-			return -1;
+			goto failure;	/* Check the proxy configuration */
 		}
 
 		g_warning("Unable to connect to %s: (%s)",
 			host_addr_port_to_string(s->addr, s->port), g_strerror(errno));
-
-		if (s->adns & SOCK_ADNS_PENDING)
-			s->adns_msg = "Connection failed";
-		else
-			socket_destroy(s, _("Connection failed"));
-		return -1;
+		goto failure;
 	}
 
 	s->local_port = socket_local_port(s);
@@ -2710,6 +2768,13 @@ socket_connect_finalize(struct gnutella_socket *s, const host_addr_t ha)
 
 	socket_evt_set(s, INPUT_EVENT_WX, socket_connected, s);
 	return 0;
+
+failure:
+	
+	if (!(s->adns & SOCK_ADNS_PENDING)) {
+		socket_destroy(s, _("Connection failed"));
+	}
+	return -1;
 }
 
 /**
@@ -2761,16 +2826,14 @@ socket_connect_by_name_helper(const host_addr_t *addrs, size_t n,
 	socket_check(s);
 	g_assert(addrs);
 
-	s->adns &= ~SOCK_ADNS_PENDING;
-
 	if (n < 1 || s->type == SOCK_TYPE_DESTROYING) {
 		s->adns |= SOCK_ADNS_FAILED | SOCK_ADNS_BADNAME;
 		s->adns_msg = "Could not resolve address";
-		return;
+		goto finish;
 	}
 
 	addr = addrs[random_raw() % n];
-	can_tls = tls_cache_lookup(addr, s->port);
+	can_tls = 0 != (SOCK_F_TLS & s->flags) || tls_cache_lookup(addr, s->port);
 
 	if (
 		s->net != host_addr_net(addr) ||
@@ -2787,13 +2850,22 @@ socket_connect_by_name_helper(const host_addr_t *addrs, size_t n,
 		}
 		if (socket_connect_prepare(s, addr, s->port, s->type, s->flags)) {
 			s->adns |= SOCK_ADNS_FAILED;
-			return;
+			s->adns_msg = "Could not resolve address";
+			goto finish;
 		}
 	}
 
+	/* SOCK_ADNS_PENDING is still set to prevent socket_destroy() on error */
 	if (socket_connect_finalize(s, addr)) {
 		s->adns |= SOCK_ADNS_FAILED;
-		return;
+		s->adns_msg = "Connection failed";
+		goto finish;
+	}
+
+finish:
+	s->adns &= ~SOCK_ADNS_PENDING;
+	if ((s->adns & SOCK_ADNS_ASYNC) && (s->adns & SOCK_ADNS_FAILED)) {
+		socket_destroy(s, s->adns_msg);
 	}
 }
 
@@ -2821,12 +2893,12 @@ socket_connect_by_name(const gchar *host, guint16 port,
 		return NULL;
 	}
 
-	s->adns |= SOCK_ADNS_PENDING;
+	s->adns = SOCK_ADNS_PENDING;
 	if (
-		!adns_resolve(host, settings_dns_net(),
-			&socket_connect_by_name_helper, s)
-		&& (s->adns & SOCK_ADNS_FAILED)
+		adns_resolve(host, settings_dns_net(), socket_connect_by_name_helper, s)
 	) {
+		s->adns |= SOCK_ADNS_ASYNC;
+	} else if (s->adns & SOCK_ADNS_FAILED) {
 		/*	socket_connect_by_name_helper() was already invoked! */
 		if (GNET_PROPERTY(socket_debug) > 0)
 			g_warning("socket_connect_by_name: "
@@ -2849,7 +2921,7 @@ socket_connect_by_name(const gchar *host, guint16 port,
  *
  * @return The new file descriptor of socket or -1 on failure.
  */
-gint
+static gint
 socket_create_and_bind(const host_addr_t bind_addr,
 	const guint16 port, const int type)
 {
@@ -3056,9 +3128,7 @@ socket_local_listen(const gchar *pathname)
 		return NULL;
 	}
 
-#ifdef HAS_GNUTLS
-	s->tls.enabled = TRUE;
-#endif /* HAS_GNUTLS */
+	s->tls.enabled = tls_enabled();
 
 	socket_enable_accept(s);
 	return s;
@@ -3122,9 +3192,7 @@ socket_tcp_listen(host_addr_t bind_addr, guint16 port)
 		s->local_port = socket_addr_get_port(&addr);
 	}
 
-#ifdef HAS_GNUTLS
-	s->tls.enabled = TRUE;
-#endif /* HAS_GNUTLS */
+	s->tls.enabled = tls_enabled();
 
 	socket_enable_accept(s);
 	return s;
@@ -3133,24 +3201,32 @@ socket_tcp_listen(host_addr_t bind_addr, guint16 port)
 static void
 socket_enable_recvdstaddr(const struct gnutella_socket *s)
 {
-	static const gint enable = 1;
-	gint level = -1, opt = -1;
+	static const int on = 1;
+	int fd;
 
 	socket_check(s);
-	g_assert(s->file_desc >= 0);
+	fd = s->file_desc;
+	g_assert(fd >= 0);
 
+	(void) on;
 	switch (s->net) {
 	case NET_TYPE_IPV4:
 #if defined(IP_RECVDSTADDR) && IP_RECVDSTADDR
-		level = sol_ip();
-		opt = IP_RECVDSTADDR;
+		if (setsockopt(fd, sol_ip(), IP_RECVDSTADDR, &on, sizeof on)) {
+			g_warning("socket_enable_recvdstaddr(): "
+				"setsockopt() for IP_RECVDSTADDR failed: %s",
+				g_strerror(errno));
+		}
 #endif /* IP_RECVDSTADDR && IP_RECVDSTADDR */
 		break;
 
 	case NET_TYPE_IPV6:
 #if defined(HAS_IPV6) && defined(IPV6_RECVPKTINFO)
-		level = sol_ipv6();
-		opt = IPV6_RECVPKTINFO;
+		if (setsockopt(fd, sol_ipv6(), IPV6_RECVPKTINFO, &on, sizeof on)) {
+			g_warning("socket_enable_recvdstaddr(): "
+				"setsockopt() for IPV6_RECVPKTINFO failed: %s",
+				g_strerror(errno));
+		}
 #endif /* HAS_IPV6 && IPV6_RECVPKTINFO */
 		break;
 
@@ -3158,15 +3234,6 @@ socket_enable_recvdstaddr(const struct gnutella_socket *s)
 	case NET_TYPE_NONE:
 		break;
 	}
-
-	if (
-		-1 != level &&
-		-1 != opt &&
-		0 != setsockopt(s->file_desc, level, opt, &enable, sizeof enable)
-	) {
-		g_warning("socket_enable_recvdstaddr(): setsockopt() failed: %s",
-			g_strerror(errno));
-	}
 }
 
 /**
@@ -3229,7 +3296,7 @@ socket_udp_listen(host_addr_t bind_addr, guint16 port)
 	 * datagrams if we are not able to read them during some time.
 	 */
 
-	sock_recv_buf(s, SOCK_UDP_RECV_BUF, FALSE);
+	socket_recv_buf(s, SOCK_UDP_RECV_BUF, FALSE);
 
 	return s;
 }
@@ -3238,14 +3305,14 @@ void
 socket_disable_token(struct gnutella_socket *s)
 {
 	socket_check(s);
-	s->omit_token = TRUE;
+	s->flags |= SOCK_F_OMIT_TOKEN;
 }
 
 gboolean
 socket_omit_token(struct gnutella_socket *s)
 {
 	socket_check(s);
-	return s->omit_token;
+	return 0 != (s->flags & SOCK_F_OMIT_TOKEN);
 }
 
 /**
@@ -3256,29 +3323,33 @@ socket_omit_token(struct gnutella_socket *s)
  * it's about 1500 bytes.
  */
 void
-sock_cork(struct gnutella_socket *s, gboolean on)
+socket_cork(struct gnutella_socket *s, gboolean on)
 #if defined(TCP_CORK) || defined(TCP_NOPUSH)
 {
-	static const gint option =
+	static const int option =
 #if defined(TCP_CORK)
 		TCP_CORK;
 #else	/* !TCP_CORK*/
 		TCP_NOPUSH;
 #endif /* TCP_CORK */
-	gint arg = on ? 1 : 0;
+	int arg = on ? 1 : 0;
 
 	socket_check(s);
-	if (!(SOCK_F_TCP & s->flags)) {
-		return;
-	}
 
-	if (-1 == setsockopt(s->file_desc, sol_tcp(), option, &arg, sizeof arg)) {
+	if (!(SOCK_F_TCP & s->flags))
+		return;
+
+	if (!(s->flags & SOCK_F_CORKED) == !on)
+		return;
+
+	if (setsockopt(s->file_desc, sol_tcp(), option, &arg, sizeof arg)) {
 		if (ECONNRESET != errno) {
 			g_warning("unable to %s TCP_CORK on fd#%d: %s",
 				on ? "set" : "clear", s->file_desc, g_strerror(errno));
 		}
 	} else {
-		s->corked = on;
+		s->flags &= ~SOCK_F_CORKED;
+		s->flags |= on ? SOCK_F_CORKED : 0;
 	}
 }
 #else
@@ -3296,26 +3367,31 @@ sock_cork(struct gnutella_socket *s, gboolean on)
 #endif /* TCP_CORK || TCP_NOPUSH */
 
 /*
- * Internal routine for sock_send_buf() and sock_recv_buf().
+ * Internal routine for socket_send_buf() and socket_recv_buf().
  * Set send/receive buffer to specified size, and warn if it cannot be done.
  * If `shrink' is false, refuse to shrink the buffer if its size is larger.
+ * If `size' is zero, the request is ignored. This is useful to stick to the
+ * system's default buffer sizes.
  */
 static void
-sock_set_intern(gint fd, gint option, gint size,
-	const gchar *type, gboolean shrink)
+socket_set_intern(int fd, int option, unsigned size,
+	const char *type, gboolean shrink)
 {
-	gint old_len = 0;
-	gint new_len = 0;
+	unsigned old_len = 0;
+	unsigned new_len = 0;
 	socklen_t len;
 
-	size = (size + 1) & ~0x1;	/* Must be even, round to upper boundary */
+	if (0 == size)
+		return;
+
+	size = (size + 1) & ~0x1U;	/* Must be even, round to upper boundary */
 
 	len = sizeof(old_len);
 	if (-1 == getsockopt(fd, SOL_SOCKET, option, &old_len, &len))
 		g_warning("cannot read old %s buffer length on fd #%d: %s",
 			type, fd, g_strerror(errno));
 
-/* XXX needs to add metaconfig test */
+/* FIXME: needs to add metaconfig test */
 #ifdef LINUX_SYSTEM
 	old_len >>= 1;		/* Linux returns twice the real amount */
 #endif
@@ -3323,13 +3399,13 @@ sock_set_intern(gint fd, gint option, gint size,
 	if (!shrink && old_len >= size) {
 		if (GNET_PROPERTY(socket_debug) > 5)
 			g_message(
-				"socket %s buffer on fd #%d NOT shrank to %d bytes (is %d)",
+				"socket %s buffer on fd #%d NOT shrank to %u bytes (is %u)",
 				type, fd, size, old_len);
 		return;
 	}
 
 	if (-1 == setsockopt(fd, SOL_SOCKET, option, &size, sizeof(size)))
-		g_warning("cannot set new %s buffer length to %d on fd #%d: %s",
+		g_warning("cannot set new %s buffer length to %u on fd #%d: %s",
 			type, size, fd, g_strerror(errno));
 
 	len = sizeof(new_len);
@@ -3342,7 +3418,7 @@ sock_set_intern(gint fd, gint option, gint size,
 #endif
 
 	if (GNET_PROPERTY(socket_debug) > 5)
-		g_message("socket %s buffer on fd #%d: %d -> %d bytes (now %d) %s",
+		g_message("socket %s buffer on fd #%d: %u -> %u bytes (now %u) %s",
 			type, fd, old_len, size, new_len,
 			(new_len == size) ? "OK" : "FAILED");
 }
@@ -3352,11 +3428,11 @@ sock_set_intern(gint fd, gint option, gint size,
  * If `shrink' is false, refuse to shrink the buffer if its size is larger.
  */
 void
-sock_send_buf(struct gnutella_socket *s, gint size, gboolean shrink)
+socket_send_buf(struct gnutella_socket *s, gint size, gboolean shrink)
 {
 	socket_check(s);
-	g_return_if_fail(!s->was_shutdown);
-	sock_set_intern(s->file_desc, SO_SNDBUF, size, "send", shrink);
+	g_return_if_fail(!(s->flags & SOCK_F_SHUTDOWN));
+	socket_set_intern(s->file_desc, SO_SNDBUF, size, "send", shrink);
 }
 
 /**
@@ -3364,31 +3440,40 @@ sock_send_buf(struct gnutella_socket *s, gint size, gboolean shrink)
  * If `shrink' is false, refuse to shrink the buffer if its size is larger.
  */
 void
-sock_recv_buf(struct gnutella_socket *s, gint size, gboolean shrink)
+socket_recv_buf(struct gnutella_socket *s, gint size, gboolean shrink)
 {
 	socket_check(s);
-	g_return_if_fail(!s->was_shutdown);
-	sock_set_intern(s->file_desc, SO_RCVBUF, size, "receive", shrink);
+	g_return_if_fail(!(s->flags & SOCK_F_SHUTDOWN));
+	socket_set_intern(s->file_desc, SO_RCVBUF, size, "receive", shrink);
 }
 
 /**
  * Turn TCP_NODELAY on or off on the socket.
  */
 void
-sock_nodelay(struct gnutella_socket *s, gboolean on)
+socket_nodelay(struct gnutella_socket *s, gboolean on)
 {
-	gint arg = on ? 1 : 0;
+	int arg = on ? 1 : 0;
 
 	socket_check(s);
-	if (!(SOCK_F_TCP & s->flags)) {
+
+	if (!(SOCK_F_TCP & s->flags))
 		return;
-	}
-	if (
-		-1 == setsockopt(s->file_desc, sol_tcp(), TCP_NODELAY, &arg, sizeof arg)
-	) {
+
+	/*
+	 * Some systems don't like enabling TCP_NODELAY if it's already enabled and
+	 * checking may also save a system call.
+	 */
+	if (!(SOCK_F_NODELAY & s->flags) == !on)
+		return;
+
+	if (setsockopt(s->file_desc, sol_tcp(), TCP_NODELAY, &arg, sizeof arg)) {
 		if (errno != ECONNRESET)
 			g_warning("unable to %s TCP_NODELAY on fd#%d: %s",
 				on ? "set" : "clear", s->file_desc, g_strerror(errno));
+	} else {
+		s->flags &= ~SOCK_F_NODELAY;
+		s->flags |= on ? SOCK_F_NODELAY : 0;
 	}
 }
 
@@ -3396,12 +3481,12 @@ sock_nodelay(struct gnutella_socket *s, gboolean on)
  * Shutdown the TX side of the socket.
  */
 void
-sock_tx_shutdown(struct gnutella_socket *s)
+socket_tx_shutdown(struct gnutella_socket *s)
 {
 	socket_check(s);
 	g_assert(s->file_desc >= 0);
 
-	if (s->was_shutdown)
+	if (s->flags & SOCK_F_SHUTDOWN)
 		return;
 
 	/* EINVAL and ENOTCONN may occur if connect() didn't succeed */
@@ -3413,7 +3498,7 @@ sock_tx_shutdown(struct gnutella_socket *s)
 		g_warning("unable to shutdown TX on fd#%d: %s",
 			s->file_desc, g_strerror(errno));
 	}
-	s->was_shutdown = TRUE;
+	s->flags |= SOCK_F_SHUTDOWN;
 }
 
 static int
@@ -3585,7 +3670,7 @@ socket_wio_link(struct gnutella_socket *s)
 		s->wio.readv = socket_no_readv;
 		s->wio.sendto = socket_no_sendto;
 	} else if (socket_uses_tls(s)) {
-		tls_wio_link(&s->wio);
+		tls_wio_link(s);
 	} else {
 		g_assert(s->flags & (SOCK_F_TCP | SOCK_F_LOCAL));
 		s->wio.write = socket_plain_write;
