@@ -1,5 +1,5 @@
 /*
- * $Id: sq.c 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: sq.c 13777 2007-05-29 00:23:11Z cbiere $
  *
  * Copyright (c) 2002-2003, Alex Bennee <alex@bennee.com> & Raphael Manfredi
  *
@@ -46,7 +46,7 @@
 
 #include "common.h"
 
-RCSID("$Id: sq.c 12147 2006-10-27 19:55:09Z cbiere $")
+RCSID("$Id: sq.c 13777 2007-05-29 00:23:11Z cbiere $")
 
 #include "sq.h"					/* search_queue structures */
 #include "pmsg.h"
@@ -81,9 +81,9 @@ typedef struct smsg {
  * react when the message queue informs us it has processed it.
  */
 struct smsg_info {
-	gpointer search;			/**< The search object which sends the query */
-	guint32 id;					/**< The unique search ID */
-	guint32 node_id;			/**< The unique node ID to which we're sending */
+	node_id_t node_id; 		/**< The unique node ID to which we're sending */
+	gpointer search;		/**< The search object which sends the query */
+	guint32 id;				/**< The unique search ID */
 };
 
 static squeue_t *global_sq = NULL;
@@ -96,7 +96,7 @@ static void cap_queue(squeue_t *sq);
 static void
 sq_pmsg_free(pmsg_t *mb, gpointer arg)
 {
-	struct smsg_info *smi = (struct smsg_info *) arg;
+	struct smsg_info *smi = arg;
 
 	g_assert(pmsg_is_extended(mb));
 
@@ -105,9 +105,10 @@ sq_pmsg_free(pmsg_t *mb, gpointer arg)
 	 * query for it to the specified node ID.
 	 */
 
-	if (current_peermode == NODE_P_LEAF)
+	if (GNET_PROPERTY(current_peermode) == NODE_P_LEAF)
 		search_notify_sent(smi->search, smi->id, smi->node_id);
 
+	node_id_unref(smi->node_id);
 	wfree(smi, sizeof(*smi));
 }
 
@@ -165,9 +166,9 @@ smsg_mutate(smsg_t *sb, struct gnutella_node *n)
 	struct smsg_info *smi;
 	pmsg_t *omb;
 
-	smi = (struct smsg_info *) walloc(sizeof(*smi));
+	smi = walloc(sizeof(*smi));
 	smi->id = search_get_id(sb->shandle, &smi->search);
-	smi->node_id = n->id;
+	smi->node_id = node_id_ref(NODE_ID(n));
 
 	omb = sb->mb;
 	sb->mb = pmsg_clone_extend(omb, sq_pmsg_free, smi);
@@ -267,13 +268,13 @@ sq_clear(squeue_t *sq)
 
 	g_assert(sq);
 
-	if (dbg > 3)
-		printf("clearing sq node %s (sent=%d, dropped=%d)\n",
+	if (GNET_PROPERTY(sq_debug) > 3)
+		g_message("clearing sq node %s (sent=%d, dropped=%d)",
 			sq->node ? node_addr(sq->node) : "GLOBAL",
 			sq->n_sent, sq->n_dropped);
 
 	for (l = sq->searches; l; l = g_list_next(l)) {
-		smsg_t *sb = (smsg_t *) l->data;
+		smsg_t *sb = l->data;
 
 		smsg_discard(sb);
 	}
@@ -317,7 +318,7 @@ sq_puthere(squeue_t *sq, gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 	sq->searches = g_list_prepend(sq->searches, sb);
 	sq->count++;
 
-	if (sq->count > search_queue_size)
+	if (sq->count > GNET_PROPERTY(search_queue_size))
 		cap_queue(sq);
 }
 
@@ -360,7 +361,7 @@ sq_global_putq(gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 void
 sq_process(squeue_t *sq, time_t now)
 {
-    time_delta_t spacing = search_queue_spacing;
+    time_delta_t spacing = GNET_PROPERTY(search_queue_spacing);
 	GList *item;
 	smsg_t *sb;
 	struct gnutella_node *n;
@@ -406,10 +407,10 @@ retry:
 		 * Processing the global SQ.
 		 */
 
-		if (current_peermode != NODE_P_ULTRA)
+		if (GNET_PROPERTY(current_peermode) != NODE_P_ULTRA)
 			return;
 
-		if (node_keep_missing() * 3 > 2 * up_connections)
+		if (node_keep_missing() * 3 > 2 * GNET_PROPERTY(up_connections))
 			return;							/* Not enough nodes for querying */
 	}
 
@@ -421,7 +422,7 @@ retry:
 	g_assert(sq->searches);
 
 	item = g_list_first(sq->searches);
-	sb = (smsg_t *) item->data;
+	sb = item->data;
 
 	g_assert(sq->count > 0);
 	sq->count--;
@@ -430,8 +431,8 @@ retry:
 	if (n == NULL) {
 		g_assert(sb->qhv != NULL);		/* Enqueued via sq_global_putq() */
 
-		if (dbg > 2)
-			printf("sq GLOBAL, queuing \"%s\" (%u left, %d sent)\n",
+		if (GNET_PROPERTY(sq_debug) > 2)
+			g_message("sq GLOBAL, queuing \"%s\" (%u left, %d sent)",
 				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
 
@@ -445,8 +446,8 @@ retry:
 
 		g_assert(sb->qhv == NULL);		/* Enqueued via sq_putq() */
 
-		if (dbg > 2)
-			printf("sq for node %s, queuing \"%s\" (%u left, %d sent)\n",
+		if (GNET_PROPERTY(sq_debug) > 2)
+			g_message("sq for node %s, queuing \"%s\" (%u left, %d sent)",
 				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
 
@@ -457,14 +458,14 @@ retry:
 		 * queries that go out.
 		 */
 
-		if (current_peermode == NODE_P_LEAF)
+		if (GNET_PROPERTY(current_peermode) == NODE_P_LEAF)
 			smsg_mutate(sb, n);
 
 		mq_putq(n->outq, sb->mb);
 
 	} else {
-		if (dbg > 4)
-			printf("sq for node %s, ignored \"%s\" (%u left, %d sent)\n",
+		if (GNET_PROPERTY(sq_debug) > 4)
+			g_message("sq for node %s, ignored \"%s\" (%u left, %d sent)",
 				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
 		pmsg_free(sb->mb);
@@ -499,9 +500,9 @@ retry:
 static void
 cap_queue(squeue_t *sq)
 {
-    while (sq->count > search_queue_size) {
+    while (sq->count > GNET_PROPERTY(search_queue_size)) {
     	GList *item = g_list_last(sq->searches);
-		smsg_t *sb = (smsg_t *) item->data;
+		smsg_t *sb = item->data;
 
 		sq->searches = g_list_remove_link(sq->searches, item);
 
@@ -509,8 +510,8 @@ cap_queue(squeue_t *sq)
 		sq->count--;
 		sq->n_dropped++;
 
-		if (dbg > 4)
-			printf("sq for node %s, dropped \"%s\" (%u left, %d dropped)\n",
+		if (GNET_PROPERTY(sq_debug) > 4)
+			g_message("sq for node %s, dropped \"%s\" (%u left, %d dropped)",
 				node_addr(sq->node),
 				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_dropped);
@@ -532,7 +533,7 @@ sq_search_closed(squeue_t *sq, gnet_search_t sh)
 	GList *next;
 
 	for (l = sq->searches; l; l = next) {
-		smsg_t *sb = (smsg_t *) l->data;
+		smsg_t *sb = l->data;
 
 		next = g_list_next(l);
 
@@ -543,8 +544,8 @@ sq_search_closed(squeue_t *sq, gnet_search_t sh)
 		sq->count--;
 		sq->searches = g_list_remove_link(sq->searches, l);
 
-		if (dbg > 4)
-			printf("sq for node %s, dropped \"%s\" on search close (%u left)\n",
+		if (GNET_PROPERTY(sq_debug) > 4)
+			g_message("sq for node %s, dropped \"%s\" on search close (%u left)",
 				sq->node ? node_addr(sq->node) : "GLOBAL",
 				gnutella_msg_search_get_text(pmsg_start(sb->mb)), sq->count);
 
