@@ -1,5 +1,5 @@
 /*
- * $Id: tls_common.h 12934 2007-02-24 22:13:10Z cbiere $
+ * $Id: tls_common.h 14928 2007-09-19 01:24:11Z cbiere $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -50,14 +50,21 @@ struct tls_context;
 
 typedef struct tls_context *tls_context_t;
 
-tls_context_t tls_init(const struct gnutella_socket *s);
-enum tls_handshake_result tls_handshake(struct gnutella_socket *s);
-void tls_bye(tls_context_t ctx, gboolean is_incoming);
-void tls_free(tls_context_t *ctx_ptr);
-void tls_wio_link(struct wrap_io *wio);
+int tls_init(struct gnutella_socket *);
+enum tls_handshake_result tls_handshake(struct gnutella_socket *);
+void tls_bye(struct gnutella_socket *);
+void tls_free(struct gnutella_socket *);
+void tls_wio_link(struct gnutella_socket *);
 
+gboolean tls_enabled(void);
 void tls_global_init(void);
 const gchar *tls_version_string(void);
 
+struct array;
+
+gboolean svn_release_notification_can_verify(void);
+gboolean svn_release_notification_verify(guint32 revision, time_t date,
+	const struct array *signature);
+
 #endif /* _core_tls_common_h_ */
 /* vi: set ts=4 sw=4 cindent: */
