@@ -1,5 +1,5 @@
 /*
- * $Id: token.h 11739 2006-08-26 12:56:50Z rmanfredi $
+ * $Id: token.h 12496 2006-11-26 09:22:49Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_token_h_
 #define _core_token_h_
 
-#include <glib.h>
+#include "common.h"
 #include "lib/host_addr.h"
 
 #define TOKEN_VERSION_SIZE	(4 + 3 + 20)	/**< stamp + seed + SHA1 */
