@@ -1,5 +1,5 @@
 /*
- * $Id: tx.h 11225 2006-06-30 09:25:25Z cbiere $
+ * $Id: tx.h 13344 2007-04-16 20:10:46Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_tx_h_
 #define _core_tx_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "if/core/hosts.h"
 
@@ -99,7 +99,7 @@ struct txdrv_ops {
  * Public interface
  */
 
-txdrv_t *tx_make(gpointer owner, gnet_host_t *host,
+txdrv_t *tx_make(gpointer owner, const gnet_host_t *host,
 	const struct txdrv_ops *ops, gpointer args);
 txdrv_t *tx_make_above(txdrv_t *ltx, const struct txdrv_ops *ops,
 	gpointer args);
