@@ -1,5 +1,5 @@
 /*
- * $Id: tx_dgram.h 8913 2005-08-01 17:37:35Z rmanfredi $
+ * $Id: tx_dgram.h 12583 2006-12-06 05:42:15Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,8 +36,11 @@
 #ifndef _core_tx_dgram_h_
 #define _core_tx_dgram_h_
 
+#include "common.h"
 #include "tx.h"
 
+#include "if/core/bsched.h"
+
 const struct txdrv_ops *tx_dgram_get_ops(void);
 
 /**
@@ -52,8 +55,8 @@ struct tx_dgram_cb {
  */
 struct tx_dgram_args {
 	struct tx_dgram_cb *cb;			/**< Callbacks */
-	struct bsched *bs;				/**< Bandwidth scheduler to use */
 	struct wrap_io *wio;			/**< I/O wrapping routines */
+	bsched_bws_t bws;				/**< Bandwidth scheduler to use */
 };
 
 #endif	/* _core_tx_dgram_h_ */
