@@ -1,5 +1,5 @@
 /*
- * $Id: uhc.c,v 1.43 2006/02/10 00:00:22 cbiere Exp $
+ * $Id: uhc.c 15122 2008-03-09 18:18:13Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,33 +35,35 @@
 
 #include "common.h"
 
-RCSID("$Id: uhc.c,v 1.43 2006/02/10 00:00:22 cbiere Exp $");
+RCSID("$Id: uhc.c 15122 2008-03-09 18:18:13Z cbiere $")
 
 #include "gnutella.h"
-#include "uhc.h"
-#include "udp.h"
-#include "nodes.h"
-#include "sockets.h"
-#include "pcache.h"
+#include "guid.h"
 #include "hcache.h"
 #include "hosts.h"
+#include "nodes.h"
+#include "pcache.h"
+#include "sockets.h"
+#include "udp.h"
+#include "uhc.h"
 
 #include "lib/adns.h"
 #include "lib/atoms.h"
 #include "lib/cq.h"
 #include "lib/endian.h"
-#include "lib/hashlist.h"
 #include "lib/glib-missing.h"
+#include "lib/hashlist.h"
 #include "lib/misc.h"
 
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
+#include "if/core/settings.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-#define UHC_MAX_ATTEMPTS	3	/**< Maximum connection / resolution attempts */
-#define UHC_TIMEOUT			20	/**< Host cache timeout, in seconds */
-#define UHC_RETRY_AFTER		3600 /**< Frequency of contacts for an UHC (secs) */
+#define UHC_MAX_ATTEMPTS 3		/**< Maximum connection / resolution attempts */
+#define UHC_TIMEOUT		 20000	/**< Host cache timeout, milliseconds */
+#define UHC_RETRY_AFTER	 3600	/**< Frequency of contacts for an UHC (secs) */
 
 /**
  * Request context, used when we decide to get hosts via the UDP host caches.
@@ -70,20 +72,20 @@ RCSID("$Id: uhc.c,v 1.43 2006/02/10 00:00:22 cbiere Exp $");
  * in the pings.
  */
 static struct uhc_context {
-	GHashTable *guids;			/**< GUIDs we sent */
+	const gchar *host;			/**< Last selected host (static buffer) */
+	cevent_t *timeout_ev;		/**< Ping timeout */
 	gint attempts;				/**< Connection / resolution attempts */
-	const gchar *host;			/**< Last selected host */
 	host_addr_t addr;			/**< Resolved IP address for host */
 	guint16 port;				/**< Port of selected host cache */
-	gpointer timeout_ev;		/**< Ping timeout */
+	gchar muid[GUID_RAW_SIZE];	/**< MUID of the ping */
 } uhc_ctx;
 
-static GList *uhc_avail = NULL;	/**< List of UHCs as string */
-static GList *uhc_used = NULL;	/**< List of used UHCs as ``struct used_uhc'' */
+static hash_list_t *uhc_list;	/**< List of ``struct uhc'' */
 
-struct used_uhc {
-	gchar 		*host;	/**< An UHC host as "<host>:<port>" */
+struct uhc {
+	const gchar	*host;	/**< An UHC host as "<host>:<port>" (string atom) */
 	time_t		stamp;	/**< Timestamp of the last request */
+	guint		used;	/**< How often have we tried to contact it */
 };
 
 /**
@@ -92,22 +94,20 @@ struct used_uhc {
 static const struct {
 	const gchar *uhc;
 } boot_hosts[] = {
-	{ "cache.kicks-ass.net:8000" },
-	{ "g6.6dns.org:1337" },
-	{ "kisama.ath.cx:8080" },
-	{ "krill.shacknet.nu:20095" },
-	{ "plankton.merseine.nu:20098" },
-	{ "pokerface.bishopston.net:3558" },
-	{ "secondary.udp-host-cache.com:9999" },
-	{ "tribaldance.bishopston.net:3558" },
-	{ "uhc.udp-host-cache.com:9999" },
-	{ "uhc2.limewire.com:20181" },
-	{ "zomfg.gtkg.org:40000" },
+#if defined(USE_LOCAL_UHC)
+	{ "localhost:6346" },
+#else	/* !USE_LOCAL_UHC */
+	{ "g6.dns6.org:1337" },
+	{ "void.ghostwhitecrab.de:9999" },
+	{ "yang.cloud.bishopston.net:33558" },
+	{ "yin.cloud.bishopston.net:33558" },
+#endif	/* USE_LOCAL_UHC */
 };
 
 static gboolean uhc_connecting = FALSE;
 
-static void uhc_host_resolved(const host_addr_t *addr, gpointer uu_udata);
+static void uhc_host_resolved(const host_addr_t *addr, size_t n,
+				gpointer uu_udata);
 
 /**
  * Parse hostname:port and return the hostname and port parts.
@@ -134,6 +134,7 @@ uhc_get_host_port(const gchar *hp, const gchar **host, guint16 *port)
 
 	*host = NULL;
 	*port = 0;
+	hostname[0] = '\0';
 
 	if (!string_to_host_or_addr(hp, &ep, NULL) || ':' != *ep)
 		return FALSE;
@@ -157,17 +158,109 @@ uhc_get_host_port(const gchar *hp, const gchar **host, guint16 *port)
 	return TRUE;
 }
 
-static void
-add_available_uhc(const gchar *hc)
+static struct uhc *
+uhc_new(const gchar *host)
 {
-	gchar *s;
+	static const struct uhc zero_uhc;
+	struct uhc *uhc;
 
-	g_assert(hc);
+	g_assert(host);
+	uhc = g_malloc(sizeof *uhc);
+	*uhc = zero_uhc;
+	uhc->host = atom_str_get(host);
+	uhc->stamp = 0;
+	uhc->used = 0;
+	return uhc;
+}
 
-	s = g_strdup(hc);
-	uhc_avail = random_value(100) < 50
-		? g_list_append(uhc_avail, s)
-		: g_list_prepend(uhc_avail, s);
+static void
+uhc_free(struct uhc **ptr)
+{
+	if (*ptr) {	
+		struct uhc *uu = *ptr;
+		atom_str_free_null(&uu->host);
+		G_FREE_NULL(uu);
+		*ptr = NULL;
+	}
+}
+
+static guint
+uhc_hash(gconstpointer key)
+{
+	const struct uhc *uhc = key;
+
+	return g_str_hash(uhc->host);
+}
+
+static gint
+uhc_equal(gconstpointer p, gconstpointer q)
+{
+	const struct uhc *a = p, *b = q;
+
+	return 0 == strcmp(a->host, b->host);
+}
+
+
+static void
+uhc_list_add(const gchar *host)
+{
+	struct uhc *uhc;
+
+	g_return_if_fail(host);
+
+	uhc = uhc_new(host);
+	if (hash_list_contains(uhc_list, uhc, NULL)) {
+		g_warning("Duplicate bootstrap UHC: \"%s\"", uhc->host);
+		uhc_free(&uhc);
+		return;
+	}
+
+	if (random_value(100) < 50) {
+		hash_list_append(uhc_list, uhc);
+	} else {
+		hash_list_prepend(uhc_list, uhc);
+	}
+}
+
+/**
+ * @return NULL on error, a newly allocated string otherwise.
+ */
+static gchar *
+uhc_get_next(void)
+{
+	struct uhc *uhc;
+	gchar *host;
+	time_t now;
+
+	g_return_val_if_fail(uhc_list, NULL);
+	
+	now = tm_time();
+	uhc = hash_list_head(uhc_list);
+	if (NULL == uhc)
+		return NULL;
+
+	/*
+	 * Wait UHC_RETRY_AFTER secs before contacting the UHC again.
+	 * Can't be too long because the UDP reply may get lost if the
+	 * requesting host already has a saturated b/w.
+	 * If we come here, it's because we're lacking hosts for establishing
+	 * a Gnutella connection, after we exhausted our caches.
+	 */
+	if (uhc->stamp && delta_time(now, uhc->stamp) < UHC_RETRY_AFTER)
+		return NULL;
+
+	uhc->stamp = now;
+	host = g_strdup(uhc->host);
+
+	if (uhc->used < UHC_MAX_ATTEMPTS) {
+		uhc->used++;
+		hash_list_moveto_tail(uhc_list, uhc);
+	} else {
+		hash_list_remove(uhc_list, uhc);
+		uhc_free(&uhc);
+	}
+
+	return host;
 }
 
 /**
@@ -178,60 +271,19 @@ add_available_uhc(const gchar *hc)
 static gboolean
 uhc_pick(void)
 {
-	gchar *hc;
-	size_t len;
-	guint idx;
-	time_t now = tm_time();
+	gboolean success = FALSE;
+	gchar *uhc;
 
-	/* First check whether used UHCs can added back */
-	while (uhc_used) {
-		struct used_uhc *uu;
-
-		uu = uhc_used->data;
-		g_assert(uu);
-
-		/*
-		 * Wait UHC_RETRY_AFTER secs before contacting the UHC again.
-		 * Can't be too long because the UDP reply may get lost if the
-		 * requesting host already has a saturated b/w.
-		 * If we come here, it's because we're lacking hosts for establishing
-		 * a Gnutella connection, after we exhausted our caches.
-		 */
-
-		if (delta_time(now, uu->stamp) < UHC_RETRY_AFTER)
-			break;
-
-		add_available_uhc(uu->host);
-
-		uhc_used = g_list_remove(uhc_used, uu);
-		G_FREE_NULL(uu->host);
-		G_FREE_NULL(uu);
-	}
-
-	len = g_list_length(uhc_avail);
-	if (len < 1) {
-		if (gwc_debug || bootstrap_debug)
+	uhc = uhc_get_next();
+	if (NULL == uhc) {
+		if (GNET_PROPERTY(bootstrap_debug))
 			g_warning("BOOT ran out of UHCs");
-		return FALSE;
+		goto finish;
 	}
 
-	idx = random_value(len - 1);
-	hc = g_list_nth_data(uhc_avail, idx);
-	g_assert(hc);
-
-	uhc_avail = g_list_remove(uhc_avail, hc);
-	{
-		struct used_uhc *uu;
-
-		uu = g_malloc(sizeof *uu);
-		uu->host = hc;
-		uu->stamp = tm_time();
-		uhc_used = g_list_append(uhc_used, uu);
-	}
-
-	if (!uhc_get_host_port(hc, &uhc_ctx.host, &uhc_ctx.port)) {
-		g_warning("cannot parse UDP host cache \"%s\"", hc);
-		return FALSE;
+	if (!uhc_get_host_port(uhc, &uhc_ctx.host, &uhc_ctx.port)) {
+		g_warning("cannot parse UDP host cache \"%s\"", uhc);
+		goto finish;
 	}
 
 	/*
@@ -240,33 +292,14 @@ uhc_pick(void)
 	{
 		gchar msg[256];
 
-		gm_snprintf(msg, sizeof msg, _("Looking for UDP host cache %s"), hc);
+		gm_snprintf(msg, sizeof msg, _("Looking for UDP host cache %s"), uhc);
 		gcu_statusbar_message(msg);
 	}
+	success = TRUE;
 
-	return TRUE;
-}
-
-/**
- * Free GUID atoms held in hash table.	-- foreach() callback
- */
-static gboolean
-uhc_guid_free(gpointer key, gpointer uu_data, gpointer uu_user)
-{
-	(void) uu_user;
-	(void) uu_data;
-
-	atom_guid_free(key);
-	return TRUE;
-}
-
-/**
- * Reset the list of ping GUIDs.
- */
-static void
-uhc_guid_reset(void)
-{
-	g_hash_table_foreach_remove(uhc_ctx.guids, uhc_guid_free, NULL);
+finish:
+	G_FREE_NULL(uhc);
+	return success;
 }
 
 /**
@@ -289,7 +322,8 @@ uhc_try_random(void)
 	 * we're protected by the `attempts' counter.
 	 */
 
-	(void) adns_resolve(uhc_ctx.host, uhc_host_resolved, NULL);
+	(void) adns_resolve(uhc_ctx.host, settings_dns_net(),
+				uhc_host_resolved, NULL);
 }
 
 /**
@@ -302,7 +336,7 @@ uhc_ping_timeout(cqueue_t *unused_cq, gpointer unused_obj)
 	(void) unused_cq;
 	(void) unused_obj;
 
-	if (gwc_debug)
+	if (GNET_PROPERTY(bootstrap_debug))
 		g_warning("no reply from UDP host cache %s:%u",
 			uhc_ctx.host, uhc_ctx.port);
 
@@ -314,67 +348,59 @@ uhc_ping_timeout(cqueue_t *unused_cq, gpointer unused_obj)
  * Send an UDP ping to the host cache.
  */
 static void
-uhc_send_ping(const host_addr_t addr, guint16 port)
+uhc_send_ping(void)
 {
-	struct gnutella_msg_init *m;
-	struct gnutella_node *n = node_udp_get_addr_port(addr, port);
-	guint32 size;
-	gchar *muid;
-	gchar msg[256];
-
 	g_assert(uhc_connecting);
 
-	m = build_ping_msg(NULL, 1, TRUE, &size);
-	muid = atom_guid_get(m->header.muid);
-	udp_send_msg(n, m, size);
+	guid_random_muid(uhc_ctx.muid);	
 
-	/*
-	 * Save the GUID of the ping we sent, to be able to determine when
-	 * we get a reply from our queries.
-	 */
+	if (udp_send_ping(uhc_ctx.muid, uhc_ctx.addr, uhc_ctx.port, TRUE)) {
 
-	if (g_hash_table_lookup(uhc_ctx.guids, muid))
-		g_warning("GUID random number generator is weak");
-	else
-		g_hash_table_insert(uhc_ctx.guids, muid, GUINT_TO_POINTER(1));
+		if (GNET_PROPERTY(bootstrap_debug))
+			g_message("BOOT sent UDP SCP ping %s to %s:%u",
+				guid_hex_str(uhc_ctx.muid), uhc_ctx.host, uhc_ctx.port);
+		/*
+		 * Give GUI feedback.
+		 */
+		{
+			gchar msg[256];
 
-	if (gwc_debug || bootstrap_debug)
-		g_message("BOOT sent UDP SCP ping %s to %s",
-			guid_hex_str(muid), host_addr_port_to_string(addr, port));
+			gm_snprintf(msg, sizeof msg,
+				_("Sent ping to UDP host cache %s:%u"),
+				uhc_ctx.host, uhc_ctx.port);
+			gcu_statusbar_message(msg);
+		}
 
-	/*
-	 * Give GUI feedback.
-	 */
+		/*
+		 * Arm a timer to see whether we should not try to ping another
+		 * host cache if we don't get a timely reply.
+		 */
 
-	gm_snprintf(msg, sizeof msg, _("Sent ping to UDP host cache %s:%u"),
-		uhc_ctx.host, uhc_ctx.port);
-	gcu_statusbar_message(msg);
+		g_assert(uhc_ctx.timeout_ev == NULL);
 
-	/*
-	 * Arm a timer to see whether we should not try to ping another
-	 * host cache if we don't get a timely reply.
-	 */
-
-	g_assert(uhc_ctx.timeout_ev == NULL);
-
-	uhc_ctx.timeout_ev = cq_insert(callout_queue,
-		UHC_TIMEOUT * 1000, uhc_ping_timeout, NULL);
+		uhc_ctx.timeout_ev = cq_insert(callout_queue,
+				UHC_TIMEOUT, uhc_ping_timeout, NULL);
+	} else {
+		g_message("BOOT failed to send UDP SCP to %s",
+			host_addr_port_to_string(uhc_ctx.addr, uhc_ctx.port));
+	}
 }
 
 /**
  * Callback for adns_resolve(), invoked when the resolution is complete.
  */
 static void
-uhc_host_resolved(const host_addr_t *addr, gpointer uu_udata)
+uhc_host_resolved(const host_addr_t *addrs, size_t n, gpointer uu_udata)
 {
 	(void) uu_udata;
+	g_assert(addrs);
 
 	/*
 	 * If resolution failed, try again if possible.
 	 */
 
-	if (!addr || !host_is_valid(*addr, uhc_ctx.port)) {
-		if (gwc_debug)
+	if (0 == n) {
+		if (GNET_PROPERTY(bootstrap_debug))
 			g_warning("could not resolve UDP host cache \"%s\"",
 				uhc_ctx.host);
 
@@ -382,17 +408,18 @@ uhc_host_resolved(const host_addr_t *addr, gpointer uu_udata)
 		return;
 	}
 
-	if (gwc_debug || bootstrap_debug)
+	uhc_ctx.addr = addrs[random_raw() % n];
+	
+	if (GNET_PROPERTY(bootstrap_debug))
 		g_message("BOOT UDP host cache \"%s\" resolved to %s",
-			uhc_ctx.host, host_addr_to_string(*addr));
+			uhc_ctx.host, host_addr_to_string(uhc_ctx.addr));
 
-	uhc_ctx.addr = *addr;
 
 	/*
 	 * Now send the ping.
 	 */
 
-	uhc_send_ping(uhc_ctx.addr, uhc_ctx.port);
+	uhc_send_ping();
 }
 
 /**
@@ -416,7 +443,7 @@ uhc_get_hosts(void)
 	 * must find out hosts another way.
 	 */
 
-	if (uhc_connecting || ancient_version)
+	if (uhc_connecting || GNET_PROPERTY(ancient_version))
 		return;
 
 	if (!udp_active())
@@ -428,7 +455,6 @@ uhc_get_hosts(void)
 
 	uhc_connecting = TRUE;
 	uhc_ctx.attempts = 0;
-	uhc_guid_reset();
 
 	g_assert(uhc_ctx.timeout_ev == NULL);
 
@@ -445,37 +471,35 @@ uhc_get_hosts(void)
 void
 uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
 {
-	const gchar *p;
-	gint i;
-	gint cnt;
-	gboolean replied = FALSE;
+	gint i, cnt;
 
 	g_assert(0 == paylen % 6);
 
 	cnt = paylen / 6;
 
-	if (gwc_debug || bootstrap_debug)
+	if (GNET_PROPERTY(bootstrap_debug))
 		g_message("extracting %d host%s in UDP IPP pong %s from %s (%s)",
 			cnt, cnt == 1 ? "" : "s",
-			guid_hex_str(n->header.muid), node_addr(n),
+			guid_hex_str(gnutella_header_get_muid(&n->header)), node_addr(n),
 			uhc_connecting ? "expected" : "unsollicited");
 
-	for (i = 0, p = payload; i < cnt; i++) {
+	for (i = 0; i < cnt; i++) {
 		host_addr_t ha;
 		guint16 port;
 
-		ha = host_addr_set_ipv4(peek_be32(p));
-		p += 4;
-		port = peek_le16(p);
-		p += 2;
+		ha = host_addr_peek_ipv4(&payload[i * 6]);
+		port = peek_le16(&payload[i * 6 + 4]);
 
 		hcache_add_caught(HOST_ULTRA, ha, port, "UDP-HC");
 
-		if (bootstrap_debug > 1)
+		if (GNET_PROPERTY(bootstrap_debug) > 1)
 			g_message("BOOT collected %s from UDP IPP pong from %s",
 				host_addr_to_string(ha), node_addr(n));
 	}
 
+	if (!uhc_connecting)
+		return;
+
 	/*
 	 * Check whether this was a reply from our request.
 	 *
@@ -484,43 +508,33 @@ uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
 	 * check whether we're still in a probing cycle.
 	 */
 
-	if (uhc_connecting && g_hash_table_lookup(uhc_ctx.guids, n->header.muid)) {
-		g_assert(uhc_ctx.timeout_ev != NULL);
+	if (!guid_eq(uhc_ctx.muid, gnutella_header_get_muid(&n->header)))
+		return;
 
-		if (gwc_debug || bootstrap_debug)
-			g_message("BOOT UDP cache \"%s\" (%s) replied: "
-				"got %d host%s from %s",
-				uhc_ctx.host,
-				host_addr_port_to_string(uhc_ctx.addr, uhc_ctx.port),
-				cnt, cnt == 1 ? "" : "s", node_addr(n));
-
-		/*
-		 * Terminate the probing cycle if we got hosts.
-		 */
-
-		if (cnt) {
-			replied = TRUE;
-			cq_cancel(callout_queue, uhc_ctx.timeout_ev);
-			uhc_ctx.timeout_ev = NULL;
-			uhc_connecting = FALSE;
-		} else
-			uhc_try_random();
+	if (GNET_PROPERTY(bootstrap_debug)) {
+		g_message("BOOT UDP cache \"%s\" replied: got %d host%s from %s",
+			uhc_ctx.host, cnt, cnt == 1 ? "" : "s", node_addr(n));
 	}
 
 	/*
-	 * Display GUI feedback, if we got a sollicited reply.
+	 * Terminate the probing cycle if we got hosts.
 	 */
 
-	if (replied) {
+	if (cnt > 0) {
 		gchar msg[256];
 
+		cq_cancel(callout_queue, &uhc_ctx.timeout_ev);
+		uhc_connecting = FALSE;
+
 		gm_snprintf(msg, sizeof(msg),
-			NG_("Got %d host from UDP host cache %s:%u",
-				"Got %d hosts from UDP host cache %s:%u",
+			NG_("Got %d host from UDP host cache %s",
+				"Got %d hosts from UDP host cache %s",
 				cnt),
-			cnt, uhc_ctx.host, uhc_ctx.port);
+			cnt, uhc_ctx.host);
 
 		gcu_statusbar_message(msg);
+	} else {
+		uhc_try_random();
 	}
 }
 
@@ -532,6 +546,9 @@ uhc_init(void)
 {
 	guint i;
 
+	g_return_if_fail(NULL == uhc_list);
+	uhc_list = hash_list_new(uhc_hash, uhc_equal);
+
 	for (i = 0; i < G_N_ELEMENTS(boot_hosts); i++) {
 		const gchar *host, *ep, *uhc;
 		guint16 port;
@@ -547,10 +564,8 @@ uhc_init(void)
 		g_assert(NULL != ep);
 		g_assert(':' == ep[0]);
 
-		add_available_uhc(uhc);
+		uhc_list_add(uhc);
 	}
-
-	uhc_ctx.guids = g_hash_table_new(guid_hash, guid_eq);
 }
 
 /**
@@ -559,13 +574,17 @@ uhc_init(void)
 void
 uhc_close(void)
 {
-	uhc_guid_reset();
-	g_hash_table_destroy(uhc_ctx.guids);
-
-	if (uhc_ctx.timeout_ev)
-		cq_cancel(callout_queue, uhc_ctx.timeout_ev);
-
+	cq_cancel(callout_queue, &uhc_ctx.timeout_ev);
 	uhc_connecting = FALSE;
+
+	if (uhc_list) {
+		struct uhc *uhc;
+
+		while (NULL != (uhc = hash_list_shift(uhc_list))) {
+			uhc_free(&uhc);
+		}
+		hash_list_free(&uhc_list);
+	}
 }
 
 /* vi: set ts=4 sw=4 cindent: */
