@@ -1,5 +1,5 @@
 /*
- * $Id: verify_tth.c 14051 2007-07-06 01:19:10Z cbiere $
+ * $Id: verify_tth.c 14141 2007-07-21 21:14:04Z cbiere $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: verify_tth.c 14051 2007-07-06 01:19:10Z cbiere $")
+RCSID("$Id: verify_tth.c 14141 2007-07-21 21:14:04Z cbiere $")
 
 #include "downloads.h"
 #include "file_object.h"
@@ -189,6 +189,24 @@ request_tigertree_callback(const struct verify *ctx, enum verify_status status,
 	return FALSE;
 }
 
+gboolean
+verify_tth_append(const char *pathname,
+	filesize_t offset, filesize_t amount,
+	verify_callback callback, void *user_data)
+{
+	return verify_enqueue(verify_tth.verify, FALSE,
+				pathname, offset, amount, callback, user_data);
+}
+
+gboolean
+verify_tth_prepend(const char *pathname,
+	filesize_t offset, filesize_t amount,
+	verify_callback callback, void *user_data)
+{
+	return verify_enqueue(verify_tth.verify, TRUE,
+				pathname, offset, amount, callback, user_data);
+}
+
 void
 request_tigertree(struct shared_file *sf, gboolean high_priority)
 {
