@@ -1,5 +1,5 @@
 /*
- * $Id: vmsg.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: vmsg.h 13917 2007-06-21 20:49:51Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -36,10 +36,11 @@
 #ifndef _core_vmsg_h_
 #define _core_vmsg_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "lib/tm.h"
 
+struct array;
 struct gnutella_node;
 struct pmsg;
 struct rnode_info;
@@ -49,9 +50,10 @@ struct rnode_info;
  */
 
 void vmsg_handle(struct gnutella_node *n);
-const gchar *vmsg_infostr(gconstpointer data, gint size);
+const gchar *vmsg_infostr(gconstpointer data, size_t size);
 
 void vmsg_send_messages_supported(struct gnutella_node *n);
+void vmsg_send_features_supported(struct gnutella_node *n);
 void vmsg_send_hops_flow(struct gnutella_node *n, guint8 hops);
 void vmsg_send_tcp_connect_back(struct gnutella_node *n, guint16 port);
 void vmsg_send_udp_connect_back(struct gnutella_node *n, guint16 port);
@@ -61,16 +63,20 @@ void vmsg_send_qstat_answer(struct gnutella_node *n,
 		const gchar *muid, guint16 hits);
 void vmsg_send_proxy_cancel(struct gnutella_node *n);
 void vmsg_send_oob_reply_ack(struct gnutella_node *n,
-		const gchar *muid, guint8 want);
+		const gchar *muid, guint8 want, const struct array *token);
 void vmsg_send_time_sync_req(struct gnutella_node *n, gboolean ntp, tm_t *);
 void vmsg_send_time_sync_reply(struct gnutella_node *n, gboolean ntp, tm_t *);
 void vmsg_send_udp_crawler_pong(struct gnutella_node *n, struct pmsg *mb);
 void vmsg_send_node_info_ans(struct gnutella_node *n,
 	const struct rnode_info *ri);
+void vmsg_send_head_ping(const struct sha1 *sha1,
+		host_addr_t addr, guint16 port, const gchar *guid);
 
-struct pmsg *vmsg_build_oob_reply_ind(const gchar *muid, guint8 hits);
+struct pmsg *vmsg_build_oob_reply_ind(const gchar *muid,
+				guint8 hits, gboolean secure);
 
 void vmsg_init(void);
+void vmsg_close(void);
 
 #endif	/* _core_vmsg_h_ */
 
