@@ -1,5 +1,5 @@
 /*
- * $Id: c2ui.c 13629 2007-05-13 04:30:01Z cbiere $
+ * $Id: c2ui.c 14372 2007-08-08 21:43:46Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -43,30 +43,30 @@
 #define GUI_SOURCES
 
 #if defined(USE_TOPLESS)
-#define gui_download_enable_start_now(running_downloads, max_downloads) \
-	((void) running_downloads, (void) max_downloads)
-#define gui_update_download(d, force) ((void) d, (void) force)
-#define gui_update_download_server(d) ((void) d)
-#define gui_update_download_range(d) ((void) d)
-#define gui_update_download_size(d) ((void) d)
-#define gui_update_download_host(d) ((void) d)
-#define gui_update_download_abort_resume()
-#define gui_update_download_clear()
-#define	gui_update_download_clear_now()
-#define	download_gui_add(d) ((void) d)
-#define	download_gui_remove(d) ((void) d)
+#define running_topless 1
+
+#define	gui_download_updates_freeze()
+#define	gui_download_updates_thaw()
+
 #define	gui_update_files_scanned()
 #define	gui_allow_rescan_dir(flag) ((void) flag)
-#define search_gui_new_search(query, flags, x) (query && flags) ? FALSE : FALSE
+#define search_gui_new_search(query, flags, x) (((query) && (flags)) ? 0 : 0)
+
 #define upload_stats_gui_add(s) ((void) s)
 #define upload_stats_gui_update(s) ((void) s)
 #define	upload_stats_gui_clear_all()
+#define upload_stats_gui_thaw()
+#define upload_stats_gui_freeze()
+
 #define statusbar_gui_warning(sec, fmt, message) g_message((fmt), (message))
 #define statusbar_gui_message(sec, fmt, message) g_message((fmt), (message))
+
 #define bitzi_gui_update(bitzi_data) ((void) bitzi_data)
+
 #endif /* USE_TOPLESS */
 
 #if defined(USE_GTK1) || defined(USE_GTK2)
+extern gboolean running_topless;
 #include "if/ui/gtk/bitzi.h"
 #include "if/ui/gtk/downloads.h"
 #include "if/ui/gtk/misc.h"
@@ -80,103 +80,61 @@
  * Functions the CORE uses to access the UI
  */
 
-/*	download interface functions (CORE -> UI) */
 void
-gcu_download_enable_start_now(guint32 running_downloads, guint32 max_downloads)
+gcu_download_gui_updates_freeze(void)
 {
-	gui_download_enable_start_now(running_downloads, max_downloads);
+	if (!running_topless) {
+		gui_download_updates_freeze();
+	}
 }
 
 void
-gcu_gui_update_download(download_t *d, gboolean force)
+gcu_download_gui_updates_thaw(void)
 {
-	gui_update_download(d, force);
+	if (!running_topless) {
+		gui_download_updates_thaw();
+	}
 }
 
-void
-gcu_gui_update_download_server(struct download *d)
-{
-	gui_update_download_server(d);
-}
-
-void
-gcu_gui_update_download_range(struct download *d)
-{
-	gui_update_download_range(d);
-}
-
-void
-gcu_gui_update_download_size(struct download *d)
-{
-	gui_update_download_size(d);
-}
-
-void
-gcu_gui_update_download_host(struct download *d)
-{
-	gui_update_download_host(d);
-}
-
-void
-gcu_gui_update_download_abort_resume(void)
-{
-	gui_update_download_abort_resume();
-}
-
-void
-gcu_gui_update_download_clear(void)
-{
-	gui_update_download_clear();
-}
-
-void
-gcu_gui_update_download_clear_now(void)
-{
-	gui_update_download_clear_now();
-}
-
-void
-gcu_download_gui_add(struct download *d)
-{
-	download_gui_add(d);
-}
-
-void
-gcu_download_gui_remove(struct download *d)
-{
-	download_gui_remove(d);
-}
-
-
 /** misc. interface functions (CORE -> UI) */
 void
 gcu_gui_update_files_scanned(void)
 {
-	gui_update_files_scanned();
+	if (!running_topless) {
+		gui_update_files_scanned();
+	}
 }
 
 void
 guc_allow_rescan_dir(gboolean flag)
 {
-	gui_allow_rescan_dir(flag);
+	if (!running_topless) {
+		gui_allow_rescan_dir(flag);
+	}
 }
 
 gint
 gcu_gtk_main_flush(void)
 {
 #if defined(USE_GTK1) || defined(USE_GTK2)
-	extern gint gtk_main_flush();	/* Don't include any GTK-related header */
-	return gtk_main_flush();
-#else
-	return 0;
+	if (!running_topless) {
+		extern gint gtk_main_flush();
+		/* Don't include any GTK-related header */
+		return gtk_main_flush();
+	}
 #endif /* GTK */
+	return 0;
 }
 
 /**	search interface functions (CORE -> UI) */
 gboolean
 gcu_search_gui_new_search(const gchar *query, flag_t flags)
 {
-	return search_gui_new_search(query, flags, NULL);
+	if (!running_topless) {
+		return search_gui_new_search(query, flags, NULL);
+	} else {
+		return FALSE;
+	}
 }
 
 
@@ -184,19 +142,41 @@ gcu_search_gui_new_search(const gchar *query, flag_t flags)
 void
 gcu_upload_stats_gui_add(const struct ul_stats *s)
 {
-	upload_stats_gui_add(s);
+	if (!running_topless) {
+		upload_stats_gui_add(s);
+	}
 }
 
 void
 gcu_upload_stats_gui_update(const struct ul_stats *s)
 {
-	upload_stats_gui_update(s);
+	if (!running_topless) {
+		upload_stats_gui_update(s);
+	}
 }
 
 void
 gcu_upload_stats_gui_clear_all(void)
 {
-	upload_stats_gui_clear_all();
+	if (!running_topless) {
+		upload_stats_gui_clear_all();
+	}
+}
+
+void
+gcu_upload_stats_gui_freeze(void)
+{
+	if (!running_topless) {
+		upload_stats_gui_freeze();
+	}
+}
+
+void
+gcu_upload_stats_gui_thaw(void)
+{
+	if (!running_topless) {
+		upload_stats_gui_thaw();
+	}
 }
 
 /**
@@ -209,20 +189,26 @@ gcu_upload_stats_gui_clear_all(void)
 void
 gcu_bitzi_result(bitzi_data_t *bitzi_data)
 {
-    bitzi_gui_update(bitzi_data);
+	if (!running_topless) {
+    	bitzi_gui_update(bitzi_data);
+	}
 }
 
 /*	statusbar interface functions (CORE -> UI) */
 void
 gcu_statusbar_warning(const gchar *message)
 {
-	statusbar_gui_warning(15, "%s", message);
+	if (!running_topless) {
+		statusbar_gui_warning(15, "%s", message);
+	}
 }
 
 void
 gcu_statusbar_message(const gchar *message)
 {
-	statusbar_gui_message(10, "%s", message);
+	if (!running_topless) {
+		statusbar_gui_message(10, "%s", message);
+	}
 }
 
 /* vi: set ts=4 sw=4 cindent: */
