@@ -1,5 +1,5 @@
 /*
- * $Id: ui2c.c 13925 2007-06-22 05:24:10Z cbiere $
+ * $Id: ui2c.c 15086 2008-01-13 01:28:03Z cbiere $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -83,6 +83,12 @@ guc_query_bitzi_by_sha1(const struct sha1 *sha1, filesize_t filesize)
     bitzi_query_by_sha1(sha1, filesize);
 }
 
+const char *
+guc_bitzi_ticket_by_sha1(const struct sha1 *sha1, filesize_t filesize)
+{
+    return bitzi_ticket_by_sha1(sha1, filesize);
+}
+
 gboolean
 guc_bitzi_has_cached_ticket(const struct sha1 *sha1)
 {
@@ -134,24 +140,6 @@ guc_download_remove_all_from_peer(const gchar *guid, const host_addr_t addr,
 	return download_remove_all_from_peer(guid, addr, port, unavailable);
 }
 
-gint
-guc_download_remove_all_named(const gchar *name)
-{
-	return download_remove_all_named(name);
-}
-
-gint
-guc_download_remove_all_with_sha1(const struct sha1 *sha1)
-{
-	return download_remove_all_with_sha1(sha1);
-}
-
-void
-guc_download_remove_file(struct download *d, gboolean reset)
-{
-	download_remove_file(d, reset);
-}
-
 gboolean
 guc_download_file_exists(const struct download *d)
 {
@@ -161,37 +149,31 @@ guc_download_file_exists(const struct download *d)
 void
 guc_download_requeue(struct download *d)
 {
-	download_requeue(d);
+	download_request_requeue(d);
 }
 
 void
-guc_download_start(struct download *d, gboolean check_allowed)
+guc_download_start(struct download *d)
 {
-	download_start(d, check_allowed);
+	download_request_start(d);
 }
 
 void
 guc_download_pause(struct download *d)
 {
-	download_pause(d);
-}
-
-gboolean
-guc_download_remove(struct download *d)
-{
-	return download_remove(d);
+	download_request_pause(d);
 }
 
 void
 guc_download_abort(struct download *d)
 {
-	download_abort(d);
+	download_request_abort(d);
 }
 
 void
 guc_download_resume(struct download *d)
 {
-	download_resume(d);
+	download_request_resume(d);
 }
 
 void
@@ -214,9 +196,9 @@ guc_download_queue_is_frozen(void)
 
 void
 guc_download_clear_stopped(gboolean complete,
-	gboolean failed, gboolean unavailable, gboolean now)
+	gboolean failed, gboolean unavailable, gboolean finished, gboolean now)
 {
-	download_clear_stopped(complete, failed, unavailable, now);
+	download_clear_stopped(complete, failed, unavailable, finished, now);
 }
 
 guint
@@ -234,6 +216,7 @@ guc_download_new(const gchar *filename,
 	const gchar *guid,
 	const gchar *hostname,
 	const struct sha1 *sha1,
+	const struct tth *tth,
 	time_t stamp,
 	fileinfo_t *fi,
 	gnet_host_vec_t *proxies,
@@ -248,6 +231,7 @@ guc_download_new(const gchar *filename,
 		   	guid,
 			hostname,
 			sha1,
+			tth,
 			stamp,
 			fi,
 			proxies,
@@ -263,6 +247,7 @@ guc_download_auto_new(const gchar *filename,
 	const gchar *guid,
 	const gchar *hostname,
 	const struct sha1 *sha1,
+	const struct tth *tth,
 	time_t stamp,
 	fileinfo_t *fi,
 	gnet_host_vec_t *proxies,
@@ -275,6 +260,7 @@ guc_download_auto_new(const gchar *filename,
 		guid,
 		hostname,
 		sha1,
+		tth,
 		stamp,
 		fi,
 		proxies,
@@ -293,6 +279,12 @@ guc_download_get_country(const struct download *d)
 	return iso3166_country_cc(download_country(d));
 }
 
+const gchar *
+guc_download_get_vendor(const struct download *d)
+{
+	return download_vendor_str(d);
+}
+
 gdouble
 guc_download_source_progress(const struct download *d)
 {
@@ -370,9 +362,21 @@ guc_fi_get_aliases(gnet_fi_t fih)
 }
 
 void
-guc_fi_purge_by_handle_list(const GSList *list)
+guc_fi_purge(gnet_fi_t fih)
 {
-	fi_purge_by_handle_list(list);
+	fi_purge(fih);
+}
+
+void
+guc_fi_pause(gnet_fi_t fih)
+{
+	fi_pause(fih);
+}
+
+void
+guc_fi_resume(gnet_fi_t fih)
+{
+	fi_resume(fih);
 }
 
 void
@@ -735,11 +739,12 @@ guc_search_is_frozen(gnet_search_t sh)
 	return search_is_frozen(sh);
 }
 
-gnet_search_t
-guc_search_new(const gchar *query,
+enum search_new_result
+guc_search_new(gnet_search_t *ptr, const gchar *query,
 	time_t create_time, guint lifetime, guint32 reissue_timeout, flag_t flags)
 {
-	return search_new(query, create_time, lifetime, reissue_timeout, flags);
+	return search_new(ptr, query,
+				create_time, lifetime, reissue_timeout, flags);
 }
 
 gboolean
@@ -836,16 +841,39 @@ guc_shared_kbytes_scanned(void)
 }
 
 void
-guc_share_add_search_request_listener(search_request_listener_t l)
+guc_search_got_results_listener_add(search_got_results_listener_t l)
 {
-	share_add_search_request_listener(l);
+	search_got_results_listener_add(l);
 }
 
+void
+guc_search_got_results_listener_remove(search_got_results_listener_t l)
+{
+	search_got_results_listener_remove(l);
+}
 
 void
-guc_share_remove_search_request_listener(search_request_listener_t l)
+guc_search_status_change_listener_add(search_status_change_listener_t l)
 {
-	share_remove_search_request_listener(l);
+	search_status_change_listener_add(l);
+}
+
+void
+guc_search_status_change_listener_remove(search_status_change_listener_t l)
+{
+	search_status_change_listener_remove(l);
+}
+
+void
+guc_search_request_listener_add(search_request_listener_t l)
+{
+	search_request_listener_add(l);
+}
+
+void
+guc_search_request_listener_remove(search_request_listener_t l)
+{
+	search_request_listener_remove(l);
 }
 
 void
