@@ -1,5 +1,5 @@
 /*
- * $Id: downloads.h 13880 2007-06-18 22:45:58Z cbiere $
+ * $Id: downloads.h 14935 2007-09-19 13:57:30Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -43,17 +43,6 @@
 
 typedef guint32 gnet_src_t;
 
-typedef void (*src_listener_t) (gnet_src_t);
-typedef enum {
-	EV_SRC_ADDED = 0,
-	EV_SRC_REMOVED,
-	EV_SRC_INFO_CHANGED,
-	EV_SRC_STATUS_CHANGED,
-	EV_SRC_RANGES_CHANGED,
-
-	EV_SRC_EVENTS /* Number of events in this domain */
-} gnet_src_ev_t;
-
 #define URN_INDEX	0xffffffff		/**< Marking index, indicates URN instead */
 
 /*
@@ -144,7 +133,7 @@ typedef enum {
     GTA_DL_ACTIVE_QUEUED,	/**< Actively queued */
     GTA_DL_PASSIVE_QUEUED,	/**< Passively queued */
     GTA_DL_REQ_SENDING,		/**< Sending HTTP request */
-	GTA_DL_IGNORING,		/**< Ignoring received data on resuming mismatch */
+	GTA_DL_IGNORING			/**< Ignoring received data on resuming mismatch */
 } download_status_t;
 
 typedef struct download download_t;
@@ -153,8 +142,8 @@ struct bio_source;
 struct http_buffer;
 
 enum dl_bufmode {
-	DL_BUF_READING = 0,
-	DL_BUF_WRITING,
+	DL_BUF_READING,
+	DL_BUF_WRITING
 };
 
 /**
@@ -176,6 +165,7 @@ enum download_magic { DOWNLOAD_MAGIC = 0x2dd6efe9 };	/**< Magic number */
 struct download {
 	enum download_magic magic;	/**< Magic number */
     gnet_src_t src_handle;      /**< Handle */
+	gboolean src_handle_valid;	/**< TRUE if src_handle is initialized */
 
 	gchar error_str[256];		/**< Used to snprintf() error strings */
 	download_status_t status;   /**< Current status of the download */
@@ -190,7 +180,6 @@ struct download {
 	struct dl_file_info *file_info;
 	guint32 record_index;		/**< Index of the file on the Gnutella server */
 	const gchar *file_name;		/**< Name of the file on the Gnutella server */
-	const gchar *escaped_name;	/**< As file_name, with control chars escaped */
 	filesize_t file_size;		/**< Total size of the file, in bytes */
 
 	filesize_t size;			/**< Total size of the next request, in bytes */
@@ -224,7 +213,7 @@ struct download {
 	const struct sha1 *sha1;	/**< Known SHA1 (binary atom), NULL if none */
 	const gchar *uri;			/**< Uri if not dealing with regular gnutella
 								 **< file download */
-	guint32 last_dmesh;			/**< Time when last download mesh was sent */
+	time_t last_dmesh;			/**< Time when last download mesh was sent */
 
 	GSList *ranges;				/**< PFSP -- known list of ranges, or NULL */
 	filesize_t ranges_size;		/**< PFSP -- size of remotely available data */
@@ -234,7 +223,6 @@ struct download {
 	guint32 cflags;
 
 	gboolean keep_alive;		/**< Keep HTTP connection? */
-	gboolean visible;			/**< The download is visible in the GUI */
 	gboolean push;				/**< Currently in push mode */
 	gboolean always_push;		/**< Always use the push method for this */
 	gboolean got_giv;			/**< Whether initiated from GIV reception */
@@ -251,42 +239,46 @@ struct download {
  */
 
 enum {
-	DL_F_URIRES			= 1 << 0,	/**< We sent a "/uri-res/N2R?" request */
-	DL_F_PUSH_IGN		= 1 << 1,	/**< Trying to ignore push flag */
-	DL_F_OVERLAPPED		= 1 << 2,	/**< We went through overlap checking */
-	DL_F_REPLIED		= 1 << 3,	/**< Servent replied to last request */
-	DL_F_CHUNK_CHOSEN	= 1 << 4,	/**< Retrying with specific chunk */
-	DL_F_SHRUNK_REPLY	= 1 << 5,	/**< Server sending less than we asked */
-	DL_F_SUNK_DATA		= 1 << 6,	/**< Whether we previously sunk data */
-	DL_F_ACTIVE_QUEUED	= 1 << 7,	/**< Download is actively queued */
-	DL_F_PASSIVE_QUEUED	= 1 << 8,	/**< Download is passively queued */
-	DL_F_DNS_LOOKUP		= 1 << 9,	/**< Attempted DNS lookup */
-	DL_F_BROWSE			= 1 << 10,	/**< Browse host type (requests "/") */
-	DL_F_TRANSIENT		= 1 << 11,	/**< Transient, don't persist */
-	DL_F_SUSPENDED		= 1 << 12,	/**< Suspended, do not schedule */
-	DL_F_MARK			= 1 << 13,	/**< Marked in traversal */
-	DL_F_PREFIX_HEAD	= 1 << 14,	/**< Sent HEAD request before GET */
-	DL_F_INITIAL		= 1 << 15,	/**< First request on this connection */
-	DL_F_PAUSED			= 1 << 16,	/**< Paused by user */
-	DL_F_THEX			= 1 << 17,	/**< THEX download */
-	DL_F_UDP_PUSH		= 1 << 18,	/**< UDP push already attempted */
+	DL_F_FAKE_G2		= 1 << 23,	/**< Trying to fake G2, intuition only */
+	DL_F_TRIED_TLS		= 1 << 22,	/**< TLS connection was tried already */
+	DL_F_TRY_TLS		= 1 << 21,	/**< Try to initiate a TLS connection */
+	DL_F_GOT_TTH		= 1 << 20,	/**< Tigertree fetching finished */
 	DL_F_FETCH_TTH		= 1 << 19,	/**< Tigertree data is being fetched */
-	DL_F_GOT_TTH		= 1 << 20	/**< Tigertree fetching finished */
+	DL_F_UDP_PUSH		= 1 << 18,	/**< UDP push already attempted */
+	DL_F_THEX			= 1 << 17,	/**< THEX download */
+	DL_F_PAUSED			= 1 << 16,	/**< Paused by user */
+	DL_F_INITIAL		= 1 << 15,	/**< First request on this connection */
+	DL_F_PREFIX_HEAD	= 1 << 14,	/**< Sent HEAD request before GET */
+	DL_F_MARK			= 1 << 13,	/**< Marked in traversal */
+	DL_F_SUSPENDED		= 1 << 12,	/**< Suspended, do not schedule */
+	DL_F_TRANSIENT		= 1 << 11,	/**< Transient, don't persist */
+	DL_F_BROWSE			= 1 << 10,	/**< Browse host type (requests "/") */
+	DL_F_DNS_LOOKUP		= 1 << 9,	/**< Attempted DNS lookup */
+	DL_F_PASSIVE_QUEUED	= 1 << 8,	/**< Download is passively queued */
+	DL_F_ACTIVE_QUEUED	= 1 << 7,	/**< Download is actively queued */
+	DL_F_SUNK_DATA		= 1 << 6,	/**< Whether we previously sunk data */
+	DL_F_SHRUNK_REPLY	= 1 << 5,	/**< Server sending less than we asked */
+	DL_F_CHUNK_CHOSEN	= 1 << 4,	/**< Retrying with specific chunk */
+	DL_F_REPLIED		= 1 << 3,	/**< Servent replied to last request */
+	DL_F_OVERLAPPED		= 1 << 2,	/**< We went through overlap checking */
+	DL_F_PUSH_IGN		= 1 << 1,	/**< Trying to ignore push flag */
+	DL_F_URIRES			= 1 << 0	/**< We sent a "/uri-res/N2R?" request */
 };
 
 /*
  * Server attributes.
  */
 enum {
-	DLS_A_UNUSED_1		= 1 << 0,	/**< UNUSED */
-	DLS_A_PUSH_IGN		= 1 << 1,	/**< Ignore pushes, connect directly */
-	DLS_A_UNUSED_2		= 1 << 2,	/**< UNUSED */
-	DLS_A_NO_HTTP_1_1	= 1 << 3,	/**< Server does NOT support HTTP/1.1 */
-	DLS_A_MINIMAL_HTTP	= 1 << 4,	/**< Use minimalist HTTP with server */
-	DLS_A_BANNING		= 1 << 5,	/**< Server might be banning us */
-	DLS_A_DNS_LOOKUP	= 1 << 6,	/**< Perform DNS lookup if possible */
+	DLS_A_G2_ONLY		= 1 << 9,	/**< Server is "G2" only */
+	DLS_A_FOOBAR		= 1 << 8,	/**< Server is foobar */
 	DLS_A_REMOVED		= 1 << 7,	/**< Server marked for removal */
-	DLS_A_FOOBAR		= 1 << 8	/**< Server is foobar */
+	DLS_A_DNS_LOOKUP	= 1 << 6,	/**< Perform DNS lookup if possible */
+	DLS_A_BANNING		= 1 << 5,	/**< Server might be banning us */
+	DLS_A_MINIMAL_HTTP	= 1 << 4,	/**< Use minimalist HTTP with server */
+	DLS_A_NO_HTTP_1_1	= 1 << 3,	/**< Server does NOT support HTTP/1.1 */
+	DLS_A_FAKE_G2		= 1 << 2,	/**< Fake a G2 server */
+	DLS_A_PUSH_IGN		= 1 << 1,	/**< Ignore pushes, connect directly */
+	DLS_A_FAKED_VENDOR	= 1 << 0	/**< Server's vendor was faked */
 };
 
 /*
@@ -317,7 +309,9 @@ enum {
  * State inspection macros.
  */
 
-#define DOWNLOAD_IS_QUEUED(d)  ((d)->status == GTA_DL_QUEUED)
+#define DOWNLOAD_IS_QUEUED(d) \
+	(	(d)->status == GTA_DL_QUEUED	\
+	||  (d)->status == GTA_DL_PASSIVE_QUEUED)
 
 #define DOWNLOAD_IS_VERIFYING(d)		 \
 	(  (d)->status == GTA_DL_VERIFY_WAIT \
@@ -363,9 +357,8 @@ enum {
 	||	DOWNLOAD_IS_ESTABLISHING(d)		)
 
 #define DOWNLOAD_IS_IN_PUSH_MODE(d) (d->push)
-#define DOWNLOAD_IS_VISIBLE(d)		(d->visible)
 
-gboolean download_has_blank_guid(const struct download *d);
+gboolean download_has_blank_guid(const struct download *);
 
 static inline void
 download_check(const struct download * const d)
@@ -375,8 +368,8 @@ download_check(const struct download * const d)
 	g_assert(!d->always_push || !download_has_blank_guid(d));
 }
 
-const gchar *download_pathname(const struct download *d);
-const gchar *download_basename(const struct download *d);
+const gchar *download_pathname(const struct download *);
+const gchar *download_basename(const struct download *);
 
 /*
  * Public interface, visible only from the bridge.
@@ -399,6 +392,7 @@ gboolean download_new(const gchar *filename,
 	const gchar *guid,
 	const gchar *hostname,
 	const struct sha1 *sha1,
+	const struct tth *tth,
 	time_t stamp,
     struct dl_file_info *fi,
 	const gnet_host_vec_t *proxies,
@@ -412,44 +406,41 @@ void download_auto_new(const gchar *filename,
 	const gchar *guid,
 	const gchar *hostname,
 	const struct sha1 *sha1,
+	const struct tth *tth,
 	time_t stamp,
 	struct dl_file_info *fi,
 	gnet_host_vec_t *proxies,
 	guint32 flags);
 
-void src_add_listener(src_listener_t, gnet_src_ev_t, frequency_t, guint32);
-void src_remove_listener(src_listener_t, gnet_src_ev_t);
-struct download *src_get_download(gnet_src_t src_handle);
-
 guint download_handle_magnet(const gchar *url);
-gchar *download_build_url(const struct download *d);
-gint download_get_http_req_percent(const struct download *d);
+gchar *download_build_url(const struct download *);
+gint download_get_http_req_percent(const struct download *);
 void download_fallback_to_push(struct download *, gboolean, gboolean);
 gint download_remove_all_from_peer(const gchar *guid,
 		host_addr_t addr, guint16 port, gboolean unavailable);
-gint download_remove_all_named(const gchar *name);
-gint download_remove_all_with_sha1(const struct sha1 *sha1);
-void download_remove_file(struct download *d, gboolean reset);
-gboolean download_file_exists(const struct download *d);
-void download_requeue(struct download *);
-void download_start(struct download *, gboolean);
-void download_pause(struct download *);
-gboolean download_remove(struct download *);
-void download_abort(struct download *);
-void download_resume(struct download *);
+void download_remove_file(struct download *, gboolean reset);
+gboolean download_file_exists(const struct download *);
+
+void download_request_requeue(struct download *);
+void download_request_start(struct download *);
+void download_request_pause(struct download *);
+gboolean download_request_remove(struct download *);
+void download_request_abort(struct download *);
+void download_request_resume(struct download *);
+
 void download_freeze_queue(void);
 void download_thaw_queue(void);
 gboolean download_queue_is_frozen(void);
-void download_clear_stopped(gboolean, gboolean, gboolean, gboolean);
-const gchar *download_get_hostname(const struct download *d);
+
+void download_gui_updates_freeze(void);
+void download_gui_updates_thaw(void);
+
+void download_clear_stopped(gboolean, gboolean, gboolean, gboolean, gboolean);
+const gchar *download_get_hostname(const struct download *);
 gdouble download_source_progress(const struct download *);
 gdouble download_total_progress(const struct download *);
 gboolean download_something_to_clear(void);
-struct download *src_get_download(gnet_src_t src_handle);
-void src_add_listener(src_listener_t cb, gnet_src_ev_t ev,
-	frequency_t t, guint32 interval);
-void src_remove_listener(src_listener_t cb, gnet_src_ev_t ev);
-guint download_speed_avg(struct download *d);
+guint download_speed_avg(struct download *);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_downloads_h_ */
