@@ -1,5 +1,5 @@
 /*
- * $Id: fileinfo.h 13995 2007-06-30 21:40:42Z cbiere $
+ * $Id: fileinfo.h 14702 2007-08-25 16:42:22Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -33,6 +33,23 @@
 struct shared_file;
 struct download;
 
+/*
+ * Operating flags.
+ */
+
+enum {
+	FI_F_SUSPEND		= 1 << 0,	/**< Marked "suspended" new downloads */
+	FI_F_DISCARD		= 1 << 1,	/**< Discard fileinfo if refcount = 0 */
+	FI_F_TRANSIENT		= 1 << 2,	/**< Don't persist to disk */
+	FI_F_MARK			= 1 << 3,	/**< Marked during traversal */
+	FI_F_PAUSED			= 1 << 4,	/**< Paused by user */
+	FI_F_SEEDING		= 1 << 5,	/**< Seeding after successful download */
+	FI_F_STRIPPED		= 1 << 6,	/**< Fileinfo trailler has been stripped */
+	FI_F_FETCH_TTH		= 1 << 7,	/**< Tigertree data is being downloaded */
+	FI_F_UNLINKED		= 1 << 8,	/**< Removed from disk */
+	FI_F_BAD_BITPRINT	= 1 << 9,	/**< SHA1 + TTH combination is bad */
+};
+
 /**
  * These used to be in fileinfo.h, but we need them now at several places.
  */
@@ -46,32 +63,43 @@ typedef guint32 gnet_fi_t;
 
 typedef struct gnet_fi_info {
 	gnet_fi_t fi_handle;
-	const gchar *file_name; /**< Name of the file on disk */
-	const struct sha1 *sha1;/**< SHA1 (binary) of the file or NULL */
-	GSList *aliases;		/**< List of aliases (NULL if none) */
+	const gchar *filename;		/**< Name of the file on disk */
+	const struct sha1 *sha1;	/**< SHA1 (binary) of the file or NULL */
+	const struct tth *tth;		/**< TTH (binary) of the file or NULL */
+	GSList *aliases;			/**< List of aliases (NULL if none) */
+	filesize_t  size;
+	filesize_t	tth_slice_size;
+	size_t 		tth_num_leaves;
+	time_t		ctime;
 } gnet_fi_info_t;
 
 typedef struct gnet_fi_status {
+	filesize_t  size;
+	filesize_t  done;
+	
+	filesize_t  uploaded;
+	filesize_t  sha1_hashed;
+	filesize_t	copied;
+
 	guint32  	recvcount;
 	guint32  	refcount;
 	guint32  	lifecount;
-	filesize_t  size;
-	filesize_t  done;
-	filesize_t  uploaded;
 	guint32  	recv_last_rate;
-	guint32  	aqueued_count;
-	guint32  	pqueued_count;
+	guint32  	active_queued;
+	guint32  	passive_queued;
+
+	unsigned	paused:1;
+	unsigned	has_sha1:1;
 
 	/*
 	 * The following are set only when file has been completely downloaded.
 	 */
 
-	filesize_t  sha1_hashed;
-	filesize_t	copied;
-	gboolean	has_sha1;
-	gboolean	sha1_matched;
-	gboolean	paused;
-	gboolean	seeding;
+	unsigned	complete:1;
+	unsigned	verifying:1;
+	unsigned 	sha1_matched:1;
+	unsigned	finished:1;
+	unsigned	seeding:1;
 } gnet_fi_status_t;
 
 typedef struct gnet_fi_chunks {
@@ -99,6 +127,7 @@ typedef struct dl_file_info {
 	struct {
 		struct tth *leaves;	/**< Tigertree leaves */
 		size_t num_leaves;	/**< Number of tigertree leaves */
+		filesize_t slice_size;	/* Slice size (bytes covered by a leaf) */
 	} tigertree;
 	gint32 refcount;		/**< Reference count of file (number of sources)*/
 	GSList *sources;        /**< list of sources (struct download *) */
@@ -120,8 +149,8 @@ typedef struct dl_file_info {
 	gboolean dirty;			/**< Does it need saving? */
 	gboolean dirty_status;  /**< Notify about status change on next interval */
 	gboolean hashed;		/**< In hash tables? */
-	guint32  aqueued_count; /**< Actively queued sources */
-	guint32  pqueued_count; /**< Passively queued sources */
+	guint32  active_queued; /**< Actively queued sources */
+	guint32  passive_queued;/**< Passively queued sources */
 
 	/*
 	 * The following group is used to compute the aggregated reception rate.
@@ -152,6 +181,20 @@ file_info_check(const fileinfo_t *fi)
 	g_assert(is_absolute_path(fi->pathname));
 }
 
+static inline void
+fi_mark_bad_bitprint(fileinfo_t *fi)
+{
+	file_info_check(fi);
+	fi->flags |= FI_F_BAD_BITPRINT;
+}
+
+static inline gboolean
+fi_has_bad_bitprint(fileinfo_t *fi)
+{
+	file_info_check(fi);
+	return (fi->flags & FI_F_BAD_BITPRINT) ? TRUE : FALSE;
+}
+
 static inline gboolean
 FILE_INFO_COMPLETE(const fileinfo_t *fi)
 {
@@ -159,6 +202,14 @@ FILE_INFO_COMPLETE(const fileinfo_t *fi)
 	return fi->file_size_known && fi->done == fi->size;
 }
 
+static inline gboolean
+FILE_INFO_FINISHED(const fileinfo_t *fi)
+{
+	file_info_check(fi);
+	return 0 != ((FI_F_STRIPPED | FI_F_TRANSIENT) & fi->flags)
+		&& FILE_INFO_COMPLETE(fi);
+}
+
 static inline gboolean
 FILE_INFO_COMPLETE_AFTER(const fileinfo_t *fi, filesize_t off)
 {
@@ -167,21 +218,29 @@ FILE_INFO_COMPLETE_AFTER(const fileinfo_t *fi, filesize_t off)
 }
 
 typedef void (*fi_listener_t) (gnet_fi_t);
-typedef void (*fi_src_listener_t) (gnet_fi_t, gnet_src_t);
+typedef void (*src_listener_t) (gnet_src_t);
 
 typedef enum {
 	EV_FI_ADDED = 0,       /**< fi_listener */
 	EV_FI_REMOVED,         /**< fi_listener */
 	EV_FI_INFO_CHANGED,    /**< fi_listener */
+	EV_FI_RANGES_CHANGED,  /**< fi_listener */
 	EV_FI_STATUS_CHANGED,  /**< fi_listener */
 	EV_FI_STATUS_CHANGED_TRANSIENT, /**< fi_listener */
-	EV_FI_SRC_ADDED,       /**< fi_src_listener */
-	EV_FI_SRC_REMOVED,     /**< fi_src_listener */
-	EV_FI_RANGES_CHANGED,  /**< fi_listener */
 
 	EV_FI_EVENTS           /**< Number of events in this domain */
 } gnet_fi_ev_t;
 
+typedef enum {
+	EV_SRC_ADDED,			/**< src_listener */
+	EV_SRC_REMOVED,			/**< src_listener */
+	EV_SRC_INFO_CHANGED,	/**< src_listener */
+	EV_SRC_RANGES_CHANGED,	/**< src_listener */
+	EV_SRC_STATUS_CHANGED,	/**< src_listener */
+
+	EV_SRC_EVENTS           /**< Number of events in this domain */
+} gnet_src_ev_t;
+
 /*
  * Public interface, visible only from the bridge.
  */
@@ -191,6 +250,11 @@ typedef enum {
 void fi_add_listener(fi_listener_t, gnet_fi_ev_t, frequency_t, guint32);
 void fi_remove_listener(fi_listener_t, gnet_fi_ev_t);
 
+void src_add_listener(src_listener_t, gnet_src_ev_t, frequency_t, guint32);
+void src_remove_listener(src_listener_t, gnet_src_ev_t);
+
+struct download *src_get_download(gnet_src_t);
+
 gnet_fi_info_t *fi_get_info(gnet_fi_t);
 void fi_free_info(gnet_fi_info_t *);
 void fi_get_status(gnet_fi_t, gnet_fi_status_t *);
@@ -199,8 +263,9 @@ void fi_free_chunks(GSList *chunks);
 GSList *fi_get_ranges(gnet_fi_t);
 void fi_free_ranges(GSList *ranges);
 gchar **fi_get_aliases(gnet_fi_t fih);
-
-void fi_purge_by_handle_list(const GSList *list);
+gboolean fi_purge(gnet_fi_t fih);
+void fi_pause(gnet_fi_t fih);
+void fi_resume(gnet_fi_t fih);
 
 const gchar *file_info_readable_filename(const struct dl_file_info *fi);
 gchar *file_info_build_magnet(gnet_fi_t fih);
