@@ -1,5 +1,5 @@
 /*
- * $Id: guid.h,v 1.1 2004/08/25 05:40:39 rmanfredi Exp $
+ * $Id: guid.h 12761 2007-01-16 16:07:30Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -26,8 +26,14 @@
 #ifndef _if_core_guid_h_
 #define _if_core_guid_h_
 
-#include <glib.h>
+#include "common.h"
 
-extern gchar blank_guid[16];
+#include "lib/misc.h"		/* For GUID_RAW_SIZE */
+
+typedef struct guid {
+	guchar v[GUID_RAW_SIZE];
+} guid_t;
+
+extern const gchar blank_guid[GUID_RAW_SIZE];
 
 #endif /* _if_core_guid_h_ */
