@@ -1,5 +1,5 @@
 /*
- * $Id: net_stats.h 13632 2007-05-13 06:40:23Z cbiere $
+ * $Id: net_stats.h 14601 2007-08-20 19:26:41Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -84,6 +84,8 @@ typedef enum msg_drop_reason {
 	MSG_DROP_INFLATE_ERROR,
 	MSG_DROP_UNKNOWN_HEADER_FLAGS,
 	MSG_DROP_OWN_RESULT,
+	MSG_DROP_ANCIENT_QUERY,
+	MSG_DROP_BLANK_SERVENT_ID,
 	
 	MSG_DROP_REASON_COUNT /**< number of known reasons to drop a message */
 } msg_drop_reason_t;
