@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 12185 2006-11-02 14:28:28Z cbiere $
+ * $Id: nodes.h 13870 2007-06-18 03:27:25Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -33,6 +33,18 @@
 #include "lib/host_addr.h"
 #include "lib/vendors.h"
 
+struct node_id;
+typedef const struct node_id *node_id_t;
+
+node_id_t node_id_ref(const node_id_t node_id);
+void node_id_unref(const node_id_t node_id);
+
+const gchar *node_id_to_string(const node_id_t node_id);
+
+#define node_id_eq_func ((GCompareFunc) node_id_eq)
+gboolean node_id_eq(const node_id_t a, const node_id_t b);
+guint node_id_hash(gconstpointer node_id);
+
 /**
  * Remote node mode, as specified for GTKG/23v1.
  */
@@ -83,7 +95,7 @@ typedef struct rnode_info {
 	guint64 cpu_sys;			/**< Total kernel CPU time used, in ms */
 	/* Information sent via optional GGEP blocks */
 	guint32 ggep_du;			/**< Daily average uptime, from "DU" */
-	gchar *ggep_ua;				/**< User-Agent string (atom), from "UA" */
+	const gchar *ggep_ua;		/**< User-Agent string (atom), from "UA" */
 	host_addr_t ggep_ipv6;		/**< IPv6 address */
 } rnode_info_t;
 
@@ -184,11 +196,6 @@ typedef struct rnode_info {
 #define RNODE_O2_LEAF_GUIDE	(1 << 11)	/**< Leaf-guided querying */
 #define RNODE_O2_OOQ_CHUNK	(1 << 12)	/**< Out-of-Queue small chunk service */
 
-/**
- * Gnet node specific types.
- */
-typedef guint32 gnet_node_t;
-
 /*
  * XXX this structure should really be inlined in a node,
  * XXX to avoid definition duplication --RAM, 2004-08-21
@@ -270,23 +277,25 @@ typedef struct gnet_node_status {
 } gnet_node_status_t;
 
 typedef struct gnet_node_info {
-    gnet_node_t node_handle;    /**< Internal node handle */
+    node_id_t node_id;    	/**< Internal node ID */
+
+	gchar gnet_guid[GUID_RAW_SIZE];		/**< Seen on network (can be blank) */
 
     gchar *error_str;       /**< To sprintf() error strings with vars */
+	const gchar *vendor;	/**< Vendor information (always UTF-8) */
+
 	gint proto_major;		/**< Protocol major number */
 	gint proto_minor;		/**< Protocol minor number */
-	gchar *vendor;			/**< Vendor information (always UTF-8) */
-	gint country;			/**< Country information */
-	union vendor_code vcode;/**< Vendor code (vcode.u32 == 0 when unknown) */
-
-	host_addr_t addr;		/**< ip of the node (connected) */
-	guint16 port;			/**< port of the node (connected) */
-
+	vendor_code_t vcode;	/**< Vendor code (vcode.u32 == 0 when unknown) */
 	gboolean is_pseudo;		/**< TRUE if it's the pseudo UDP node */
 
+	host_addr_t addr;		/**< ip of the node (connected) */
 	host_addr_t gnet_addr;	/**< Advertised Gnutella address for connecting */
+
+	guint16 port;			/**< port of the node (connected) */
 	guint16 gnet_port;		/**< Advertised Gnutella listening port */
-	gchar gnet_guid[GUID_RAW_SIZE];		/**< Seen on network (can be blank) */
+	guint16 country;		/**< Country information */
+
 } gnet_node_info_t;
 
 /*
@@ -354,10 +363,10 @@ typedef enum {
 /*
  * Nodes callback definitions
  */
-typedef void (*node_added_listener_t) (gnet_node_t);
-typedef void (*node_removed_listener_t) (gnet_node_t);
-typedef void (*node_info_changed_listener_t) (gnet_node_t);
-typedef void (*node_flags_changed_listener_t) (gnet_node_t);
+typedef void (*node_added_listener_t) (node_id_t);
+typedef void (*node_removed_listener_t) (node_id_t);
+typedef void (*node_info_changed_listener_t) (node_id_t);
+typedef void (*node_flags_changed_listener_t) (node_id_t);
 
 #define node_add_listener(signal, callback) \
     CAT3(node_add_,signal,_listener)(callback);
@@ -385,14 +394,14 @@ void node_remove_node_flags_changed_listener(node_flags_changed_listener_t);
  */
 void node_add(const host_addr_t addr, guint16, guint32 flags);
 void node_add_by_name(const gchar *host, guint16, guint32 flags);
-void node_remove_by_handle(gnet_node_t n);
-void node_remove_nodes_by_handle(GSList *node_list);
-void node_get_status(const gnet_node_t n, gnet_node_status_t *s);
-gnet_node_info_t *node_get_info(const gnet_node_t n);
+void node_remove_by_id(const node_id_t node_id);
+void node_remove_nodes_by_id(const GSList *node_list);
+gboolean node_get_status(const node_id_t node_id, gnet_node_status_t *s);
+gnet_node_info_t *node_get_info(const node_id_t node_id);
 void node_clear_info(gnet_node_info_t *info);
 void node_free_info(gnet_node_info_t *info);
-void node_fill_flags(gnet_node_t n, gnet_node_flags_t *flags);
-void node_fill_info(const gnet_node_t n, gnet_node_info_t *info);
+gboolean node_fill_flags(const node_id_t node_id, gnet_node_flags_t *flags);
+gboolean node_fill_info(const node_id_t node_id, gnet_node_info_t *info);
 const gchar *node_flags_to_string(const gnet_node_flags_t *flags);
 const gchar *node_peermode_to_string(node_peer_t m);
 
