@@ -1,5 +1,5 @@
 /*
- * $Id: share.h 9244 2005-08-17 13:44:05Z cbiere $
+ * $Id: share.h 14132 2007-07-21 01:30:20Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -28,21 +28,6 @@
 
 #include "lib/host_addr.h"
 
-/**
- * Search query types.
- */
-typedef enum {
-    QUERY_STRING,
-    QUERY_SHA1
-} query_type_t;
-
-/**
- * Sharing callbacks.
- */
-
-typedef void (*search_request_listener_t) (
-    query_type_t, const gchar *query, const host_addr_t addr, guint16);
-
 /*
  * Public interface, visible from the bridge.
  */
@@ -53,8 +38,6 @@ void shared_dir_add(const gchar *);
 void share_scan(void);
 guint64 shared_files_scanned(void);
 guint64 shared_kbytes_scanned(void);
-void share_add_search_request_listener(search_request_listener_t l);
-void share_remove_search_request_listener(search_request_listener_t l);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_share_h */
