@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h 13870 2007-06-18 03:27:25Z cbiere $
+ * $Id: uploads.h 14533 2007-08-18 09:06:19Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -60,6 +60,7 @@ typedef struct gnet_upload_status {
 	guint reqnum;		 /**< Count of uploaded chunks */
 
 	gboolean parq_quick;
+	gboolean parq_frozen;
 	guint	parq_position;
 	guint	parq_size;
 	guint32	parq_lifetime;
@@ -110,6 +111,9 @@ typedef struct gnet_upload_info {
 #define UPLOAD_IS_SENDING(u)	\
 	((u)->status == GTA_UL_SENDING)
 
+#define UPLOAD_IS_QUEUED(u)	\
+	((u)->status == GTA_UL_QUEUED)
+
 #define UPLOAD_WAITING_FOLLOWUP(u) \
 	((u)->status == GTA_UL_WAITING || (u)->status == GTA_UL_EXPECTING)
 
@@ -128,12 +132,9 @@ struct ul_stats {
 /*
  * Uploads callback definitions
  */
-typedef void (*upload_added_listener_t) (
-    gnet_upload_t, guint32, guint32);
-typedef void (*upload_removed_listener_t) (
-    gnet_upload_t, const gchar *, guint32, guint32);
-typedef void (*upload_info_changed_listener_t) (
-    gnet_upload_t, guint32, guint32);
+typedef void (*upload_added_listener_t)(gnet_upload_t);
+typedef void (*upload_removed_listener_t)(gnet_upload_t, const gchar *);
+typedef void (*upload_info_changed_listener_t)(gnet_upload_t);
 
 #define upload_add_listener(signal, callback) \
     CAT3(upload_add_,signal,_listener)(callback);
@@ -147,19 +148,17 @@ typedef void (*upload_info_changed_listener_t) (
 
 #ifdef CORE_SOURCES
 
-gnet_upload_info_t *upload_get_info(gnet_upload_t uh);
-void upload_free_info(gnet_upload_info_t *info);
-void upload_get_status(gnet_upload_t uh, gnet_upload_status_t *si);
-void upload_kill_addr(const host_addr_t addr);
-void upload_add_upload_added_listener(upload_added_listener_t l);
-void upload_remove_upload_added_listener(upload_added_listener_t l);
-void upload_add_upload_removed_listener(upload_removed_listener_t l);
-void upload_remove_upload_removed_listener(upload_removed_listener_t l);
-void upload_add_upload_info_changed_listener(
-	upload_info_changed_listener_t l);
-void upload_remove_upload_info_changed_listener(
-	upload_info_changed_listener_t l);
-void upload_kill(gnet_upload_t upload);
+gnet_upload_info_t *upload_get_info(gnet_upload_t);
+void upload_free_info(gnet_upload_info_t *);
+void upload_get_status(gnet_upload_t, gnet_upload_status_t *);
+void upload_kill_addr(const host_addr_t);
+void upload_add_upload_added_listener(upload_added_listener_t);
+void upload_remove_upload_added_listener(upload_added_listener_t);
+void upload_add_upload_removed_listener(upload_removed_listener_t);
+void upload_remove_upload_removed_listener(upload_removed_listener_t);
+void upload_add_upload_info_changed_listener(upload_info_changed_listener_t);
+void upload_remove_upload_info_changed_listener(upload_info_changed_listener_t);
+void upload_kill(gnet_upload_t);
 void upload_stats_prune_nonexistent(void);
 
 #endif /* CORE_SOURCES */
