@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.h 13057 2007-03-11 23:04:51Z cbiere $
+ * $Id: upload_stats.h 14319 2007-08-05 02:08:51Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -40,6 +40,8 @@ void upload_stats_gui_init(void);
 void upload_stats_gui_clear_all(void);
 void upload_stats_gui_add(const struct ul_stats *s);
 void upload_stats_gui_update(const struct ul_stats *s);
+void upload_stats_gui_freeze(void);
+void upload_stats_gui_thaw(void);
 
 #endif /* GUI_SOURCES */
 #endif /* _if_ui_gtk_upload_stats_h_ */
