@@ -1,5 +1,5 @@
 /*
- * $Id: base64.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: base64.c 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,10 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: base64.c 11437 2006-08-05 00:24:20Z cbiere $")
-
-#include <glib.h>
-#include <string.h>
+RCSID("$Id: base64.c 12500 2006-11-26 10:53:21Z cbiere $")
 
 #include "base64.h"
 #include "override.h"		/* Must be the last header included */
