@@ -1,5 +1,5 @@
 /*
- * $Id: bg.c 12242 2006-11-06 20:41:55Z cbiere $
+ * $Id: bg.c 13773 2007-05-27 21:29:05Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,16 +39,21 @@
 
 #include "common.h"
 
-RCSID("$Id: bg.c 12242 2006-11-06 20:41:55Z cbiere $")
+RCSID("$Id: bg.c 13773 2007-05-27 21:29:05Z rmanfredi $")
 
 #include "bg.h"
+#include "misc.h"
 #include "tm.h"
 #include "walloc.h"
+
 #include "override.h"		/* Must be the last header included */
 
-#define BT_MAGIC		0xbacc931dU		/**< Internal bgtask magic number */
+enum bgtask_magic {
+	BGTASK_MAGIC	  = 0xbacc931dU,
+	BGTASK_DEAD_MAGIC = 0x6f5c8a03U
+};
 
-#define MAX_LIFE		150000			/**< In useconds, MUST be << 1 sec */
+#define MAX_LIFE		350000			/**< In useconds, MUST be << 1 sec */
 #define MIN_LIFE		40000			/**< Min lifetime per task, in usecs */
 #define DELTA_FACTOR	4				/**< Max variations are 400% */
 
@@ -68,7 +73,7 @@ static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
  * `stepvec' is the set of steps we have to run (normally in sequence).
  */
 struct bgtask {
-	guint magic;			/**< Magic number */
+	enum bgtask_magic magic;/**< Magic number */
 	guint32 flags;			/**< Operating flags */
 	const gchar *name;		/**< Task name */
 	gint step;				/**< Current processing step */
@@ -109,33 +114,45 @@ struct bgtask {
  */
 
 enum {
-	TASK_F_EXITED	=	0x00000001,	/**< Exited */
-	TASK_F_SIGNAL	=	0x00000002,	/**< Signal received */
-	TASK_F_RUNNING	=	0x00000004,	/**< Task is running */
-	TASK_F_ZOMBIE	=	0x00000008,	/**< Task waiting status collect */
-	TASK_F_NOTICK	=	0x00000010,	/**< Do no recompute tick info */
-	TASK_F_SLEEPING	=	0x00000020,	/**< Task is sleeping */
-	TASK_F_RUNNABLE	=	0x00000040,	/**< Task is runnable */
-	TASK_F_DAEMON	=	0x80000000	/**< Task is a daemon */
+	TASK_F_EXITED	=	1 << 0,	/**< Exited */
+	TASK_F_SIGNAL	=	1 << 1,	/**< Signal received */
+	TASK_F_RUNNING	=	1 << 2,	/**< Task is running */
+	TASK_F_ZOMBIE	=	1 << 3,	/**< Task waiting status collect */
+	TASK_F_NOTICK	=	1 << 4,	/**< Do no recompute tick info */
+	TASK_F_SLEEPING	=	1 << 5,	/**< Task is sleeping */
+	TASK_F_RUNNABLE	=	1 << 6,	/**< Task is runnable */
+	TASK_F_DAEMON	=	1 << 7	/**< Task is a daemon */
 };
 
+static inline void
+bg_task_check(const struct bgtask * const bt)
+{
+	g_assert(bt);
+	g_assert(BGTASK_MAGIC == bt->magic);
+}
+
 /*
  * Access routines to internal fields.
  */
 
-static inline struct bgtask *
-TASK(gpointer x)
+gint
+bg_task_seqno(const struct bgtask *bt)
 {
-	return x;
+	bg_task_check(bt);
+	return bt->seqno;
 }
 
-gint bg_task_seqno(gpointer h)			{ return TASK(h)->seqno; }
-gpointer bg_task_context(gpointer h)	{ return TASK(h)->ucontext; }
+gpointer
+bg_task_context(const struct bgtask *bt)
+{
+	bg_task_check(bt);
+	return bt->ucontext;
+}
 
-static GSList *runq = NULL;
-static GSList *sleepq = NULL;
-static gint runcount = 0;
-static GSList *dead_tasks = NULL;
+static gint runcount;
+static GSList *runq;
+static GSList *sleepq;
+static GSList *dead_tasks;
 
 /**
  * Add new task to the scheduler (run queue).
@@ -143,6 +160,7 @@ static GSList *dead_tasks = NULL;
 static void
 bg_sched_add(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_RUNNABLE));	/* Not already in list */
 
 	/*
@@ -160,6 +178,7 @@ bg_sched_add(struct bgtask *bt)
 static void
 bg_sched_remove(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	/*
 	 * We currently have only one run queue: we don't handle priorities.
 	 */
@@ -177,8 +196,13 @@ bg_sched_pick(void)
 	/*
 	 * All task in run queue have equal priority, pick the first.
 	 */
-
-	return runq != NULL ? TASK(runq->data) : NULL;
+	if (runq) {
+		struct bgtask *bt = runq->data;
+		bg_task_check(bt);
+		return bt;
+	} else {
+		return NULL;
+	}
 }
 
 /**
@@ -187,9 +211,10 @@ bg_sched_pick(void)
 static void
 bg_task_suspend(struct bgtask *bt)
 {
-	tm_t end;
-	gint elapsed;
+	tm_t end, delta;
+	time_delta_t elapsed;
 
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	bg_sched_add(bt);
@@ -200,8 +225,8 @@ bg_task_suspend(struct bgtask *bt)
 	 */
 
 	tm_now_exact(&end);
-	elapsed = (glong) ((end.tv_sec - bt->start.tv_sec) * 1000 * 1000 +
-		(end.tv_usec - bt->start.tv_usec));
+	tm_elapsed(&delta, &end, &bt->start);
+	elapsed = tm2us(&delta); 
 
 	/*
 	 * Compensate any clock adjustment by reusing the previous value we
@@ -232,9 +257,9 @@ bg_task_suspend(struct bgtask *bt)
 			(4 * bt->tick_cost + (elapsed / bt->ticks_used)) / 5.0;
 
 		if (common_dbg > 4)
-			printf("BGTASK \"%s\" total=%d msecs, elapsed=%d, ticks=%d, "
-				"used=%d, tick_cost=%f usecs (was %f)\n",
-				bt->name, bt->wtime, elapsed, bt->ticks, bt->ticks_used,
+			g_message("BGTASK \"%s\" total=%d msecs, elapsed=%lu, ticks=%d, "
+				"used=%d, tick_cost=%f usecs (was %f)",
+				bt->name, bt->wtime, (gulong)elapsed, bt->ticks, bt->ticks_used,
 				new_cost, bt->tick_cost);
 
 		bt->tick_cost = new_cost;
@@ -247,6 +272,7 @@ bg_task_suspend(struct bgtask *bt)
 static void
 bg_task_resume(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 
 	bg_sched_remove(bt);
@@ -261,6 +287,7 @@ bg_task_resume(struct bgtask *bt)
 static void
 bg_sched_sleep(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_SLEEPING));
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 	g_assert(runcount > 0);
@@ -277,6 +304,7 @@ bg_sched_sleep(struct bgtask *bt)
 static void
 bg_sched_wakeup(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_SLEEPING);
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 
@@ -287,7 +315,7 @@ bg_sched_wakeup(struct bgtask *bt)
 }
 
 
-static struct bgtask *current_task = NULL;
+static struct bgtask *current_task;
 
 /**
  * Switch to new task `bt'.
@@ -306,16 +334,26 @@ bg_task_switch(struct bgtask *bt)
 		bg_task_suspend(old);
 		current_task = NULL;
 	}
-
-	if (bt == NULL)
-		return old;
-
-	bg_task_resume(bt);
-	current_task = bt;
-
+	if (bt) {
+		bg_task_check(bt);
+		bg_task_resume(bt);
+		current_task = bt;
+	}
 	return old;
 }
 
+static struct bgtask *
+bg_task_alloc(void)
+{
+	static const struct bgtask zero_bt;
+	struct bgtask *bt;
+
+	bt = walloc(sizeof *bt);
+	*bt = zero_bt;
+	bt->magic = BGTASK_MAGIC;
+	return bt;
+}
+
 /**
  * Create a new background task.
  * The `steps' array is cloned, so it can be built on the caller's stack.
@@ -331,13 +369,14 @@ bg_task_switch(struct bgtask *bt)
  *
  * @returns an opaque handle.
  */
-gpointer
-bg_task_create(const gchar *name,		/**< Task name (for tracing) */
-	bgstep_cb_t *steps, gint stepcnt,	/**< Work to perform (copied) */
-	gpointer ucontext,					/**< User context */
-	bgclean_cb_t ucontext_free,			/**< Free routine for context */
-	bgdone_cb_t done_cb,				/**< Notification callback when done */
-	gpointer done_arg)					/**< Callback argument */
+struct bgtask *
+bg_task_create(const gchar *name,	/**< Task name (for tracing) */
+	const bgstep_cb_t *steps,		/**< Work to perform (copied) */
+	gint stepcnt,					/**< Number of steps */
+	gpointer ucontext,				/**< User context */
+	bgclean_cb_t ucontext_free,		/**< Free routine for context */
+	bgdone_cb_t done_cb,			/**< Notification callback when done */
+	gpointer done_arg)				/**< Callback argument */
 {
 	struct bgtask *bt;
 	gint stepsize;
@@ -345,9 +384,7 @@ bg_task_create(const gchar *name,		/**< Task name (for tracing) */
 	g_assert(stepcnt > 0);
 	g_assert(steps);
 
-	bt = walloc0(sizeof(*bt));
-
-	bt->magic = BT_MAGIC;
+	bt = bg_task_alloc();
 	bt->name = name;
 	bt->ucontext = ucontext;
 	bt->uctx_free = ucontext_free;
@@ -384,16 +421,17 @@ bg_task_create(const gchar *name,		/**< Task name (for tracing) */
  *
  * Use bg_daemon_enqueue() to enqueue more work to the daemon.
  */
-gpointer
+struct bgtask *
 bg_daemon_create(
-	const gchar *name,				  /**< Task name (for tracing) */
-	bgstep_cb_t *steps, gint stepcnt, /**< Work to perform (copied) */
-	gpointer ucontext,				  /**< User context */
-	bgclean_cb_t ucontext_free,		  /**< Free routine for context */
-	bgstart_cb_t start_cb,			  /**< Starting working on an item */
-	bgend_cb_t end_cb,				  /**< Done working on an item */
-	bgclean_cb_t item_free,			  /**< Free routine for work queue items */
-	bgnotify_cb_t notify)			  /**< Start/Stop notify (optional) */
+	const gchar *name,			/**< Task name (for tracing) */
+	const bgstep_cb_t *steps,	/**< Work to perform (copied) */
+	gint stepcnt,				/**< Number of steps */
+	gpointer ucontext,			/**< User context */
+	bgclean_cb_t ucontext_free,	/**< Free routine for context */
+	bgstart_cb_t start_cb,		/**< Starting working on an item */
+	bgend_cb_t end_cb,			/**< Done working on an item */
+	bgclean_cb_t item_free,		/**< Free routine for work queue items */
+	bgnotify_cb_t notify)		/**< Start/Stop notify (optional) */
 {
 	struct bgtask *bt;
 	gint stepsize;
@@ -401,9 +439,7 @@ bg_daemon_create(
 	g_assert(stepcnt > 0);
 	g_assert(steps);
 
-	bt = walloc0(sizeof(*bt));
-
-	bt->magic = BT_MAGIC;
+	bt = bg_task_alloc();
 	bt->flags |= TASK_F_DAEMON;
 	bt->name = name;
 	bt->ucontext = ucontext;
@@ -429,19 +465,16 @@ bg_daemon_create(
  * If task was sleeping, wake it up.
  */
 void
-bg_daemon_enqueue(gpointer h, gpointer item)
+bg_daemon_enqueue(struct bgtask *bt, gpointer item)
 {
-	struct bgtask *bt = TASK(h);
-
-	g_assert(h);
-	g_assert(bt->magic == BT_MAGIC);
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_DAEMON);
 
 	bt->wq = g_slist_append(bt->wq, item);
 
 	if (bt->flags & TASK_F_SLEEPING) {
 		if (common_dbg > 1)
-			printf("BGTASK waking up daemon \"%s\" task\n", bt->name);
+			g_message("BGTASK waking up daemon \"%s\" task", bt->name);
 
 		bg_sched_wakeup(bt);
 		if (bt->notify)
@@ -459,6 +492,9 @@ bg_task_free(struct bgtask *bt)
 	gint stepsize;
 	gint count;
 
+	g_assert(bt);
+	g_assert(BGTASK_DEAD_MAGIC == bt->magic);
+	
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 	g_assert(bt->flags & TASK_F_EXITED);
 
@@ -476,7 +512,8 @@ bg_task_free(struct bgtask *bt)
 		g_warning("freed %d pending item%s for daemon \"%s\" task",
 			count, count == 1 ? "" : "s", bt->name);
 
-	wfree(bt, sizeof(*bt));
+	bt->magic = 0;
+	wfree(bt, sizeof *bt);
 }
 
 /**
@@ -487,6 +524,7 @@ bg_task_terminate(struct bgtask *bt)
 {
 	bgstatus_t status;
 
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_EXITED));
 
 	/*
@@ -502,7 +540,7 @@ bg_task_terminate(struct bgtask *bt)
 	 */
 
 	if (common_dbg > 1)
-		printf("BGTASK terminating \"%s\"%s, ran %d msecs\n",
+		g_message("BGTASK terminating \"%s\"%s, ran %d msecs",
 			bt->name, (bt->flags & TASK_F_DAEMON) ? " daemon" : "", bt->wtime);
 
 	g_assert(!(bt->flags & TASK_F_RUNNING));
@@ -557,7 +595,7 @@ bg_task_terminate(struct bgtask *bt)
 	 */
 
 	(*bt->uctx_free)(bt->ucontext);
-	bt->magic = 0;							/* Prevent further uses! */
+	bt->magic = BGTASK_DEAD_MAGIC;	/* Prevent further uses! */
 
 	/*
 	 * Do not free the task structure immediately, in case the calling
@@ -575,12 +613,9 @@ bg_task_terminate(struct bgtask *bt)
  * We exit immediately, not returning to the user code.
  */
 void
-bg_task_exit(gpointer h, gint code)
+bg_task_exit(struct bgtask *bt, gint code)
 {
-	struct bgtask *bt = TASK(h);
-
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	bt->exitcode = code;
@@ -599,6 +634,7 @@ bg_task_exit(gpointer h, gint code)
 static void
 bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 {
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	bt->flags |= TASK_F_SIGNAL;
@@ -616,14 +652,11 @@ bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
  * @returns -1 if the task could not be signalled.
  */
 static gint
-bg_task_kill(gpointer h, bgsig_t sig)
+bg_task_kill(struct bgtask *bt, bgsig_t sig)
 {
-	struct bgtask *bt = TASK(h);
 	bgsig_cb_t sighandler;
 
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
-
+	bg_task_check(bt);
 	if (bt->flags & TASK_F_EXITED)		/* Already exited */
 		return -1;
 
@@ -658,7 +691,7 @@ bg_task_kill(gpointer h, bgsig_t sig)
 	 */
 
 	if (!(bt->flags & TASK_F_RUNNING) || (bt->flags & TASK_F_SIGNAL)) {
-		bt->signals = g_slist_append(bt->signals, (gpointer) sig);
+		bt->signals = g_slist_append(bt->signals, GUINT_TO_POINTER(sig));
 		return 1;
 	}
 
@@ -678,14 +711,11 @@ bg_task_kill(gpointer h, bgsig_t sig)
  * @returns previously installed signal handler.
  */
 bgsig_cb_t
-bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
+bg_task_signal(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 {
-	struct bgtask *bt = TASK(h);
 	bgsig_cb_t oldhandler;
 
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
-
+	bg_task_check(bt);
 	oldhandler = bt->sigh[sig];
 	bt->sigh[sig] = handler;
 
@@ -698,6 +728,7 @@ bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
 static void
 bg_task_deliver_signals(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	/*
@@ -727,14 +758,11 @@ bg_task_deliver_signals(struct bgtask *bt)
  * Cancel a given task.
  */
 void
-bg_task_cancel(gpointer h)
+bg_task_cancel(struct bgtask *bt)
 {
-	struct bgtask *bt = TASK(h);
 	struct bgtask *old = NULL;
 
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
-
+	bg_task_check(bt);
 	if (bt->flags & TASK_F_EXITED)		/* Already exited */
 		return;
 
@@ -756,7 +784,7 @@ bg_task_cancel(gpointer h)
 		}
 
 		g_assert(bt->flags & TASK_F_RUNNING);
-		bg_task_kill(h, BG_SIG_TERM);		/* Let task cleanup nicely */
+		bg_task_kill(bt, BG_SIG_TERM);		/* Let task cleanup nicely */
 
 		/*
 		 * We only come back if the signal did not kill the task, i.e.
@@ -769,7 +797,7 @@ bg_task_cancel(gpointer h)
 		}
 	}
 
-	bg_task_kill(h, BG_SIG_KILL);			/* Kill task immediately */
+	bg_task_kill(bt, BG_SIG_KILL);			/* Kill task immediately */
 
 	g_assert(bt->flags & TASK_F_EXITED);	/* Task is now terminated */
 }
@@ -779,12 +807,9 @@ bg_task_cancel(gpointer h)
  * all its ticks and it matters for the computation of the cost per tick.
  */
 void
-bg_task_ticks_used(gpointer h, gint used)
+bg_task_ticks_used(struct bgtask *bt, gint used)
 {
-	struct bgtask *bt = TASK(h);
-
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 	g_assert(used >= 0);
 	g_assert(used <= bt->ticks);
@@ -804,7 +829,7 @@ bg_reclaim_dead(void)
 	GSList *sl;
 
 	for (sl = dead_tasks; sl; sl = g_slist_next(sl)) {
-		bg_task_free(TASK(sl->data));
+		bg_task_free(sl->data);
 	}
 	g_slist_free(dead_tasks);
 	dead_tasks = NULL;
@@ -818,6 +843,8 @@ bg_task_ended(struct bgtask *bt)
 {
 	gpointer item;
 
+	bg_task_check(bt);
+
 	/*
 	 * Non-daemon task: reroute to bg_task_terminate().
 	 */
@@ -836,7 +863,7 @@ bg_task_ended(struct bgtask *bt)
 	item = bt->wq->data;
 
 	if (common_dbg > 2)
-		printf("BGTASK daemon \"%s\" done with item 0x%lx\n",
+		g_message("BGTASK daemon \"%s\" done with item 0x%lx",
 			bt->name, (gulong) item);
 
 	(*bt->end_cb)(bt, bt->ucontext, item);
@@ -858,7 +885,7 @@ bg_task_ended(struct bgtask *bt)
 
 	if (bt->wq == NULL) {
 		if (common_dbg > 1)
-			printf("BGTASK daemon \"%s\" going back to sleep\n", bt->name);
+			g_message("BGTASK daemon \"%s\" going back to sleep", bt->name);
 
 		bg_sched_sleep(bt);
 		if (bt->notify)
@@ -932,10 +959,11 @@ bg_sched_timer(gboolean overloaded)
 				}
 			}
 			g_assert(ticks > 0);
-		} else
+		} else {
 			ticks = 1;
-
-		bt->ticks = bt->ticks_used = ticks;
+		}
+		bt->ticks = ticks;
+		bt->ticks_used = ticks;
 
 		/*
 		 * Switch to the selected task.
@@ -957,7 +985,7 @@ bg_sched_timer(gboolean overloaded)
 			 */
 
 			if (common_dbg > 1)
-				printf("BGTASK back from setjmp() for \"%s\"\n", bt->name);
+				g_message("BGTASK back from setjmp() for \"%s\"", bt->name);
 
 			bt->flags |= TASK_F_NOTICK;
 			bg_task_switch(NULL);
@@ -970,7 +998,7 @@ bg_sched_timer(gboolean overloaded)
 		 */
 
 		if (common_dbg > 4)
-			printf("BGTASK \"%s\" running step #%d.%d with %d tick%s\n",
+			g_message("BGTASK \"%s\" running step #%d.%d with %d tick%s",
 				bt->name, bt->step, bt->seqno, ticks, ticks == 1 ? "" : "s");
 
 		bg_task_deliver_signals(bt);	/* Send any queued signal */
@@ -988,7 +1016,7 @@ bg_sched_timer(gboolean overloaded)
 			item = bt->wq->data;
 
 			if (common_dbg > 2)
-				printf("BGTASK daemon \"%s\" starting with item 0x%lx\n",
+				g_message("BGTASK daemon \"%s\" starting with item 0x%lx",
 					bt->name, (gulong) item);
 
 			(*bt->start_cb)(bt, bt->ucontext, item);
@@ -1002,8 +1030,8 @@ bg_sched_timer(gboolean overloaded)
 		remain -= bt->elapsed;
 
 		if (common_dbg > 4)
-			printf("BGTASK \"%s\" step #%d.%d ran %d tick%s "
-				"in %d usecs [ret=%d]\n",
+			g_message("BGTASK \"%s\" step #%d.%d ran %d tick%s "
+				"in %d usecs [ret=%d]",
 				bt->name, bt->step, bt->seqno,
 				bt->ticks_used, bt->ticks_used == 1 ? "" : "s",
 				bt->elapsed, ret);
@@ -1039,35 +1067,47 @@ bg_sched_timer(gboolean overloaded)
 		bg_reclaim_dead();			/* Free dead tasks */
 }
 
+static guint
+bg_task_terminate_all(GSList **ptr)
+{
+	guint count;
+
+	count = 0;
+	if (*ptr) {
+		GSList *iter, *copy;
+
+		copy = g_slist_copy(*ptr);
+		for (iter = copy; NULL != iter; iter = g_slist_next(iter)) {
+			count++;
+			bg_task_terminate(iter->data);
+		}
+		g_slist_free(copy);
+		copy = NULL;
+		g_slist_free(*ptr);
+		*ptr = NULL;
+	}
+	return count;
+}
+
 /**
  * Called at shutdown time.
  */
 void
 bg_close(void)
 {
-	GSList *l;
-	GSList *c;
-	gint count;
+	guint count;
 
-	for (count = 0, c = g_slist_copy(runq), l = c; l; l = l->next) {
-		count++;
-		bg_task_terminate(l->data);
+	count = bg_task_terminate_all(&runq);
+	if (count > 0) {
+		g_warning("terminated %u running task%s",
+			count, count == 1 ? "" : "s");
 	}
-	g_slist_free(runq);
-	g_slist_free(c);
 
-	if (count)
-		g_warning("terminated %d running task%s", count, count == 1 ? "" : "s");
-
-	for (count = 0, c = g_slist_copy(sleepq), l = c; l; l = l->next) {
-		count++;
-		bg_task_terminate(l->data);
+	count = bg_task_terminate_all(&sleepq);
+	if (count > 0) {
+		g_warning("terminated %d daemon task%s",
+			count, count == 1 ? "" : "s");
 	}
-	g_slist_free(sleepq);
-	g_slist_free(c);
-
-	if (count)
-		g_warning("terminated %d daemon task%s", count, count == 1 ? "" : "s");
 
 	bg_reclaim_dead();				/* Free dead tasks */
 }
