@@ -1,5 +1,5 @@
 /*
- * $Id: cobs.c,v 1.13 2005/09/10 10:10:36 daichik Exp $
+ * $Id: cobs.c 11846 2006-08-31 22:00:15Z cbiere $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -39,7 +39,7 @@
 
 #include "common.h"
 
-RCSID("$Id: cobs.c,v 1.13 2005/09/10 10:10:36 daichik Exp $");
+RCSID("$Id: cobs.c 11846 2006-08-31 22:00:15Z cbiere $")
 
 #include "cobs.h"
 #include "walloc.h"
@@ -267,9 +267,9 @@ cobs_decode(gchar *buf, size_t len, size_t *retlen, gboolean inplace)
 	if (cobs_decode_into(buf, len, out, len, retlen))
 		return out;
 
-	if (!inplace)
+	if (!inplace) {
 		G_FREE_NULL(out);
-
+	}
 	return NULL;
 }
 
