@@ -1,5 +1,5 @@
 /*
- * $Id: eval.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: eval.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _eval_h_
 #define _eval_h_
 
-#include <glib.h>
+#include "common.h"
 
 /*
  * Public interface
@@ -45,7 +45,7 @@
 void eval_init(void);
 void eval_close(void);
 
-gchar *eval_subst(const gchar *str);
+const gchar *eval_subst(const gchar *str);
 
 #endif	/* _eval_h_ */
 
