@@ -1,5 +1,5 @@
 /*
- * $Id: event.c 12270 2006-11-09 06:07:54Z cbiere $
+ * $Id: event.c 13809 2007-06-11 22:06:53Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -27,15 +27,13 @@
  * @ingroup lib
  * @file
  *
- * Functions that should be in GTK+ 1.2 or GTK+ 2.x but are not.
- *
  * @author Richard Eckart
  * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: event.c 12270 2006-11-09 06:07:54Z cbiere $")
+RCSID("$Id: event.c 13809 2007-06-11 22:06:53Z cbiere $")
 
 #include "event.h"
 #include "misc.h"
@@ -63,7 +61,7 @@ subscriber_destroy(struct subscriber *s)
 	wfree(s, sizeof *s);
 }
 
-inline struct event *
+struct event *
 event_new(const gchar *name)
 {
     struct event *evt;
