@@ -1,5 +1,5 @@
 /*
- * $Id: getdate.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: getdate.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,8 +36,7 @@
 #ifndef _getdate_h_
 #define _getdate_h_
 
-#include <time.h>
-#include <sys/time.h>
+#include "common.h"
 
 time_t date2time(const char *p, time_t now);
 
