@@ -1,5 +1,5 @@
 /*
- * $Id: getline.h 12500 2006-11-26 10:53:21Z cbiere $
+ * $Id: getline.h 14336 2007-08-06 15:12:47Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -68,13 +68,14 @@ typedef struct getline {
  */
 
 getline_t *getline_make(size_t maxsize);
-void getline_free(getline_t *o);
-void getline_reset(getline_t *o);
-getline_result_t getline_read(getline_t *o,
+void getline_free(getline_t *);
+void getline_reset(getline_t *);
+getline_result_t getline_read(getline_t *,
 					const gchar *data, size_t len, size_t *used);
-const gchar *getline_str(getline_t *o);
-size_t getline_length(getline_t *o);
+const gchar *getline_str(getline_t *);
+size_t getline_length(getline_t *);
 void getline_copy(getline_t *source, getline_t *dest);
+void getline_set_maxlen(getline_t *, size_t);
 
 #endif	/* _getline_h_ */
 
