@@ -1,5 +1,5 @@
 /*
- * $Id: host_addr.c 12229 2006-11-05 15:23:10Z cbiere $
+ * $Id: host_addr.c 14954 2007-09-28 19:23:13Z bpringlemeir $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2005, Christian Biere
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: host_addr.c 12229 2006-11-05 15:23:10Z cbiere $")
+RCSID("$Id: host_addr.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
 
 #ifdef I_NET_IF
 #include <net/if.h>		/* For IFF_* flags */
@@ -50,6 +50,7 @@ RCSID("$Id: host_addr.c 12229 2006-11-05 15:23:10Z cbiere $")
 #include <netdb.h>				/* For gethostbyname() */
 #endif /* I_NETDB */
 
+#include "atoms.h"				/* For binary_hash */
 #include "host_addr.h"
 #include "misc.h"
 #include "endian.h"
@@ -71,7 +72,7 @@ host_addr_family(const host_addr_t ha)
 	case NET_TYPE_IPV4:
 		return AF_INET;
 	case NET_TYPE_IPV6:
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 		return AF_INET6;
 #else
 		return -1;
@@ -85,15 +86,18 @@ host_addr_family(const host_addr_t ha)
 }
 
 /**
- * Checks for RFC1918 private addresses.
+ * Checks for RFC1918 private addresses but also IPv6 link-local and site-local
+ * addresses.
  *
  * @return TRUE if is a private address.
  */
 gboolean
-is_private_addr(const host_addr_t ha)
+is_private_addr(const host_addr_t addr)
 {
-	if (NET_TYPE_IPV4 == host_addr_net(ha)) {
-		guint32 ip = host_addr_ipv4(ha);
+	host_addr_t addr_ipv4;
+
+	if (host_addr_convert(addr, &addr_ipv4, NET_TYPE_IPV4)) {
+		guint32 ip = host_addr_ipv4(addr_ipv4);
 
 		/* 10.0.0.0 -- (10/8 prefix) */
 		if ((ip & 0xff000000) == 0xa000000)
@@ -110,11 +114,21 @@ is_private_addr(const host_addr_t ha)
 		/* 192.168.0.0 -- (192.168/16 prefix) */
 		if ((ip & 0xffff0000) == 0xc0a80000)
 			return TRUE;
-		
-	} else if (NET_TYPE_IPV6 == host_addr_net(ha)) {
-		return host_addr_equal(ha, ipv6_loopback);
-	}
+	} else {
 
+		switch (host_addr_net(addr)) {
+		case NET_TYPE_IPV4:
+			g_assert_not_reached();
+		case NET_TYPE_IPV6:
+			return	host_addr_equal(addr, ipv6_loopback) ||
+					host_addr_matches(addr, ipv6_link_local, 10) ||
+					host_addr_matches(addr, ipv6_site_local, 10);
+		case NET_TYPE_LOCAL:
+			return TRUE;
+		case NET_TYPE_NONE:
+			break;
+		}
+	}
 	return FALSE;
 }
 
@@ -141,11 +155,11 @@ ipv4_addr_is_routable(guint32 ip)
 	return TRUE;
 }
 
-static inline guint32
+static inline gboolean
 host_addr_is_6to4(const host_addr_t ha)
 {
 	return NET_TYPE_IPV6 == host_addr_net(ha) &&
-		htons(0x2002) == ha.addr.u16[0];
+		0x2002 == peek_be16(&ha.addr.ipv6[0]);
 }
 
 static inline guint32
@@ -159,7 +173,7 @@ host_addr_6to4_to_ipv4(const host_addr_t from, host_addr_t *to)
 {
 	if (host_addr_is_6to4(from)) {
 		if (to) {
-			*to = host_addr_get_ipv4(peek_be32(&from.addr.ipv6[2]));
+			*to = host_addr_peek_ipv4(&from.addr.ipv6[2]);
 		}
 		return TRUE;
 	} else {
@@ -228,7 +242,7 @@ host_addr_is_routable(const host_addr_t addr)
 
 	case NET_TYPE_LOCAL:
 	case NET_TYPE_NONE:
-		break;
+		return FALSE;
 	}
 
 	g_assert_not_reached();
@@ -297,8 +311,7 @@ host_addr_convert(const host_addr_t from, host_addr_t *to,
 		switch (from.net) {
 		case NET_TYPE_IPV6:
 			if (host_addr_can_convert(from, NET_TYPE_IPV4)) {
-				to->net = NET_TYPE_IPV4;
-				to->addr.ipv4 = peek_be32(&from.addr.ipv6[12]);
+				*to = host_addr_peek_ipv4(&from.addr.ipv6[12]);
 				return TRUE;
 			}
 			break;
@@ -315,7 +328,7 @@ host_addr_convert(const host_addr_t from, host_addr_t *to,
 			memset(to->addr.ipv6, 0, 10);
 			to->addr.ipv6[10] = 0xff;
 			to->addr.ipv6[11] = 0xff;
-			poke_be32(&to->addr.ipv6[12], from.addr.ipv4);
+			poke_be32(&to->addr.ipv6[12], host_addr_ipv4(from));
 			return TRUE;
 		case NET_TYPE_NONE:
 			break;
@@ -447,6 +460,22 @@ host_addr_port_to_string2(const host_addr_t ha, guint16 port)
 	return buf;
 }
 
+const char *
+host_port_to_string(const char *hostname, host_addr_t addr, guint16 port)
+{
+	static char buf[MAX_HOSTLEN + 32];
+
+	if (hostname) {
+		char port_buf[UINT32_DEC_BUFLEN];
+
+		uint32_to_string_buf(port, port_buf, sizeof port_buf);
+		concat_strings(buf, sizeof buf, hostname, ":", port_buf, (void *) 0);
+	} else {
+		host_addr_port_to_string_buf(addr, port, buf, sizeof buf);
+	}
+	return buf;
+}
+
 /**
  * Parses IPv4 and IPv6 addresses. The latter requires IPv6 support to be
  * enabled.
@@ -474,7 +503,7 @@ string_to_host_addr(const char *s, const gchar **endptr, host_addr_t *addr_ptr)
 		guint8 ipv6[16];
 		if (parse_ipv6_addr(s, ipv6, endptr)) {
 			if (addr_ptr) {
-				*addr_ptr = host_addr_get_ipv6(ipv6);
+				*addr_ptr = host_addr_peek_ipv6(ipv6);
 			}
 			return TRUE;
 		}
@@ -505,6 +534,7 @@ gboolean
 string_to_host_or_addr(const char *s, const gchar **endptr, host_addr_t *ha)
 {
 	const gchar *ep;
+	size_t len;
 	host_addr_t addr;
 
 	if ('[' == s[0]) {
@@ -513,7 +543,7 @@ string_to_host_or_addr(const char *s, const gchar **endptr, host_addr_t *ha)
 		if (parse_ipv6_addr(&s[1], ipv6, &ep) && ']' == *ep) {
 
 			if (ha) {
-				*ha = host_addr_get_ipv6(ipv6);
+				*ha = host_addr_peek_ipv6(ipv6);
 			}
 			if (endptr)
 				*endptr = ++ep;
@@ -539,7 +569,8 @@ string_to_host_or_addr(const char *s, const gchar **endptr, host_addr_t *ha)
 	if (endptr)
 		*endptr = ep;
 
-	return s != ep ? TRUE : FALSE;
+	len = ep - s;
+	return len > 0 && len <= MAX_HOSTLEN;
 }
 
 gboolean
@@ -574,7 +605,7 @@ string_to_host_addr_port(const gchar *str, const gchar **endptr,
 	return ret;
 }
 
-void
+static void
 gethostbyname_error(const gchar *host)
 {
 #if defined(HAS_HSTRERROR)
@@ -605,21 +636,16 @@ socket_addr_set(socket_addr_t *sa_ptr, const host_addr_t addr, guint16 port)
 			static const struct sockaddr_in zero_sin;
 
 			sa_ptr->inet4 = zero_sin;
-			/* Note: The next line is a cheap(?) hack to be used until
-			 *       HAS_SIN_LEN is initialized by Configure. sin_len if
-			 *       available.
-			 */
-			sa_ptr->len = sizeof sa_ptr->inet4;
-#ifdef HAS_SIN_LEN
+#ifdef HAS_SOCKADDR_IN_SIN_LEN
 			sa_ptr->inet4.sin_len = sizeof sa_ptr->inet4;
-#endif /* HAS_SIN_LEN */
+#endif /* HAS_SOCKADDR_IN_SIN_LEN */
 			sa_ptr->inet4.sin_family = AF_INET;
 			sa_ptr->inet4.sin_port = htons(port);
 			sa_ptr->inet4.sin_addr.s_addr = htonl(host_addr_ipv4(addr));
 		}
 		return sizeof sa_ptr->inet4;
 	case NET_TYPE_IPV6:
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 		if (sa_ptr) {
 			static const struct sockaddr_in6 zero_sin6;
 
@@ -632,7 +658,7 @@ socket_addr_set(socket_addr_t *sa_ptr, const host_addr_t addr, guint16 port)
 			memcpy(sa_ptr->inet6.sin6_addr.s6_addr, addr.addr.ipv6, 16);
 		}
 		return sizeof sa_ptr->inet6;
-#endif	/* USE_IPV6 */
+#endif	/* HAS_IPV6 */
 	case NET_TYPE_LOCAL:
 	case NET_TYPE_NONE:
 		if (sa_ptr) {
@@ -652,7 +678,7 @@ socket_addr_init(socket_addr_t *sa_ptr, enum net_type net)
 	case NET_TYPE_IPV4:
 		return socket_addr_set(sa_ptr, ipv4_unspecified, 0);
 	case NET_TYPE_IPV6:
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 		return socket_addr_set(sa_ptr, ipv6_unspecified, 0);
 #endif
 	case NET_TYPE_LOCAL:
@@ -686,7 +712,7 @@ host_addr_to_name(host_addr_t addr)
 		static gchar host[1025];
 		gint error;
 
-		error = getnameinfo(socket_addr_get_sockaddr(&sa),
+		error = getnameinfo(socket_addr_get_const_sockaddr(&sa),
 					socket_addr_get_len(&sa), host, sizeof host, NULL, 0, 0);
 		if (error) {
 			gchar buf[HOST_ADDR_BUFLEN];
@@ -710,11 +736,11 @@ host_addr_to_name(host_addr_t addr)
 			len = sizeof sa.inet4.sin_addr;
 			break;
 		case NET_TYPE_IPV6:
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 			ptr = cast_to_gchar_ptr(&sa.inet6.sin6_addr);
 			len = sizeof sa.inet6.sin6_addr;
 			break;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 		case NET_TYPE_LOCAL:
 		case NET_TYPE_NONE:
 			return NULL;
@@ -769,24 +795,24 @@ resolve_hostname(const gchar *host, enum net_type net)
 		switch (ai->ai_family) {
 		case PF_INET:
 			if (ai->ai_addrlen >= 4) {
-				const struct sockaddr_in *sin;
+				const struct sockaddr_in *sin4;
 
-				sin = cast_to_gconstpointer(ai->ai_addr);
-				addr = host_addr_get_ipv4(ntohl(sin->sin_addr.s_addr));
+				sin4 = cast_to_gconstpointer(ai->ai_addr);
+				addr = host_addr_peek_ipv4(&sin4->sin_addr.s_addr);
 			}
 			break;
 
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 		case PF_INET6:
 			if (ai->ai_addrlen >= 16) {
 				const struct sockaddr_in6 *sin6;
 
 				sin6 = cast_to_gconstpointer(ai->ai_addr);
-				addr = host_addr_get_ipv6(cast_to_gconstpointer(
+				addr = host_addr_peek_ipv6(cast_to_gconstpointer(
 							sin6->sin6_addr.s6_addr));
 			}
 			break;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 		}
 
 		if (is_host_addr(addr) && !g_hash_table_lookup(ht, &addr)) {
@@ -834,7 +860,7 @@ resolve_hostname(const gchar *host, enum net_type net)
 		}
 		break;
 		
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 	case AF_INET6:
 		if (16 != he->h_length) {
 			g_warning("host_to_addr: Wrong length of IPv6 address (\"%s\")",
@@ -842,7 +868,7 @@ resolve_hostname(const gchar *host, enum net_type net)
 			return NULL;
 		}
 		break;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 		
 	default:
 		return NULL;
@@ -855,15 +881,15 @@ resolve_hostname(const gchar *host, enum net_type net)
 
 		switch (he->h_addrtype) {
 		case AF_INET:
-			addr = host_addr_get_ipv4(peek_be32(he->h_addr_list[i]));
+			addr = host_addr_peek_ipv4(&he->h_addr_list[i]);
 			break;
 
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 		case AF_INET6:
-			addr = host_addr_get_ipv6(
+			addr = host_addr_peek_ipv6(
 						cast_to_gconstpointer(he->h_addr_list[i]));
 			break;
-#endif /* !USE_IPV6 */
+#endif /* !HAS_IPV6 */
 		default:
 			g_assert_not_reached();
 		}
@@ -999,7 +1025,7 @@ wfree_host_addr(gpointer key, gpointer unused_data)
  *			of the machine.
  */
 GSList *
-host_addr_get_interface_addrs(void)
+host_addr_get_interface_addrs(const enum net_type net)
 #if defined(HAS_GETIFADDRS)
 {
 	struct ifaddrs *ifa0, *ifa;
@@ -1024,22 +1050,25 @@ host_addr_get_interface_addrs(void)
 			continue;
 
 		if (AF_INET == ifa->ifa_addr->sa_family) {
-            const struct sockaddr_in *sin;
+            const struct sockaddr_in *sin4;
 			
-			sin = cast_to_gconstpointer(ifa->ifa_addr);
-			addr = host_addr_get_ipv4(ntohl(sin->sin_addr.s_addr));
-#ifdef USE_IPV6
+			sin4 = cast_to_gconstpointer(ifa->ifa_addr);
+			addr = host_addr_peek_ipv4(&sin4->sin_addr.s_addr);
+#ifdef HAS_IPV6
 		} else if (AF_INET6 == ifa->ifa_addr->sa_family) {
             const struct sockaddr_in6 *sin6;
 
 			sin6 = cast_to_gconstpointer(ifa->ifa_addr);
-			addr = host_addr_get_ipv6(sin6->sin6_addr.s6_addr);
-#endif /* USE_IPV6 */
+			addr = host_addr_peek_ipv6(sin6->sin6_addr.s6_addr);
+#endif /* HAS_IPV6 */
 		} else {
 			addr = zero_host_addr;
 		}
 
-		if (is_host_addr(addr)) {
+		if (
+			(NET_TYPE_NONE == net || host_addr_net(addr) == net) &&
+			is_host_addr(addr)
+		) {
 			sl_addrs = g_slist_prepend(sl_addrs, wcopy(&addr, sizeof addr));
 		}
 	}
@@ -1049,6 +1078,7 @@ host_addr_get_interface_addrs(void)
 }
 #else	/* !HAS_GETIFADDRS */
 {
+	(void) net;
 	return NULL;
 }
 #endif	/* HAS_GETIFADDRS */
@@ -1104,10 +1134,11 @@ packed_host_addr_unpack(const struct packed_host_addr paddr)
 {
 	switch (paddr.net) {
 	case NET_TYPE_IPV4:
-		return host_addr_get_ipv4(peek_be32(paddr.addr));
+		return host_addr_peek_ipv4(paddr.addr);
 	case NET_TYPE_IPV6:
-		return host_addr_get_ipv6(paddr.addr);
+		return host_addr_peek_ipv6(paddr.addr);
 	case NET_TYPE_LOCAL:
+		return local_host_addr;
 	case NET_TYPE_NONE:
 		return zero_host_addr;
 	}
@@ -1148,15 +1179,19 @@ packed_host_unpack(const struct packed_host phost,
 	switch (phost.ha.net) {
 	case NET_TYPE_IPV4:
 		if (addr_ptr) {
-			*addr_ptr = host_addr_get_ipv4(peek_be32(phost.ha.addr));
+			*addr_ptr = host_addr_peek_ipv4(phost.ha.addr);
 		}
 		return TRUE;
 	case NET_TYPE_IPV6:
 		if (addr_ptr) {
-			*addr_ptr = host_addr_get_ipv6(phost.ha.addr);
+			*addr_ptr = host_addr_peek_ipv6(phost.ha.addr);
 		}
 		return TRUE;
 	case NET_TYPE_LOCAL:
+		if (addr_ptr) {
+			*addr_ptr = local_host_addr;
+		}
+		return TRUE;
 	case NET_TYPE_NONE:
 		if (addr_ptr) {
 			*addr_ptr = zero_host_addr;
@@ -1167,11 +1202,13 @@ packed_host_unpack(const struct packed_host phost,
 	return FALSE;
 }
 
-
-guint
-packed_host_size(const struct packed_host phost)
+/**
+ * Significant size of a packed host (serialization size), given by address.
+ */
+static inline guint
+packed_host_size_ptr(const struct packed_host *phost)
 {
-	switch (phost.ha.net) {
+	switch (phost->ha.net) {
 	case NET_TYPE_IPV4:	 return 1 + 4 + 2;
 	case NET_TYPE_IPV6:  return 1 + 16 + 2;
 	case NET_TYPE_LOCAL: return 1 + 2;
@@ -1181,4 +1218,76 @@ packed_host_size(const struct packed_host phost)
 	return 0;
 }
 
+
+/**
+ * Significant size of a packed host (serialization size).
+ */
+guint
+packed_host_size(const struct packed_host phost)
+{
+	return packed_host_size_ptr(&phost);
+}
+
+/***
+ *** The following routines are meant to be used for hash tables indexed
+ *** by packed_host structures, which are variable-sized entities.
+ ***/
+
+/**
+ * Hash a packed host buffer (variable-sized).
+ */
+guint
+packed_host_hash_func(gconstpointer key)
+{
+	const struct packed_host *p = key;
+	return binary_hash(key, packed_host_size_ptr(p));
+}
+
+/**
+ * Compare two packed host buffers (variable-sized).
+ */
+gboolean
+packed_host_eq_func(gconstpointer p, gconstpointer q)
+{
+	const struct packed_host *a = p, *b = q;
+	guint asize = packed_host_size_ptr(a);
+	guint bsize = packed_host_size_ptr(b);
+
+	if (asize != bsize)
+		return FALSE;
+
+	return 0 == memcmp(a, b, asize);
+}
+
+/**
+ * Allocate a packed_host key.
+ */
+gpointer
+walloc_packed_host(const host_addr_t addr, guint16 port)
+{
+	struct packed_host packed;
+	gpointer key;
+	guint len;
+
+	packed = host_pack(addr, port);
+	len = packed_host_size_ptr(&packed);
+	key = walloc(len);
+	memcpy(key, &packed, len);
+
+	return key;
+}
+
+/**
+ * Release packed_host key.
+ */
+void
+wfree_packed_host(gpointer key, gpointer unused_data)
+{
+	const struct packed_host *p = key;
+
+	(void) unused_data;
+
+	wfree(key, packed_host_size_ptr(p));
+}
+
 /* vi: set ts=4 sw=4 cindent: */
