@@ -1,5 +1,5 @@
 /*
- * $Id: inputevt.c 13371 2007-04-19 23:30:54Z cbiere $
+ * $Id: inputevt.c 14430 2007-08-13 00:32:24Z cbiere $
  *
  * Copyright (c) 2002, ko (ko-@wanadoo.fr)
  * Copyright (c) 2005, Christian Biere
@@ -44,7 +44,7 @@
 
 #include "common.h"
 
-RCSID("$Id: inputevt.c 13371 2007-04-19 23:30:54Z cbiere $")
+RCSID("$Id: inputevt.c 14430 2007-08-13 00:32:24Z cbiere $")
 
 #ifdef HAS_KQUEUE
 #include <sys/event.h>
@@ -55,8 +55,8 @@ RCSID("$Id: inputevt.c 13371 2007-04-19 23:30:54Z cbiere $")
  * should be used to access this struct member.
  */
 #if defined(HAS_KEVENT_INT_UDATA)
-#define KEVENT_UDATA_TO_PTR(x) cast_uintptr_to_ptr(x)
-#define PTR_TO_KEVENT_UDATA(x) cast_ptr_to_uintptr(x)
+#define KEVENT_UDATA_TO_PTR(x) ulong_to_pointer(x)
+#define PTR_TO_KEVENT_UDATA(x) pointer_to_ulong(x)
 #else
 #define KEVENT_UDATA_TO_PTR(x) (x)
 #define PTR_TO_KEVENT_UDATA(x) (x)
@@ -189,7 +189,7 @@ static inline gint
 get_poll_event_fd(gpointer p)
 {
 	struct kevent *ev = p;
-	return GPOINTER_TO_INT(KEVENT_UDATA_TO_PTR(ev->udata));
+	return pointer_to_uint(KEVENT_UDATA_TO_PTR(ev->udata));
 }
 
 static inline inputevt_cond_t 
@@ -1014,6 +1014,8 @@ inputevt_init(void)
 	} else {
 		GIOChannel *ch;
 
+		set_close_on_exec(poll_ctx->fd);	/* Just in case */
+
 #ifdef HAS_DEV_POLL
 		g_main_set_poll_func(poll_func);
 #endif	/* HAS_DEV_POLL */
