@@ -1,5 +1,5 @@
 /*
- * $Id: iso3166.h 7505 2004-12-09 04:28:08Z cbiere $
+ * $Id: iso3166.h 13870 2007-06-18 03:27:25Z cbiere $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -26,13 +26,15 @@
 #ifndef _lib_iso3166_h_
 #define _lib_iso3166_h_
 
-#include <glib.h>
+#include "common.h"
 
-gint iso3166_encode_cc(const gchar *cc);
+#define ISO3166_INVALID ((guint16)-1)
+
+guint16 iso3166_encode_cc(const gchar *cc);
 void iso3166_init(void);
 void iso3166_close(void);
-const gchar *iso3166_country_name(gint code);
-const gchar *iso3166_country_cc(gint code);
+const gchar *iso3166_country_name(guint16 code);
+const gchar *iso3166_country_cc(guint16 code);
 
 #endif /* _lib_iso3166_h_ */
 /* vi: set ts=4 sw=4 cindent: */
