@@ -1,5 +1,5 @@
 /*
- * $Id: magnet.c 13966 2007-06-24 10:54:27Z cbiere $
+ * $Id: magnet.c 14954 2007-09-28 19:23:13Z bpringlemeir $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -37,7 +37,7 @@
 
 #include "common.h"
 
-RCSID("$Id: magnet.c 13966 2007-06-24 10:54:27Z cbiere $")
+RCSID("$Id: magnet.c 14954 2007-09-28 19:23:13Z bpringlemeir $")
 
 #include "lib/atoms.h"
 #include "lib/glib-missing.h"
@@ -60,12 +60,13 @@ RCSID("$Id: magnet.c 13966 2007-06-24 10:54:27Z cbiere $")
 
 enum magnet_key {
 	MAGNET_KEY_NONE,
-	MAGNET_KEY_DISPLAY_NAME,	/* Display Name */
-	MAGNET_KEY_KEYWORD_TOPIC,	/* Keyword Topic */
-	MAGNET_KEY_EXACT_LENGTH,	/* eXact file Length */
-	MAGNET_KEY_EXACT_SOURCE,	/* eXact Source */
-	MAGNET_KEY_EXACT_TOPIC,		/* eXact Topic */
-	MAGNET_KEY_PARQ_ID,			/* PARQ ID */
+	MAGNET_KEY_DISPLAY_NAME,		/* Display Name */
+	MAGNET_KEY_KEYWORD_TOPIC,		/* Keyword Topic */
+	MAGNET_KEY_EXACT_LENGTH,		/* eXact file Length */
+	MAGNET_KEY_ALTERNATE_SOURCE,	/* Alternate Source */
+	MAGNET_KEY_EXACT_SOURCE,		/* eXact Source */
+	MAGNET_KEY_EXACT_TOPIC,			/* eXact Topic */
+	MAGNET_KEY_PARQ_ID,				/* PARQ ID */
 	
 	NUM_MAGNET_KEYS
 };
@@ -75,6 +76,7 @@ static const struct {
 	const enum magnet_key id;
 } magnet_keys[] = {
 	{ "",			MAGNET_KEY_NONE },
+	{ "as",			MAGNET_KEY_ALTERNATE_SOURCE },
 	{ "dn",			MAGNET_KEY_DISPLAY_NAME },
 	{ "kt",			MAGNET_KEY_KEYWORD_TOPIC },
 	{ "xl",			MAGNET_KEY_EXACT_LENGTH },
@@ -270,13 +272,28 @@ static void
 magnet_handle_key(struct magnet_resource *res,
 	const gchar *name, const gchar *value)
 {
+	char *to_free = NULL;
+
 	g_return_if_fail(res);
 	g_return_if_fail(name);
 	g_return_if_fail(value);
 	
 	if (!utf8_is_valid_string(value)) {
+		const char *encoding;
+		char *result;
+			
 		g_message("MAGNET URI key \"%s\" is not UTF-8 encoded", name);
-		return;
+
+		if (MAGNET_KEY_DISPLAY_NAME != magnet_key_get(name))
+			return;
+
+		result = unknown_to_utf8(value, &encoding);
+		if (result != value) {
+			to_free = result;
+		}
+		value = result;
+		g_message("Assuming MAGNET URI key \"%s\" is %s encoded",
+			name, encoding);
 	}
 
 	switch (magnet_key_get(name)) {
@@ -286,6 +303,7 @@ magnet_handle_key(struct magnet_resource *res,
 		}
 		break;
 
+	case MAGNET_KEY_ALTERNATE_SOURCE:
 	case MAGNET_KEY_EXACT_SOURCE:
 		{
 			struct magnet_source *ms;
@@ -337,6 +355,8 @@ magnet_handle_key(struct magnet_resource *res,
 	case NUM_MAGNET_KEYS:
 		g_assert_not_reached();
 	}
+
+	G_FREE_NULL(to_free);
 }
 
 struct magnet_resource * 
@@ -479,7 +499,7 @@ magnet_source_new(void)
 	return wcopy(&zero_source, sizeof zero_source);
 }
 
-void
+static void
 magnet_add_source(struct magnet_resource *res, struct magnet_source *s)
 {
 	g_return_if_fail(res);
@@ -646,7 +666,7 @@ magnet_append_item(GString **gs_ptr, gboolean escape_value,
 	*gs_ptr = gs;
 }
 
-gchar *
+static gchar *
 magnet_source_to_string(struct magnet_source *s)
 {
 	gchar *url;
