@@ -1,5 +1,5 @@
 /*
- * $Id: malloc.h 11606 2006-08-18 21:03:16Z cbiere $
+ * $Id: malloc.h 13945 2007-06-24 00:42:32Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -82,6 +82,9 @@
 #define g_memdup(p,s)	((p) ? memdup_track((p), (s), _WHERE_, __LINE__) : 0)
 #define g_strfreev(v)	strfreev_track((v), _WHERE_, __LINE__)
 
+/* FIXME: This only correct if xmlFree() is equivalent to free(). */
+#define xmlFree(o)		free_track(o, _WHERE_, __LINE__)
+
 #undef g_new
 #undef g_new0
 #undef g_renew
@@ -101,7 +104,7 @@
 #define g_hash_table_new(x,y)	hashtable_new_track(x, y, _WHERE_, __LINE__)
 #define g_hash_table_destroy(x)	hashtable_destroy_track(x, _WHERE_, __LINE__)
 
-#define hash_list_new_(h,c)		hash_list_new_track((h),(c),_WHERE_, __LINE__)
+#define hash_list_new(h,c)		hash_list_new_track((h),(c),_WHERE_, __LINE__)
 #define hash_list_free(h)		hash_list_free_track((h), _WHERE_, __LINE__)
 
 #define g_slist_alloc()			track_slist_alloc(_WHERE_, __LINE__)
@@ -174,35 +177,41 @@
  * Likewise, use MEMTRACK() to track some random memory buffer known to have
  * been allocated by a routine and not via any of the trapped calls.
  */
-#define STRTRACK(s)		string_record((s), _WHERE_, __LINE__)
+#define STRTRACK(o)		string_record((o), _WHERE_, __LINE__)
 #define MEMTRACK(o,s)	malloc_record((o), (s), _WHERE_, __LINE__)
+#define GSLISTTRACK(o)	gslist_record((o), _WHERE_, __LINE__)
+#define GLISTTRACK(o)	glist_record((o), _WHERE_, __LINE__)
 
 #else	/* !TRACK_MALLOC || MALLOC_SOURCE */
 
-#define STRTRACK(s)		s
+#define STRTRACK(o)		o
 #define MEMTRACK(o,s)	o
+#define GSLISTTRACK(o)	o
+#define GLISTTRACK(o)	o
 
 #endif	/* TRACK_MALLOC && !MALLOC_SOURCE */
 
 #if defined(TRACK_MALLOC) || defined(MALLOC_SOURCE)
 
-gpointer string_record(const gchar *s, gchar *file, gint line);
-gpointer malloc_record(gpointer o, guint32 s, gchar *file, gint line);
+gchar *string_record(const gchar *s, gchar *file, gint line);
+gpointer malloc_record(gconstpointer o, size_t size, gchar *file, gint line);
+GSList *gslist_record(const GSList *, gchar *file, gint line);
+GList *glist_record(const GList *, gchar *file, gint line);
 
-gpointer malloc_track(guint32 s, gchar *file, gint line);
-gpointer malloc0_track(guint32 s, gchar *file, gint line);
+gpointer malloc_track(size_t size, gchar *file, gint line);
+gpointer malloc0_track(size_t size, gchar *file, gint line);
 void free_track(gpointer o, gchar *file, gint line);
 void strfreev_track(gchar **v, gchar *file, gint line);
-gpointer realloc_track(gpointer o, guint32 s, gchar *file, gint line);
+gpointer realloc_track(gpointer o, size_t size, gchar *file, gint line);
 gchar *strdup_track(const gchar *s, gchar *file, gint line);
-gchar *strndup_track(const gchar *s, gint n, gchar *file, gint line);
-gpointer memdup_track(gconstpointer p, guint size, gchar *file, gint line);
+gchar *strndup_track(const gchar *s, size_t n, gchar *file, gint line);
+gpointer memdup_track(gconstpointer p, size_t size, gchar *file, gint line);
 gchar *strjoinv_track(const gchar *s, gchar **vec, gchar *file, gint line);
 gchar *strconcat_track(gchar *file, gint line, const gchar *s, ...);
 gchar *strdup_printf_track(gchar *file, gint line, const gchar *fmt, ...)
 	G_GNUC_PRINTF(3, 4);
 gchar **strsplit_track(
-	const gchar *s, const gchar *d, gint m, gchar *file, gint line);
+	const gchar *s, const gchar *d, size_t m, gchar *file, gint line);
 
 GHashTable *hashtable_new_track(
 	GHashFunc h, GCompareFunc y, gchar *file, gint line);
@@ -210,7 +219,7 @@ void hashtable_destroy_track(GHashTable *h, gchar *file, gint line);
 
 hash_list_t *hash_list_new_track(
 	GHashFunc hash_func, GEqualFunc eq_func, gchar *file, gint line);
-void hash_list_free_track(hash_list_t *h, gchar *file, gint line);
+void hash_list_free_track(hash_list_t **hl_ptr, gchar *file, gint line);
 
 GSList *track_slist_alloc(gchar *file, gint line);
 GSList *track_slist_append(GSList *l, gpointer data, gchar *file, gint line);
@@ -243,7 +252,7 @@ GList *track_list_insert_after(
 GList *track_list_delete_link(GList *l, GList *lk, gchar *file, gint line);
 
 GString *string_new_track(const gchar *p, gchar *file, gint line);
-GString *string_sized_new_track(guint size, gchar *file, gint line);
+GString *string_sized_new_track(size_t size, gchar *file, gint line);
 GString *string_append_track(
 	GString *s, const gchar *p, gchar *file, gint line);
 GString *string_append_c_track(
@@ -277,7 +286,7 @@ void malloc_close(void);
 #if defined(TRACK_MALLOC) || defined(TRACK_ZALLOC)
 
 gpointer leak_init(void);
-void leak_add(gpointer o, guint32 size, gchar *file, gint line);
+void leak_add(gpointer o, size_t size, gchar *file, gint line);
 void leak_dump(gpointer o);
 void leak_close(gpointer o);
 
