@@ -1,5 +1,5 @@
 /*
- * $Id: misc.c 14036 2007-07-04 20:45:48Z cbiere $
+ * $Id: misc.c 15049 2007-12-07 18:48:45Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: misc.c 14036 2007-07-04 20:45:48Z cbiere $")
+RCSID("$Id: misc.c 15049 2007-12-07 18:48:45Z cbiere $")
 
 #include "atoms.h"
 #include "base32.h"
@@ -48,6 +48,7 @@ RCSID("$Id: misc.c 14036 2007-07-04 20:45:48Z cbiere $")
 #include "tm.h"
 #include "walloc.h"
 #include "utf8.h"
+#include "lib/misc.h"
 
 #include "override.h"			/* Must be the last header included */
 
@@ -64,7 +65,7 @@ RCSID("$Id: misc.c 14036 2007-07-04 20:45:48Z cbiere $")
 static const char hex_alphabet[] = "0123456789ABCDEF";
 const char hex_alphabet_lower[] = "0123456789abcdef";
 
-#ifndef HAS_STRLCPY
+#if !defined(HAS_STRLCPY) && !defined(USE_GLIB2)
 size_t
 strlcpy(gchar *dst, const gchar *src, size_t dst_size)
 {
@@ -90,7 +91,7 @@ strlcpy(gchar *dst, const gchar *src, size_t dst_size)
 }
 #endif /* HAS_STRLCPY */
 
-#ifndef HAS_STRLCAT
+#if !defined(HAS_STRLCAT) && !defined(USE_GLIB2)
 size_t
 strlcat(gchar *dst, const gchar *src, size_t dst_size)
 {
@@ -281,7 +282,7 @@ file_exists(const gchar *pathname)
   	struct stat st;
 
     g_assert(pathname);
-    return 0 == do_stat(pathname, &st);
+    return 0 == stat(pathname, &st);
 }
 
 /**
@@ -293,7 +294,7 @@ file_does_not_exist(const gchar *pathname)
   	struct stat st;
 
     g_assert(pathname);
-	return 0 != do_stat(pathname, &st) && ENOENT == do_errno;
+	return stat(pathname, &st) && ENOENT == errno;
 }
 
 /**
@@ -812,9 +813,7 @@ is_directory(const gchar *pathname)
 	struct stat st;
 
 	g_assert(pathname);
-	if (0 != do_stat(pathname, &st))
-		return FALSE;
-	return S_ISDIR(st.st_mode);
+	return 0 == stat(pathname, &st) && S_ISDIR(st.st_mode);
 }
 
 /**
@@ -826,9 +825,7 @@ is_regular(const gchar *pathname)
 	struct stat st;
 
 	g_assert(pathname);
-	if (0 != do_stat(pathname, &st))
-		return FALSE;
-	return S_ISREG(st.st_mode);
+	return 0 == stat(pathname, &st) && S_ISREG(st.st_mode);
 }
 
 /**
@@ -1072,6 +1069,17 @@ compact_kb_size(guint32 size, gboolean metric)
 	return b;
 }
 
+const gchar *
+nice_size(guint64 size, gboolean metric)
+{
+	static gchar buf[256];
+	gchar bytes[UINT64_DEC_BUFLEN];
+
+	uint64_to_string_buf(size, bytes, sizeof bytes);
+	gm_snprintf(buf, sizeof buf,
+		_("%s (%s bytes)"), short_size(size, metric), bytes);
+	return buf;
+}
 
 gchar *
 compact_value(gchar *buf, size_t size, guint64 v, gboolean metric)
@@ -1128,7 +1136,7 @@ compact_rate(guint64 rate, gboolean metric)
 	return buf;
 }
 
-size_t
+static size_t
 short_rate_to_string_buf(guint64 rate, gboolean metric, gchar *dst, size_t size)
 {
 	short_value(dst, size, rate, metric);
@@ -1358,7 +1366,7 @@ hex2int(guchar c)
  * @param c the decimal ASCII character to convert.
  * @return "0..9" for valid decimal ASCII characters.
  */
-gint
+static gint
 dec2int(guchar c)
 {
 	gint ret;
@@ -1377,7 +1385,7 @@ dec2int(guchar c)
  * @param c the decimal ASCII character to convert.
  * @return "0..36" for valid decimal ASCII characters.
  */
-gint
+static gint
 alnum2int(guchar c)
 {
 	gint ret;
@@ -1571,8 +1579,8 @@ base32_to_guid(const gchar *base32)
  *             truncated.
  * @return dst.
  */
-gchar *
-sha1_to_base32_buf(const struct sha1 *sha1, gchar *dst, size_t size)
+char *
+sha1_to_base32_buf(const struct sha1 *sha1, char *dst, size_t size)
 {
 	g_assert(sha1);
 	if (size > 0) {
@@ -1610,9 +1618,9 @@ sha1_to_string(const struct sha1 sha1)
  * @return The SHA-1 converted to an URN string.
  */
 size_t
-sha1_to_urn_string_buf(const struct sha1 *sha1, gchar *dst, size_t size)
+sha1_to_urn_string_buf(const struct sha1 *sha1, char *dst, size_t size)
 {
-	static const gchar prefix[] = "urn:sha1:";
+	static const char prefix[] = "urn:sha1:";
 	size_t n;
 
 	g_assert(sha1);
@@ -1627,26 +1635,26 @@ sha1_to_urn_string_buf(const struct sha1 *sha1, gchar *dst, size_t size)
 	return CONST_STRLEN(prefix) + SHA1_BASE32_SIZE + 1;
 }
 
-const gchar *
+const char *
 sha1_to_urn_string(const struct sha1 *sha1)
 {
-	static gchar buf[CONST_STRLEN("urn:sha1:") + SHA1_BASE32_SIZE + 1];
+	static char buf[CONST_STRLEN("urn:sha1:") + SHA1_BASE32_SIZE + 1];
 
 	g_assert(sha1);
 	sha1_to_urn_string_buf(sha1, buf, sizeof buf);
 	return buf;
 }
 
-const gchar *
+const char *
 bitprint_to_urn_string(const struct sha1 *sha1, const struct tth *tth)
 {
 	g_assert(sha1);
 
 	if (tth) {
-		static const gchar prefix[] = "urn:bitprint:";
-		static gchar buf[CONST_STRLEN(prefix) + BITPRINT_BASE32_SIZE + 1];
-		const gchar * const end = &buf[sizeof buf];
-		gchar *p = buf;
+		static const char prefix[] = "urn:bitprint:";
+		static char buf[CONST_STRLEN(prefix) + BITPRINT_BASE32_SIZE + 1];
+		const char * const end = &buf[sizeof buf];
+		char *p = buf;
 
 		memcpy(p, prefix, CONST_STRLEN(prefix));
 		p += CONST_STRLEN(prefix);
@@ -1662,7 +1670,7 @@ bitprint_to_urn_string(const struct sha1 *sha1, const struct tth *tth)
 		
 		return buf;
 	} else {
-		static gchar buf[CONST_STRLEN("urn:sha1:") + SHA1_BASE32_SIZE + 1];
+		static char buf[CONST_STRLEN("urn:sha1:") + SHA1_BASE32_SIZE + 1];
 
 		sha1_to_urn_string_buf(sha1, buf, sizeof buf);
 		return buf;
@@ -1678,7 +1686,7 @@ bitprint_to_urn_string(const struct sha1 *sha1, const struct tth *tth)
  *			validly base32 encoded SHA1.
  */
 const struct sha1 *
-base32_sha1(const gchar *base32)
+base32_sha1(const char *base32)
 {
 	static struct sha1 sha1;
 	size_t len;
@@ -1693,10 +1701,10 @@ base32_sha1(const gchar *base32)
  *
  * @return pointer to static data.
  */
-const gchar *
+const char *
 tth_base32(const struct tth *tth)
 {
-	static gchar buf[TTH_BASE32_SIZE + 1];
+	static char buf[TTH_BASE32_SIZE + 1];
 
 	g_assert(tth);
 	base32_encode(buf, sizeof buf, tth->data, sizeof tth->data);
@@ -1713,7 +1721,7 @@ tth_base32(const struct tth *tth)
  *			validly base32 encoded TTH.
  */
 const struct tth *
-base32_tth(const gchar *base32)
+base32_tth(const char *base32)
 {
 	static struct tth tth;
 	size_t len;
@@ -1723,6 +1731,79 @@ base32_tth(const gchar *base32)
 	return TTH_RAW_SIZE == len ? &tth : NULL;
 }
 
+/**
+ * Convert binary TTH into a base32 string.
+ *
+ * @param dst The destination buffer for the string.
+ * @param size The size of "dst" in bytes; should be larger than
+ *             TTH_BASE32_SIZE, otherwise the resulting string will be
+ *             truncated.
+ * @return dst.
+ */
+char *
+tth_to_base32_buf(const struct tth *tth, char *dst, size_t size)
+{
+	g_assert(tth);
+	if (size > 0) {
+		base32_encode(dst, size, tth->data, sizeof tth->data);
+		dst[size - 1] = '\0';
+	}
+	return dst;
+}
+
+/**
+ * Convert binary TTH into a urn:ttroot:<base32> string.
+ *
+ * @param tth A binary TTH.
+ * @return The TTH converted to an URN string.
+ */
+size_t
+tth_to_urn_string_buf(const struct tth *tth, char *dst, size_t size)
+{
+	static const char prefix[] = "urn:ttroot:";
+	size_t n;
+
+	g_assert(tth);
+
+	n = MIN(size, CONST_STRLEN(prefix));
+	memcpy(dst, prefix, n);
+	size -= n;
+	if (size > 0) {
+		n = MIN(size, (TTH_BASE32_SIZE + 1));
+		tth_to_base32_buf(tth, &dst[CONST_STRLEN(prefix)], n);
+	}
+	return CONST_STRLEN(prefix) + TTH_BASE32_SIZE + 1;
+}
+
+const char *
+tth_to_urn_string(const struct tth *tth)
+{
+	static char buf[CONST_STRLEN("urn:ttroot:") + TTH_BASE32_SIZE + 1];
+
+	g_assert(tth);
+	tth_to_urn_string_buf(tth, buf, sizeof buf);
+	return buf;
+}
+
+/**
+ * Convert time to ISO 8601 date plus time, e.g. "2002-06-09 14:54:42Z".
+ *
+ * @return The length of the created string.
+ */
+size_t
+timestamp_utc_to_string_buf(time_t date, gchar *dst, size_t size)
+{
+	const struct tm *tm = localtime(&date);
+	size_t len;
+
+	g_assert(size > 0);
+	tm = gmtime(&date);
+	len = strftime(dst, size, "%Y-%m-%d %H:%M:%SZ", tm);
+	dst[len] = '\0';		/* Be really sure */
+
+	return len;
+}
+
 /**
  * Convert time to an ISO 8601 timestamp, e.g. "2002-06-09T14:54:42Z".
  *
@@ -1732,13 +1813,7 @@ const gchar *
 timestamp_utc_to_string(time_t date)
 {
 	static gchar buf[80];
-	const struct tm *tm;
-	size_t len;
-
-	tm = gmtime(&date);
-	len = strftime(buf, sizeof buf, "%Y-%m-%dT%H:%M:%SZ", tm);
-	buf[len] = '\0';		/* Be really sure */
-
+	timestamp_utc_to_string_buf(date, buf, sizeof buf);
 	return buf;
 }
 
@@ -2150,7 +2225,7 @@ random_init(void)
 
 	sha1_feed_string(&ctx, __DATE__);
 	sha1_feed_string(&ctx, __TIME__);
-	sha1_feed_string(&ctx, "$Id: misc.c 14036 2007-07-04 20:45:48Z cbiere $");
+	sha1_feed_string(&ctx, "$Id: misc.c 15049 2007-12-07 18:48:45Z cbiere $");
 
 #if GLIB_CHECK_VERSION(2,6,0)
 	/*
@@ -2701,6 +2776,9 @@ unique_filename(const gchar *path, const gchar *name, const gchar *ext,
 	if (pathname)
 		goto finish;
 
+	if (!is_directory(path))
+		return NULL;
+
 	/*
 	 * Looks like we need to make the filename more unique.  Append .00, then
 	 * .01, etc... until .99.
@@ -3001,63 +3079,6 @@ string_to_ip_and_mask(const gchar *str, guint32 *ip, guint32 *netmask)
 	return TRUE;
 }
 
-/***
- *** System call wrapping with errno remapping.
- ***/
-
-gint do_errno;
-
-/**
- * Wrapper for the stat() system call.
- */
-gint
-do_stat(const gchar *path, struct stat *buf)
-{
-	gint ret;
-
-	/*
-	 * On my system, since I upgraded to libc6 2.3.2, I have system calls
-	 * that fail with errno = 0.  I assume this is a multi-threading issue,
-	 * since my kernel is SMP and gcc 3.3 requires a libpthread.  Or whatever,
-	 * but it did not occur before with the same kernel and a previous libc6
-	 * along with gcc 2.95.
-	 *
-	 * So... Assume that if stat() returns -1 and errno is 0, then it
-	 * really means ENOENT.
-	 *
-	 *		--RAM, 27/10/2003
-	 */
-
-	ret = stat(path, buf);
-	do_errno = errno;
-
-	if (-1 == ret && 0 == do_errno) {
-		g_warning("stat(\"%s\") returned -1 with errno = 0, assuming ENOENT",
-			path);
-		do_errno = errno = ENOENT;
-	}
-
-	/*
-	 * Perform some remapping.  Stats through NFS may return EXDEV?
-	 */
-
-	switch (do_errno) {
-	case EXDEV:
-		g_warning("stat(\"%s\") failed with weird errno = %d (%s), "
-			"assuming ENOENT", path, do_errno, g_strerror(do_errno));
-		do_errno = errno = ENOENT;
-		break;
-	default:
-		break;
-	}
-
-	if (-1 == ret && ENOENT != do_errno)
-		g_warning("stat(\"%s\") returned -1 with errno = %d (%s)",
-			path, do_errno, g_strerror(do_errno));
-
-	return ret;
-}
-
 /**
  * Create new pathname from the concatenation of the dirname and the basename
  * of the file. A directory separator is insert, unless "dir" already ends
@@ -3244,15 +3265,12 @@ filepath_directory(const gchar *pathname)
 gboolean
 filepath_exists(const gchar *dir, const gchar *file)
 {
-	gchar *path;
 	struct stat buf;
-	gboolean exists = TRUE;
+	gboolean exists;
+	gchar *path;
 
 	path = make_pathname(dir, file);
-
-	if (-1 == do_stat(path, &buf))
-		exists = FALSE;
-
+	exists = 0 == stat(path, &buf);
 	G_FREE_NULL(path);
 
 	return exists;
@@ -4116,6 +4134,29 @@ close_file_descriptors(const int first_fd)
 	}
 }
 
+/*
+ * Ensures that fd 0, 1 and 2 are opened.
+ *
+ * @return 0 on success, -1 on failure.
+ */
+int
+reserve_standard_file_descriptors(void)
+{
+	int fd;
+
+	/*
+	 * POSIX guarantees that open() and dup() return the lowest unassigned file
+	 * descriptor. Check this but don't rely on it.
+	 */
+	for (fd = 0; fd < 3; fd++) {
+		if (-1 != fcntl(fd, F_GETFL))
+			continue;
+		if (open("/dev/null", O_RDWR, 0) != fd)
+			return -1;
+	}
+	return 0; 
+}
+
 /**
  * Equivalent to strstr() for raw memory without NUL-termination.
  *
