@@ -1,5 +1,5 @@
 /*
- * $Id: misc.h 13970 2007-06-24 20:17:48Z cbiere $
+ * $Id: misc.h 15055 2007-12-09 01:31:13Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -372,6 +372,7 @@ const gchar *timestamp_rfc822_to_string2(time_t date);
 const gchar *timestamp_rfc1123_to_string(time_t date);
 
 size_t timestamp_to_string_buf(time_t date, gchar *dst, size_t size);
+size_t timestamp_utc_to_string_buf(time_t date, gchar *dst, size_t size);
 size_t time_locale_to_string_buf(time_t date, gchar *dst, size_t size);
 
 short_string_t timestamp_get_string(time_t date);
@@ -455,6 +456,7 @@ const gchar *short_rate(guint64 rate, gboolean metric);
 const gchar *compact_size(guint64 size, gboolean metric);
 const gchar *compact_rate(guint64 rate, gboolean metric);
 const gchar *compact_kb_size(guint32 size, gboolean metric);
+const gchar *nice_size(guint64 size, gboolean metric);
 gchar *short_value(gchar *buf, size_t size, guint64 v, gboolean metric);
 gchar *compact_value(gchar *buf, size_t size, guint64 v, gboolean metric);
 
@@ -464,14 +466,17 @@ short_string_t short_rate_get_string(guint64 rate, gboolean metric);
  * SHA1<->base32 string conversion
  */
 typedef struct sha1 {
-	gchar data[SHA1_RAW_SIZE];
+	char data[SHA1_RAW_SIZE];
 } sha1_t;
 
-const gchar *sha1_to_string(const struct sha1 sha1);
-const gchar *sha1_to_urn_string(const struct sha1 *sha1);
-gchar *sha1_to_base32_buf(const struct sha1 *sha1, gchar *dst, size_t size);
-const gchar *sha1_base32(const struct sha1 *sha1);
-const struct sha1 *base32_sha1(const gchar *base32);
+#define SHA1_URN_LENGTH	(CONST_STRLEN("urn:sha1:") + SHA1_BASE32_SIZE)
+
+const char *sha1_to_string(const struct sha1 sha1);
+const char *sha1_to_urn_string(const struct sha1 *);
+size_t sha1_to_urn_string_buf(const struct sha1 *, char *dst, size_t size);
+char *sha1_to_base32_buf(const struct sha1 *, char *dst, size_t size);
+const char *sha1_base32(const struct sha1 *);
+const struct sha1 *base32_sha1(const char *base32);
 
 static inline int
 sha1_cmp(const struct sha1 *a, const struct sha1 *b)
@@ -483,14 +488,19 @@ sha1_cmp(const struct sha1 *a, const struct sha1 *b)
  * TTH <-> base32 string conversion
  */
 typedef struct tth {
-	gchar data[TTH_RAW_SIZE];
+	char data[TTH_RAW_SIZE];
 } tth_t;
 
-const gchar *tth_base32(const struct tth *tth);
-const struct tth *base32_tth(const gchar *base32);
+#define TTH_URN_LENGTH	(CONST_STRLEN("urn:ttroot:") + TTH_BASE32_SIZE)
+
+const char *tth_base32(const struct tth *);
+const struct tth *base32_tth(const char *base32);
+const char *tth_to_urn_string(const struct tth *);
+size_t tth_to_urn_string_buf(const struct tth *, char *dst, size_t size);
+char *tth_to_base32_buf(const struct tth *, char *dst, size_t size);
 
 
-const gchar *bitprint_to_urn_string(const struct sha1 *, const struct tth *);
+const char *bitprint_to_urn_string(const struct sha1 *, const struct tth *);
 
 /*
  * GUID<->hex string conversion
@@ -608,6 +618,7 @@ guint32 html_decode_entity(const gchar *src, const gchar **endptr);
 gint canonize_path(gchar *dst, const gchar *path);
 guint compat_max_fd(void);
 void close_file_descriptors(const int first_fd);
+int reserve_standard_file_descriptors(void);
 gboolean compat_is_superuser(void);
 int compat_daemonize(const char *directory);
 void set_close_on_exec(gint fd);
@@ -756,10 +767,6 @@ round_size(size_t align, size_t n)
 
 struct stat;
 
-extern gint do_errno;
-
-gint do_stat(const gchar *path, struct stat *buf);
-
 static inline gboolean
 is_temporary_error(gint error)
 {
