@@ -1,5 +1,5 @@
 /*
- * $Id: tiger.h 11145 2006-06-22 19:32:48Z cbiere $
+ * $Id: tiger.h 13285 2007-04-13 01:03:20Z cbiere $
  *
  * Copyright (c) 2003, Jeroen Asselman
  *
@@ -36,10 +36,10 @@
 #ifndef _tiger_h_
 #define _tiger_h_
 
-#include <glib.h>
+#include "common.h"
 
 void tiger_check(void);
-void tiger(gconstpointer data, guint64 length, guchar hash[24]);
+void tiger(gconstpointer data, guint64 length, gchar hash[24]);
 
 #endif /* _tiger_h_ */
 /* vi: set ts=4 sw=4 cindent: */
