@@ -1,5 +1,5 @@
 /*
- * $Id: url.c,v 1.23 2005/10/20 07:17:12 cbiere Exp $
+ * $Id: url.c 13958 2007-06-24 04:31:11Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,19 +35,16 @@
 
 #include "common.h"
 
-RCSID("$Id: url.c,v 1.23 2005/10/20 07:17:12 cbiere Exp $");
+RCSID("$Id: url.c 13958 2007-06-24 04:31:11Z cbiere $")
 
+#include "glib-missing.h"
 #include "host_addr.h"
 #include "url.h"
 #include "misc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
-extern guint32 lib_debug;
-
 #define ESCAPE_CHAR		'%'
-#define TRANSPARENT_CHAR(x,m) \
-	((x) >= 32 && (x) < 128 && (is_transparent[(x)-32] & (m)))
 
 /**
  * - Reserved chars: ";", "/", "?", ":", "@", "=" and "&"
@@ -56,19 +53,37 @@ extern guint32 lib_debug;
  *
  * - Bit 0 encodes regular transparent set (pathnames, '/' is transparent).
  * - Bit 1 encodes regular transparent set minus '+' (query string).
+ * - Bit 2 encodes the set for fixing an incomplete escaping.
+ * - Bit 3 encodes the set for use in a shell.
  */
 static const guint8 is_transparent[96] = {
-/*  0 1 2 3 4 5 6 7 8 9 a b c d e f */	/* 0123456789abcdef -            */
-    0,3,0,0,3,0,0,3,3,3,3,1,3,3,3,3,	/*  !"#$%&'()*+,-./ -  32 -> 47  */
-    3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,0,	/* 0123456789:;<=>? -  48 -> 63  */
-    0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,	/* @ABCDEFGHIJKLMNO -  64 -> 79  */
-    3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,3,	/* PQRSTUVWXYZ[\]^_ -  80 -> 95  */
-    0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,	/* `abcdefghijklmno -  96 -> 111 */
-    3,3,3,3,3,3,3,3,3,3,3,0,0,0,0,0,	/* pqrstuvwxyz{|}~  - 112 -> 127 */
+/*  0   1   2   3   4   5   6   7 */	/* 01234567 -            */
+    0x0,0x0,0x0,0x0,0x7,0x0,0x4,0x0,	/*  !"#$%&' -	 32..39  */
+	0x7,0x7,0x7,0x0,0xf,0xf,0xf,0xf,	/* ()*+,-./ -	 40..47	 */
+    0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* 01234567 -	 48..55  */
+	0xf,0xf,0x6,0x0,0x0,0x4,0x0,0x4,	/* 89:;<=>? -    56..63  */
+    0x0,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* @ABCDEFG -	 64..71	 */
+	0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* HIJKLMNO - 	 72..79  */
+    0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* PQRSTUVW -    80..87	 */
+	0xf,0xf,0xf,0x0,0x0,0x0,0x0,0xf,	/* XYZ[\]^_ -	 88..95  */
+    0x0,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* `abcdefg	-	 96..103 */
+	0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* hijklmno -	104..111 */
+    0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* pqrstuvw -   112..119 */
+	0xf,0xf,0xf,0x0,0x0,0x0,0x0,0x0,	/* xyz{|}~  -	120..127 */
 };
 
-#define PATH_MASK		0x1
-#define QUERY_MASK		0x2
+enum escape_mask {
+	PATH_MASK	= (1 << 0),
+	QUERY_MASK	= (1 << 1),
+	FIX_MASK	= (1 << 2),
+	SHELL_MASK	= (1 << 3)
+};
+
+static inline gboolean
+is_transparent_char(const gint c, const enum escape_mask m)
+{
+	return c >= 32 && c < 128 && (is_transparent[c - 32] & (guint8) m);
+}
 
 static const char hex_alphabet[] = "0123456789ABCDEF";
 
@@ -90,7 +105,7 @@ url_escape_mask(const gchar *url, guint8 mask)
 	gchar *new;
 
 	for (p = url, c = *p++; c; c = *p++)
-		if (!TRANSPARENT_CHAR(c, mask))
+		if (!is_transparent_char(c, mask))
 			need_escape++;
 
 	if (need_escape == 0)
@@ -99,7 +114,7 @@ url_escape_mask(const gchar *url, guint8 mask)
 	new = g_malloc(p - url + (need_escape << 1));
 
 	for (p = url, q = new, c = *p++; c; c = *p++) {
-		if (TRANSPARENT_CHAR(c, mask))
+		if (is_transparent_char(c, mask))
 			*q++ = c;
 		else {
 			*q++ = ESCAPE_CHAR;
@@ -129,7 +144,7 @@ url_escape_mask_into(const gchar *url, gchar *target, gint len, guint8 mask)
 	gchar *end = target + len;
 
 	for (q = target, c = *p++; c && q < end; c = *p++) {
-		if (TRANSPARENT_CHAR(c, mask))
+		if (is_transparent_char(c, mask))
 			*q++ = c;
 		else if (end - q >= 3) {
 			*q++ = ESCAPE_CHAR;
@@ -161,7 +176,10 @@ url_escape(const gchar *url)
 }
 
 /**
- * Same as url_escape(), but '+' are also escaped for the query string.
+ * Same as url_escape() except for:
+ *
+ * '+' is also escaped for the query string.
+ * ':' is not escaped.
  *
  * @return argument if no escaping is necessary, or a new string otherwise.
  */
@@ -171,6 +189,35 @@ url_escape_query(const gchar *url)
 	return url_escape_mask(url, QUERY_MASK);
 }
 
+/**
+ * Escapes the given string for safe use in a shell or file: URL.
+ * @return argument if no escaping is necessary, or a new string otherwise.
+ */
+gchar *
+url_escape_shell(const gchar *url)
+{
+	return url_escape_mask(url, SHELL_MASK);
+}
+
+/**
+ * Creates a file: URL from the given absolute path and escapes it properly
+ * for safe use in a shell too.
+ * @return A new string or NULL on failure.
+ */
+char *
+url_from_absolute_path(const gchar *path)
+{
+	gchar *escaped, *url;
+	
+	g_return_val_if_fail(is_absolute_path(path), NULL);
+	escaped = url_escape_mask(path, SHELL_MASK);
+	url = g_strconcat("file://", escaped, (void *) 0);
+	if (escaped != path) {
+		G_FREE_NULL(escaped);
+	}
+	return url;
+}
+
 /**
  * Escape undesirable characters using %xx, where xx is an hex code.
  * This is done in the `target' buffer, whose size is `len'.
@@ -184,41 +231,76 @@ url_escape_into(const gchar *url, gchar *target, gint len)
 	return url_escape_mask_into(url, target, len, PATH_MASK);
 }
 
+/**
+ * Don't touch '?', '&', '=', ':', %HH.
+ */
+gchar *
+url_fix_escape(const gchar *url)
+{
+	const gchar *p;
+	GString *gs;
+	guchar c;
+
+	gs = g_string_new(NULL);
+
+	for (p = url; '\0' != (c = *p); p++) {
+		if (
+			is_transparent_char(c, FIX_MASK) ||
+			('%' == c && is_ascii_xdigit(p[1]) && is_ascii_xdigit(p[2]))
+		) {
+			gs = g_string_append_c(gs, c);
+		} else {
+			gchar buf[3];
+
+			buf[0] = ESCAPE_CHAR;
+			buf[1] = hex_alphabet[c >> 4];
+			buf[2] = hex_alphabet[c & 0xf];
+			gs = g_string_append_len(gs, buf, sizeof buf);
+		}
+	}
+
+	return gm_string_finalize(gs);
+}
+
 /**
  * Escape control characters using %xx, where xx is an hex code.
  *
  * @return argument if no escaping is necessary, or a new string otherwise.
  */
 gchar *
-url_escape_cntrl(gchar *url)
+url_escape_cntrl(const gchar *url)
 {
-	gchar *p;
-	gchar *q;
-	int need_escape = 0;
-	guchar c;
-	gchar *new;
+	size_t need_escape = 0;
+	const gchar *p;
 
-	for (p = url, c = *p++; c; c = *p++)
-		if (iscntrl(c) || c == ESCAPE_CHAR)
+	for (p = url; '\0' != *p; p++) {
+		if (is_ascii_cntrl(*p) || ESCAPE_CHAR == *p)
 			need_escape++;
-
-	if (need_escape == 0)
-		return url;
-
-	new = g_malloc(p - url + (need_escape << 1));
-
-	for (p = url, q = new, c = *p++; c; c = *p++) {
-		if (!iscntrl(c) && c != ESCAPE_CHAR)
-			*q++ = c;
-		else {
-			*q++ = ESCAPE_CHAR;
-			*q++ = hex_alphabet[c >> 4];
-			*q++ = hex_alphabet[c & 0xf];
-		}
 	}
-	*q = '\0';
 
-	return new;
+	if (need_escape > 0) {
+		gchar *escaped, *q;
+		size_t size;
+		guchar c;
+
+		size = p - url + 1 + need_escape * 2;
+		escaped = g_malloc(size);
+		q = escaped;
+
+		for (p = url; '\0' != (c = *p); p++) {
+			if (!is_ascii_cntrl(c) && ESCAPE_CHAR != c)
+				*q++ = c;
+			else {
+				*q++ = ESCAPE_CHAR;
+				*q++ = hex_alphabet[c >> 4];
+				*q++ = hex_alphabet[c & 0xf];
+			}
+		}
+		*q = '\0';
+		return escaped;
+	} else {
+		return deconstify_gchar(url);
+	}
 }
 
 /**
@@ -325,8 +407,7 @@ url_params_parse(gchar *query)
 				*q = '\0';
 				value = url_unescape(start, FALSE);
 				if (!value) {
-					if (name)
-						G_FREE_NULL(name);
+					G_FREE_NULL(name);
 					url_params_free(up);
 					return NULL;
 				}
@@ -407,8 +488,7 @@ url_safe_char(gchar c, url_policy_t p)
 			!is_ascii_digit(c) &&
 			NULL == strchr("/._-~", c)
 		) {
-			if (lib_debug)
-      			g_warning("Unsafe character in GWC URL; rejected");
+      		/* Unsafe character in GWC URL; rejected */
       		return FALSE;
     	}
 	}
@@ -434,17 +514,15 @@ gchar *
 url_normalize(gchar *url, url_policy_t pol)
 {
 	static const char http_prefix[] = "http://";
-	const gchar *p, *uri, *endptr, *tld = NULL;
-	gchar c, *q, *warn = NULL;
+	const gchar *p, *uri, *endptr, *tld = NULL, *warn = NULL;
+	gchar c, *q;
 	host_addr_t addr;
 
 	g_assert(url);
 
-	if (NULL == (q = is_strcaseprefix(url, http_prefix))) {
-		if (lib_debug)
-			g_warning("URL \"%s\" isn't preceded by \"%s\"", url, http_prefix);
+	if (NULL == (q = is_strcaseprefix(url, http_prefix)))
 		return NULL;
-	}
+
 	/* Make sure the prefix is all-lowercase */
 	memcpy(url, http_prefix, CONST_STRLEN(http_prefix));
 
@@ -499,10 +577,6 @@ url_normalize(gchar *url, url_policy_t pol)
 			} else if ('\0' == c || '/' == c || ':' == c) {
 				break;
 			} else {
-				if (lib_debug)
-					g_warning("Bad URL \"%s\": "
-						"invalid character '%c' in ``host:port'' part.",
-						url, is_ascii_print(c) ? c : '?');
 				return NULL;
 			}
 		}
@@ -534,8 +608,13 @@ url_normalize(gchar *url, url_policy_t pol)
 			goto bad;
 		}
 
-		if (!(URL_POLICY_ALLOW_ANY_PORT & pol) && 80 != port && port < 1024) {
-			warn = "Ports below 1024 other than 80 are disallowed";
+		if (
+			!(URL_POLICY_ALLOW_ANY_PORT & pol) &&
+			port < 1024 &&
+			80 != port &&
+			443 != port
+		) {
+			warn = "Ports below 1024 other than 80 and 443 are disallowed";
 			goto bad;
 		}
 
@@ -614,14 +693,9 @@ url_normalize(gchar *url, url_policy_t pol)
 		url = s;
 	}
 
-	if (lib_debug > 1)
-		g_message("url=\"%s\"", url);
-
 	return url;
 
 bad:
-	if (lib_debug)
-		g_warning("invalid URL \"%s\": %s", url, warn);
 	return NULL;
 }
 
