@@ -1,5 +1,5 @@
 /*
- * $Id: walloc.c 12017 2006-09-21 14:53:10Z cbiere $
+ * $Id: walloc.c 13984 2007-06-29 13:31:06Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: walloc.c 12017 2006-09-21 14:53:10Z cbiere $")
+RCSID("$Id: walloc.c 13984 2007-06-29 13:31:06Z cbiere $")
 
 #include "glib-missing.h"
 #include "misc.h"
@@ -302,6 +302,7 @@ wdestroy(void)
 {
 	size_t i;
 
+#if defined(USE_HALLOC) && !defined(TRACK_MALLOC)
 	/*
 	 * We cannot do this currently for GLib 2.0 because g_malloc() is
 	 * mapped to halloc() and a g_warning() or other GLib functions may
@@ -314,6 +315,7 @@ wdestroy(void)
 
 	if (!g_mem_is_system_malloc())
 		return;
+#endif	/* USE_HALLOC */
 
 	for (i = 0; i < WZONE_SIZE; i++) {
 		if (wzone[i] != NULL) {
