@@ -1,5 +1,5 @@
 /*
- * $Id: walloc.h 12246 2006-11-07 02:53:05Z cbiere $
+ * $Id: walloc.h 13881 2007-06-18 23:48:50Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,11 +36,12 @@
 #ifndef _walloc_h_
 #define _walloc_h_
 
-#include <glib.h>
+#include "common.h"
+#include "malloc.h"
 
 /*
  * Under REMAP_ZALLOC control, those routines are remapped to malloc/free.
- * Under TRACK_ZALLOC, we keep tack of the allocation places.
+ * Under TRACK_ZALLOC, we keep track of the allocation places.
  */
 
 #if defined(USE_DMALLOC) && !defined(REMAP_ZALLOC)
@@ -53,7 +54,7 @@
 #error "TRACK_ZALLOC and REMAP_ZALLOC are mutually exclusive"
 #endif
 
-#if GLIB_CHECK_VERSION(2, 10, 0)
+#if 0 && GLIB_CHECK_VERSION(2, 10, 0)
 static inline gpointer walloc(size_t size) { return g_slice_alloc(size); }
 static inline gpointer walloc0(size_t size) { return g_slice_alloc0(size); }
 static inline void wfree(gpointer p, size_t size) { g_slice_free1(size, p); }
