@@ -1,5 +1,5 @@
 /*
- * $Id: watcher.c 13015 2007-03-08 19:07:02Z cbiere $
+ * $Id: watcher.c 14446 2007-08-13 14:06:15Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -38,7 +38,7 @@
 
 #include "common.h"
 
-RCSID("$Id: watcher.c 13015 2007-03-08 19:07:02Z cbiere $")
+RCSID("$Id: watcher.c 14446 2007-08-13 14:06:15Z cbiere $")
 
 #include "watcher.h"
 #include "atoms.h"
@@ -71,7 +71,7 @@ watcher_mtime(const gchar *filename)
 {
 	struct stat buf;
 
-	if (-1 == do_stat(filename, &buf))
+	if (-1 == stat(filename, &buf))
 		return 0;
 
 	return buf.st_mtime;
