@@ -1,5 +1,5 @@
 /*
- * $Id: watcher.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: watcher.h 12500 2006-11-26 10:53:21Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _watcher_h_
 #define _watcher_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "file.h"
 
