@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 14052 2007-07-06 21:28:36Z cbiere $
+ * $Id: main.c 14979 2007-10-16 19:17:13Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -51,6 +51,7 @@
 #include "core/features.h"
 #include "core/fileinfo.h"
 #include "core/file_object.h"
+#include "core/g2_cache.h"
 #include "core/geo_ip.h"
 #include "core/gmsg.h"
 #include "core/gnet_stats.h"
@@ -76,12 +77,12 @@
 #include "core/search.h"
 #include "core/settings.h"
 #include "core/share.h"
-#include "core/shell.h"
 #include "core/sockets.h"
 #include "core/spam.h"
 #include "core/sq.h"
 #include "core/tls_cache.h"
 #include "core/tls_common.h"
+#include "core/topless.h"
 #include "core/tsync.h"
 #include "core/tx.h"
 #include "core/udp.h"
@@ -116,27 +117,24 @@
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 #include "lib/wordvec.h"
+#include "shell/shell.h"
 
-#if defined(USE_TOPLESS)
 #include "ui/gtk/gui.h"
-#endif
 
 #if defined(USE_GTK1) || defined(USE_GTK2)
-#include "ui/gtk/gui.h"
-#include "ui/gtk/icon.h"
 #include "ui/gtk/main.h"
 #include "ui/gtk/settings.h"
 #include "ui/gtk/upload_stats.h"
-#include "if/ui/gtk/search.h"
 #endif /* GTK */
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
+#include "if/core/main.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: main.c 14052 2007-07-06 21:28:36Z cbiere $")
+RCSID("$Id: main.c 14979 2007-10-16 19:17:13Z rmanfredi $")
 
 #define SLOW_UPDATE_PERIOD		20	/**< Update period for `main_slow_update' */
 #define EXIT_GRACE				30	/**< Seconds to wait before exiting */
@@ -157,6 +155,7 @@ static tm_t start_time;
 
 void gtk_gnutella_exit(gint n);
 static gint reopen_log_files(void);
+static gboolean stderr_disabled;
 
 /**
  * Force immediate shutdown of SIGALRM reception.
@@ -267,6 +266,12 @@ debugging(guint t)
 		0;
 }
 
+const char *
+gtk_gnutella_interface(void)
+{
+	return running_topless ? "Topless" : GTA_INTERFACE;
+}
+
 /**
  * Invoked as an atexit() callback when someone does an exit().
  */
@@ -394,7 +399,9 @@ gtk_gnutella_exit(gint n)
 	 */
 
 	DO(settings_save_if_dirty);
-	DO(settings_gui_save_if_dirty);
+	if (!running_topless) {
+		DO(settings_gui_save_if_dirty);
+	}
 
 	safe_to_exit = TRUE;	/* Will immediately exit if re-entered */
 
@@ -403,8 +410,9 @@ gtk_gnutella_exit(gint n)
 
 #undef DO
 
-	main_gui_update_coords();
-	main_gui_shutdown();
+	if (!running_topless) {
+		main_gui_shutdown();
+	}
 
     hcache_shutdown();		/* Save host caches to disk */
 	settings_shutdown();
@@ -413,7 +421,9 @@ gtk_gnutella_exit(gint n)
 	search_shutdown();
 	bsched_shutdown();
 
-	settings_gui_shutdown();
+	if (!running_topless) {
+		settings_gui_shutdown();
+	}
 
 	/*
 	 * Show total CPU used, and the amount spent in user / kernel, before
@@ -444,7 +454,9 @@ gtk_gnutella_exit(gint n)
 		if ((d = delta_time(now, exit_time)) >= exit_grace)
 			break;
 
-		main_gui_shutdown_tick(exit_grace - d);
+		if (!running_topless) {
+			main_gui_shutdown_tick(exit_grace - d);
+		}
 		sleep(1);
 	}
 
@@ -493,6 +505,7 @@ gtk_gnutella_exit(gint n)
 	adns_close();
 	dbus_util_close();  /* After adns_close() to avoid strange crashes */
 	tls_cache_close();
+	g2_cache_close();
 	file_object_close();
 	settings_close();	/* Must come after hcache_close() */
 	atoms_close();
@@ -502,14 +515,13 @@ gtk_gnutella_exit(gint n)
 	malloc_close();
 #endif
 
-	if (debugging(0) || signal_received || shutdown_requested)
+	if (debugging(0) || signal_received || shutdown_requested) {
 		g_message("gtk-gnutella shut down cleanly.");
-
-#if defined(USE_GTK1) || defined(USE_GTK2)
-	gtk_exit(n);
-#else
+	}
+	if (!running_topless) {
+		main_gui_exit(n);
+	}
 	exit(n);
-#endif
 }
 
 static void
@@ -521,13 +533,6 @@ sig_terminate(int n)
 		exit(1);				/* Terminate ASAP */
 }
 
-#if !defined(USE_TOPLESS)
-/* FIXME: this is declared in search_gui.c and should be called in the
- *        main timer loop of the gui.
- */
-void search_gui_store_searches(void);
-#endif /* USE_TOPLESS */
-
 static void
 slow_main_timer(time_t now)
 {
@@ -550,7 +555,6 @@ slow_main_timer(time_t now)
 		dmesh_ban_store();
 		break;
 	case 1:
-		search_gui_store_searches();
 		hcache_store_if_dirty(HOST_ANY);
 		break;
 	case 2:
@@ -569,7 +573,9 @@ slow_main_timer(time_t now)
 
 	download_store_if_dirty();		/* Important, so always attempt it */
 	settings_save_if_dirty();		/* Nice to have, and file is small */
-	settings_gui_save_if_dirty();	/* Ditto */
+	if (!running_topless) {
+		settings_gui_save_if_dirty();	/* Ditto */
+	}
 	tx_collect();					/* Collect freed TX stacks */
 	rx_collect();					/* Idem for freed RX stacks */
 	prune_page_cache();
@@ -578,10 +584,6 @@ slow_main_timer(time_t now)
 	ignore_timer(now);
 }
 
-#if !defined(USE_TOPLESS)
-void icon_timer(void);
-#endif /* USE_TOPLESS */
-
 /**
  * Check CPU usage.
  *
@@ -728,7 +730,10 @@ main_timer(gpointer p)
 	 */
 
 	if (!exiting) {
-		main_gui_timer(now);
+
+		if (!running_topless) {
+			main_gui_timer(now);
+		}
 
 		/* Update for things that change slowly */
 		if (main_slow_update++ > SLOW_UPDATE_PERIOD) {
@@ -741,11 +746,6 @@ main_timer(gpointer p)
 	 * The following are low-priority tasks, not called if we've
 	 * detected a high CPU load.
 	 */
-
-	if (!GNET_PROPERTY(overloaded_cpu)) {
-		icon_timer();
-	}
-
 	bg_sched_timer(GNET_PROPERTY(overloaded_cpu));	/* Background tasks */
 
 	return TRUE;
@@ -781,6 +781,10 @@ log_handler(const gchar *domain, GLogLevelFlags level,
 
 	(void) domain;
 	(void) user_data;
+
+	if (stderr_disabled)
+		return;
+
 	now = tm_time();
 	ct = localtime(&now);
 
@@ -853,6 +857,7 @@ enum main_arg {
 	main_arg_pause_on_crash,
 	main_arg_ping,
 	main_arg_shell,
+	main_arg_topless,
 	main_arg_version,
 
 	/* Passed through for Gtk+/GDK/GLib */
@@ -900,6 +905,11 @@ static struct {
 	OPTION(pause_on_crash, 	NONE, "Pause the process on crash."),
 	OPTION(ping,			NONE, "Check whether gtk-gnutella is running."),
 	OPTION(shell,			NONE, "Access the local shell interface."),
+#ifdef USE_TOPLESS
+	OPTION(topless,			NONE, NULL),	/* accept by hide */
+#else
+	OPTION(topless,			NONE, "Disable the graphical user-interface."),
+#endif	/* USE_TOPLESS */
 	OPTION(version,			NONE, "Show version information."),
 
 	/* These are handled by Gtk+/GDK/GLib */
@@ -975,18 +985,25 @@ reopen_log_files(void)
 		} else {
 			fprintf(stderr, "freopen(..., \"a\", stdout) failed: %s",
 				g_strerror(errno));
-			failure |= TRUE;
+			failure = TRUE;
 		}
 	}
 	if (options[main_arg_log_stderr].used) {
-		if (freopen(options[main_arg_log_stderr].arg, "a", stderr)) {
+		const char *pathname = options[main_arg_log_stderr].arg;
+		
+		if (freopen(pathname, "a", stderr)) {
+			stderr_disabled = 0 == strcmp(pathname, "/dev/null");
 			setvbuf(stderr, NULL, _IOLBF, 0);
 		} else {
 			fprintf(stderr, "freopen(..., \"a\", stderr) failed: %s",
 				g_strerror(errno));
-			failure |= TRUE;
+			failure = TRUE;
+			stderr_disabled = TRUE;
 		}
+	} else if (options[main_arg_daemonize].used) {
+		stderr_disabled = TRUE;
 	}
+
 	return failure ? -1 : 0;
 }
 
@@ -1076,7 +1093,6 @@ prehandle_arguments(char **argv)
 	}
 }
 
-
 static void
 handle_arguments(int argc, char **argv)
 {
@@ -1088,6 +1104,10 @@ handle_arguments(int argc, char **argv)
 		g_assert(options[i].id == i);
 	}
 
+#ifdef USE_TOPLESS
+	options[main_arg_topless].used = TRUE;
+#endif	/* USE_TOPLESS */
+
 	argv0 = argv[0];
 	argv++;
 	argc--;
@@ -1154,22 +1174,43 @@ handle_arguments(int argc, char **argv)
 	if (options[main_arg_help].used) {
 		usage(EXIT_SUCCESS);
 	}
+#ifndef USE_TOPLESS
+	if (options[main_arg_topless].used) {
+		running_topless = TRUE;
+	}
+#endif	/* USE_TOPLESS */
 	if (options[main_arg_version].used) {
 		printf("%s\n", version_build_string());
+		
+		printf("GLib %u.%u.%u",
+			glib_major_version, glib_minor_version, glib_micro_version);
+		if (
+			GLIB_MAJOR_VERSION != glib_major_version ||
+			GLIB_MINOR_VERSION != glib_minor_version ||
+			GLIB_MICRO_VERSION != glib_micro_version
+		) {
+			printf(" (compiled against %u.%u.%u)",
+				GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+		}
+		printf("\n");
 
-		printf("GLib %u.%u.%u (compiled against %u.%u.%u)\n",
-			glib_major_version, glib_minor_version, glib_micro_version,
-			GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+#if defined(GTK_MAJOR_VERSION) && defined(GTK_MINOR_VERSION)
+		printf("Gtk+ %u.%u.%u",
+			gtk_major_version, gtk_minor_version, gtk_micro_version);
+		if (
+			GTK_MAJOR_VERSION != gtk_major_version ||
+			GTK_MINOR_VERSION != gtk_minor_version ||
+			GTK_MICRO_VERSION != gtk_micro_version
+		) {
+			printf(" (compiled against %u.%u.%u)",
+				GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
+		}
+		printf("\n");
+#endif	/* Gtk+ */
 
-#ifndef USE_TOPLESS
-		printf("Gtk+ %u.%u.%u (compiled against %u.%u.%u)\n",
-			gtk_major_version, gtk_minor_version, gtk_micro_version,
-			GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
-#endif	/* USE_TOPLESS */
-
-		if (tls_version_string())
+		if (tls_version_string()) {
 			printf("%s\n", tls_version_string());
-
+		}
 		exit(EXIT_SUCCESS);
 	}
 	if (options[main_arg_ping].used) {
@@ -1194,7 +1235,6 @@ handle_arguments(int argc, char **argv)
 			exit(EXIT_FAILURE);
 		}
 	}
-
 }
 
 int
@@ -1211,6 +1251,9 @@ main(int argc, char **argv)
 	 * file descriptor.
 	 */
 	close_file_descriptors(3); /* Just in case */
+	if (reserve_standard_file_descriptors()) {
+		exit(EXIT_FAILURE);
+	}
 
 	prehandle_arguments(argv);
 
@@ -1261,6 +1304,10 @@ main(int argc, char **argv)
 	STATIC_ASSERT(sizeof(size_t) == sizeof(gsize));
 	STATIC_ASSERT(sizeof(ssize_t) == sizeof(gssize));
 
+	STATIC_ASSERT(UNSIGNED(INT_MIN) > 0);
+	STATIC_ASSERT(UNSIGNED(LONG_MIN) > 0);
+	STATIC_ASSERT(UNSIGNED(-1) > 0);
+
 	inputevt_init();
 	tiger_check();
 	tt_check();
@@ -1275,7 +1322,9 @@ main(int argc, char **argv)
 	dbus_util_init();
 	vendor_init();
 
-	main_gui_early_init(argc, argv, options[main_arg_no_xshm].used);
+	if (!running_topless) {
+		main_gui_early_init(argc, argv, options[main_arg_no_xshm].used);
+	}
 
 	cq_init();
 	vmsg_init();
@@ -1286,6 +1335,7 @@ main(int argc, char **argv)
 	settings_init();
 	tls_global_init();
 	tls_cache_init();
+	g2_cache_init();
 	hostiles_init();
 	spam_init();
 	bogons_init();
@@ -1330,7 +1380,9 @@ main(int argc, char **argv)
 	dht_route_init();
 	dht_rpc_init();
 
-	main_gui_init();
+	if (!running_topless) {
+		main_gui_init();
+	}
 	node_post_init();
 
 	download_restore_state();
@@ -1360,7 +1412,11 @@ main(int argc, char **argv)
 	bsched_enable_all();
 	version_ancient_warn();
 
-	main_gui_run(options[main_arg_geometry].arg);
+	if (running_topless) {
+		topless_main_run();
+	} else {
+		main_gui_run(options[main_arg_geometry].arg);
+	}
 
 	return 0;
 }
