@@ -1,5 +1,5 @@
 /*
- * $Id: bitzi.c 13925 2007-06-22 05:24:10Z cbiere $
+ * $Id: bitzi.c 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
@@ -35,15 +35,15 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: bitzi.c 13925 2007-06-22 05:24:10Z cbiere $")
+RCSID("$Id: bitzi.c 14457 2007-08-14 04:23:12Z cbiere $")
 
-#include "gtk/search.h"			/* search_t */
+#include "gtk/search.h"
 #include "gtk/misc.h"			/* gui_record_sha1_eq() */
-#include "gtk/gtk-missing.h"
 
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 #include "if/core/bitzi.h"    	/* bitzi_data_t */
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /**
