@@ -1,5 +1,5 @@
 /*
- * $Id: filter.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: filter.c 14592 2007-08-19 18:57:35Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,26 +36,19 @@
 
 #include "gui.h"
 
-RCSID("$Id: filter.c 13777 2007-05-29 00:23:11Z cbiere $")
-
-#include "filter.h"
-#include "filter_cb.h"
-#include "gtk-missing.h"
+RCSID("$Id: filter.c 14592 2007-08-19 18:57:35Z cbiere $")
 
 #include "gtk/misc.h"
 
-#ifdef USE_GTK1
-#include "gtk/gtk1/interface-glade.h"
-#endif
-#ifdef USE_GTK2
-#include "gtk/gtk2/interface-glade.h"
-#endif
+#include "filter.h"
+#include "filter_cb.h"
 
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
 
 #include "lib/glib-missing.h"
 #include "lib/utf8.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 #define DEFAULT_TARGET (filter_get_drop_target())
@@ -117,11 +110,11 @@ add_column(GtkTreeView *tv, const gchar *name, gint id)
 				(void *) 0);
 
 	g_object_set(G_OBJECT(column),
-		"fixed-width", 100,
-		"min-width", 1,
+		"fixed-width", 200,
+		"min-width", 10,
 		"reorderable", FALSE,
 		"resizable", TRUE,
-		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
+		"sizing", GTK_TREE_VIEW_COLUMN_AUTOSIZE,
 		(void *) 0);
 	
     gtk_tree_view_append_column(tv, column);
@@ -182,153 +175,6 @@ getFilterRoot(filter_t *f)
     }
 }
 
-/**
- * Initialize the contents of the dialog editor and some
- * internal variables like the roots in the filter list etc.
- */
-void
-filter_gui_init(void)
-{
-	static const struct {
-		const gchar *name;
-		const guint id;
-	} radio_buttons[] = {
-#define D(x) "radiobutton_filter_" x 
-
-		{ D("flag_stable_set"),		RULE_FLAG_SET },
-		{ D("flag_stable_unset"),	RULE_FLAG_UNSET },
-		{ D("flag_stable_ignore"),	RULE_FLAG_IGNORE },
-
-		{ D("flag_busy_set"),		RULE_FLAG_SET },
-		{ D("flag_busy_unset"),		RULE_FLAG_UNSET },
-		{ D("flag_busy_ignore"),	RULE_FLAG_IGNORE },
-
-		{ D("flag_push_set"),		RULE_FLAG_SET },
-		{ D("flag_push_unset"),		RULE_FLAG_UNSET },
-		{ D("flag_push_ignore"),	RULE_FLAG_IGNORE },
-
-		/*
-		 * The user_data set here is later relevant for
-		 * filter_gui_get_state_rule().
-		 */
-		{ D("state_display_do"),		FILTER_PROP_STATE_DO },
-		{ D("state_display_dont"),		FILTER_PROP_STATE_DONT },
-		{ D("state_display_ignore"),	FILTER_PROP_STATE_IGNORE },
-		{ D("state_display_undef"), 	FILTER_PROP_STATE_UNKNOWN },
-
-		{ D("state_download_do"),		FILTER_PROP_STATE_DO },
-		{ D("state_download_dont"), 	FILTER_PROP_STATE_DONT },
-		{ D("state_download_ignore"),	FILTER_PROP_STATE_IGNORE },
-		{ D("state_download_undef"), 	FILTER_PROP_STATE_UNKNOWN },
-#undef D
-	};
-	static const struct {
-		const guint id;
-	} menu_items[] = {
-		{ RULE_TEXT_PREFIX },
-		{ RULE_TEXT_WORDS },
-		{ RULE_TEXT_SUFFIX },
-		{ RULE_TEXT_SUBSTR },
-		{ RULE_TEXT_REGEXP },
-		{ RULE_TEXT_EXACT },
-	};
-    GtkMenu *m;
-    guint i;
-
-    if (gui_filter_dialog() == NULL)
-        return;
-
-#ifdef USE_GTK1
-	{
-		GtkCList *clist_filter_rules;
-		GtkCTree *ctree_filter_filters;
-
-		clist_filter_rules = GTK_CLIST
-			(gui_filter_dialog_lookup("clist_filter_rules"));
-		ctree_filter_filters = GTK_CTREE
-			(gui_filter_dialog_lookup("ctree_filter_filters"));
-
-		gtk_clist_set_reorderable(clist_filter_rules, TRUE);
-		for (i = 0; i < 4; i++)
-			gtk_clist_set_column_width(GTK_CLIST(clist_filter_rules), i,
-					GUI_PROPERTY(filter_rules_col_widths)[i]);
-
-		for (i = 0; i < 3; i++)
-			gtk_clist_set_column_width(GTK_CLIST(ctree_filter_filters), i,
-					GUI_PROPERTY(filter_filters_col_widths)[i]);
-	}
-#endif /* USE_GTK1 */
-#ifdef USE_GTK2
-	{
-		GtkTreeView *tv_rules;
-		GtkTreeView *tv_filters;
-		GtkTreeModel *model;
-
-		tv_rules = GTK_TREE_VIEW(lookup_widget(gui_filter_dialog(),
-									"treeview_filter_rules"));
-		tv_filters = GTK_TREE_VIEW(lookup_widget(gui_filter_dialog(),
-									"treeview_filter_filters"));
-
-		model = create_filters_model();
-		gtk_tree_view_set_model(tv_filters, model);	
-		add_column(tv_filters, _("Filter"), 1);
-		add_column(tv_filters, _("Rule"), 2);
-		add_column(tv_filters, _("Match"), 3);
-		gtk_tree_view_set_rules_hint(tv_filters, TRUE);
-		g_signal_connect(G_OBJECT(tv_filters), "cursor-changed",
-			G_CALLBACK(on_treeview_filter_filters_select_row), NULL);
-
-		model = create_rules_model();
-		add_column(tv_rules, _("!"), 1);
-		add_column(tv_rules, _("Condition"), 2);
-		add_column(tv_rules, _("Target"), 3);
-		add_column(tv_rules, _("Match"), 4);
-		gtk_tree_view_set_model(tv_rules, model);	
-		gtk_tree_view_set_rules_hint(tv_rules, TRUE);
-		g_signal_connect(G_OBJECT(tv_rules), "cursor-changed",
-			G_CALLBACK(on_treeview_filter_rules_select_row), NULL);
-		g_signal_connect(GTK_OBJECT(tv_rules), "button-press-event",
-			G_CALLBACK(on_treeview_filter_rules_button_press_event), NULL);
-
-		gtk_tree_view_set_reorderable(tv_rules, TRUE);
-#if 0
-		for (i = 0; i < 4; i++)
-			gtk_clist_set_column_width(GTK_CLIST(clist_filter_rules), i,
-					GUI_PROPERTY(filter_rules_col_widths)[i]);
-
-		for (i = 0; i < 3; i++)
-			gtk_clist_set_column_width(GTK_CLIST(ctree_filter_filters), i,
-					GUI_PROPERTY(filter_filters_col_widths)[i]);
-#endif
-	}
-#endif /* USE_GTK2 */
-
-    gtk_notebook_set_show_tabs(
-        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
-        FALSE);
-
-    m = GTK_MENU(gtk_menu_new());
-	for (i = 0; i < G_N_ELEMENTS(menu_items); i++) {
-		guint id = menu_items[i].id;
-    	menu_new_item_with_data(m, _(rule_text_type_labels[id]),
-        	GUINT_TO_POINTER(id));
-	}
-
-    gtk_option_menu_set_menu(GTK_OPTION_MENU(
-			gui_filter_dialog_lookup("optionmenu_filter_text_type")),
-        GTK_WIDGET(m));
-
-    /*
-     * The user_data set here is later relevant for filter_gui_get_flag_rule()
-     */
-
-	for (i = 0; i < G_N_ELEMENTS(radio_buttons); i++) {
-    	gtk_object_set_user_data(
-			GTK_OBJECT(gui_filter_dialog_lookup(radio_buttons[i].name)),
-			GUINT_TO_POINTER(radio_buttons[i].id));
-	}
-}
-
 /**
  * Show the dialog on screen and set position.
  */
@@ -366,7 +212,7 @@ filter_gui_filter_clear_list(void)
 	static const struct {
 		const gchar *title;
 	} nodes[] = {
-		{ N_("Builtin targets (not editable)") },
+		{ N_("Built-in targets") },
 		{ N_("Global filters") },
 		{ N_("Search filters") },
 		{ N_("Free filters") },
@@ -425,7 +271,7 @@ filter_gui_filter_clear_list(void)
 	static const struct {
 		const gchar *title;
 	} nodes[] = {
-		{ N_("Builtin targets (not editable)") },
+		{ N_("Built-in targets") },
 		{ N_("Global filters") },
 		{ N_("Search filters") },
 		{ N_("Free filters") },
@@ -738,7 +584,7 @@ filter_gui_filter_set(filter_t *f, gboolean removable,
 
     if (f != NULL) {
         gtk_mass_widget_set_sensitive(gui_filter_dialog(),
-			widgets, G_N_ELEMENTS(widgets), TRUE);
+			widgets, G_N_ELEMENTS(widgets), filter_is_modifiable(f));
 
         gtk_widget_set_sensitive
             (gui_filter_dialog_lookup("button_filter_remove"), removable);
@@ -813,14 +659,6 @@ filter_gui_filter_set(filter_t *f, gboolean removable,
         filter_gui_set_ruleset(NULL);
         filter_gui_filter_set_enabled(NULL, FALSE);
 
-#ifdef USE_GTK1
-        gtk_clist_unselect_all(GTK_CLIST(ctree));
-#endif /* USE_GTK1 */
-#ifdef USE_GTK2
-		gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(tv));
-#endif /* USE_GTK2 */
-
-
         gtk_widget_set_sensitive
             (gui_filter_dialog_lookup("button_filter_remove"), FALSE);
         gtk_mass_widget_set_sensitive(gui_filter_dialog(),
@@ -928,7 +766,7 @@ filter_get_filter_stats(const filter_t *filter)
 /**
  * Update the filter list with the current stats data from the filters.
  */
-void
+static void
 filter_gui_update_filter_stats(void)
 {
     GtkCTree *ctree;
@@ -985,7 +823,7 @@ filter_update_filter_stats_helper(GtkTreeModel *model, GtkTreePath *unused_path,
 /**
  * Update the filter list with the current stats data from the filters.
  */
-void
+static void
 filter_gui_update_filter_stats(void)
 {
     if (gui_filter_dialog() == NULL)
@@ -1027,7 +865,7 @@ filter_get_rule_stats(const rule_t *rule)
 /**
  * Update the rules list with the current stats data from the rules.
  */
-void
+static void
 filter_gui_update_rule_stats(void)
 {
     GtkCList *clist;
@@ -1080,7 +918,7 @@ filter_update_rule_stats_helper(GtkTreeModel *model, GtkTreePath *unused_path,
 /**
  * Update the rules list with the current stats data from the rules.
  */
-void
+static void
 filter_gui_update_rule_stats(void)
 {
     if (gui_filter_dialog() == NULL || work_filter == NULL)
@@ -1196,7 +1034,7 @@ filter_gui_edit_rule(rule_t *r)
     if (gui_filter_dialog() == NULL)
         return;
 
-    if (r != NULL) {
+    if (r && filter_is_modifiable(work_filter)) {
         switch (r->type) {
         case RULE_TEXT:
             filter_gui_edit_text_rule(r);
@@ -1223,9 +1061,8 @@ filter_gui_edit_rule(rule_t *r)
             g_error("Unknown rule type: %d", r->type);
         }
     } else {
-        gtk_notebook_set_page(
-            GTK_NOTEBOOK
-                (gui_filter_dialog_lookup("notebook_filter_detail")),
+        gtk_notebook_set_current_page(
+            GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
             nb_filt_page_buttons);
 
 #ifdef USE_GTK1
@@ -1261,7 +1098,7 @@ filter_gui_edit_ip_rule(rule_t *r)
 
     if (r != NULL) {
         ip   = g_strdup_printf("%s/%u",
-					host_addr_to_string(r->u.ip.addr), r->u.ip.mask);
+					host_addr_to_string(r->u.ip.addr), r->u.ip.cidr);
         target = r->target;
         invert = RULE_IS_NEGATED(r);
         active = RULE_IS_ACTIVE(r);
@@ -1291,7 +1128,7 @@ filter_gui_edit_ip_rule(rule_t *r)
 
    	G_FREE_NULL(ip);
 
-    gtk_notebook_set_page(
+    gtk_notebook_set_current_page(
         GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_ip);
 }
@@ -1348,9 +1185,8 @@ filter_gui_edit_sha1_rule(rule_t *r)
             (gui_filter_dialog_lookup("checkbutton_filter_sha1_soft")),
         soft);
 
-    gtk_notebook_set_page(
-        GTK_NOTEBOOK
-            (gui_filter_dialog_lookup("notebook_filter_detail")),
+    gtk_notebook_set_current_page(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_sha1);
 }
 
@@ -1412,9 +1248,8 @@ filter_gui_edit_text_rule(rule_t *r)
             (gui_filter_dialog_lookup("checkbutton_filter_text_soft")),
         soft);
 
-    gtk_notebook_set_page(
-        GTK_NOTEBOOK
-            (gui_filter_dialog_lookup("notebook_filter_detail")),
+    gtk_notebook_set_current_page(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_text);
 }
 
@@ -1469,9 +1304,8 @@ filter_gui_edit_size_rule(rule_t *r)
             (gui_filter_dialog_lookup("checkbutton_filter_size_soft")),
         soft);
 
-    gtk_notebook_set_page(
-        GTK_NOTEBOOK
-            (gui_filter_dialog_lookup("notebook_filter_detail")),
+    gtk_notebook_set_current_page(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_size);
 }
 
@@ -1504,9 +1338,8 @@ filter_gui_edit_jump_rule(rule_t *r)
             (gui_filter_dialog_lookup("checkbutton_filter_jump_active")),
         active);
 
-    gtk_notebook_set_page(
-        GTK_NOTEBOOK
-            (gui_filter_dialog_lookup("notebook_filter_detail")),
+    gtk_notebook_set_current_page(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_jump);
 }
 
@@ -1603,9 +1436,8 @@ filter_gui_edit_flag_rule(rule_t *r)
             (gui_filter_dialog_lookup("checkbutton_filter_flag_soft")),
         soft);
 
-    gtk_notebook_set_page(
-        GTK_NOTEBOOK
-            (gui_filter_dialog_lookup("notebook_filter_detail")),
+    gtk_notebook_set_current_page(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_flag);
 }
 
@@ -1697,9 +1529,8 @@ filter_gui_edit_state_rule(rule_t *r)
             (gui_filter_dialog_lookup("checkbutton_filter_state_soft")),
         soft);
 
-    gtk_notebook_set_page(
-        GTK_NOTEBOOK
-            (gui_filter_dialog_lookup("notebook_filter_detail")),
+    gtk_notebook_set_current_page(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_state);
 }
 
@@ -1755,7 +1586,7 @@ filter_gui_set_ruleset(GList *ruleset)
 
     gtk_widget_set_sensitive(
         gui_filter_dialog_lookup("button_filter_clear"),
-        count != 0);
+        count > 0 && filter_is_modifiable(work_filter));
 
     if (GUI_PROPERTY(gui_debug) >= 5)
         g_message("updated %d items\n", count);
@@ -1807,7 +1638,7 @@ filter_gui_set_ruleset(GList *ruleset)
 
     gtk_widget_set_sensitive(
         gui_filter_dialog_lookup("button_filter_clear"),
-        count != 0);
+        count > 0 && filter_is_modifiable(work_filter));
 
     if (GUI_PROPERTY(gui_debug) >= 5)
         g_message("updated %d items\n", count);
@@ -2360,4 +2191,168 @@ filter_gui_create_dlg_filters(void)
 }
 #endif	/* USE_GTK2 */
 
+/**
+ * Periodically update the filter display with current data.
+ */
+static void
+filter_gui_timer(time_t now)
+{
+	static time_t last_update;
+
+	if (last_update != now) {
+		last_update = now;
+		filter_gui_update_filter_stats();
+		filter_gui_update_rule_stats();
+	}
+}
+
+/**
+ * Initialize the contents of the dialog editor and some
+ * internal variables like the roots in the filter list etc.
+ */
+void
+filter_gui_init(void)
+{
+	static const struct {
+		const gchar *name;
+		const guint id;
+	} radio_buttons[] = {
+#define D(x) "radiobutton_filter_" x 
+
+		{ D("flag_stable_set"),		RULE_FLAG_SET },
+		{ D("flag_stable_unset"),	RULE_FLAG_UNSET },
+		{ D("flag_stable_ignore"),	RULE_FLAG_IGNORE },
+
+		{ D("flag_busy_set"),		RULE_FLAG_SET },
+		{ D("flag_busy_unset"),		RULE_FLAG_UNSET },
+		{ D("flag_busy_ignore"),	RULE_FLAG_IGNORE },
+
+		{ D("flag_push_set"),		RULE_FLAG_SET },
+		{ D("flag_push_unset"),		RULE_FLAG_UNSET },
+		{ D("flag_push_ignore"),	RULE_FLAG_IGNORE },
+
+		/*
+		 * The user_data set here is later relevant for
+		 * filter_gui_get_state_rule().
+		 */
+		{ D("state_display_do"),		FILTER_PROP_STATE_DO },
+		{ D("state_display_dont"),		FILTER_PROP_STATE_DONT },
+		{ D("state_display_ignore"),	FILTER_PROP_STATE_IGNORE },
+		{ D("state_display_undef"), 	FILTER_PROP_STATE_UNKNOWN },
+
+		{ D("state_download_do"),		FILTER_PROP_STATE_DO },
+		{ D("state_download_dont"), 	FILTER_PROP_STATE_DONT },
+		{ D("state_download_ignore"),	FILTER_PROP_STATE_IGNORE },
+		{ D("state_download_undef"), 	FILTER_PROP_STATE_UNKNOWN },
+#undef D
+	};
+	static const struct {
+		const guint id;
+	} menu_items[] = {
+		{ RULE_TEXT_PREFIX },
+		{ RULE_TEXT_WORDS },
+		{ RULE_TEXT_SUFFIX },
+		{ RULE_TEXT_SUBSTR },
+		{ RULE_TEXT_REGEXP },
+		{ RULE_TEXT_EXACT },
+	};
+    GtkMenu *m;
+    guint i;
+
+	main_gui_add_timer(filter_gui_timer);
+
+    if (gui_filter_dialog() == NULL)
+        return;
+
+#ifdef USE_GTK1
+	{
+		GtkCList *clist;
+
+		clist = GTK_CLIST(gui_filter_dialog_lookup("clist_filter_rules"));
+		gtk_clist_set_reorderable(clist, TRUE);
+		clist_restore_widths(clist, PROP_FILTER_RULES_COL_WIDTHS);
+
+		clist = GTK_CLIST(gui_filter_dialog_lookup("ctree_filter_filters"));
+		clist_restore_widths(clist, PROP_FILTER_FILTERS_COL_WIDTHS);
+	}
+#endif /* USE_GTK1 */
+#ifdef USE_GTK2
+	{
+		GtkTreeView *tv_rules;
+		GtkTreeView *tv_filters;
+		GtkTreeModel *model;
+
+		tv_rules = GTK_TREE_VIEW(lookup_widget(gui_filter_dialog(),
+									"treeview_filter_rules"));
+		tv_filters = GTK_TREE_VIEW(lookup_widget(gui_filter_dialog(),
+									"treeview_filter_filters"));
+
+		model = create_filters_model();
+		gtk_tree_view_set_model(tv_filters, model);	
+		add_column(tv_filters, _("Filter"), 1);
+		add_column(tv_filters, _("Rule"), 2);
+		add_column(tv_filters, _("Match"), 3);
+		gtk_tree_view_set_rules_hint(tv_filters, TRUE);
+		gui_signal_connect(tv_filters,
+			"cursor-changed", on_treeview_filter_filters_select_row, NULL);
+
+		model = create_rules_model();
+		add_column(tv_rules, _("!"), 1);
+		add_column(tv_rules, _("Condition"), 2);
+		add_column(tv_rules, _("Target"), 3);
+		add_column(tv_rules, _("Match"), 4);
+		gtk_tree_view_set_model(tv_rules, model);	
+		gtk_tree_view_set_rules_hint(tv_rules, TRUE);
+		gui_signal_connect(tv_rules,
+			"cursor-changed", on_treeview_filter_rules_select_row, NULL);
+		gui_signal_connect(tv_rules,
+			"button-press-event", on_treeview_filter_rules_button_press_event,
+			NULL);
+
+		gtk_tree_view_set_reorderable(tv_rules, TRUE);
+	}
+#endif /* USE_GTK2 */
+
+    gtk_notebook_set_show_tabs(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
+        FALSE);
+
+    m = GTK_MENU(gtk_menu_new());
+	for (i = 0; i < G_N_ELEMENTS(menu_items); i++) {
+		guint id = menu_items[i].id;
+    	menu_new_item_with_data(m, _(rule_text_type_labels[id]),
+        	GUINT_TO_POINTER(id));
+	}
+
+    gtk_option_menu_set_menu(GTK_OPTION_MENU(
+			gui_filter_dialog_lookup("optionmenu_filter_text_type")),
+        GTK_WIDGET(m));
+
+    /*
+     * The user_data set here is later relevant for filter_gui_get_flag_rule()
+     */
+
+	for (i = 0; i < G_N_ELEMENTS(radio_buttons); i++) {
+    	gtk_object_set_user_data(
+			GTK_OBJECT(gui_filter_dialog_lookup(radio_buttons[i].name)),
+			GUINT_TO_POINTER(radio_buttons[i].id));
+	}
+}
+
+void
+filter_gui_shutdown(void)
+{
+#ifdef USE_GTK1
+    if (gui_filter_dialog()) {
+		GtkCList *clist;
+
+		clist = GTK_CLIST(gui_filter_dialog_lookup("clist_filter_rules"));
+		clist_save_widths(clist, PROP_FILTER_RULES_COL_WIDTHS);
+
+		clist = GTK_CLIST(gui_filter_dialog_lookup("ctree_filter_filters"));
+		clist_save_widths(clist, PROP_FILTER_FILTERS_COL_WIDTHS);
+	}
+#endif /* USE_GTK1 */
+}
+
 /* vi: set ts=4 sw=4 cindent: */
