@@ -1,5 +1,5 @@
 /*
- * $Id: filter.h 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: filter.h 14389 2007-08-09 22:10:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -59,10 +59,9 @@ void filter_gui_filter_remove(filter_t *f);
 void filter_gui_filter_set_enabled(filter_t *f, gboolean active);
 void filter_gui_filter_set(filter_t *, gboolean, gboolean, GList *);
 void filter_gui_init(void);
+void filter_gui_shutdown(void);
 void filter_gui_rebuild_target_combos(GList *filters);
 void filter_gui_set_ruleset(GList *ruleset);
-void filter_gui_update_filter_stats(void);
-void filter_gui_update_rule_stats(void);
 void filter_gui_show_dialog(void);
 rule_t *filter_gui_get_rule(void);
 void filter_gui_update_rule_count(filter_t *f, GList *ruleset);
