@@ -1,5 +1,5 @@
 /*
- * $Id: filter_core.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: filter_core.c 15039 2007-12-01 19:09:17Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -34,30 +34,26 @@
  * @date 2001-2003
  */
 
-#include "common.h"
+#include "gui.h"
 
-RCSID("$Id: filter_core.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: filter_core.c 15039 2007-12-01 19:09:17Z cbiere $")
 
-#include "filter.h"
-#include "filter_core.h"
-#include "search.h"
-#include "settings.h"
-#include "gtk-missing.h"
-
-#ifdef USE_GTK1
-#include "gtk/gtk1/interface-glade.h"
-#endif
-#ifdef USE_GTK2
-#include "gtk/gtk2/interface-glade.h"
-#endif
+#include "gtk/filter.h"
+#include "gtk/filter_core.h"
+#include "gtk/search.h"
+#include "gtk/search_result.h"
+#include "gtk/settings.h"
 
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
 
+#include "if/core/search.h"
+
 #include "lib/atoms.h"
 #include "lib/walloc.h"
 #include "lib/utf8.h"
 #include "lib/glib-missing.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
 /**
@@ -105,29 +101,29 @@ static void filter_free(filter_t *f);
 /**
  * Public variables.
  */
-filter_t *work_filter = NULL;
+filter_t *work_filter;
 
 /*
  * Private variables
  */
-static GList *shadow_filters = NULL;
-static GList *filters_added = NULL;
-static GList *filters_removed = NULL;
+static GList *shadow_filters;
+static GList *filters_added;
+static GList *filters_removed;
 
 /* built-in targets */
-static filter_t *filter_drop = NULL;
-static filter_t *filter_show = NULL;
-static filter_t *filter_download = NULL;
-static filter_t *filter_nodownload = NULL;
-static filter_t *filter_return = NULL;
+static filter_t *filter_drop;
+static filter_t *filter_show;
+static filter_t *filter_download;
+static filter_t *filter_nodownload;
+static filter_t *filter_return;
 
 /* global filters */
-static filter_t *filter_global_pre = NULL;
-static filter_t *filter_global_post = NULL;
+static filter_t *filter_global_pre;
+static filter_t *filter_global_post;
 
 /* not static because needed in search_xml. */
-GList *filters = NULL;
-GList *filters_current = NULL;
+GList *filters;
+GList *filters_current;
 
 
 /***
@@ -426,7 +422,7 @@ filter_refresh_display(GList *filter_list)
 void
 filter_open_dialog(void)
 {
-    search_t *current_search;
+    struct search *current_search;
 
     current_search = search_gui_get_current_search();
 
@@ -438,7 +434,11 @@ filter_open_dialog(void)
         filter_refresh_display(filters_current);
     }
 
-    filter_set(current_search ? current_search->filter : NULL);
+	if (current_search) {
+    	filter_set(search_gui_get_filter(current_search));
+	} else {
+    	filter_set(NULL);
+	}
     filter_gui_show_dialog();
 }
 
@@ -453,6 +453,7 @@ filter_close_dialog(gboolean commit)
 {
     if (commit) {
         filter_apply_changes();
+		search_gui_store_searches();
     } else
         filter_revert_changes();
 
@@ -494,7 +495,7 @@ filter_duplicate_rule(const rule_t *r)
         return filter_new_text_rule(r->u.text.match, r->u.text.type,
 					r->u.text.case_sensitive, r->target, r->flags);
     case RULE_IP:
-        return filter_new_ip_rule(r->u.ip.addr, r->u.ip.mask,
+        return filter_new_ip_rule(r->u.ip.addr, r->u.ip.cidr,
 					r->target, r->flags);
     case RULE_SIZE:
         return filter_new_size_rule(r->u.size.lower, r->u.size.upper,
@@ -565,13 +566,13 @@ filter_new_text_rule(const gchar *match, gint type,
 
 		if (err) {
 			gchar regbuf[1000];
-			regerror(err, re, regbuf, sizeof(regbuf));
 
-			g_warning(
-                "problem in regular expression: %s"
+			regerror(err, re, regbuf, sizeof(regbuf));
+			g_warning("problem in regular expression: %s"
 				"; falling back to substring match", buf);
 
 			r->u.text.type = RULE_TEXT_SUBSTR;
+			regfree(re);
             G_FREE_NULL(re);
 		} else {
 			r->u.text.u.re = re;
@@ -579,9 +580,9 @@ filter_new_text_rule(const gchar *match, gint type,
 	}
 
 	/* no "else" because REGEXP can fall back here */
-	if (r->u.text.type == RULE_TEXT_SUBSTR)
+	if (r->u.text.type == RULE_TEXT_SUBSTR) {
 		r->u.text.u.pattern = pattern_compile(buf);
-
+	}
     G_FREE_NULL(buf);
 
     return r;
@@ -590,7 +591,7 @@ filter_new_text_rule(const gchar *match, gint type,
 
 
 rule_t *
-filter_new_ip_rule(const host_addr_t addr, guint8 mask,
+filter_new_ip_rule(const host_addr_t addr, guint8 cidr,
 	filter_t *target, guint16 flags)
 {
 	rule_t *r;
@@ -601,8 +602,16 @@ filter_new_ip_rule(const host_addr_t addr, guint8 mask,
 
    	r->type = RULE_IP;
 
+	switch (host_addr_net(addr)) {
+	case NET_TYPE_IPV4:	cidr = MIN(cidr, 32); break;
+	case NET_TYPE_IPV6:	cidr = MIN(cidr, 128); break;
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		break;
+	}
+
 	r->u.ip.addr  = addr;
-	r->u.ip.mask  = mask;
+	r->u.ip.cidr  = cidr;
     r->target     = target;
     r->flags      = flags;
     set_flags(r->flags, RULE_FLAG_VALID);
@@ -742,23 +751,23 @@ filter_new_state_rule(enum filter_prop_state display,
 void
 filter_set(filter_t *f)
 {
-    if (f != NULL) {
+    if (f) {
         shadow_t *shadow;
         gboolean removable;
         gboolean active;
         GList *ruleset;
 
+		removable = filter_is_modifiable(f) &&
+			!filter_is_global(f) &&
+			!filter_is_bound(f);
+
         shadow = shadow_find(f);
         if (shadow != NULL) {
-            removable =
-                (shadow->refcount == 0) && !filter_is_builtin(f) &&
-                !filter_is_global(f) && !filter_is_bound(f);
+            removable = removable && 0 == shadow->refcount;
             active = filter_is_active(shadow);
             ruleset = shadow->current;
         } else {
-            removable =
-                (f->refcount == 0) && !filter_is_builtin(f) &&
-                !filter_is_global(f) && !filter_is_bound(f);
+            removable = removable && 0 == f->refcount;
             active = filter_is_active(f);
             ruleset = f->ruleset;
         }
@@ -779,30 +788,31 @@ filter_set(filter_t *f)
 
 /**
  * Clear the searches shadow, update the combobox and the filter
- * bound to this search (search->ruleser).
+ * bound to this search (search->ruleset).
  */
 void
-filter_close_search(search_t *s)
+filter_close_search(struct search *s)
 {
     shadow_t *shadow;
+	filter_t *filter;
 
-    g_assert(s != NULL);
-    g_assert(s->filter != NULL);
+	g_return_if_fail(s);
+	filter = search_gui_get_filter(s);
+	g_return_if_fail(filter);
 
     if (GUI_PROPERTY(gui_debug) >= 6)
         g_message("closing search (freeing filter): %s", search_gui_query(s));
 
-    shadow = shadow_find(s->filter);
+    shadow = shadow_find(filter);
     if (shadow != NULL) {
 		GList *copy;
 
 		copy = g_list_copy(shadow->removed);
-		G_LIST_FOREACH_SWAPPED(copy, filter_append_rule_to_session, s->filter);
+		G_LIST_FOREACH_SWAPPED(copy, filter_append_rule_to_session, filter);
         g_list_free(copy);
 
 		copy = g_list_copy(shadow->added);
-		G_LIST_FOREACH_SWAPPED(copy,
-			filter_remove_rule_from_session, s->filter);
+		G_LIST_FOREACH_SWAPPED(copy, filter_remove_rule_from_session, filter);
         g_list_free(copy);
 
         shadow_cancel(shadow);
@@ -811,17 +821,15 @@ filter_close_search(search_t *s)
     /*
      * If this is the filter currently worked on, clear the display.
      */
-    if (s->filter == work_filter)
+    if (filter == work_filter) {
         filter_set(NULL);
+	}
+    filter_gui_filter_remove(filter);
 
-    filter_gui_filter_remove(s->filter);
-
-    filter_free(s->filter);
-    s->filter = NULL;
+    filter_free(filter);
+	search_gui_set_filter(s, NULL);
 }
 
-
-
 /**
  * Go through all the shadow filters, and commit the recorded
  * changes to the associated filter. We walk through the
@@ -1012,7 +1020,7 @@ filter_rule_condition_to_string(const rule_t *r)
         break;
     case RULE_IP:
 		gm_snprintf(tmp, sizeof tmp, _("If IP address matches %s/%u"),
-			host_addr_to_string(r->u.ip.addr), r->u.ip.mask);
+			host_addr_to_string(r->u.ip.addr), r->u.ip.cidr);
         break;
     case RULE_SIZE:
 		if (r->u.size.upper == r->u.size.lower) {
@@ -1276,7 +1284,7 @@ filter_add_to_session(filter_t *f)
  * Create a new filter bound to a search and register it.
  */
 void
-filter_new_for_search(search_t *s)
+filter_new_for_search(struct search *s)
 {
 	const gchar *query;
     filter_t *f;
@@ -1304,7 +1312,7 @@ filter_new_for_search(search_t *s)
      * Crosslink filter and search
      */
     f->search = s;
-    s->filter = f;
+	search_gui_set_filter(s, f);
 
     /*
      * It's important to add the filter here, because it was not
@@ -1414,7 +1422,7 @@ filter_free_rule(rule_t *r)
             break;
         case RULE_TEXT_REGEXP:
             regfree(r->u.text.u.re);
-            r->u.text.u.re = NULL;
+            G_FREE_NULL(r->u.text.u.re);
             break;
         case RULE_TEXT_PREFIX:
         case RULE_TEXT_SUFFIX:
@@ -2061,10 +2069,8 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
     g_assert(filter != NULL);
     g_assert(ctx != NULL);
 	rec = ctx->rec;
-    g_assert(rec != NULL);
+	record_check(rec);
     g_assert(res != NULL);
-	g_assert(rec->magic == RECORD_MAGIC);
-	g_assert(rec->refcount >= 0 && rec->refcount < INT_MAX);
 
     /*
      * We only try to prevent circles or the filter is inactive.
@@ -2195,7 +2201,7 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
 			}
             case RULE_IP:
 				match = host_addr_matches(rec->results_set->addr,
-							r->u.ip.addr, r->u.ip.mask);
+							r->u.ip.addr, r->u.ip.cidr);
                 break;
             case RULE_SIZE:
                 if (rec->size >= r->u.size.lower &&
@@ -2352,16 +2358,14 @@ filter_apply(filter_t *filter, struct filter_context *ctx, filter_result_t *res)
  * rows. This must be freed with filter_free_properties.
  */
 filter_result_t *
-filter_record(search_t *sch, const struct record *rec)
+filter_record(struct search *search, const struct record *rec)
 {
     filter_result_t *result;
 	struct filter_context ctx;
     gint i;
 
-    g_assert(sch != NULL);
-    g_assert(rec != NULL);
-	g_assert(rec->magic == RECORD_MAGIC);
-	g_assert(rec->refcount >= 0 && rec->refcount < INT_MAX);
+    g_assert(search != NULL);
+	record_check(rec);
 
 	ctx.rec = rec;
 	ctx.l_name = ctx.utf8_name = NULL;
@@ -2379,7 +2383,7 @@ filter_record(search_t *sch, const struct record *rec)
      * If not decided check if the filters for this search apply.
      */
     if (result->props_set < MAX_FILTER_PROP)
-        filter_apply(sch->filter, &ctx, result);
+        filter_apply(search_gui_get_filter(search), &ctx, result);
 
     /*
      * If it has not yet been decided, try the global filter
@@ -2432,6 +2436,8 @@ filter_shutdown(void)
     if (GUI_PROPERTY(gui_debug) >= 5)
         g_message("shutting down filters");
 
+	filter_gui_shutdown();
+
     /*
      * It is important that all searches have already been closed.
      * Since it is not allowd to use a bound filter as a target,
@@ -2470,7 +2476,110 @@ filter_shutdown(void)
         filter_free(f->data);
 }
 
+static void
+filter_preset_init(const char *name, const char *regexp, filesize_t minsize)
+{
+	filter_t *filter;
 
+	filter = filter_find_by_name_in_session(name);
+	if (filter) {
+		/* Remove all rules, we want to keep this filters up-to-date */
+		while (NULL != filter->ruleset) {
+			rule_t *rule;
+		
+			rule = g_list_nth_data(filter->ruleset, 0);
+			g_assert(rule->target);
+			g_assert(rule->target->refcount > 0);
+			rule->target->refcount--;
+
+			filter->ruleset = g_list_remove(filter->ruleset, rule);
+			filter_free_rule(rule);
+		}
+		filter_remove_from_session(filter);
+	} else {
+		filter = filter_new(lazy_ui_string_to_utf8(name));
+	}
+
+	if (minsize > 0) {
+		rule_t *rule;
+
+		rule = filter_new_size_rule(0, minsize - 1,
+				filter_get_drop_target(),
+				RULE_FLAG_ACTIVE);
+		filter_append_rule(filter, rule);
+	}
+
+	if (regexp) {
+		rule_t *rule;
+
+		rule = filter_new_text_rule(regexp,
+			RULE_TEXT_REGEXP,
+			FALSE,	/* case-insensitive */
+			filter_get_drop_target(),
+			RULE_FLAG_ACTIVE | RULE_FLAG_NEGATE);
+		filter_append_rule(filter, rule);
+	}
+
+	filter_add_to_session(filter);
+	set_flags(filter->flags, FILTER_FLAG_PRESET);
+}
+
+/**
+ *  Adds simple filter rules, for use by novice users.
+ */
+void
+filter_init_presets(void)
+{
+	static const struct {
+		const char *name;
+		const char *regex;
+		filesize_t minsize;
+	} tab[] = {
+		{ N_("<Archive>"),	  "[.](bz2|gz|zip|rar|iso)$", 0 },
+		{ N_("<Audio>"), 	  "[.](mp3|m4a|ogg|flac)$", 1000000 },
+		{ N_("<Image>"), 	  "[.](bmp|gif|jpg|jpeg|png|psd|tif|tiff)$", 0 },
+		{ N_("<Literature>"), "[.](pdf|doc|lit|djvu|ps)$", 10000 },
+		{ N_("<Video>"), 	  "[.](avi|mpg|mp4|mpeg|mkv|ogm)$", 10000000 },
+	};
+	unsigned i;
+
+	for (i = 0; i < G_N_ELEMENTS(tab); i++) {
+		filter_preset_init(_(tab[i].name), tab[i].regex, tab[i].minsize);
+	}
+	filter_update_targets();
+}
+
+static filter_t *
+filters_add(const char *name, unsigned flags)
+{
+	filter_t *filter;
+
+    filter = filter_new(lazy_ui_string_to_utf8(name));
+	set_flags(filter->flags, flags);
+    filters = g_list_append(filters, filter);
+	return filter;
+}
+
+static void
+filter_init_globals(void)
+{
+	const unsigned flags = FILTER_FLAG_GLOBAL;
+
+    filter_global_pre  = filters_add(_("Global (pre)"), flags);
+    filter_global_post = filters_add(_("Global (post)"), flags);
+}
+
+static void
+filter_init_builtins(void)
+{
+	const unsigned flags = FILTER_FLAG_BUILTIN;
+
+    filter_show        = filters_add(_("DISPLAY"), flags);
+    filter_drop        = filters_add(_("DON'T DISPLAY"), flags);
+    filter_download    = filters_add(_("DOWNLOAD"), flags);
+    filter_nodownload  = filters_add(_("DON'T DOWNLOAD"), flags);
+    filter_return      = filters_add(_("RETURN"), flags);
+}
 
 /**
  * Initialize global filters.
@@ -2478,29 +2587,13 @@ filter_shutdown(void)
 void
 filter_init(void)
 {
-    filter_global_pre  = filter_new(lazy_ui_string_to_utf8(_("Global (pre)")));
-    filter_global_post = filter_new(lazy_ui_string_to_utf8(_("Global (post)")));
-    filter_show        = filter_new(lazy_ui_string_to_utf8(_("DISPLAY")));
-    filter_drop        = filter_new(lazy_ui_string_to_utf8(_("DON'T DISPLAY")));
-    filter_download    = filter_new(lazy_ui_string_to_utf8(_("DOWNLOAD")));
-    filter_nodownload = filter_new(lazy_ui_string_to_utf8(_("DON'T DOWNLOAD")));
-    filter_return      = filter_new(lazy_ui_string_to_utf8(_("RETURN")));
-
-    filters = g_list_append(filters, filter_global_pre);
-    filters = g_list_append(filters, filter_global_post);
-    filters = g_list_append(filters, filter_show);
-    filters = g_list_append(filters, filter_drop);
-    filters = g_list_append(filters, filter_download);
-    filters = g_list_append(filters, filter_nodownload);
-    filters = g_list_append(filters, filter_return);
-
+	filter_init_globals();
+	filter_init_builtins();
     filters_current = g_list_copy(filters);
 
     gui_popup_filter_rule_set(create_popup_filter_rule());
 }
 
-
-
 /**
  * Trigger a rebuild of the target combos.
  */
@@ -2510,20 +2603,6 @@ filter_update_targets(void)
     filter_gui_rebuild_target_combos(filters_current);
 }
 
-
-
-/**
- * Periodically update the filter display with current data.
- */
-void
-filter_timer(void)
-{
-    filter_gui_update_filter_stats();
-    filter_gui_update_rule_stats();
-}
-
-
-
 /**
  * Reset the rule stats for a given rule.
  */
@@ -2644,17 +2723,22 @@ filter_find_by_name_in_session(const gchar *name)
 gboolean
 filter_is_global(const filter_t *f)
 {
-    return f == filter_global_pre || f == filter_global_post;
+	g_return_val_if_fail(f, FALSE);
+    return 0 != (FILTER_FLAG_GLOBAL & f->flags);
 }
 
 gboolean
 filter_is_builtin(const filter_t *f)
 {
-    return	f == filter_show ||
-			f == filter_drop ||
-            f == filter_download ||
-			f == filter_nodownload ||
-            f == filter_return;
+	g_return_val_if_fail(f, FALSE);
+    return 0 != (FILTER_FLAG_BUILTIN & f->flags);
+}
+
+gboolean
+filter_is_modifiable(const filter_t *f)
+{
+	g_return_val_if_fail(f, FALSE);
+    return !((FILTER_FLAG_BUILTIN | FILTER_FLAG_PRESET) & f->flags);
 }
 
 filter_t *
@@ -2708,10 +2792,8 @@ filter_add_drop_sha1_rule(const struct record *rec, filter_t *filter)
     rule_t *rule;
 	gchar *s;
 
-    g_assert(rec != NULL);
+	record_check(rec);
     g_assert(filter != NULL);
-	g_assert(rec->magic == RECORD_MAGIC);
-	g_assert(rec->refcount >= 0 && rec->refcount < INT_MAX);
 
 	s = unknown_to_utf8_normalized(rec->name, UNI_NORM_GUI, FALSE);
     rule = filter_new_sha1_rule(rec->sha1, s,
@@ -2732,10 +2814,8 @@ filter_add_drop_name_rule(const struct record *rec, filter_t *filter)
     rule_t *rule;
 	gchar *s;
 
-    g_assert(rec != NULL);
+	record_check(rec);
     g_assert(filter != NULL);
-	g_assert(rec->magic == RECORD_MAGIC);
-	g_assert(rec->refcount >= 0 && rec->refcount < INT_MAX);
 
 	s = unknown_to_utf8_normalized(rec->name, UNI_NORM_GUI, FALSE);
     rule = filter_new_text_rule(s, RULE_TEXT_EXACT, TRUE,
@@ -2755,10 +2835,8 @@ filter_add_drop_host_rule(const struct record *rec, filter_t *filter)
 {
     rule_t *rule;
 
-    g_assert(rec != NULL);
+	record_check(rec);
     g_assert(filter != NULL);
-	g_assert(rec->magic == RECORD_MAGIC);
-	g_assert(rec->refcount >= 0 && rec->refcount < INT_MAX);
 
     rule = filter_new_ip_rule(rec->results_set->addr, -1,
         		filter_get_drop_target(), RULE_FLAG_ACTIVE);
@@ -2772,10 +2850,8 @@ filter_add_drop_host_rule(const struct record *rec, filter_t *filter)
 void
 filter_add_download_sha1_rule(const struct record *rec, filter_t *filter)
 {
-    g_assert(rec != NULL);
+	record_check(rec);
     g_assert(filter != NULL);
-	g_assert(rec->magic == RECORD_MAGIC);
-	g_assert(rec->refcount >= 0 && rec->refcount < INT_MAX);
 
     if (rec->sha1) {
         rule_t *rule;
@@ -2801,10 +2877,8 @@ filter_add_download_name_rule(const struct record *rec, filter_t *filter)
     rule_t *rule;
 	gchar *s;
 
-    g_assert(rec != NULL);
+	record_check(rec);
     g_assert(filter != NULL);
-	g_assert(rec->magic == RECORD_MAGIC);
-	g_assert(rec->refcount >= 0 && rec->refcount < INT_MAX);
 
 	s = unknown_to_utf8_normalized(rec->name, UNI_NORM_GUI, FALSE);
     rule = filter_new_text_rule(s, RULE_TEXT_EXACT, TRUE,
