@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats_common.c 12952 2007-03-03 10:31:59Z cbiere $
+ * $Id: gnet_stats_common.c 14389 2007-08-09 22:10:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,10 +25,11 @@
 
 #include "gui.h"
 
-RCSID("$Id: gnet_stats_common.c 12952 2007-03-03 10:31:59Z cbiere $")
+RCSID("$Id: gnet_stats_common.c 14389 2007-08-09 22:10:53Z cbiere $")
+
+#include "gtk/notebooks.h"
 
 #include "gnet_stats_common.h"
-#include "gtk-missing.h"
 #include "settings.h"
 
 #include "if/core/net_stats.h"
@@ -225,4 +226,22 @@ gnet_stats_gui_horizon_update(hsep_triple *table, guint32 triples)
 		"%s", short_kb_size(val, show_metric_units()));
 }
 
+static gboolean
+gnet_stats_gui_is_visible(void)
+{
+	return main_gui_window_visible() &&
+		nb_main_page_stats == main_gui_notebook_get_page();
+}
+
+void
+gnet_stats_gui_timer(time_t now)
+{
+	static time_t last_update;
+
+	if (last_update != now && gnet_stats_gui_is_visible()) {
+		last_update = now;
+		gnet_stats_gui_update_display(now);
+	}
+}
+
 /* vi: set ts=4 sw=4 cindent: */
