@@ -1,5 +1,5 @@
 /*
- * $Id: gtk-missing.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: gtk-missing.c 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -37,9 +37,8 @@
 
 #include "gui.h"
 
-RCSID("$Id: gtk-missing.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: gtk-missing.c 14457 2007-08-14 04:23:12Z cbiere $")
 
-#include "gtk-missing.h"
 #include "columns.h"
 
 #include "if/gui_property.h"
@@ -75,15 +74,88 @@ gtk_paned_get_position(GtkPaned *paned)
  * @warning EVIL HACK
  */
 void
-gtk_clist_set_column_name(GtkCList * clist, gint col, gchar * t)
+gtk_clist_set_column_name(GtkCList *clist, int col, const char *title)
 {
-    if (col < 0 || col >= clist->columns)
-        return;
+    if (col >= 0 && col < clist->columns) {
+		G_FREE_NULL(clist->column[col].title);
+		clist->column[col].title = g_strdup(title);
+	}
+}
 
-    if (clist->column[col].title)
-        G_FREE_NULL(clist->column[col].title);
+void
+clist_sync_rows(GtkCList *clist, void (*func)(int, void *))
+{
+	GList *iter;
+	int i;
 
-    clist->column[col].title = g_strdup(t);
+	g_return_if_fail(clist);
+	g_return_if_fail(func);
+	
+	i = 0;
+	for (iter = clist->row_list; NULL != iter; iter = g_list_next(iter), i++) {
+		GtkCListRow *row;
+
+		row = iter->data;
+		(*func)(i, row->data);
+	}
+}
+
+char *
+clist_copy_text(GtkCList *clist, int row, int column)
+{
+	char *text;
+
+	g_return_val_if_fail(clist, NULL);
+	
+	if (
+		row < 0 ||
+		column < 0 ||
+		!gtk_clist_get_text(GTK_CLIST(clist), row, column, &text)
+	) {
+		text = NULL;
+	}
+	return g_strdup(text);
+}
+
+static int
+clist_get_focus_row(GtkCList *clist)
+{
+	g_return_val_if_fail(clist, -1);
+
+	if (
+		GTK_WIDGET_HAS_FOCUS(GTK_WIDGET(clist))
+		&& clist->focus_row >= 0
+		&& clist->focus_row < clist->rows
+	) {
+		return clist->focus_row;
+	}
+	return -1;
+}
+
+static gboolean
+clist_row_is_selected(GtkCList *clist, int row)
+{
+	const GList *node;
+
+	g_return_val_if_fail(clist, FALSE);
+	g_return_val_if_fail(row >= 0, FALSE);
+	g_return_val_if_fail(row < clist->rows, FALSE);
+
+	node = g_list_nth(clist->row_list, row);
+	g_return_val_if_fail(node, FALSE);
+
+	return GTK_STATE_SELECTED == GTK_CLIST_ROW(node)->state;
+}
+
+int
+clist_get_cursor_row(GtkCList *clist)
+{
+	int row;
+
+	g_return_val_if_fail(clist, -1);
+	
+	row = clist_get_focus_row(clist);
+	return row >= 0 && clist_row_is_selected(clist, row) ? row : -1;
 }
 #endif /* USE_GTK1 */
 
@@ -243,6 +315,19 @@ void gtk_label_printf(GtkLabel *label, const gchar *format, ...)
     va_end(args);
 }
 
+
+#ifdef USE_GTK1
+const char *
+gtk_label_get_text(GtkLabel *label)
+{
+	const char *text = NULL;
+
+	g_return_val_if_fail(label, NULL);
+	gtk_label_get(GTK_LABEL(label), (char **) &text);
+	return text;
+}
+#endif	/* USE_GTK1 */
+
 /**
  * Takes an array of strings which are supposed to be widgets names found with
  * the given top-level widget. Sets all to the given sentitivity state.
@@ -289,7 +374,7 @@ clist_collect_data(GtkCList *clist, gboolean allow_null, GCompareFunc cfn)
         if ((data != NULL) || allow_null) {
             if (cfn != NULL) {
                 if (g_slist_find_custom(result_list, data, cfn) != NULL) {
-                    if (gui_debug >= 3) {
+                    if (GUI_PROPERTY(gui_debug) >= 3) {
                         const gchar *name =
                             gtk_widget_get_name(GTK_WIDGET(clist));
                         printf("%s has duplicate data: %p\n",
@@ -303,7 +388,7 @@ clist_collect_data(GtkCList *clist, gboolean allow_null, GCompareFunc cfn)
             result_list = g_slist_prepend(result_list, data);
             to_unselect = g_slist_prepend(to_unselect, GINT_TO_POINTER(row));
         } else {
-            if (gui_debug >= 3) {
+            if (GUI_PROPERTY(gui_debug) >= 3) {
                 const gchar *name = gtk_widget_get_name(GTK_WIDGET(clist));
                 printf("%s contains NULL data in row %d\n",
                     (name != NULL) ? name : "<UNKNOWN>", row);
@@ -400,7 +485,7 @@ tree_selection_collect_data_record(GtkTreeModel *model, GtkTreeIter *iter,
 
 	if (NULL != cdata->cfn &&
 		NULL != g_slist_find_custom(cdata->results, data, cdata->cfn)) {
-		if (gui_debug >= 3)
+		if (GUI_PROPERTY(gui_debug) >= 3)
 			g_warning("%s has duplicate data: %p", cdata->name, data);
 		return;
 	}
@@ -462,7 +547,7 @@ tree_selection_collect_data(GtkTreeSelection *selection,
 	cdata.cfn = cfn;
 	cdata.gdf = gdf;
 	cdata.tv = gtk_tree_selection_get_tree_view(selection);
-	if (gui_debug >= 3) {
+	if (GUI_PROPERTY(gui_debug) >= 3) {
 		cdata.name = gtk_widget_get_name(
 			GTK_WIDGET(gtk_tree_selection_get_tree_view(selection)));
 		if (NULL == cdata.name)
@@ -606,29 +691,85 @@ tree_view_motion_set_callback(GtkTreeView *tv,
 {
 	tree_view_motion_t *tvm;
 
-	g_assert(tv != NULL);
-	g_assert(cb != NULL);
+	g_return_val_if_fail(tv, NULL);
+	g_return_val_if_fail(cb, NULL);
 
 	tvm = g_malloc(sizeof *tvm);
-	tvm->tv = tv;
+	tvm->tv = GTK_TREE_VIEW(g_object_ref(tv));
 	tvm->cb = cb;
 	tvm->timeout_id = g_timeout_add(interval, tree_view_motion_timeout, tvm);
-	tvm->signal_id = g_signal_connect(GTK_OBJECT(tv),
-		"motion-notify-event", G_CALLBACK(on_tree_view_motion_notify), tvm);
+	tvm->signal_id = gui_signal_connect(tv,
+						"motion-notify-event", on_tree_view_motion_notify, tvm);
 	return tvm;
 }
 
 void
-tree_view_motion_clear_callback(GtkTreeView *tv, tree_view_motion_t *tvm)
+tree_view_motion_clear_callback(tree_view_motion_t **ptr)
 {
-	g_assert(tv != NULL);
-	g_assert(tvm != NULL);
+	if (*ptr) {
+		tree_view_motion_t *tvm = *ptr;
 
-	g_signal_handler_disconnect(GTK_OBJECT(tv), tvm->signal_id);
-	g_source_remove(tvm->timeout_id);
-	G_FREE_NULL(tvm);
+		g_signal_handler_disconnect(tvm->tv, tvm->signal_id);
+		g_source_remove(tvm->timeout_id);
+		g_object_unref(tvm->tv);
+		tvm->tv = NULL;
+		G_FREE_NULL(tvm);
+		*ptr = NULL;
+	}
 }
 
+void
+tree_view_set_fixed_height_mode(GtkTreeView *tv, gboolean fixed)
+{
+	g_return_if_fail(tv);
+	
+#if GTK_CHECK_VERSION(2, 4, 0)
+    g_object_set(GTK_TREE_VIEW(tv), "fixed_height_mode", fixed, (void *) 0);
+#endif /* GTK+ >= 2.4.0 */
+}
+
+void
+tree_model_iter_changed(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	GtkTreePath *path;
+
+	path = gtk_tree_model_get_path(model, iter);
+	gtk_tree_model_row_changed(model, path, iter);
+	gtk_tree_path_free(path);
+}
+
+void
+list_store_set_pointer(GtkListStore *store, GtkTreeIter *iter,
+	int column, void *data)
+{
+	static const GValue zero_value;
+	GValue value = zero_value;
+
+	g_value_init(&value, G_TYPE_POINTER);
+	g_value_set_pointer(&value, data);
+	gtk_list_store_set_value(store, iter, column, &value);
+}
+
+void
+list_store_append_pointer(GtkListStore *store, GtkTreeIter *iter,
+	int column, void *data)
+#if GTK_CHECK_VERSION(2,6,0)
+{
+	static const GValue zero_value;
+	GValue value = zero_value;
+	const int row = INT_MAX;
+
+	g_value_init(&value, G_TYPE_POINTER);
+	g_value_set_pointer(&value, data);
+	gtk_list_store_insert_with_valuesv(store, iter, row, &column, &value, 1);
+}
+#else	/* Gtk+ < 2.6 */
+{
+	gtk_list_store_append(store, iter);
+	list_store_set_pointer(store, iter, column, data);
+}
+#endif	/* Gtk+ >= 2.6 */
+
 #endif /* USE_GTK2 */
 
 gdouble
@@ -691,10 +832,9 @@ widget_init_choices(GtkWidget *widget, GtkSignalFunc func,
 
 			gtk_widget_show(list_item);
 
-			gtk_signal_connect_after(GTK_OBJECT(list_item),
-					"select", func, user_data);
+			gui_signal_connect_after(list_item, "select", func, user_data);
 
-			l = g_list_prepend(NULL, (gpointer) list_item);
+			l = g_list_prepend(NULL, list_item);
 			gtk_list_append_items(GTK_LIST(combo->list), l);
 
 			if (def->data.guint32.choices[i].value == original)
@@ -715,14 +855,7 @@ widget_init_choices(GtkWidget *widget, GtkSignalFunc func,
 			gtk_object_set_user_data(GTK_OBJECT(item),
 				GUINT_TO_POINTER(def->data.guint32.choices[i].value));
 			gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-#ifdef USE_GTK1
-			gtk_signal_connect_after(GTK_OBJECT(item), "activate",
-				func, user_data);
-#endif /* USE_GTK1 */
-#ifdef USE_GTK2
-			g_signal_connect_after(GTK_OBJECT(item), "activate",
-				G_CALLBACK(func), user_data);
-#endif /* USE_GTK2 */
+			gui_signal_connect_after(item, "activate", func, user_data);
 		}
 		gtk_option_menu_set_menu(option_menu, GTK_WIDGET(menu));
 	}
@@ -1019,6 +1152,7 @@ gtk_widget_fix_width(GtkWidget *w, GtkWidget *l, guint chars, guint extra)
     max_width = PANGO_PIXELS((gint)
 		(pango_font_metrics_get_approximate_digit_width(pfm) * chars + extra));
 
+	max_width *= 1.2;	/* The above still allocates less than it should */
     gtk_widget_set_size_request(w, max_width, -1);
 
     pango_font_metrics_unref(pfm);
@@ -1066,4 +1200,30 @@ gtk_paned_get_child2(GtkPaned *paned)
 }
 #endif
 
+void
+statusbar_set_shadow_type(GtkStatusbar *sb, GtkShadowType shadow)
+{
+	g_return_if_fail(sb);
+	(void) shadow;
+
+#if !GTK_CHECK_VERSION(2,0,0)
+	gtk_frame_set_shadow_type(GTK_FRAME(sb->frame), shadow);
+#endif	/* Gtk+ 1.2 */
+}
+
+/**
+ * @return TRUE if the runtime Gtk+ version is the given version or newer.
+ */
+gboolean
+check_gtk_version(unsigned major, unsigned minor, unsigned micro)
+{
+	if (gtk_major_version != major)
+		return gtk_major_version > major;
+	if (gtk_minor_version != minor)
+		return gtk_minor_version > minor;
+	if (gtk_micro_version != micro)
+		return gtk_micro_version > micro;
+	return TRUE;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
