@@ -1,5 +1,5 @@
 /*
- * $Id: callbacks.c 13990 2007-06-30 00:59:50Z cbiere $
+ * $Id: callbacks.c 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -25,7 +25,7 @@
 
 #include "common.h"
 
-RCSID("$Id: callbacks.c 13990 2007-06-30 00:59:50Z cbiere $")
+RCSID("$Id: callbacks.c 14457 2007-08-14 04:23:12Z cbiere $")
 
 #include "gtk/gui.h"
 #include "gtk/misc.h"
@@ -59,26 +59,9 @@ static GtkWidget *add_dir_filesel = NULL;
 
 
 /***
- *** Left panel (selection tree)
+ *** Left panel
  ***/
 
-void
-on_ctree_menu_tree_select_row(GtkCTree *ctree, GList *node,
-	gint unused_column, gpointer unused_udata)
-{
-    gint tab;
-
-	(void) unused_column;
-	(void) unused_udata;
-
-    tab = GPOINTER_TO_INT(
-		gtk_ctree_node_get_row_data(ctree, GTK_CTREE_NODE(node)));
-
-	gtk_notebook_set_page(
-		GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")), tab);
-}
-
-
 gboolean
 on_progressbar_bws_in_button_press_event(GtkWidget *unused_widget,
 		GdkEventButton *unused_event, gpointer unused_udata)
@@ -211,15 +194,6 @@ on_button_search_stats_reset_clicked(GtkButton *unused_button,
 	search_stats_gui_reset();
 }
 
-void
-on_clist_search_stats_resize_column(GtkCList *unused_clist, gint column,
-	gint width, gpointer unused_udata)
-{
-	(void) unused_clist;
-	(void) unused_udata;
-	*(gint *) &GUI_PROPERTY(search_stats_col_widths)[column] = width;
-}
-
 /***
  *** Config pane
  ***/
@@ -491,36 +465,4 @@ on_entry_config_netmask_activate(GtkEditable *editable, gpointer unused_data)
 }
 FOCUS_TO_ACTIVATE(entry_config_netmask)
 
-/***
- *** search list (sidebar)
- ***/
-void
-on_clist_search_resize_column(GtkCList *unused_clist, gint column,
-	gint width, gpointer unused_udata)
-{
-	(void) unused_clist;
-	(void) unused_udata;
-    *(gint *) &GUI_PROPERTY(search_list_col_widths)[column] = width;
-}
-
-void
-on_popup_search_browse_host_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-	search_gui_browse_selected();
-}
-
-void
-on_popup_search_sort_defaults_activate(GtkMenuItem *unused_menuitem,
-	gpointer unused_udata)
-{
-	(void) unused_menuitem;
-	(void) unused_udata;
-	
-	search_gui_set_sort_defaults();
-}
-
 /* vi: set ts=4 sw=4 cindent: */
