@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: gnet_stats.c 14389 2007-08-09 22:10:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,11 +25,11 @@
 
 #include "common.h"
 
-RCSID("$Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: gnet_stats.c 14389 2007-08-09 22:10:53Z cbiere $")
 
 #include "gtk/gui.h"
 #include "gtk/gnet_stats.h"
-#include "gtk/notebooks.h"
+#include "gtk/misc.h"
 #include "gtk/settings.h"
 
 #include "if/core/hsep.h"
@@ -42,134 +42,6 @@ RCSID("$Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $")
 #include "lib/tm.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-/***
- *** Callbacks
- ***/
-void
-on_clist_gnet_stats_msg_resize_column(GtkCList *unused_clist, gint column,
-	gint width, gpointer unused_udata)
-{
-    static gboolean lock = FALSE;
-    guint32 buf = width;
-
-	(void) unused_clist;
-	(void) unused_udata;
-
-    if (lock)
-        return;
-
-    lock = TRUE;
-
-    /* remember the width for storing it to the config file later */
-    gui_prop_set_guint32(PROP_GNET_STATS_MSG_COL_WIDTHS, &buf, column, 1);
-
-    lock = FALSE;
-}
-
-
-void
-on_clist_gnet_stats_fc_ttl_resize_column(GtkCList *unused_clist,
-	gint unused_column, gint width, gpointer unused_udata)
-{
-    static gboolean lock = FALSE;
-    guint32 buf[9];
-    guint n;
-
-	(void) unused_clist;
-	(void) unused_column;
-	(void) unused_udata;
-
-    if (lock)
-        return;
-
-    lock = TRUE;
-
-    /* remember the width for storing it to the config file later */
-    for (n = 0; n < G_N_ELEMENTS(buf); n ++)
-        buf[n] = width;
-
-    gui_prop_set_guint32(PROP_GNET_STATS_FC_TTL_COL_WIDTHS,
-		buf, 1, G_N_ELEMENTS(buf));
-
-    lock = FALSE;
-}
-
-void
-on_clist_gnet_stats_fc_hops_resize_column(GtkCList *unused_clist,
-	gint unused_column, gint width, gpointer unused_udata)
-{
-    static gboolean lock = FALSE;
-    guint32 buf[9];
-    guint n;
-
-	(void) unused_clist;
-	(void) unused_column;
-	(void) unused_udata;
-
-    if (lock)
-        return;
-
-    lock = TRUE;
-
-    /* remember the width for storing it to the config file later */
-    for (n = 0; n < G_N_ELEMENTS(buf); n ++)
-        buf[n] = width;
-
-    gui_prop_set_guint32(PROP_GNET_STATS_FC_HOPS_COL_WIDTHS,
-		buf, 1, G_N_ELEMENTS(buf));
-
-    lock = FALSE;
-}
-
-void
-on_clist_gnet_stats_horizon_resize_column(GtkCList *unused_clist,
-	gint column, gint width, gpointer unused_udata)
-{
-    static gboolean lock = FALSE;
-    guint32 buf = width;
-
-	(void) unused_clist;
-	(void) unused_udata;
-
-    if (lock)
-        return;
-
-    lock = TRUE;
-
-    /* remember the width for storing it to the config file later */
-    gui_prop_set_guint32(PROP_GNET_STATS_HORIZON_COL_WIDTHS, &buf, column, 1);
-
-    lock = FALSE;
-}
-
-void
-on_clist_gnet_stats_drop_reasons_resize_column(GtkCList *unused_clist,
-	gint column, gint width, gpointer unused_udata)
-{
-    guint32 buf = width;
-
-	(void) unused_clist;
-	(void) unused_udata;
-
-    /* remember the width for storing it to the config file later */
-    gui_prop_set_guint32(PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS,
-        &buf, column, 1);
-}
-
-void
-on_clist_gnet_stats_general_resize_column(GtkCList *unused_clist, gint column,
-	gint width, gpointer unused_udata)
-{
-    guint32 buf = width;
-
-	(void) unused_clist;
-	(void) unused_udata;
-
-    /* remember the width for storing it to the config file later */
-    gui_prop_set_guint32(PROP_GNET_STATS_GENERAL_COL_WIDTHS,
-        &buf, column, 1);
-}
-
 /***
  *** Private functions
  ***/
@@ -355,6 +227,14 @@ gnet_stats_gui_init(void)
 	gtk_clist_column_titles_passive(clist_general);
 	gtk_clist_column_titles_passive(clist_horizon);
 
+	clist_restore_widths(clist_stats_msg, PROP_GNET_STATS_MSG_COL_WIDTHS);
+	clist_restore_widths(clist_stats_fc_ttl, PROP_GNET_STATS_FC_TTL_COL_WIDTHS);
+	clist_restore_widths(clist_stats_fc_hops,
+		PROP_GNET_STATS_FC_HOPS_COL_WIDTHS);
+	clist_restore_widths(clist_reason, PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS);
+	clist_restore_widths(clist_general, PROP_GNET_STATS_GENERAL_COL_WIDTHS);
+	clist_restore_widths(clist_horizon, PROP_GNET_STATS_HORIZON_COL_WIDTHS);
+
     /*
      * Initialize stats tables.
      */
@@ -394,19 +274,39 @@ gnet_stats_gui_init(void)
 
     guc_hsep_add_global_table_listener(
 		(GCallback) gnet_stats_gui_horizon_update, FREQ_UPDATES, 0);
+
+	main_gui_add_timer(gnet_stats_gui_timer);
 }
 
 void
 gnet_stats_gui_shutdown(void)
 {
 	guc_hsep_remove_global_table_listener(
-	    (GCallback) gnet_stats_gui_horizon_update);
+		(GCallback) gnet_stats_gui_horizon_update);
+	
+	clist_save_widths(
+		GTK_CLIST(gui_main_window_lookup("clist_gnet_stats_msg")),
+		PROP_GNET_STATS_MSG_COL_WIDTHS);
+	clist_save_widths(
+		GTK_CLIST(gui_main_window_lookup("clist_gnet_stats_fc_ttl")),
+		PROP_GNET_STATS_FC_TTL_COL_WIDTHS);
+	clist_save_widths(
+		GTK_CLIST(gui_main_window_lookup("clist_gnet_stats_fc_hops")),
+		PROP_GNET_STATS_FC_HOPS_COL_WIDTHS);
+	clist_save_widths(
+		GTK_CLIST(gui_main_window_lookup("clist_gnet_stats_drop_reasons")),
+		PROP_GNET_STATS_DROP_REASONS_COL_WIDTHS);
+	clist_save_widths(
+		GTK_CLIST(gui_main_window_lookup("clist_gnet_stats_general")),
+		PROP_GNET_STATS_GENERAL_COL_WIDTHS);
+	clist_save_widths(
+		GTK_CLIST(gui_main_window_lookup("clist_gnet_stats_horizon")),
+		PROP_GNET_STATS_HORIZON_COL_WIDTHS);
 }
 
 void
-gnet_stats_gui_update(time_t now)
+gnet_stats_gui_update_display(time_t now)
 {
-	static time_t last_update = 0;
     GtkCList *clist_stats_msg;
     GtkCList *clist_reason;
     GtkCList *clist_general;
@@ -420,17 +320,6 @@ gnet_stats_gui_update(time_t now)
 	gint global_table_size;
 	gnet_stats_t *xstats = NULL;
 
-    gint current_page;
-
-	if (last_update == now)
-		return;
-	last_update = now;
-    current_page = gtk_notebook_get_current_page(
-        GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")));
-
-    if (current_page != nb_main_page_gnet_stats)
-        return;
-
     guc_gnet_stats_get(&stats);
 
     clist_stats_msg = GTK_CLIST(
