@@ -1,5 +1,5 @@
 /*
- * $Id: nodes_cb.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: nodes_cb.c 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -27,7 +27,7 @@
 
 #include "common.h"
 
-RCSID("$Id: nodes_cb.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: nodes_cb.c 14457 2007-08-14 04:23:12Z cbiere $")
 
 #include "gtk/gui.h"
 #include "nodes_cb.h"
@@ -35,8 +35,7 @@ RCSID("$Id: nodes_cb.c 13777 2007-05-29 00:23:11Z cbiere $")
 #include "gtk/gtkcolumnchooser.h"
 #include "gtk/nodes_common.h"
 #include "gtk/settings.h"
-#include "gtk/gtk-missing.h"
-#include "gtk/search_common.h"
+#include "gtk/search.h"
 
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
@@ -81,17 +80,6 @@ on_clist_nodes_unselect_row(GtkCList *clist, gint unused_row,
 	update_sensitivity(clist->selection != NULL);
 }
 
-void
-on_clist_nodes_resize_column(GtkCList *unused_clist, gint column,
-		gint width, gpointer unused_udata)
-{
-	(void) unused_clist;
-	(void) unused_udata;
-	g_assert(UNSIGNED(column) < G_N_ELEMENTS(GUI_PROPERTY(nodes_col_widths)));
-	/* override const */
-    *(guint32 *) &GUI_PROPERTY(nodes_col_widths)[column] = width;
-}
-
 gboolean
 on_clist_nodes_button_press_event(GtkWidget *unused_widget,
 		GdkEventButton *event, gpointer unused_udata)
@@ -206,7 +194,8 @@ on_popup_nodes_config_cols_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     cc = gtk_column_chooser_new(gui_main_window_lookup("clist_nodes"));
-    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1, 0);
+    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1,
+		gtk_get_current_event_time());
 
     /* GtkColumnChooser takes care of cleaning up itself */
 }
