@@ -1,5 +1,5 @@
 /*
- * $Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: gnet_stats.c 14389 2007-08-09 22:10:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,11 +25,10 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: gnet_stats.c 14389 2007-08-09 22:10:53Z cbiere $")
 
 #include "gtk/gnet_stats.h"
 #include "gtk/notebooks.h"
-#include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
 #include "gtk/settings.h"
 
@@ -42,14 +41,13 @@ RCSID("$Id: gnet_stats.c 13777 2007-05-29 00:23:11Z cbiere $")
 #include "lib/glib-missing.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-static GtkTreeView *treeview_gnet_stats_messages = NULL;
-static GtkTreeView *treeview_gnet_stats_drop_reasons = NULL;
-static GtkTreeView *treeview_gnet_stats_flowc = NULL;
-static GtkTreeView *treeview_gnet_stats_recv = NULL;
-static GtkTreeView *treeview_gnet_stats_general = NULL;
-static GtkTreeView *treeview_gnet_stats_horizon = NULL;
-static GtkNotebook *notebook_main = NULL;
-static GtkNotebook *notebook_gnet_stats = NULL;
+static GtkTreeView *treeview_gnet_stats_messages;
+static GtkTreeView *treeview_gnet_stats_drop_reasons;
+static GtkTreeView *treeview_gnet_stats_flowc;
+static GtkTreeView *treeview_gnet_stats_recv;
+static GtkTreeView *treeview_gnet_stats_general;
+static GtkTreeView *treeview_gnet_stats_horizon;
+static GtkNotebook *notebook_gnet_stats;
 
 static const gchar * const msg_stats_label[] = {
 	N_("Type"),
@@ -189,7 +187,7 @@ add_column(GtkTreeView *treeview, gint column_id, gint width, gfloat xalign,
 		"min-width", 1,
 		"reorderable", TRUE,
 		"resizable", TRUE,
-		"sizing", GTK_TREE_VIEW_COLUMN_AUTOSIZE,
+		"sizing", GTK_TREE_VIEW_COLUMN_FIXED,
 		(void *) 0);
 	gtk_tree_view_append_column(treeview, column);
 }
@@ -318,14 +316,13 @@ gnet_stats_update_recv(const gnet_stats_t *stats)
 }
 
 static void
-gnet_stats_update_horizon(void)
+gnet_stats_update_horizon(time_t now)
 {
+	static time_t last_horizon_update;
 	GtkTreeView *treeview = treeview_gnet_stats_horizon;
 	GtkListStore *store;
 	GtkTreeIter iter;
 	gint i;
-	static time_t last_horizon_update = 0;
-	time_t now = tm_time();
 	gint global_table_size;
 
 	/*
@@ -404,6 +401,7 @@ gnet_stats_gui_horizon_init(void)
 	}
 	gtk_tree_view_set_model(treeview, model);
 	g_object_unref(model);
+	tree_view_set_fixed_height_mode(treeview, TRUE);
 }
 
 static void
@@ -455,6 +453,7 @@ gnet_stats_gui_flowc_init(void)
 	}
 	gtk_tree_view_set_model(treeview, model);
 	g_object_unref(model);
+	tree_view_set_fixed_height_mode(treeview, TRUE);
 }
 
 static void
@@ -492,6 +491,7 @@ gnet_stats_gui_drop_reasons_init(void)
 	}
 	gtk_tree_view_set_model(treeview, model);
 	g_object_unref(model);
+	tree_view_set_fixed_height_mode(treeview, TRUE);
 }
 
 static void
@@ -528,6 +528,7 @@ gnet_stats_gui_general_init(void)
 	}
 	gtk_tree_view_set_model(treeview, model);
 	g_object_unref(model);
+	tree_view_set_fixed_height_mode(treeview, TRUE);
 }
 
 static void
@@ -577,6 +578,7 @@ gnet_stats_gui_messages_init(void)
 
 	gtk_tree_view_set_model(treeview, model);
 	g_object_unref(model);
+	tree_view_set_fixed_height_mode(treeview, TRUE);
 }
 
 static void
@@ -640,7 +642,6 @@ gnet_stats_gui_recv_init(void)
 void
 gnet_stats_gui_init(void)
 {
-	notebook_main = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
 	notebook_gnet_stats = GTK_NOTEBOOK(
 							gui_main_window_lookup("gnet_stats_notebook"));
 
@@ -657,6 +658,8 @@ gnet_stats_gui_init(void)
 
 	guc_hsep_add_global_table_listener(
 		(GCallback) gnet_stats_gui_horizon_update, FREQ_UPDATES, 0);
+
+	main_gui_add_timer(gnet_stats_gui_timer);
 }
 
 void
@@ -690,22 +693,11 @@ gnet_stats_gui_shutdown(void)
 }
 
 void
-gnet_stats_gui_update(time_t now)
+gnet_stats_gui_update_display(time_t now)
 {
 	static gnet_stats_t stats;
-	static gboolean locked = FALSE;
-	static time_t last_update = 0;
 	gint current_page;
 
-	if (last_update == now || locked)
-		return;
-	last_update = now;
-	locked = TRUE;
-
-	current_page = gtk_notebook_get_current_page(notebook_main);
-	if (current_page != nb_main_page_gnet_stats)
-		goto cleanup;
-
 	guc_gnet_stats_get(&stats);
 
 	current_page = gtk_notebook_get_current_page(notebook_gnet_stats);
@@ -713,10 +705,10 @@ gnet_stats_gui_update(time_t now)
 	case GNET_STATS_NB_PAGE_STATS:
 		gnet_stats_update_general(&stats);
 		gnet_stats_update_drop_reasons(&stats);
-		goto cleanup;
+		break;
 	case GNET_STATS_NB_PAGE_HORIZON:
-		gnet_stats_update_horizon();
-		goto cleanup;
+		gnet_stats_update_horizon(now);
+		break;
 	case GNET_STATS_NB_PAGE_MESSAGES:
 		switch (GUI_PROPERTY(gnet_stats_source)) {
 		case GNET_STATS_FULL:
@@ -731,21 +723,16 @@ gnet_stats_gui_update(time_t now)
 			g_assert_not_reached();
 		}
 		gnet_stats_update_messages(&stats);
-		goto cleanup;
+		break;
 	case GNET_STATS_NB_PAGE_FLOWC:
 		gnet_stats_update_flowc(&stats);
-		goto cleanup;
+		break;
 	case GNET_STATS_NB_PAGE_RECV:
 		gnet_stats_update_recv(&stats);
-		goto cleanup;
-
+		break;
 	case NUM_GNET_STATS_NB_PAGES:
 		break;
 	}
-	g_assert_not_reached();
-
-cleanup:
-	locked = FALSE;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
