@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.c 13870 2007-06-18 03:27:25Z cbiere $
+ * $Id: nodes.c 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,11 +36,8 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: nodes.c 13870 2007-06-18 03:27:25Z cbiere $")
+RCSID("$Id: nodes.c 14457 2007-08-14 04:23:12Z cbiere $")
 
-#include "interface-glade.h"
-
-#include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
 #include "gtk/nodes_common.h"
 #include "gtk/nodes.h"
@@ -48,7 +45,7 @@ RCSID("$Id: nodes.c 13870 2007-06-18 03:27:25Z cbiere $")
 #include "gtk/notebooks.h"
 #include "gtk/settings.h"
 #include "gtk/statusbar.h"
-#include "gtk/search_common.h"
+#include "gtk/search.h"
 
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
@@ -59,9 +56,8 @@ RCSID("$Id: nodes.c 13870 2007-06-18 03:27:25Z cbiere $")
 #include "lib/iso3166.h"
 #include "lib/utf8.h"
 #include "lib/walloc.h"
-#include "lib/override.h"	/* Must be the last header included */
 
-#define UPDATE_MIN	300		/**< Update screen every 5 minutes at least */
+#include "lib/override.h"	/* Must be the last header included */
 
 /*
  * These hash tables record which information about which nodes has
@@ -546,14 +542,14 @@ nodes_gui_early_init(void)
 void
 nodes_gui_init(void)
 {
-	treeview_nodes = GTK_TREE_VIEW(gui_main_window_lookup( "treeview_nodes"));
+	GtkTreeView *tv;
+	
+	tv = GTK_TREE_VIEW(gui_main_window_lookup( "treeview_nodes"));
+	treeview_nodes = tv;
 
-	tree_view_restore_widths(treeview_nodes, PROP_NODES_COL_WIDTHS);
-	tree_view_restore_visibility(treeview_nodes, PROP_NODES_COL_VISIBLE);
-
-#if GTK_CHECK_VERSION(2, 4, 0)
-    g_object_set(treeview_nodes, "fixed_height_mode", TRUE, (void *) 0);
-#endif /* GTK+ >= 2.4.0 */
+	tree_view_restore_widths(tv, PROP_NODES_COL_WIDTHS);
+	tree_view_restore_visibility(tv, PROP_NODES_COL_VISIBLE);
+	tree_view_set_fixed_height_mode(tv, TRUE);
 
 	nodes_handles = g_hash_table_new(node_id_hash, node_id_eq_func);
     ht_node_info_changed = g_hash_table_new(node_id_hash, node_id_eq_func);
@@ -565,14 +561,13 @@ nodes_gui_init(void)
     guc_node_add_node_info_changed_listener(nodes_gui_node_info_changed);
     guc_node_add_node_flags_changed_listener(nodes_gui_node_flags_changed);
 
-	g_signal_connect(GTK_OBJECT(treeview_nodes), "cursor-changed",
-		G_CALLBACK(on_cursor_changed), treeview_nodes);
+	widget_add_popup_menu(GTK_WIDGET(tv), nodes_gui_get_popup_menu);
+	gui_signal_connect(tv, "cursor-changed", on_cursor_changed, tv);
+	gui_signal_connect(tv, "leave-notify-event", on_leave_notify, tv);
 
-	g_signal_connect(GTK_OBJECT(treeview_nodes), "leave-notify-event",
-		G_CALLBACK(on_leave_notify), treeview_nodes);
+	tvm_nodes = tree_view_motion_set_callback(tv, update_tooltip, 400);
 
-	tvm_nodes = tree_view_motion_set_callback(treeview_nodes,
-					update_tooltip, 400);
+	main_gui_add_timer(nodes_gui_timer);
 }
 
 /**
@@ -581,11 +576,7 @@ nodes_gui_init(void)
 void
 nodes_gui_shutdown(void)
 {
-	if (tvm_nodes) {
-		tree_view_motion_clear_callback(treeview_nodes, tvm_nodes);
-		tvm_nodes = NULL;
-	}
-
+	tree_view_motion_clear_callback(&tvm_nodes);
 	tree_view_save_widths(treeview_nodes, PROP_NODES_COL_WIDTHS);
 	tree_view_save_visibility(treeview_nodes, PROP_NODES_COL_VISIBLE);
 
@@ -732,6 +723,8 @@ update_row(gpointer key, gpointer value, gpointer user_data)
 			memcpy(data->info, s, size);
 		}
 	}
+
+	tree_model_iter_changed(GTK_TREE_MODEL(nodes_model), &data->iter);
 }
 
 /**
@@ -746,63 +739,11 @@ update_row(gpointer key, gpointer value, gpointer user_data)
  */
 
 void
-nodes_gui_update_nodes_display(time_t now)
+nodes_gui_update_display(time_t now)
 {
-#define DO_FREEZE FALSE
-	static const gboolean do_freeze = DO_FREEZE;
-    static time_t last_update = 0;
-	gint current_page;
-	static GtkNotebook *notebook = NULL;
-    GtkTreeModel *model;
-
-	if (GUI_PROPERTY(gui_debug) > 0) {
-    	g_message("recorded changed: flags: %d info: %d",
-        	g_hash_table_size(ht_node_flags_changed),
-        	g_hash_table_size(ht_node_info_changed));
-	}
-
-    if (delta_time(now, last_update) < 2)
-        return;
-
-	/*
-	 * Usually don't perform updates if nobody is watching.  However,
-	 * we do need to perform periodic cleanup of dead entries or the
-	 * memory usage will grow.  Perform an update every UPDATE_MIN minutes
-	 * at least.
-	 *		--RAM, 28/12/2003
-	 */
-
-	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
-
-    current_page = gtk_notebook_get_current_page(notebook);
-	if (
-		current_page != nb_main_page_gnet &&
-		delta_time(now, last_update) < UPDATE_MIN
-	) {
-		return;
-	}
-
-    last_update = now;
-
-	if (do_freeze) {
-    	/* "Freeze" view */
-    	model = gtk_tree_view_get_model(treeview_nodes);
-    	g_object_ref(model);
-    	gtk_tree_view_set_model(treeview_nodes, NULL);
-	} else {
-		model = NULL; /* dumb compiler */
-	}
-
+	g_object_freeze_notify(G_OBJECT(treeview_nodes));
 	g_hash_table_foreach(nodes_handles, update_row, &now);
-
-	if (do_freeze) {
-    	/* "Thaw" view */
-    	gtk_tree_view_set_model(treeview_nodes, model);
-    	g_object_unref(model);
-	} else {
-    	gtk_widget_queue_draw(GTK_WIDGET(treeview_nodes));
-	}
+	g_object_thaw_notify(G_OBJECT(treeview_nodes));
 }
 
 /***
