@@ -1,5 +1,5 @@
 /*
- * $Id: search_stats.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: search_stats.c 14389 2007-08-09 22:10:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
@@ -46,10 +46,9 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: search_stats.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: search_stats.c 14389 2007-08-09 22:10:53Z cbiere $")
 
 #include "gtk/search_stats.h"
-#include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
 
 #include "if/gui_property.h"
@@ -259,9 +258,9 @@ static void
 search_stats_gui_disable(void)
 {
     if (callback_registered) {
-        guc_share_remove_search_request_listener(search_stats_notify_word);
-        guc_share_remove_search_request_listener(search_stats_notify_whole);
-        guc_share_remove_search_request_listener(search_stats_notify_routed);
+        guc_search_request_listener_remove(search_stats_notify_word);
+        guc_search_request_listener_remove(search_stats_notify_whole);
+        guc_search_request_listener_remove(search_stats_notify_routed);
         callback_registered = FALSE;
     }
 
@@ -366,6 +365,48 @@ add_column(GtkTreeView *treeview, gint id, gfloat xalign,
 	gtk_tree_view_column_set_sort_column_id(column, id);
 }
 
+/**
+ * Display the data gathered during the last time period.
+ * Perhaps it would be better to have this done on a button click(?)
+ */
+static void
+search_stats_gui_update_display(void)
+{
+	stat_count = 0;
+	gtk_list_store_clear(store_search_stats);
+	g_object_freeze_notify(G_OBJECT(treeview_search_stats));
+	/* insert the hash table contents into the sorted treeview */
+	g_hash_table_foreach_remove(stat_hash, stats_hash_to_treeview, NULL);
+	g_object_thaw_notify(G_OBJECT(treeview_search_stats));
+
+	/* update the counter */
+	gtk_label_printf(GTK_LABEL(label_search_stats_count),
+		NG_("%u term counted", "%u terms counted", stat_count),
+		stat_count);
+}
+
+static gboolean
+search_stats_gui_is_visible(void)
+{
+	return main_gui_window_visible() &&
+		nb_main_page_search_stats == main_gui_notebook_get_page();
+}
+
+static void
+search_stats_gui_timer(time_t now)
+{
+	static time_t last_update;
+	time_delta_t interval = GUI_PROPERTY(search_stats_update_interval);
+
+    if (
+		search_stats_gui_is_visible() &&
+		delta_time(now, last_update) > interval
+	) {
+    	last_update = now;
+		search_stats_gui_update_display();
+	}
+}
+
 void
 search_stats_gui_init(void)
 {
@@ -406,8 +447,10 @@ search_stats_gui_init(void)
 		add_column(treeview, cols[i].id, cols[i].align, _(cols[i].title));
 	}
 	tree_view_restore_widths(treeview, PROP_SEARCH_STATS_COL_WIDTHS);
+	tree_view_set_fixed_height_mode(treeview, TRUE);
 
 	stat_hash = g_hash_table_new(NULL, NULL);
+	main_gui_add_timer(search_stats_gui_timer);
 }
 
 void
@@ -419,32 +462,4 @@ search_stats_gui_shutdown(void)
 	stat_hash = NULL;
 }
 
-/**
- * Display the data gathered during the last time period.
- * Perhaps it would be better to have this done on a button click(?)
- */
-void
-search_stats_gui_update(time_t now)
-{
-	static time_t last_update = 0;
-	const time_delta_t interval = GUI_PROPERTY(search_stats_update_interval);
-
-    if (delta_time(now, last_update) < interval)
-        return;
-
-    last_update = now;
-
-	stat_count = 0;
-	gtk_list_store_clear(store_search_stats);
-	g_object_freeze_notify(G_OBJECT(treeview_search_stats));
-	/* insert the hash table contents into the sorted treeview */
-	g_hash_table_foreach_remove(stat_hash, stats_hash_to_treeview, NULL);
-	g_object_thaw_notify(G_OBJECT(treeview_search_stats));
-
-	/* update the counter */
-	gtk_label_printf(GTK_LABEL(label_search_stats_count),
-		NG_("%u term counted", "%u terms counted", stat_count),
-		stat_count);
-}
-
 /* vi: set ts=4 sw=4 cindent: */
