@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.c 13057 2007-03-11 23:04:51Z cbiere $
+ * $Id: upload_stats.c 14425 2007-08-12 17:05:22Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
@@ -56,12 +56,11 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: upload_stats.c 13057 2007-03-11 23:04:51Z cbiere $")
+RCSID("$Id: upload_stats.c 14425 2007-08-12 17:05:22Z cbiere $")
 
 #include "interface-glade.h" /* for create_popup_upload_stats */
 
 #include "gtk/columns.h"
-#include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
 #include "gtk/settings.h"
 
@@ -349,10 +348,8 @@ upload_stats_gui_init_intern(gboolean intern)
 				cell_renderer_func);
 		}
 
-		g_signal_connect(GTK_OBJECT(upload_stats_treeview),
-			"button_press_event",
-			G_CALLBACK(on_button_press_event),
-			NULL);
+		gui_signal_connect(upload_stats_treeview,
+			"button_press_event", on_button_press_event, NULL);
 	}
 
 	if (!intern) {
@@ -413,6 +410,20 @@ upload_stats_gui_init(void)
 	upload_stats_gui_init_intern(FALSE);
 }
 
+void
+upload_stats_gui_freeze(void)
+{
+	upload_stats_gui_init_intern(TRUE);
+	g_object_freeze_notify(G_OBJECT(upload_stats_treeview));
+}
+
+void
+upload_stats_gui_thaw(void)
+{
+	g_return_if_fail(upload_stats_treeview);
+	g_object_thaw_notify(G_OBJECT(upload_stats_treeview));
+}
+
 /**
  * Update the visible statistics for a given file.
  *
