@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.h 6139 2004-08-24 22:14:43Z rmanfredi $
+ * $Id: hcache.h 14389 2007-08-09 22:10:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -29,13 +29,7 @@
 #include "gui.h"
 #include "hcache_common.h"
 
-#ifdef USE_GTK1
-void on_clist_hcache_resize_column(
-    GtkCList *clist, gint column, gint width, gpointer user_data);
-#endif /* USE_GTK1 */
-
 void hcache_gui_init(void);
-void hcache_gui_update(time_t now);
 void hcache_gui_shutdown(void);
 
 #endif /* _gtk_hcache_h_ */
