@@ -1,5 +1,5 @@
 /*
- * $Id: main.c 13990 2007-06-30 00:59:50Z cbiere $
+ * $Id: main.c 14967 2007-10-06 20:37:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -36,19 +36,12 @@
 
 #include "gui.h"
 
-RCSID("$Id: main.c 13990 2007-06-30 00:59:50Z cbiere $")
+RCSID("$Id: main.c 14967 2007-10-06 20:37:53Z cbiere $")
 
 #ifdef I_PWD
 #include <pwd.h>
 #endif /* I_PWD */
 
-#ifdef USE_GTK1
-#include "gtk1/interface-glade.h"
-#endif
-#ifdef USE_GTK2
-#include "gtk2/interface-glade.h"
-#endif
-
 #include "notebooks.h"
 #include "main.h"
 #include "misc.h"
@@ -67,12 +60,9 @@ RCSID("$Id: main.c 13990 2007-06-30 00:59:50Z cbiere $")
 #include "icon.h"
 #include "filter_cb.h"
 #include "filter_core.h"
-#include "upload_stats_cb.h" /* FIXME: remove dependency (compare_ul_norm) */
 #include "fileinfo.h"
 #include "visual_progress.h"
 
-#include "gtk-missing.h"
-
 #include "gtk/misc.h"
 
 #include "if/bridge/ui2c.h"
@@ -115,9 +105,9 @@ WIDGET(main_window)
 WIDGET(popup_downloads)
 WIDGET(popup_monitor)
 WIDGET(popup_nodes)
-WIDGET(popup_queue)
 WIDGET(popup_search)
 WIDGET(popup_search_list)
+WIDGET(popup_sources)
 WIDGET(popup_uploads)
 WIDGET(shutdown_window)
 #undef WIDGET
@@ -141,175 +131,145 @@ gui_init_window_title(void)
 	gtk_window_set_title(GTK_WINDOW(gui_main_window()), title);
 }
 
-/**
- * The contents of the navigation tree menu in exact order.
- */
-static const struct {
-	const gint	depth;	/**< Depth in tree */
-	const gchar *title; /**< Translatable title for the node */
-	const gint	page;	/**< Page reference ("the target") for the node */
-} menu[] = {
-	{   0,	 N_("GnutellaNet"),		nb_main_page_gnet },
-	{     1, N_("Stats"),			nb_main_page_gnet_stats },
-	{     1, N_("Hostcache"),		nb_main_page_hostcache },
-	{   0,	 N_("Uploads"),			nb_main_page_uploads },
-	{     1, N_("History"), 		nb_main_page_uploads_stats },
-#ifdef USE_GTK1
-	{   0,	 N_("Downloads"),		nb_main_page_dl_files },
-	{     1, N_("Active"),			nb_main_page_dl_active },
-	{     1, N_("Queue"),			nb_main_page_dl_queue },
-#endif /* USE_GTK1 */
-#ifdef USE_GTK2
-	{   0,	 N_("Downloads"),		nb_main_page_downloads },
-#endif /* USE_GTK1 */
-	{   0,	 N_("Search"),			nb_main_page_search },
-	{     1, N_("Monitor"),			nb_main_page_monitor },
-	{     1, N_("Stats"),			nb_main_page_search_stats },
-};
+static GSList *visibility_listeners[nb_main_page_num];
+static int notebook_main_current_page;
+static gboolean main_window_is_visible;
 
+gboolean
+main_gui_window_visible(void)
+{
+	return main_window_is_visible;
+}
 
-#ifdef USE_GTK2
+static void
+main_gui_page_visibility_change(int page_num, gboolean visible)
+{
+	GSList *iter;
+
+	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
+
+	iter = visibility_listeners[page_num];
+	for (/* NOTHING */; NULL != iter; iter = g_slist_next(iter)) {
+		main_gui_visibility_cb func = iter->data;
+
+		g_assert(func);
+		(*func)(visible);
+	}
+}
 
 static gboolean
-gui_init_menu_helper(GtkTreeModel *model, GtkTreePath *path,
-	GtkTreeIter *iter, gpointer data)
+on_main_gui_map_event(GtkWidget *unused_widget,
+	GdkEvent *event, gpointer unused_udata)
 {
-	static const GValue zero_value;
-	GValue value;
-	guint32 expanded;
-	gint id;
+	const gboolean was_visible = main_window_is_visible;
 
-	value = zero_value;
-	gtk_tree_model_get_value(model, iter, 1, &value);
-	id = GPOINTER_TO_UINT(g_value_get_pointer(&value));
-	
-	gui_prop_get_guint32(PROP_TREEMENU_NODES_EXPANDED, &expanded, id, 1);
-	if (expanded)
-		gtk_tree_view_expand_row(GTK_TREE_VIEW(data), path, FALSE);
-	return FALSE;
-}
+	(void) unused_widget;
+	(void) unused_udata;
 
-static void
-gui_init_menu(void)
-{
-	static GType types[] = {
-		G_TYPE_STRING,	/* Label */
-		G_TYPE_POINTER,	/* Notebook page number (casted to a pointer) */
-	};
-	GtkTreeView	*treeview;
-	GtkTreeIter	parent, iter;
-	GtkTreeStore *store;
-	GtkTreeViewColumn *column;
-    GtkCellRenderer *renderer;
-	guint i;
-	gint depth = -1;
+	switch (event->type) {
+	case GDK_MAP:
+		main_window_is_visible = TRUE;
+		break;
 
-	STATIC_ASSERT(G_N_ELEMENTS(types) == 2);
+	case GDK_UNMAP:
+		main_window_is_visible = FALSE;
+		break;
 
-    renderer = gtk_cell_renderer_text_new();
-    g_object_set(renderer, "ypad", GUI_CELL_RENDERER_YPAD, (void *) 0);
-	treeview = GTK_TREE_VIEW(gui_main_window_lookup("treeview_menu"));
-	store = gtk_tree_store_newv(G_N_ELEMENTS(types), types);
+#if GTK_CHECK_VERSION(2,0,0)
+	case GDK_WINDOW_STATE:
+		main_window_is_visible = !(event->window_state.new_window_state
+				& (GDK_WINDOW_STATE_WITHDRAWN | GDK_WINDOW_STATE_ICONIFIED));
+		break;
+#endif	/* Gtk+ >= 2.0 */
 
-	for (i = 0; i < G_N_ELEMENTS(menu); i++) {
-		if (depth < menu[i].depth) {
-			g_assert(menu[i].depth == depth + 1);
-	
-			parent = iter;
-			depth = menu[i].depth;
-		} else if (depth > menu[i].depth) {
-			do {
-				gboolean valid;
-				
-				valid = gtk_tree_model_iter_parent(GTK_TREE_MODEL(store),
-							&parent, &iter);	
-				g_assert(valid);
-				iter = parent;
-			} while (--depth > menu[i].depth);
-			depth = menu[i].depth;
-		}
-		
-		gtk_tree_store_append(store, &iter, depth > 0 ? &parent : NULL);
-		gtk_tree_store_set(store, &iter,
-				0, _(menu[i].title),
-				1, GUINT_TO_POINTER(menu[i].page),
-				(-1));
+	default:
+		break;
 	}
-
-	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(store));
-
-	column = gtk_tree_view_column_new_with_attributes(
-				NULL, renderer, "text", 0, (void *) 0);
-    gtk_tree_view_column_set_resizable(column, TRUE);
-    gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-    gtk_tree_view_append_column(treeview, column);
-	gtk_tree_view_columns_autosize(treeview);
-
-	gtk_tree_model_foreach(GTK_TREE_MODEL(store),
-		gui_init_menu_helper, treeview);
-
-	g_object_unref(store);
-
-	gui_signal_connect(treeview, "cursor-changed",
-		on_main_gui_treeview_menu_cursor_changed, NULL);
-	gui_signal_connect(treeview, "row-collapsed",
-		on_main_gui_treeview_menu_row_collapsed, NULL);
-	gui_signal_connect(treeview, "row-expanded",
-		on_main_gui_treeview_menu_row_expanded, NULL);
-}
-
-static void
-gui_menu_shutdown(void)
-{
-	g_signal_handlers_disconnect_by_func(
-		G_OBJECT(gui_main_window_lookup("treeview_menu")),
-		on_main_gui_treeview_menu_cursor_changed,
-		NULL);
-	g_signal_handlers_disconnect_by_func(
-		G_OBJECT(gui_main_window_lookup("treeview_menu")),
-		on_main_gui_treeview_menu_row_collapsed,
-		NULL);
-	g_signal_handlers_disconnect_by_func(
-		G_OBJECT(gui_main_window_lookup("treeview_menu")),
-		on_main_gui_treeview_menu_row_expanded,
-		NULL);
-}
-
-#else
-
-static void
-gui_init_menu(void)
-{
-    GtkCTree *ctree_menu = GTK_CTREE(gui_main_window_lookup("ctree_menu"));
-	GtkCTreeNode *parent_node = NULL;
-	guint i;
-
-	for (i = 0; i < G_N_ELEMENTS(menu); i++) {
-		GtkCTreeNode *node;
-		const gchar *title[1];
-
-		title[0] = _(menu[i].title);
-    	node = gtk_ctree_insert_node(ctree_menu,
-					menu[i].depth == 0 ? NULL : parent_node, NULL,
-					(gchar **) title, /* Override const */
-					0, NULL, NULL, NULL, NULL, FALSE, TRUE);
-		if (i == 0 || menu[i].depth < menu[i - 1].depth)
-			parent_node = node;
-
-    	gtk_ctree_node_set_row_data(ctree_menu, node,
-			GINT_TO_POINTER(menu[i].page));
+	if (was_visible != main_window_is_visible) {
+		main_gui_page_visibility_change(notebook_main_current_page,
+			main_window_is_visible);
 	}
-
-	gtk_clist_select_row(GTK_CLIST(ctree_menu), 0, 0);
+	return FALSE;	/* propagate further */
 }
 
 static void
-gui_menu_shutdown(void)
+on_notebook_main_switch_page(GtkNotebook *unused_notebook,
+	GtkNotebookPage *unused_page, int page_num, void *unused_udata)
 {
-	/* NOTHING */
+	int old_page;
+
+	(void) unused_notebook;
+	(void) unused_page;
+	(void) unused_udata;
+
+	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
+
+	old_page = notebook_main_current_page;
+	notebook_main_current_page = page_num;
+
+	if (main_window_is_visible) {
+		main_gui_page_visibility_change(old_page, FALSE);
+		main_gui_page_visibility_change(notebook_main_current_page, TRUE);
+	}
 }
-	
-#endif /* USE_GTK2 */
+
+void
+main_gui_add_page_visibility_listener(main_gui_visibility_cb func,
+	int page_num)
+{
+	g_return_if_fail(func);
+	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
+
+	visibility_listeners[page_num] = g_slist_append(
+										visibility_listeners[page_num], func);
+}
+
+void
+main_gui_remove_page_visibility_listener(main_gui_visibility_cb func,
+	int page_num)
+{
+	g_return_if_fail(func);
+	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
+
+	visibility_listeners[page_num] = g_slist_remove(
+										visibility_listeners[page_num], func);
+}
+
+int
+main_gui_notebook_get_page(void)
+{
+	g_assert(UNSIGNED(notebook_main_current_page) < nb_main_page_num);
+	return notebook_main_current_page;
+}
+
+void
+main_gui_notebook_set_page(int page_num)
+{
+	g_return_if_fail(UNSIGNED(page_num) < nb_main_page_num);
+	gtk_notebook_set_current_page(
+		GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")),
+		page_num);
+}
+
+#ifdef USE_GTK2
+static const gchar *
+notebook_main_page_label(gint page)
+{
+	g_return_val_if_fail(UNSIGNED(page) < nb_main_page_num, NULL);
+	switch (page) {
+	case nb_main_page_network:			return _("Network");
+	case nb_main_page_search:			return _("Searches");
+	case nb_main_page_downloads:		return _("Downloads");
+	case nb_main_page_uploads:			return _("Uploads");
+	case nb_main_page_stats:			return _("Statistics");
+	case nb_main_page_monitor:			return _("Search Monitor");
+	case nb_main_page_uploads_stats:	return _("Upload History");
+	case nb_main_page_hostcache:		return _("Hostcache");
+	case nb_main_page_search_stats:		return _("Search Stats");
+	}
+	return NULL;
+}
+#endif	/* Gtk+ >= 2.0 */
 
 /**
  * Handles main window UI joining.
@@ -329,11 +289,12 @@ gui_init_main_window(void)
 	 * First create the main window without the tab contents.
 	 */
 	notebook = gui_main_window_lookup("notebook_main");
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), TRUE);
 
 	/*
 	 * Then create all the tabs in their own window.
 	 */
-	tab_window[nb_main_page_gnet] = create_main_window_gnet_tab();
+	tab_window[nb_main_page_network] = create_main_window_gnet_tab();
 	tab_window[nb_main_page_uploads] = create_main_window_uploads_tab();
 	tab_window[nb_main_page_uploads_stats] =
 		create_main_window_upload_stats_tab();
@@ -344,7 +305,7 @@ gui_init_main_window(void)
 	tab_window[nb_main_page_monitor] = create_main_window_monitor_tab();
 	tab_window[nb_main_page_search_stats] =
 		create_main_window_search_stats_tab();
-	tab_window[nb_main_page_gnet_stats] = create_main_window_gnet_stats_tab();
+	tab_window[nb_main_page_stats] = create_main_window_gnet_stats_tab();
 	tab_window[nb_main_page_hostcache] = create_main_window_hostcache_tab();
 
 	/*
@@ -354,6 +315,7 @@ gui_init_main_window(void)
 		GtkWidget *w = tab_window[i];
 		gui_merge_window_as_tab(gui_main_window(), notebook, w);
 		gtk_object_destroy(GTK_OBJECT(w));
+
 	}
 
 	/*
@@ -364,7 +326,21 @@ gui_init_main_window(void)
 	gtk_container_remove(GTK_CONTAINER(notebook),
 		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 0));
 
+	for (i = 0; i < nb_main_page_num; i++) {
+		gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(notebook),
+			gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), i),
+			notebook_main_page_label(i));
+	}
+
+	gui_signal_connect(gui_main_window(), "window-state-event",
+		on_main_gui_map_event, NULL);
+
 #endif	/* USE_GTK2 */
+
+	gui_signal_connect(gui_main_window(), "map-event",
+		on_main_gui_map_event, NULL);
+	gui_signal_connect(gui_main_window(), "unmap-event",
+		on_main_gui_map_event, NULL);
 }
 
 static void 
@@ -423,68 +399,68 @@ text_widget_append(GtkWidget *widget, const gchar *line)
 static void
 gui_init_dlg_about(void)
 {
-    /* NB: These strings are UTF-8 encoded. */
-    static const char * const contributors[] = {
-        "Yann Grossel <olrick@users.sourceforge.net>",
-        "Steven Wilcoxon <swilcoxon@users.sourceforge.net>",
-        "Jason Lingohr <lingman@users.sourceforge.net>",
-        "Brian St Pierre <bstpierre@users.sourceforge.net>",
-        "Chuck Homic <homic@users.sourceforge.net>",
-        "Ingo Saitz <salz@users.sourceforge.net>",
-        "Ben Hochstedler <hochstrb@users.sourceforge.net>",
-        "Daniel Walker <axiom@users.sourceforge.net>",
-        "Paul Cassella <pwc@users.sourceforge.net>",
-        "Jared Mauch <jaredmauch@users.sourceforge.net>",
-        "Nate E <web1 (at) users dot sourceforge dot net>",
-        "Rapha\303\253l Manfredi <Raphael_Manfredi@pobox.com>",
-        "Kenn Brooks Hamm <khamm@andrew.cmu.edu>",
-        "Mark Schreiber <mark7@andrew.cmu.edu>",
-        "Sam Varshavchik <mrsam@courier-mta.com>",
-        "Vladimir Klebanov <unny@rz.uni-karlsruhe.de>",
-        "Roman Shterenzon <roman@xpert.com>",
-        "Robert Bihlmeyer <robbe@orcus.priv.at>",
-        "Noel T.Nunkovich <ntnunk@earthlink.net>",
-        "Michael Tesch <tesch@users.sourceforge.net>",
-        "Markus 'guruz' Goetz <guruz@guruz.info>",
-        "Richard Eckart <wyldfire@users.sourceforge.net>",
-        "Christophe Tronche <ch.tronche@computer.org>",
-        "Alex Bennee <alex@bennee.com>",
-        "Mike Perry <mikepery@fscked.org>",
-        "Zygo Blaxell <zblaxell@feedme.hungrycats.org>",
-        "Vidar Madsen <vidar@gimp.org>",
-        "Christian Biere <christianbiere@gmx.de>",
-        "ko <junkpile@free.fr>",
-        "Jeroen Asselman <jeroen@asselman.com>",
-        "T'aZ <tazdev@altern.org>",
-        "Andrew Barnert <barnert@myrealbox.com>",
-        "Michael Gray <mrgray01@louisville.edu>",
-        "Nicol\303\241s Lichtmaier <nick@technisys.com.ar>",
-        "Rafael R. Reilova <rreilova@magepr.net>",
-        "Stephane Corbe <noubi@users.sourceforge.net>",
-        "Emile le Vivre <emile@struggle.ca>",
-        "Angelo Cano <angelo_cano@fastmail.fm>",
-        "Thomas Schuerger <thomas@schuerger.com>",
-        "Russell Francis <rf358197@ohio.edu>",
-        "Richard Hyde <email@richardhyde.net>",
-        "Thadeu Lima de Souza Cascardo <cascardo@dcc.ufmg.br>",
-        "Paco Arjonilla <pacoarjonilla@yahoo.es>",
-        "Clayton Rollins <clayton.rollins@asu.edu>",
-        "Hans de Graaff <hans@degraaff.org>",
-        "Globuz",
-		"Daichi Kawahata <daichik@users.sourceforge.net>",
-		"Dave Rutherford <polymath69@users.sourceforge.net>",
-		"Ian Sheldon <shellgeekorguk@users.sourceforge.net>",
-		"Bill Pringlemeir <bpringle@sympatico.ca>",
-		"Bouklis Panos <panos@echidna-band.com>",
-		"Sulyok P\303\251ter <sp@elte.hu>",
-		"Alexander N. S\303\270rnes <alex@thehandofagony.com>",
-		"Vitaliy Buyar <vetal18@users.sourceforge.net>",
-		"Wu Xiaoguang <wxgnj@yahoo.com.cn>",
-		"Lorenzo Gaifas <lorenzo@artiemestieri.tn.it>",
-		"FlashCode <flashcode@flashtux.org>",
-		"U\304\237ur \303\207etin <ugur.jnmbk@gmail.com>",
-		"Lloyd Bryant <lloydbaz@msn.com>",
-		"Martijn van Oosterhout <kleptog@svana.org>",
+	/* NB: These strings are UTF-8 encoded. */
+	static const char * const contributors[] = {
+		"Yann Grossel <olrick\100users.sourceforge.net>",
+		"Steven Wilcoxon <swilcoxon\100users.sourceforge.net>",
+		"Jason Lingohr <lingman\100users.sourceforge.net>",
+		"Brian St Pierre <bstpierre\100users.sourceforge.net>",
+		"Chuck Homic <homic\100users.sourceforge.net>",
+		"Ingo Saitz <salz\100users.sourceforge.net>",
+		"Ben Hochstedler <hochstrb\100users.sourceforge.net>",
+		"Daniel Walker <axiom\100users.sourceforge.net>",
+		"Paul Cassella <pwc\100users.sourceforge.net>",
+		"Jared Mauch <jaredmauch\100users.sourceforge.net>",
+		"Nate E <web1 (at) users dot sourceforge dot net>",
+		"Rapha\303\253l Manfredi <Raphael_Manfredi\100pobox.com>",
+		"Kenn Brooks Hamm <khamm\100andrew.cmu.edu>",
+		"Mark Schreiber <mark7\100andrew.cmu.edu>",
+		"Sam Varshavchik <mrsam\100courier-mta.com>",
+		"Vladimir Klebanov <unny\100rz.uni-karlsruhe.de>",
+		"Roman Shterenzon <roman\100xpert.com>",
+		"Robert Bihlmeyer <robbe\100orcus.priv.at>",
+		"Noel T.Nunkovich <ntnunk\100earthlink.net>",
+		"Michael Tesch <tesch\100users.sourceforge.net>",
+		"Markus 'guruz' Goetz <guruz\100guruz.info>",
+		"Richard Eckart <wyldfire\100users.sourceforge.net>",
+		"Christophe Tronche <ch.tronche\100computer.org>",
+		"Alex Bennee <alex\100bennee.com>",
+		"Mike Perry <mikepery\100fscked.org>",
+		"Zygo Blaxell <zblaxell\100feedme.hungrycats.org>",
+		"Vidar Madsen <vidar\100gimp.org>",
+		"Christian Biere <christianbiere\100gmx.de>",
+		"ko <junkpile\100free.fr>",
+		"Jeroen Asselman <jeroen\100asselman.com>",
+		"T'aZ <tazdev\100altern.org>",
+		"Andrew Barnert <barnert\100myrealbox.com>",
+		"Michael Gray <mrgray01\100louisville.edu>",
+		"Nicol\303\241s Lichtmaier <nick\100technisys.com.ar>",
+		"Rafael R. Reilova <rreilova\100magepr.net>",
+		"Stephane Corbe <noubi\100users.sourceforge.net>",
+		"Emile le Vivre <emile\100struggle.ca>",
+		"Angelo Cano <angelo_cano\100fastmail.fm>",
+		"Thomas Schuerger <thomas\100schuerger.com>",
+		"Russell Francis <rf358197\100ohio.edu>",
+		"Richard Hyde <email\100richardhyde.net>",
+		"Thadeu Lima de Souza Cascardo <cascardo\100dcc.ufmg.br>",
+		"Paco Arjonilla <pacoarjonilla\100yahoo.es>",
+		"Clayton Rollins <clayton.rollins\100asu.edu>",
+		"Hans de Graaff <hans\100degraaff.org>",
+		"Globuz",
+		"Daichi Kawahata <daichik\100users.sourceforge.net>",
+		"Dave Rutherford <polymath69\100users.sourceforge.net>",
+		"Ian Sheldon <shellgeekorguk\100users.sourceforge.net>",
+		"Bill Pringlemeir <bpringle\100sympatico.ca>",
+		"Bouklis Panos <panos\100echidna-band.com>",
+		"Sulyok P\303\251ter <sp\100elte.hu>",
+		"Alexander N. S\303\270rnes <alex\100thehandofagony.com>",
+		"Vitaliy Buyar <vetal18\100users.sourceforge.net>",
+		"Wu Xiaoguang <wxgnj\100yahoo.com.cn>",
+		"Lorenzo Gaifas <lorenzo\100artiemestieri.tn.it>",
+		"FlashCode <flashcode\100flashtux.org>",
+		"U\304\237ur \303\207etin <ugur.jnmbk\100gmail.com>",
+		"Lloyd Bryant <lloydbaz\100msn.com>",
+		"Martijn van Oosterhout <kleptog\100svana.org>",
     };
 	GtkWidget *text;
     guint i;
@@ -518,7 +494,7 @@ gui_init_dlg_about(void)
  *
  * Where the last one can overrule settings from earlier resource files.
  */
-void
+static void
 main_gui_gtkrc_init(void)
 {
 #ifdef USE_GTK2
@@ -559,6 +535,16 @@ main_gui_gtkrc_init(void)
  *** Public functions
  ***/
 
+void
+main_gui_show_prefences(void)
+{
+	g_return_if_fail(gui_dlg_prefs());
+	
+    gtk_widget_show(gui_dlg_prefs());
+	gui_restore_window(gui_dlg_prefs(), PROP_PREFS_DLG_COORDS);
+	gdk_window_raise(gui_dlg_prefs()->window);
+}
+
 /**
  * Some setup of the gui side which I wanted out of main.c but must be done
  * before the backend can be initialized since the core code is not free of
@@ -600,20 +586,14 @@ main_gui_early_init(gint argc, gchar **argv, gboolean disable_xshm)
 	gui_popup_search_set(create_popup_search());
 	gui_popup_search_list_set(create_popup_search_list());
 
-#if defined(USE_GTK2)
 	gui_popup_downloads_set(create_popup_downloads());
-#elif defined(USE_GTK1)
-	gui_popup_downloads_set(create_popup_dl_active());
-	gui_popup_queue_set(create_popup_dl_queued());
-#endif /* USE_GTK1 */
-
+	gui_popup_sources_set(create_popup_sources());
 	gui_popup_monitor_set(create_popup_monitor());
 
 	gui_popup_nodes_set(create_popup_nodes());
     nodes_gui_early_init();
 
     gui_popup_uploads_set(create_popup_uploads());
-    uploads_gui_early_init();
     statusbar_gui_init();
 
 	gui_init_window_title();
@@ -622,9 +602,13 @@ main_gui_early_init(gint argc, gchar **argv, gboolean disable_xshm)
     gtk_combo_disable_activate(GTK_COMBO(
 		gui_main_window_lookup("combo_search")));
 
-    /* copy url selection stuff */
-    gtk_selection_add_target(gui_popup_downloads(),
-		GDK_SELECTION_PRIMARY, GDK_SELECTION_TYPE_STRING, 1);
+	clipboard_attach(gui_main_window());
+}
+
+void
+main_gui_exit(int n)
+{
+	gtk_exit(n);
 }
 
 void
@@ -644,43 +628,16 @@ main_gui_init(void)
 
     gtk_clist_column_titles_passive(
         GTK_CLIST(gui_main_window_lookup("clist_search")));
-    gtk_clist_set_compare_func(
-        GTK_CLIST(gui_main_window_lookup("clist_ul_stats")),
-        compare_ul_norm);
 #endif /* USE_GTK1 */
 
 #ifdef USE_GTK2
 	GTK_WINDOW(gui_main_window())->allow_shrink = TRUE;
 #endif /* USE_GTK2 */
 
-#ifdef USE_GTK1
-    /* FIXME: those gtk_widget_set_sensitive should become obsolete when
-     * all property-change callbacks are set up properly
-     */
-	gtk_widget_set_sensitive
-        (gui_popup_downloads_lookup("popup_downloads_remove_file"), FALSE);
-    gtk_widget_set_sensitive
-        (gui_popup_downloads_lookup("popup_downloads_copy_url"), FALSE);
-	gtk_widget_set_sensitive
-        (gui_popup_queue_lookup("popup_queue_abort"), FALSE);
-	gtk_widget_set_sensitive
-        (gui_popup_queue_lookup("popup_queue_abort_named"), FALSE);
-	gtk_widget_set_sensitive
-        (gui_popup_queue_lookup("popup_queue_abort_host"), FALSE);
-    gtk_widget_set_sensitive(
-        gui_popup_downloads_lookup("popup_downloads_push"),
-    	!gtk_toggle_button_get_active(
-            GTK_TOGGLE_BUTTON
-                (lookup_widget(gui_main_window(),
-                               "checkbutton_downloads_never_push"))));
-#endif /* USE_GTK1 */
-
     settings_gui_init();
     fi_gui_init();
-	downloads_gui_init();
     vp_gui_init();
     nodes_gui_init();
-    gui_init_menu();
     hcache_gui_init();
     gnet_stats_gui_init();
     search_stats_gui_init();
@@ -689,6 +646,7 @@ main_gui_init(void)
     /* Must come before search_init() so searches/filters can be loaded.*/
 	filter_init();
     search_gui_init();
+	filter_init_presets();
     filter_update_targets(); /* Make sure the default filters are ok */
     monitor_gui_init();
 	gui_update_files_scanned();
@@ -700,7 +658,8 @@ main_gui_run(const gchar *geometry_spec)
 {
 	time_t now = tm_time_exact();
 
-    gtk_widget_show(gui_main_window());		/* Display the main window */
+    gtk_widget_show_now(gui_main_window());		/* Display the main window */
+	gtk_widget_map(gui_main_window());
 
 	if (geometry_spec) {
 		guint32 coord[4] = { 0, 0, 0, 0 };
@@ -712,7 +671,7 @@ main_gui_run(const gchar *geometry_spec)
 		}
 	}
 	gui_restore_window(gui_main_window(), PROP_WINDOW_COORDS);
- 
+
     icon_init();
     main_gui_timer(now);
 
@@ -721,71 +680,40 @@ main_gui_run(const gchar *geometry_spec)
         gui_main_window_lookup("label_statusbar_uptime"),
         8, 8);
 
-	gui_signal_connect(gui_main_window_lookup("notebook_main"), "switch-page",
-		on_notebook_main_switch_page, NULL);
-
 	/*
 	 * Make sure the application starts in the Gnet pane.
 	 */
 
-	gtk_notebook_set_page(
-		GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")),
-		nb_main_page_gnet);
-
-#ifdef USE_GTK1
-	{
-		GtkCTree *ctree_menu =
-			GTK_CTREE(gui_main_window_lookup("ctree_menu"));
-		GtkCTreeNode *node;
-
-		node = gtk_ctree_find_by_row_data(ctree_menu,
-			gtk_ctree_node_nth(ctree_menu, 0),
-			GINT_TO_POINTER(nb_main_page_gnet));
-
-		if (node != NULL)
-			gtk_ctree_select(ctree_menu, node);
-	}
-#endif /* USE_GTK1 */
+	gui_signal_connect(GTK_NOTEBOOK(gui_main_window_lookup("notebook_main")),
+		"switch-page", on_notebook_main_switch_page, NULL);
+	main_gui_notebook_set_page(nb_main_page_network);
 
 	settings_gui_restore_panes();
     gtk_main();
 }
 
+static slist_t *timers;
+
 void
-main_gui_shutdown(void)
+main_gui_add_timer(main_gui_timer_cb func)
 {
-	icon_close();
+	g_return_if_fail(func);
 
-    /*
-     * Discard all changes and close the dialog.
-     */
+	if (NULL == timers) {
+		timers = slist_new();
+	}
+	g_return_if_fail(!slist_contains_identical(timers, func));
 
-    filter_close_dialog(FALSE);
-	gtk_widget_hide(gui_main_window());
-	if (gui_dlg_prefs())
-		gtk_widget_hide(gui_dlg_prefs());
-
-	gui_menu_shutdown();
-    search_stats_gui_shutdown();
-    filter_cb_close();
-    monitor_gui_shutdown();
-    search_gui_flush(tm_time(), TRUE);
-    search_gui_shutdown(); /* must be done before filter_shutdown! */
- 	downloads_gui_shutdown();
-	filter_shutdown();
-	vp_gui_shutdown();
-    fi_gui_shutdown();
-    nodes_gui_shutdown();
-    uploads_gui_shutdown();
-    upload_stats_gui_shutdown();
-	gnet_stats_gui_shutdown();
-    hcache_gui_shutdown();
+	slist_append(timers, func);
 }
 
 void
-main_gui_update_coords(void)
+main_gui_remove_timer(main_gui_timer_cb func)
 {
-	gui_save_window(gui_main_window(), PROP_WINDOW_COORDS);
+	g_return_if_fail(func);
+	g_return_if_fail(timers);
+
+	slist_remove(timers, func);
 }
 
 /**
@@ -794,9 +722,8 @@ main_gui_update_coords(void)
 void
 main_gui_timer(time_t now)
 {
-	static const gint num_states = 10;
 	gboolean overloaded;
-	gint i;
+	size_t length;
 
 	gnet_prop_get_boolean_val(PROP_OVERLOADED_CPU, &overloaded);
 
@@ -808,26 +735,15 @@ main_gui_timer(time_t now)
 	 * updated every second.
 	 */
 
-	for (i = 0; i < num_states; i++) {
-		static guint counter = 0;
+	length = timers ? slist_length(timers) : 0;
+	while (length-- > 0) {
+		main_gui_timer_cb func;
 
-		switch (counter) {
-		case 0: hcache_gui_update(now);					break;
-		case 1: gnet_stats_gui_update(now);				break;
-		case 2: search_stats_gui_update(now);			break;
-		case 3: nodes_gui_update_nodes_display(now);	break;
-		case 4: uploads_gui_update_display(now);		break;
-		case 5: fi_gui_update_display(now);				break;
-		case 6: statusbar_gui_clear_timeouts(now);		break;
-		case 7: filter_timer();							break;
-		case 8: downloads_gui_update_display(now);		break;
-		case 9: search_gui_flush(now, FALSE);			break;
-		default:
-			g_error("bad modulus computation (counter is %d)", counter);
-			break;
+		func = slist_shift(timers);
+		if (func) {
+			slist_append(timers, func);
+			(*func)(now);
 		}
-		counter = (counter + 1) % num_states;
-
 		if (overloaded)
 			break;
 	}
@@ -849,4 +765,42 @@ main_gui_shutdown_tick(guint left)
     gtk_main_flush();
 }
 
+void
+main_gui_shutdown(void)
+{
+	unsigned i;
+	
+	gui_save_window(gui_main_window(), PROP_WINDOW_COORDS);
+
+	for (i = 0; i < G_N_ELEMENTS(visibility_listeners); i++) {
+		g_slist_free(visibility_listeners[i]);
+		visibility_listeners[i] = NULL;
+	}
+	slist_free(&timers);
+
+	icon_close();
+
+    /*
+     * Discard all changes and close the dialog.
+     */
+
+    filter_close_dialog(FALSE);
+	gtk_widget_hide(gui_main_window());
+	if (gui_dlg_prefs()) {
+		gtk_widget_hide(gui_dlg_prefs());
+	}
+    search_stats_gui_shutdown();
+    filter_cb_close();
+    monitor_gui_shutdown();
+    search_gui_shutdown(); /* must be done before filter_shutdown! */
+	filter_shutdown();
+	vp_gui_shutdown();
+    fi_gui_shutdown();
+    nodes_gui_shutdown();
+    uploads_gui_shutdown();
+    upload_stats_gui_shutdown();
+	gnet_stats_gui_shutdown();
+    hcache_gui_shutdown();
+}
+
 /* vi: set ts=4 sw=4 cindent: */
