@@ -1,5 +1,5 @@
 /*
- * $Id: main.h 13180 2007-03-21 17:46:22Z cbiere $
+ * $Id: main.h 14389 2007-08-09 22:10:53Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -45,20 +45,36 @@ WIDGET(popup_downloads)
 WIDGET(popup_filter_rule)
 WIDGET(popup_monitor)
 WIDGET(popup_nodes)
-WIDGET(popup_queue)
 WIDGET(popup_search)
 WIDGET(popup_search_list)
+WIDGET(popup_sources)
 WIDGET(popup_uploads)
 WIDGET(shutdown_window)
 #undef WIDGET
 
 void main_gui_early_init(gint argc, gchar **argv, gboolean disable_xshm);
 void main_gui_init(void);
+void main_gui_exit(int n);
 void main_gui_run(const gchar *geometry_spec);
 void main_gui_shutdown(void);
 void main_gui_timer(time_t now);
-void main_gui_update_coords(void);
 void main_gui_shutdown_tick(guint);
+void main_gui_show_prefences(void);
+
+gboolean main_gui_window_visible(void);
+
+typedef void (*main_gui_visibility_cb)(gboolean visible);
+
+void main_gui_add_page_visibility_listener(main_gui_visibility_cb, int page);
+void main_gui_remove_page_visibility_listener(main_gui_visibility_cb, int page);
+
+int main_gui_notebook_get_page(void);
+void main_gui_notebook_set_page(int);
+
+typedef void (*main_gui_timer_cb)(time_t);
+
+void main_gui_add_timer(main_gui_timer_cb func);
+void main_gui_remove_timer(main_gui_timer_cb func);
 
 #endif /* _gtk_main_h_ */
 /* vi: set ts=4 sw=4 cindent: */
