@@ -1,5 +1,5 @@
 /*
- * $Id: misc.c 13908 2007-06-20 23:31:17Z cbiere $
+ * $Id: misc.c 14675 2007-08-23 18:42:13Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi & Richard Eckart
  *
@@ -36,15 +36,13 @@
 
 #include "gui.h"
 
-RCSID("$Id: misc.c 13908 2007-06-20 23:31:17Z cbiere $")
+RCSID("$Id: misc.c 14675 2007-08-23 18:42:13Z cbiere $")
 
 #include "gtk-shared/callbacks.h"
 
-#include "nodes.h"
-#include "downloads.h"
+#include "gtk/misc.h"
+
 #include "settings.h"
-#include "search.h"
-#include "gtk-missing.h"
 
 #include "if/gnet_property.h"
 #include "if/gui_property.h"
@@ -54,6 +52,7 @@ RCSID("$Id: misc.c 13908 2007-06-20 23:31:17Z cbiere $")
 #include "if/bridge/ui2c.h"
 
 #include "lib/glib-missing.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 /*
@@ -88,13 +87,13 @@ gui_allow_rescan_dir(gboolean flag)
 void
 gui_general_timer(time_t now)
 {
-	static GtkLabel *label = NULL;
+	static GtkLabel *label;
 	const gchar *uptime;
 	time_t start;
 
-	if (label == NULL)
+	if (label == NULL) {
 		label = GTK_LABEL(gui_main_window_lookup("label_statusbar_uptime"));
-
+	}
 	gnet_prop_get_timestamp_val(PROP_START_STAMP, &start);
 	uptime = short_uptime(delta_time(now, start));
 
@@ -102,7 +101,7 @@ gui_general_timer(time_t now)
 	{
 		gchar buf[128];
 
-		gm_snprintf(buf, sizeof buf, "<tt> %s </tt>", uptime);
+		gm_snprintf(buf, sizeof buf, "<tt>%s</tt>", uptime);
 		gtk_label_set_use_markup(label, TRUE);
 		gtk_label_set_markup(label, buf);
 	}
@@ -271,80 +270,6 @@ gui_update_stats_frames(void)
         gtk_widget_hide(frame_bws_glinout);
 }
 
-/**
- * Tells if two hit records have the same filename.
- */
-gint
-gui_record_name_eq(gconstpointer rec1, gconstpointer rec2)
-{
-    gint result;
-
-    result = strcmp(((const record_t *) rec1)->name,
-       ((const record_t *) rec2)->name);
-
-	if (GUI_PROPERTY(gui_debug) > 4)
-    	g_message("[%s] == [%s] -> %d\n", ((const record_t *) rec1)->name,
-			((const record_t *) rec2)->name, result);
-
-    return result;
-}
-
-/**
- * Tells if two hit records have the same SHA1.
- */
-gint
-gui_record_sha1_eq(gconstpointer p1, gconstpointer p2)
-{
-	const record_t *a = p1, *b = p2;
-
-    if (a->sha1 == b->sha1)
-        return 0;
-
-    if (a->sha1 == NULL || b->sha1 == NULL)
-		return 1;
-
-    return memcmp(a->sha1, b->sha1, SHA1_RAW_SIZE);
-}
-
-/**
- * Tells if two hit records come from the same host.
- */
-gint
-gui_record_host_eq(gconstpointer rec1, gconstpointer rec2)
-{
-	const record_t *r1 = rec1, *r2 = rec2;
-    return !host_addr_equal(r1->results_set->addr, r2->results_set->addr);
-}
-
-/**
- * Tells if two hit records have the same SHA1 or the same name.
- *
- * The targetted search feature by Andrew Meredith (andrew@anvil.org)
- * now uses this function to filter input and avoid duplicates.
- * Andrew, if this somehow breaks the intent, let me know at
- * junkpile@free.fr.
- *
- * This provides the following behavior :
- *
- * - If several hits with the same SHA1 are selected, only one SHA1 rule
- *   will be added even if the filenames differ (same as before).
- *
- * - If several hits with the same filename and no SHA1 are selected,
- *   only one filename rule will be added.
- *
- * - If two selected hits have the same filename, but one has an SHA1
- *   and the other doesn't, both rules (filename and SHA1) will be added.
- *
- */
-gint
-gui_record_sha1_or_name_eq(gconstpointer rec1, gconstpointer rec2)
-{
-    if (((const record_t *) rec1)->sha1 || ((const record_t *) rec2)->sha1)
-        return gui_record_sha1_eq(rec1, rec2);
-    else
-        return gui_record_name_eq(rec1, rec2);
-}
-
 /**
  * If the given coordinates are not reasonable, they're appriopriately
  * adjusted.
@@ -358,7 +283,7 @@ gui_fix_coords(guint32 *coord)
 	gint x, y, w, h;
 	gint screen_w, screen_h;
 
-	g_assert(coord != NULL);
+	g_return_if_fail(coord);
 
 	screen_w = gdk_screen_width();
 	screen_h = gdk_screen_height();
@@ -394,11 +319,11 @@ gui_fix_coords(guint32 *coord)
 }
 
 /**
- * Parses an geometry string of the form WIDTHxHEIGHT+X+Y. If the string
+ * Parses an geometry string of the form [WIDTHxHEIGHT][+X+Y]. If the string
  * is not parsable a failure occurs. In case of failure, the coord[] array
  * is not touched. The coord[] array should be initialized with default
- * values before calling this function because the x and y coordinates
- * are optional and are not touched if not specified.
+ * values before calling this function because the coordinates and the
+ * dimensions are both optional and are not touched if omitted.
  *
  * @param spec  The string holding the geometry specification.
  * @param coord An array of four 32-bit integers representing
@@ -422,18 +347,22 @@ gui_parse_geometry_spec(const gchar *spec, guint32 coord[4])
 	h = coord[3];
 
 	s = spec;
-	w = parse_uint32(s, &endptr, 10, &error);
-	if (error || 'x' != endptr[0]) {
-		return -1;
+
+	if (is_ascii_digit(s[0])) {
+		w = parse_uint32(s, &endptr, 10, &error);
+		if (error || 'x' != endptr[0]) {
+			return -1;
+		}
+
+		s = &endptr[1];
+		h = parse_uint32(s, &endptr, 10, &error);
+		if (error) {
+			return -1;
+		}
+		s = endptr;
 	}
 
-	s = &endptr[1];
-	h = parse_uint32(s, &endptr, 10, &error);
-	if (error) {
-		return -1;
-	}
-
-	switch (endptr[0]) {
+	switch (s[0]) {
 	case '-':
 		sign = -1;
 		break;
@@ -445,17 +374,18 @@ gui_parse_geometry_spec(const gchar *spec, guint32 coord[4])
 	default:
 		return -1;
 	}
+	s++;
 
-	s = &endptr[1];
 	x = parse_uint32(s, &endptr, 10, &error);
 	if (error) {
 		return -1;
 	}
+	s = endptr;
 	if (sign < 0) {
 		x = gdk_screen_width() - 1 - x - w;
 	}
 
-	switch (endptr[0]) {
+	switch (s[0]) {
 	case '-':
 		sign = -1;
 		break;
@@ -465,8 +395,8 @@ gui_parse_geometry_spec(const gchar *spec, guint32 coord[4])
 	default:
 		return -1;
 	}
+	s++;
 
-	s = &endptr[1];
 	y = parse_uint32(s, &endptr, 10, &error);
 	if (error) {
 		return -1;
@@ -483,13 +413,94 @@ done:
 	return 0;
 }
 
+#if !GTK_CHECK_VERSION(2,0,0)
+static void
+gtk_window_get_position(GtkWindow *widget, int *x, int *y)
+{
+	g_assert(widget);
+	gdk_window_get_root_origin(GTK_WIDGET(widget)->window, x, y);
+}
+
+static void
+gtk_window_get_size(GtkWindow *widget, int *width, int *height)
+{
+	g_assert(widget);
+	gdk_window_get_size(GTK_WIDGET(widget)->window, width, height);
+}
+
+static void
+gtk_window_move(GtkWindow *widget, int x, int y)
+{
+	g_return_if_fail(widget);
+	gdk_window_move(GTK_WIDGET(widget)->window, x, y);
+}
+#endif	/* Gtk+ < 2.x */
+
+static void
+gui_window_move_resize(GtkWidget *widget, int x, int y, int width, int height)
+{
+	g_return_if_fail(widget);
+#if GTK_CHECK_VERSION(2,0,0)
+	gtk_window_move(GTK_WINDOW(widget), x, y);
+	gtk_window_resize(GTK_WINDOW(widget), width, height);
+#else
+	gdk_window_move_resize(GTK_WIDGET(widget)->window, x, y, width, height);
+#endif	/* Gtk+ 2.x */
+}
+
+
+static void
+anti_window_shift_hack(GtkWidget *widget, int x, int y, int width, int height)
+{
+	int ax, ay, dx, dy;
+	
+	/* First, move the window to the supposed location. Next make the
+	 * window visible by gtk_window_get_position()... */
+
+	gui_window_move_resize(widget, x, y, width, height);
+
+	ax = x;
+	ay = y;
+	gtk_window_get_position(GTK_WINDOW(widget), &ax, &ay);
+
+	/*
+	 * (At least) FVWM2 doesn't take the window decoration into account
+	 * when handling positions requests. Readjust the window position
+	 * if we detect that the window manager added an offset.
+	 */
+
+	dx = ax - x;
+	dy = ay - y;
+	if (0 == dx && 0 == dy)
+		return;
+
+	if (abs(dx) > 64 || abs(dy) > 64)
+		return;
+	
+	g_message("anti_window_shift_hack: "
+		"x=%d, y=%d, ax=%d, ay=%d , dx=%d, dy=%d",
+			x, y, ax, ay, dx, dy);
+
+	gtk_window_move(GTK_WINDOW(widget), x - dx, y - dy);
+	gtk_window_get_position(GTK_WINDOW(widget), &ax, &ay);
+	if (ax == x && ay == y)
+		return;
+		
+	g_message("anti_window_shift_hack failed: ax=%d, ay=%d", ax, ay);
+}
+
 void
 gui_restore_window(GtkWidget *widget, property_t prop)
 {
     guint32 coord[4] = { 0, 0, 0, 0 };
+	int x, y, width, height;
 
     gui_prop_get_guint32(prop, coord, 0, G_N_ELEMENTS(coord));
 	gui_fix_coords(coord);
+	x = coord[0];
+	y = coord[1];
+	width = coord[2];
+	height = coord[3];
 
     /*
      * We need to tell Gtk the size of the window, otherwise we'll get
@@ -498,99 +509,21 @@ gui_restore_window(GtkWidget *widget, property_t prop)
      *      -- Richard, 8/9/2002
      */
 
-    gtk_window_set_default_size(GTK_WINDOW(widget), coord[2], coord[3]);
-
-
-#ifdef USE_GTK2
-    if (coord[2] != 0 && coord[3] != 0) {
-
-		/* First, move the window to the supposed location. Next make the
-		 * window visible by gtk_window_get_position()... */
-
-       	gtk_window_move(GTK_WINDOW(widget), coord[0], coord[1]);
-		gtk_window_resize(GTK_WINDOW(widget), coord[2], coord[3]);
-
-		{
-			gint x, y, dx, dy;
-			gint i;
-
-			/* The first call to gtk_window_get_position() makes the window
-			 * visible but x and y are always set to zero. The second call
-			 * yields the *real* values.
-			 *
-			 * Must wait some time between subsequent calls, and if we get both
-			 * x and y set to 0, assume we did not correctly compute the
-			 * position and leave it unaltered..
-			 */
-
-			for (i = 0; i < 2; i++) {
-#ifdef HAS_USLEEP
-				usleep(20000);
-#endif
-				gtk_window_get_position(GTK_WINDOW(widget), &x, &y);
-				if (x || y)
-					break;
-			}
-
-			/*
-			 * (At least) FVWM2 doesn't take the window decoration into account
-			 * when handling positions requests. Readjust the window position
-			 * if we detect that the window manager added an offset.
-			 */
-
-			dx = (gint) coord[0] - x;
-			dy = (gint) coord[1] - y;
-			if ((x || y) && (dx || dy)) {
-				gtk_window_move(GTK_WINDOW(widget),
-					coord[0] + dx, coord[1] + dy);
-			}
-		}
+    gtk_window_set_default_size(GTK_WINDOW(widget), width, height);
+    if (width != 0 && height != 0) {
+		anti_window_shift_hack(widget, x, y, width, height);
 	}
-#else	/* !USE_GTK2 */
-    if (coord[2] != 0 && coord[3] != 0) {
-        gdk_window_move_resize(widget->window,
-			coord[0], coord[1], coord[2], coord[3]);
 
-		/* This causes a wandering window -- make it optional */
-		{
-			gint x, y, dx, dy;
-			gint i;
-
-			/* (At least) FVWM2 doesn't take the window decoration into account
-			 * when handling positions requests. Readjust the window position
-			 * if we detect that the window manager added an offset.
-			 */
-
-			for (i = 0; i < 2; i++) {
-#ifdef HAS_USLEEP
-				usleep(20000);
-#endif
-				gdk_window_get_root_origin(widget->window, &x, &y);
-				if (x || y)
-					break;
-			}
-			dx = (gint) coord[0] - x;
-			dy = (gint) coord[1] - y;
-			if ((x || y) && (dx || dy))
-				gdk_window_move(widget->window, coord[0] + dx, coord[1] + dy);
-		}
-	}
-#endif /* USE_GTK2 */
 }
 
 void
 gui_save_window(GtkWidget *widget, property_t prop)
 {
     guint32 coord[4] = { 0, 0, 0, 0};
-	gint x, y, w, h;
+	int x, y, w, h;
 
-#ifdef USE_GTK1
-	gdk_window_get_root_origin(widget->window, &x, &y);
-	gdk_window_get_size(widget->window, &w, &h);
-#else	/* !USE_GTK1 */
 	gtk_window_get_position(GTK_WINDOW(widget), &x, &y);
 	gtk_window_get_size(GTK_WINDOW(widget), &w, &h);
-#endif /* USE_GTK1 */
 	coord[0] = x;
 	coord[1] = y;
 	coord[2] = w;
@@ -624,10 +557,10 @@ static void
 gui_steal_widget_dict_recursive(GtkWidget *widget, gpointer user_data)
 {
 	const gchar *name;
-	steal_dict_params_t *params = (steal_dict_params_t *) user_data;
+	steal_dict_params_t *params = user_data;
 
-	g_assert(widget != NULL);
-	g_assert(user_data != NULL);
+	g_return_if_fail(widget);
+	g_return_if_fail(user_data);
 
 	name = gtk_widget_get_name(widget);
 	if (name != NULL) {
@@ -738,7 +671,7 @@ paned_save_position(GtkPaned *paned, property_t prop)
 {
 	guint32 pos;
 	
-	g_assert(paned);
+	g_return_if_fail(paned);
 
 	pos = gtk_paned_get_position(paned);
 	gui_prop_set_guint32_val(prop, pos);
@@ -749,7 +682,7 @@ paned_restore_position(GtkPaned *paned, property_t prop)
 {
 	guint32 pos;
 	
-	g_assert(paned);
+	g_return_if_fail(paned);
 
 	gui_prop_get_guint32_val(prop, &pos);
 	gtk_paned_set_position(paned, pos);
@@ -761,7 +694,8 @@ tree_view_save_widths(GtkTreeView *treeview, property_t prop)
 {
 	gint i;
 
-	g_assert(treeview);
+	g_return_if_fail(treeview);
+
 	for (i = 0; i < INT_MAX; i++) {
 		GtkTreeViewColumn *c;
 		guint32 width;
@@ -781,7 +715,8 @@ tree_view_restore_widths(GtkTreeView *treeview, property_t prop)
 {
 	gint i;
 
-	g_assert(treeview);
+	g_return_if_fail(treeview);
+
 	for (i = 0; i < INT_MAX; i++) {
 		GtkTreeViewColumn *c;
 		guint32 width;
@@ -801,7 +736,8 @@ tree_view_save_visibility(GtkTreeView *treeview, property_t prop)
 {
 	guint i;
 
-	g_assert(treeview);
+	g_return_if_fail(treeview);
+
 	for (i = 0; i < INT_MAX; i++) {
 		GtkTreeViewColumn *c;
 		gboolean val;
@@ -819,7 +755,8 @@ tree_view_restore_visibility(GtkTreeView *treeview, property_t prop)
 {
 	guint i;
 
-	g_assert(treeview);
+	g_return_if_fail(treeview);
+
 	for (i = 0; i < INT_MAX; i++) {
 		GtkTreeViewColumn *c;
 		gboolean val;
@@ -834,20 +771,54 @@ tree_view_restore_visibility(GtkTreeView *treeview, property_t prop)
 #endif /* USE_GTK2 */
 
 #ifdef USE_GTK1
+/**
+ * Save widths of columns in given property which must by a boolean array
+ * property with at least as many elements as there are columns.
+ */
+void
+clist_save_widths(GtkCList *clist, property_t prop)
+{
+	gint i;
+
+	g_return_if_fail(clist);
+
+    for (i = 0; i < clist->columns; i++) {
+		guint32 width = clist->column[i].width;
+		gui_prop_set_guint32(prop, &width, i, 1);
+	}
+}
+
+/**
+ * Restore widths of columns from given property which must by a boolean
+ * array property with at least as many elements as there are columns.
+ */
+void
+clist_restore_widths(GtkCList *clist, property_t prop)
+{
+	gint i;
+
+	g_return_if_fail(clist);
+
+    for (i = 0; i < clist->columns; i++) {
+		guint32 width;
+
+		gui_prop_get_guint32(prop, &width, i, 1);
+    	gtk_clist_set_column_width(clist, i, width);
+	}
+}
 /**
  * Save visibility of columns in given property which must by a boolean array
  * property with at least as many elements as there are columns.
  */
 void
-gtk_clist_save_visibility(GtkCList *clist, property_t prop)
+clist_save_visibility(GtkCList *clist, property_t prop)
 {
 	gint i;
-	gboolean val;
 
-	g_assert(clist);
+	g_return_if_fail(clist);
 
     for (i = 0; i < clist->columns; i++) {
-		val = clist->column[i].visible;
+		gboolean val = clist->column[i].visible;
 		gui_prop_set_boolean(prop, &val, i, 1);
 	}
 }
@@ -857,18 +828,109 @@ gtk_clist_save_visibility(GtkCList *clist, property_t prop)
  * array property with at least as many elements as there are columns.
  */
 void
-gtk_clist_restore_visibility(GtkCList *clist, property_t prop)
+clist_restore_visibility(GtkCList *clist, property_t prop)
 {
 	gint i;
-	gboolean val;
 
-	g_assert(clist);
+	g_return_if_fail(clist);
 
     for (i = 0; i < clist->columns; i++) {
+		gboolean val;
+
 		gui_prop_get_boolean(prop, &val, i, 1);
     	gtk_clist_set_column_visibility(clist, i, val);
 	}
 }
 #endif /* USE_GTK1 */
 
+static gboolean
+show_popup_menu(widget_popup_menu_cb handler)
+{
+	GtkMenu *menu;
+
+	g_return_val_if_fail(handler, FALSE);
+
+	menu = (*handler)();
+	if (menu) {
+		int button;
+		
+		/*
+		 * NOTE: Use 0 as button here even though 3 was probably pressed
+		 * (right-click) because under Gtk+ 1.2 the popup will otherwise
+		 * immediately disappear on button release if we use 3. Button 1
+		 * does not work well for Gtk+ 2.x, thus use 0 then.
+		 */
+		button = GTK_CHECK_VERSION(2,0,0) ? 0 : 1;
+		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+			button, gtk_get_current_event_time());
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+#if GTK_CHECK_VERSION(2,0,0)
+static gboolean
+on_popup_menu(GtkWidget *unused_widget, void *user_data)
+{
+	(void) unused_widget;
+	return show_popup_menu(user_data);
+}
+#endif	/* Gtk+ >= 2.0*/
+
+static gboolean
+on_key_press_event(GtkWidget *unused_widget,
+	GdkEventKey *event, void *user_data)
+{
+	(void) unused_widget;
+
+	switch (event->keyval) {
+	unsigned modifier;
+	case GDK_F10:
+	case 0x1008FE0A:	/* Shift+F10 under Xnest */
+		modifier = gtk_accelerator_get_default_mod_mask() & event->state;
+		if (GDK_SHIFT_MASK == modifier) {
+			return show_popup_menu(user_data);
+		}
+		break;
+	}
+	return FALSE;
+}
+
+static gboolean
+on_button_press_event(GtkWidget *unused_widget,
+	GdkEventButton *event, void *user_data)
+{
+	(void) unused_widget;
+
+	if (
+		GDK_BUTTON_PRESS == event->type &&
+		3 == event->button &&
+		0 == (gtk_accelerator_get_default_mod_mask() & event->state)
+	) {
+		return show_popup_menu(user_data);
+	}
+	return FALSE;
+}
+
+/**
+ * "handler" will be called whenever a popup menu is requested for "widget".
+ */
+void
+widget_add_popup_menu(GtkWidget *widget, widget_popup_menu_cb handler)
+{
+	g_return_if_fail(widget);
+	g_return_if_fail(handler);
+
+	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_FOCUS);
+	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
+#if GTK_CHECK_VERSION(2,0,0)
+	gui_signal_connect(widget, "popup-menu", on_popup_menu, handler);
+#endif	/* Gtk+ >= 2.0*/
+
+	gui_signal_connect(widget, "key-press-event", on_key_press_event, handler);
+	gui_signal_connect(widget, "button-press-event", on_button_press_event,
+		handler);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
