@@ -1,5 +1,5 @@
 /*
- * $Id: misc.h 11876 2006-09-05 18:24:29Z cbiere $
+ * $Id: misc.h 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi, Richard Eckart
  *
@@ -27,7 +27,11 @@
 #define _gtk_misc_h_
 
 #include "gui.h"
+
+#include "clipboard.h"
+
 #include "if/ui/gtk/misc.h"
+
 #include "lib/prop.h"
 
 void gui_general_timer(time_t now);
@@ -38,15 +42,6 @@ void gui_save_window(GtkWidget *widget, property_t prop);
 void gui_restore_window(GtkWidget *widget, property_t prop);
 gint gui_parse_geometry_spec(const gchar *spec, guint32 coord[4]);
 
-/*
- * Hit record comparison functions.
- */
-
-gint gui_record_name_eq(gconstpointer rec1, gconstpointer rec2);
-gint gui_record_sha1_eq(gconstpointer rec1, gconstpointer rec2);
-gint gui_record_host_eq(gconstpointer rec1, gconstpointer rec2);
-gint gui_record_sha1_or_name_eq(gconstpointer rec1, gconstpointer rec2);
-
 #ifdef USE_GTK2
 void gui_merge_window_as_tab(GtkWidget *toplvl, GtkWidget *notebook,
 	GtkWidget *window);
@@ -59,11 +54,16 @@ void tree_view_restore_widths(GtkTreeView *treeview, property_t prop);
 #endif /* USE_GTK2 */
 
 #ifdef USE_GTK1
-void gtk_clist_save_visibility(GtkCList *clist, property_t prop);
-void gtk_clist_restore_visibility(GtkCList *clist, property_t prop);
+void clist_save_visibility(GtkCList *clist, property_t prop);
+void clist_save_widths(GtkCList *clist, property_t prop);
+void clist_restore_visibility(GtkCList *clist, property_t prop);
+void clist_restore_widths(GtkCList *clist, property_t prop);
 #endif /* USE_GTK1 */
 
 void paned_save_position(GtkPaned *paned, property_t prop);
 void paned_restore_position(GtkPaned *paned, property_t prop);
 
+typedef GtkMenu *(*widget_popup_menu_cb)(void);
+void widget_add_popup_menu(GtkWidget *, widget_popup_menu_cb);
+
 #endif /* _gtk_misc_h_ */
