@@ -1,5 +1,5 @@
 /*
- * $Id: search_xml.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: search_xml.c 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -35,7 +35,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: search_xml.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: search_xml.c 14457 2007-08-14 04:23:12Z cbiere $")
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -82,7 +82,7 @@ extern GList *filters_current;
  */
 static void parse_xml(xmlNodePtr xmlnode, gpointer user_data);
 static void builtin_to_xml(xmlNodePtr);
-static void search_to_xml(xmlNodePtr, search_t *);
+static void search_to_xml(xmlNodePtr, const struct search *);
 static void filter_to_xml(xmlNodePtr, filter_t *);
 static void rule_to_xml(xmlNodePtr, rule_t *);
 static void xml_to_builtin(xmlNodePtr, gpointer);
@@ -396,7 +396,7 @@ xml_prop_printf(xmlNodePtr node, const gchar *name, const char *fmt, ...)
 void
 search_store_xml(void)
 {
-	const GList *l;
+	const GList *iter;
 	time_t now = tm_time();
     xmlDocPtr doc;
     xmlNodePtr root;
@@ -428,15 +428,17 @@ search_store_xml(void)
     /*
      * Iterate over the searches and add them to the tree
      */
-    for (l = search_gui_get_searches(); l; l = g_list_next(l))
-        search_to_xml(root, l->data);
+    for (iter = search_gui_get_searches(); iter; iter = g_list_next(iter)) {
+        search_to_xml(root, iter->data);
+	}
 
     /*
      * Iterate over the rulesets and add them to the tree.
      * Only those that are not bound to a search.
      */
-    for (l = filters; l; l = g_list_next(l))
-        filter_to_xml(root, l->data);
+    for (iter = filters; NULL != iter; iter = g_list_next(iter)) {
+        filter_to_xml(root, iter->data);
+	}
 
     /*
      * Try to save the file
@@ -494,6 +496,8 @@ search_retrieve_xml(void)
 	gchar *path = NULL;
 	gchar *path_orig = NULL;
 
+	LIBXML_TEST_VERSION
+
 	/*
 	 * We can't use routines from file.c here because libxml2 only defines
 	 * interfaces for parsing a path or memory, but not for parsing a FILE
@@ -552,7 +556,7 @@ search_retrieve_xml(void)
 	    /* if it doesn't have a name */
 	    (root->name == NULL) ||
 	    /* if it isn't a Genealogy node */
-	    g_ascii_strcasecmp((const gchar *) root->name, "Searches") != 0
+	    ascii_strcasecmp((const gchar *) root->name, "Searches") != 0
     ) {
         g_warning("searches file has invalid format: %s", path);
 		xmlFreeDoc(doc);
@@ -652,9 +656,11 @@ search_retrieve_xml(void)
             g_message("verifying bindings...");
 
         for (s = search_gui_get_searches(); s != NULL; s = g_list_next(s)) {
-            search_t *search = s->data;
+            const struct search *search = s->data;
+            const struct filter *filter;
 
-            if (search->filter->search == search) {
+			filter = search_gui_get_filter(search);
+            if (filter->search == search) {
                 if (GUI_PROPERTY(gui_debug) >= 6)
                     g_message("binding ok for: %s", search_gui_query(search));
             } else {
@@ -709,45 +715,56 @@ builtin_to_xml(xmlNodePtr parent)
 }
 
 static void
-search_to_xml(xmlNodePtr parent, search_t *s)
+search_to_xml(xmlNodePtr parent, const struct search *search)
 {
+	gnet_search_t search_handle;
     xmlNodePtr newxml;
     GList *iter;
 
-    g_assert(s != NULL);
-    g_assert(search_gui_query(s) != NULL);
+    g_assert(search != NULL);
+	search_handle = search_gui_get_handle(search);
+    g_assert(guc_search_query(search_handle) != NULL);
     g_assert(parent != NULL);
 
-	if (search_gui_is_browse(s) || search_gui_is_local(s))
-		return;			/* Don't persist "browse host" searches. */
+	if (guc_search_is_browse(search_handle))
+		return;	
+
+	if (guc_search_is_local(search_handle))
+		return;
 
     if (GUI_PROPERTY(gui_debug) >= 6) {
         g_message(
-			"saving search: %s (%p enabled=%d)\n"
+			"saving search: %s (enabled=%d)\n"
 			"  -- filter is bound to: %p\n"
 			"  -- search is         : %p",
-			search_gui_query(s), cast_to_gconstpointer(s),
-			search_gui_is_enabled(s),
-			cast_to_gconstpointer(s->filter->search),
-			cast_to_gconstpointer(s));
+			guc_search_query(search_handle),
+			!guc_search_is_frozen(search_handle),
+			cast_to_gconstpointer(search_gui_get_filter(search)->search),
+			cast_to_gconstpointer(search));
     }
 
     newxml = xml_new_empty_child(parent, NODE_SEARCH);
-    xml_prop_set(newxml, TAG_SEARCH_QUERY, search_gui_query(s));
-
-	xml_prop_printf(newxml, TAG_SEARCH_ENABLED, "%u", search_gui_is_enabled(s));
-    xml_prop_printf(newxml, TAG_SEARCH_PASSIVE, "%u", search_gui_is_passive(s));
+    xml_prop_set(newxml, TAG_SEARCH_QUERY,
+		guc_search_query(search_handle));
+	xml_prop_printf(newxml, TAG_SEARCH_ENABLED, "%u",
+		!guc_search_is_frozen(search_handle));
+    xml_prop_printf(newxml, TAG_SEARCH_PASSIVE, "%u",
+		guc_search_is_passive(search_handle));
     xml_prop_printf(newxml, TAG_SEARCH_REISSUE_TIMEOUT, "%u",
-		guc_search_get_reissue_timeout(s->search_handle));
+		guc_search_get_reissue_timeout(search_handle));
     xml_prop_printf(newxml, TAG_SEARCH_CREATE_TIME, "%s",
-		timestamp_to_string(guc_search_get_create_time(s->search_handle)));
+		timestamp_to_string(guc_search_get_create_time(search_handle)));
     xml_prop_printf(newxml, TAG_SEARCH_LIFETIME, "%u",
-		guc_search_get_lifetime(s->search_handle));
-    xml_prop_printf(newxml, TAG_SEARCH_SORT_COL, "%d", s->sort_col);
-    xml_prop_printf(newxml, TAG_SEARCH_SORT_ORDER, "%d", s->sort_order);
+		guc_search_get_lifetime(search_handle));
+    xml_prop_printf(newxml, TAG_SEARCH_SORT_COL, "%d",
+		search_gui_get_sort_column(search));
+    xml_prop_printf(newxml, TAG_SEARCH_SORT_ORDER, "%d",
+		search_gui_get_sort_order(search));
 
-    for (iter = s->filter->ruleset; iter != NULL; iter = g_list_next(iter))
+    iter = search_gui_get_filter(search)->ruleset;
+    for (/* NOTHING */; iter != NULL; iter = g_list_next(iter)) {
         rule_to_xml(newxml, iter->data);
+	}
 }
 
 
@@ -835,7 +852,7 @@ rule_to_xml(xmlNodePtr parent, rule_t *r)
         newxml = xml_new_empty_child(parent, NODE_RULE_IP);
         xml_prop_set(newxml, TAG_RULE_IP_ADDR,
 			host_addr_to_string(r->u.ip.addr));
-        xml_prop_printf(newxml, TAG_RULE_IP_MASK, "%u", r->u.ip.mask);
+        xml_prop_printf(newxml, TAG_RULE_IP_MASK, "%u", r->u.ip.cidr);
         break;
     case RULE_SIZE:
 		{
@@ -909,7 +926,7 @@ parse_xml(xmlNodePtr xmlnode, gpointer user_data)
 
     for (n = 0; parser_map[n].name != NULL; n ++) {
         if (
-			0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+			0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					(const gchar *) parser_map[n].name)
 		) {
             parser_map[n].parser_func(xmlnode, user_data);
@@ -930,7 +947,7 @@ xml_to_builtin(xmlNodePtr xmlnode, gpointer unused_udata)
 	(void) unused_udata;
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_BUILTIN));
     g_assert(filter_get_show_target() != NULL);
     g_assert(filter_get_drop_target() != NULL);
@@ -1007,7 +1024,7 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
     gint sort_col = SORT_NO_COL, sort_order = SORT_NONE;
     guint32 reissue_timeout;
     xmlNodePtr node;
-    search_t *search;
+    struct search *search;
     guint flags = 0;
 	guint lifetime;
 	time_t create_time;
@@ -1015,7 +1032,7 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
 	(void) unused_udata;
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_SEARCH));
 
     gnet_prop_get_guint32_val(PROP_SEARCH_REISSUE_TIMEOUT, &reissue_timeout);
@@ -1115,7 +1132,7 @@ xml_to_search(xmlNodePtr xmlnode, gpointer unused_udata)
 		 * Also parse all children.
 		 */
 		for (node = xmlnode->children; node != NULL; node = node->next) {
-			parse_xml(node, search->filter);
+			parse_xml(node, search_gui_get_filter(search));
 		}
 	}
 }
@@ -1135,7 +1152,7 @@ xml_to_filter(xmlNodePtr xmlnode, gpointer unused_udata)
 	(void) unused_udata;
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_FILTER));
 
 	buf = STRTRACK(xml_get_string(xmlnode, TAG_FILTER_NAME));
@@ -1226,7 +1243,7 @@ xml_to_text_rule(xmlNodePtr xmlnode, gpointer data)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_RULE_TEXT));
 
     match = STRTRACK(xml_get_string(xmlnode, TAG_RULE_TEXT_MATCH));
@@ -1286,7 +1303,7 @@ xml_to_ip_rule(xmlNodePtr xmlnode, gpointer data)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_RULE_IP));
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_IP_ADDR));
@@ -1353,7 +1370,7 @@ xml_to_size_rule(xmlNodePtr xmlnode, gpointer data)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_RULE_SIZE));
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_SIZE_LOWER));
@@ -1416,7 +1433,7 @@ xml_to_jump_rule(xmlNodePtr xmlnode, gpointer data)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_RULE_JUMP));
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_TARGET));
@@ -1457,7 +1474,7 @@ xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer data)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_RULE_SHA1));
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_SHA1_FILENAME));
@@ -1517,7 +1534,7 @@ xml_to_flag_rule(xmlNodePtr xmlnode, gpointer data)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_RULE_FLAG));
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_FLAG_STABLE));
@@ -1595,7 +1612,7 @@ xml_to_state_rule(xmlNodePtr xmlnode, gpointer data)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(0 == g_ascii_strcasecmp((const gchar *) xmlnode->name,
+    g_assert(0 == ascii_strcasecmp((const gchar *) xmlnode->name,
 					NODE_RULE_STATE));
 
     buf = STRTRACK(xml_get_string(xmlnode, TAG_RULE_STATE_DISPLAY));
