@@ -1,5 +1,5 @@
 /*
- * $Id: search_xml.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: search_xml.h 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -36,7 +36,10 @@
 #ifndef _gtk_search_xml_h_
 #define _gtk_search_xml_h_
 
+#include "if/core/search.h"
+
 void search_store_xml(void);
 gboolean search_retrieve_xml(void);
+gnet_search_t search_gui_get_handle(const struct search *);
 
 #endif	/* _gtk_search_xml_h_ */
