@@ -1,5 +1,5 @@
 /*
- * $Id: settings_cb.c 13990 2007-06-30 00:59:50Z cbiere $
+ * $Id: settings_cb.c 14457 2007-08-14 04:23:12Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,9 +36,8 @@
 
 #include "gui.h"
 
-RCSID("$Id: settings_cb.c 13990 2007-06-30 00:59:50Z cbiere $")
+RCSID("$Id: settings_cb.c 14457 2007-08-14 04:23:12Z cbiere $")
 
-#include "gtk-missing.h"
 #include "misc.h"
 #include "settings_cb.h"
 #include "settings.h"
@@ -130,7 +129,7 @@ on_entry_config_extensions_activate(GtkEditable *editable, gpointer unused_data)
 	(void) unused_data;
     ext = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
     gnet_prop_set_string(PROP_SCAN_EXTENSIONS, ext);
-    g_free(ext);
+    G_FREE_NULL(ext);
 }
 FOCUS_TO_ACTIVATE(entry_config_extensions)
 
@@ -142,7 +141,7 @@ on_entry_config_path_activate(GtkEditable *editable, gpointer unused_udata)
 
 	(void) unused_udata;
     gnet_prop_set_string(PROP_SHARED_DIRS_PATHS, path);
-    g_free(path);
+    G_FREE_NULL(path);
 }
 FOCUS_TO_ACTIVATE(entry_config_path)
 #endif /* USE_GTK1 */
@@ -436,10 +435,7 @@ on_leave_notify(GtkWidget *widget, GdkEventCrossing *unused_event,
 
 	tv = GTK_TREE_VIEW(data);
 	update_tooltip(GTK_TREE_VIEW(widget), NULL);
-	if (tvm_dbg_property) {
-		tree_view_motion_clear_callback(tv, tvm_dbg_property);
-		tvm_dbg_property = NULL;
-	}
+	tree_view_motion_clear_callback(&tvm_dbg_property);
 	return FALSE;
 }
 
@@ -584,8 +580,7 @@ dbg_tree_init(void)
 		renderer = gtk_cell_renderer_text_new();
 
 		if (columns[i].editable) {
-			g_signal_connect(renderer, "edited",
-					G_CALLBACK(on_cell_edited), NULL);
+			gui_signal_connect(renderer, "edited", on_cell_edited, NULL);
 			g_object_set(renderer,
 					"editable", TRUE,
 					(void *) 0);
@@ -623,12 +618,9 @@ dbg_tree_init(void)
 		gtk_tree_view_append_column(tv, column);
 	}
 
-	g_signal_connect(GTK_OBJECT(tv),
-			"enter-notify-event", G_CALLBACK(on_enter_notify), tv);
-	g_signal_connect(GTK_OBJECT(tv),
-			"leave-notify-event", G_CALLBACK(on_leave_notify), tv);
-	g_signal_connect(GTK_OBJECT(tv),
-			"cursor-changed", G_CALLBACK(on_cursor_changed), tv);
+	gui_signal_connect(tv, "enter-notify-event", on_enter_notify, tv);
+	gui_signal_connect(tv, "leave-notify-event", on_leave_notify, tv);
+	gui_signal_connect(tv, "cursor-changed", on_cursor_changed, tv);
 }
 
 void
@@ -968,43 +960,12 @@ on_menu_sidebar_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
 }
 
 void
-on_menu_navtree_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
-{
-	gboolean sidebar, navtree;
-	
-	(void) unused_udata;
-
-	checkmenu_changed(gui, PROP_NAVTREE_VISIBLE, menuitem);
-
-	gui_prop_get_boolean_val(PROP_NAVTREE_VISIBLE, &navtree);
-	gui_prop_get_boolean_val(PROP_SIDEBAR_VISIBLE, &sidebar);
-	if (navtree && !sidebar) {
-		gui_prop_set_boolean_val(PROP_SIDEBAR_VISIBLE, TRUE);
-	}
-	/*
-	 * Gtk+ 2.x automagically moves the gutter when a child's
-	 * visibility status changes.
-	 */
-#ifdef USE_GTK1
-	{
-		GtkPaned *paned;
-		
-		paned = GTK_PANED(gui_main_window_lookup("vpaned_sidebar"));
-		if (navtree) {
-			paned_restore_position(paned, PROP_SIDE_DIVIDER_POS);
-		} else {
-			paned_save_position(paned, PROP_SIDE_DIVIDER_POS);
-			gtk_paned_set_position(paned, 0);
-		}
-	}
-#endif /* USE_GTK1 */
-}
-
-void
-on_menu_toolbar_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
+on_menu_menubar_visible_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
 {
 	(void) unused_udata;
-	checkmenu_changed(gui, PROP_TOOLBAR_VISIBLE, menuitem);
+
+	checkmenu_changed(gui, PROP_MENUBAR_VISIBLE, menuitem);
 }
 
 void
@@ -1086,18 +1047,4 @@ on_menu_autohide_bws_gleaf_activate(GtkMenuItem *menuitem,
 	checkmenu_changed(gui, PROP_AUTOHIDE_BWS_GLEAF, menuitem);
 }
 
-void
-on_popup_search_toggle_tabs_activate(GtkMenuItem *unused_menuitem,
-		gpointer unused_udata)
-{
-    gboolean val;
-
-	(void) unused_menuitem;
-	(void) unused_udata;
-
-    gui_prop_get_boolean_val(PROP_SEARCH_RESULTS_SHOW_TABS, &val);
-    val = !val;
-    gui_prop_set_boolean_val(PROP_SEARCH_RESULTS_SHOW_TABS, val);
-}
-
 /* vi: set ts=4 sw=4 cindent: */
