@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats_cb.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: upload_stats_cb.c 14270 2007-08-01 07:49:39Z cbiere $
  *
  * Copyright (c) 2003, Raphael Manfredi
  * Copyright (c) 2001-2003, Richard Eckart
@@ -38,7 +38,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: upload_stats_cb.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: upload_stats_cb.c 14270 2007-08-01 07:49:39Z cbiere $")
 
 #include "upload_stats_cb.h"
 #include "upload_stats.h"
@@ -185,16 +185,6 @@ on_clist_ul_stats_click_column(GtkCList *clist, gint column,
 	gtk_clist_sort(clist);
 }
 
-void
-on_clist_ul_stats_resize_column(GtkCList *unused_clist,
-	gint column, gint width, gpointer unused_udata)
-{
-	(void) unused_clist;
-	(void) unused_udata;
-    /* FIXME: use properties */
-	*(gint *) &GUI_PROPERTY(ul_stats_col_widths)[column] = width;
-}
-
 #endif /* USE_GTK1 */
 
 #ifdef USE_GTK2
@@ -208,7 +198,8 @@ on_popup_upload_stats_config_cols_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     cc = gtk_column_chooser_new(gui_main_window_lookup("treeview_ul_stats"));
-    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
+    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 0,
+		gtk_get_current_event_time());
 }
 #endif /* USE_GTK2 */
 
