@@ -1,5 +1,5 @@
 /*
- * $Id: uploads_common.c 13500 2007-05-05 17:54:08Z rmanfredi $
+ * $Id: uploads_common.c 14617 2007-08-21 17:46:37Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -25,12 +25,13 @@
 
 #include "gui.h"
 
-RCSID("$Id: uploads_common.c 13500 2007-05-05 17:54:08Z rmanfredi $")
+RCSID("$Id: uploads_common.c 14617 2007-08-21 17:46:37Z rmanfredi $")
 
 #include "uploads.h"			/* For upload_row_data_t */
 #include "uploads_common.h"
-#include "search_common.h"
+#include "search.h"
 #include "settings.h"
+#include "notebooks.h"
 
 #include "if/gui_property.h"
 #include "if/gnet_property.h"
@@ -43,7 +44,8 @@ RCSID("$Id: uploads_common.c 13500 2007-05-05 17:54:08Z rmanfredi $")
 #include "lib/tm.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-#define IO_STALLED		60	/**< If nothing exchanged after that many secs */
+#define IO_STALLED	60	/**< If nothing exchanged after that many secs */
+#define UPDATE_MIN	60	/**< Update screen every minute at least */
 
 /**
  *
@@ -81,7 +83,7 @@ uploads_gui_progress(const gnet_upload_status_t *u,
 
 			requested = data->range_end - data->range_start + 1;
 			done = requested > 0 ? u->pos - data->range_start : 0;
-			progress = filesize_per_100(requested, done) / 100.0;
+			progress = filesize_per_10000(requested, done) / 10000.0;
 		}
 		break;
 	}
@@ -158,34 +160,39 @@ uploads_gui_status_str(const gnet_upload_status_t *u,
     case GTA_UL_CLOSED:
         return _("Transmission complete");
 
-	case GTA_UL_QUEUED:
+	case GTA_UL_QUEUED:		/* Actively queued */
 		{
 			guint32 max_up, cur_up;
 			gboolean queued;
+			guint available = 0;
 			gchar tbuf[64];
 
-			/*
-			 * Status: GTA_UL_QUEUED. When PARQ is enabled, and all upload
-			 * slots are full an upload is placed into the PARQ-upload. Clients
-			 * supporting Queue 0.1 and 1.0 will get an active slot. We
-			 * probably want to display this information
-			 *		-- JA, 06/02/2003
-			 */
-
-
 			gnet_prop_get_guint32_val(PROP_MAX_UPLOADS, &max_up);
 			gnet_prop_get_guint32_val(PROP_UL_RUNNING, &cur_up);
-			queued = u->parq_position > max_up - cur_up;
 
-			/* position 1 should always get an upload slot */
+			if (cur_up < max_up)
+				available = max_up - cur_up;
+
+			/*
+			 * We'll flag as "Waiting" instead of "Queued" uploads
+			 * that are actively queued and whose position is low
+			 * enough to possibly get scheduled at the next request,
+			 * given the amount of free slots.
+			 *		--RAM, 2007-08-21
+			 */
+
+			queued = u->parq_position > available;
+
 			if (u->parq_retry > 0) {
 				gm_snprintf(tbuf, sizeof tbuf,
 							" %s,", short_time(u->parq_retry));
 			} else {
 				tbuf[0] = '\0';
 			}
+
 			gm_snprintf(tmpstr, sizeof tmpstr,
 						_("%s [%d] (slot %d/%d)%s %s %s"),
+						u->parq_frozen ? _("Frozen") :
 						queued ? _("Queued") : _("Waiting"),
 						u->parq_queue_no,
 						u->parq_position,
@@ -302,4 +309,33 @@ uploads_gui_browse_host(host_addr_t addr, guint16 port)
 		search_gui_new_browse_host(NULL, addr, port, NULL, NULL, 0);
 }
 
+static gboolean
+uploads_gui_is_visible(void)
+{
+	return main_gui_window_visible() &&
+		nb_main_page_uploads == main_gui_notebook_get_page();
+}
+
+gboolean
+uploads_gui_update_required(time_t now)
+{
+	static time_t last_update;
+	time_delta_t delta;
+
+	/*
+	 * Usually don't perform updates if nobody is watching.  However,
+	 * we do need to perform periodic cleanup of dead entries or the
+	 * memory usage will grow.  Perform an update every UPDATE_MIN minutes
+	 * at least.
+	 *		--RAM, 28/12/2003
+	 */
+
+	delta = last_update ? delta_time(now, last_update) : UPDATE_MIN;
+	if (0 == delta || (delta < UPDATE_MIN && !uploads_gui_is_visible()))
+		return FALSE;
+
+	last_update = now;
+	return TRUE;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
